@@ -4,7 +4,7 @@
  * by Joern Thyssen <joern@thyssen.nu>, 2000
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,13 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: analysis.h,v 1.35 2006/10/04 12:28:39 c_anthon Exp $
+ * $Id: analysis.h,v 1.46 2009/03/24 23:48:45 c_anthon Exp $
  */
 
 #ifndef _ANALYSIS_H_
 #define _ANALYSIS_H_
 
-#include <list.h>
+#include "list.h"
+#include "gnubg-types.h"
 
 typedef enum _lucktype {
     LUCK_VERYBAD, LUCK_BAD, LUCK_NONE, LUCK_GOOD, LUCK_VERYGOOD
@@ -32,15 +33,15 @@ typedef enum _skilltype {
   SKILL_VERYBAD,
   SKILL_BAD,
   SKILL_DOUBTFUL,
-  SKILL_NONE,
-  SKILL_GOOD
+  SKILL_NONE
 } skilltype;
 
-#define badSkill(st)  ((st) != SKILL_NONE && (st) != SKILL_GOOD)
+#define badSkill(st)  ((st) < SKILL_NONE)
 
-#define N_SKILLS (SKILL_GOOD + 1)
+#define N_SKILLS ((int)SKILL_NONE + 1)
+#define N_LUCKS ((int)LUCK_VERYGOOD + 1)
 
-typedef struct {
+typedef struct _statcontext {
   int fMoves, fCube, fDice; /* which statistics have been computed? */
     
   int anUnforcedMoves[ 2 ];
@@ -54,7 +55,7 @@ typedef struct {
 
   int anMoves[ 2 ][ N_SKILLS ];
 
-  int anLuck[ 2 ][ LUCK_VERYGOOD + 1 ];
+  int anLuck[ 2 ][ N_LUCKS ];
 
   int anCubeMissedDoubleDP[ 2 ];
   int anCubeMissedDoubleTG[ 2 ];
@@ -87,13 +88,6 @@ typedef struct {
   float arVarianceLuckAdj[ 2 ];
   int nGames;
 
-#if USE_TIMECONTROL
-  /* time control */
-
-  int anTimePenalties[ 2 ];
-  float aarTimeLoss[ 2 ][ 2 ];
-#endif /* USE_TIMECONTROL */
-
 } statcontext;
 
 typedef enum {
@@ -101,9 +95,10 @@ typedef enum {
   RAT_BEGINNER, RAT_CASUAL_PLAYER, RAT_INTERMEDIATE, RAT_ADVANCED,
   RAT_EXPERT, RAT_WORLD_CLASS, RAT_SUPERNATURAL, RAT_UNDEFINED
 } ratingtype;
+#define N_RATINGS ((int)RAT_UNDEFINED + 1)
 
-extern const char* aszRating [ RAT_UNDEFINED + 1 ];
-extern const char* aszLuckRating[ 7 ];
+extern const char* aszRating [ N_RATINGS ];
+extern const char* aszLuckRating[ N_LUCKS ];
 
 extern int afAnalysePlayers[ 2 ];
 
@@ -111,17 +106,15 @@ extern ratingtype GetRating ( const float rError );
 extern void IniStatcontext ( statcontext* psc );
 extern void AddStatcontext ( const statcontext* pscA, statcontext* pscB );
 
-extern void
-DumpStatcontext ( char *szOutput, const statcontext *psc, const char * pl, const char * op, const char * sz,
-                  const int fIsMatch );
+extern void DumpStatcontext(char *szOutput, const statcontext *psc, const char *pl, const char *op, int nMatchTo);
 
 extern void
-updateStatisticsGame ( const list* plGame );
+updateStatisticsGame ( const listOLD* plGame );
 
 extern void
-updateStatisticsMatch ( list* plMatch );
+updateStatisticsMatch ( listOLD* plMatch );
 
-extern int getLuckRating(float rLuck);
+extern lucktype getLuckRating(float rLuck);
 
 extern float
 relativeFibsRating(float r, int n);
@@ -150,10 +143,15 @@ absoluteFibsRatingCube( const float rCube, const int n );
 #define NORMALISED   0
 #define UNNORMALISED 1
 
-extern float
+extern void
 getMWCFromError ( const statcontext* psc, float aaaar[ 3 ][ 2 ][ 2 ][ 2 ] );
 
 extern skilltype
 Skill( float r );
 
+extern int MatchAnalysed(void);
+extern float LuckAnalysis(const TanBoard anBoard, int n0, int n1,
+			  matchstate *pms);
+extern lucktype Luck( float r );
+
 #endif
