@@ -4,6 +4,6 @@ set -x
 aclocal -I m4
 libtoolize --force --copy
 autoheader
-automake --add-missing --copy
+automake --add-missing --copy -Wno-portability
 autoconf
 
