@@ -4,7 +4,7 @@
  * by Gary Wong <gary@cs.arizona.edu>, 1997-1999.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,100 +16,94 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: bearoffdump.c,v 1.3 2005/02/21 23:23:07 jsegrave Exp $
+ * $Id: bearoffdump.c,v 1.16 2008/03/18 22:51:34 c_anthon Exp $
  */
 
 #include "config.h"
 
-#include <assert.h>
+#include <glib.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include "getopt.h"
 #include "positionid.h"
 #include "bearoff.h"
+#include "multithread.h"
+#include "backgammon.h"
 
-/* ugly fixes */
-char *aszRNG[ 1 ];
-char *aszSkillType[ 1 ];
-int exsExport;
-int ap;
-
-extern bearoffcontext *
-BearoffInitBuiltin ( void ) {
-
-  printf ( "Make makebearoff build (avoid resursive rules Makefile :-)\n" );
-  return NULL;
-
+#if USE_MULTITHREAD
+extern int MT_GetThreadID(void)
+{
+  return (0);
 }
-/* end ugly fixes */
-
-static void
-show_usage( const char *arg0 ) {
-
-  printf( "Usage: %s file -n id | -p PosID\n"
-          "where file is a bearoff database and id is the number "
-          "of the position to dump. Alternatively, give a "
-          "position ID.\n", arg0 );
-
+extern void MT_Release(void)
+{
+  return;
 }
-
+extern void MT_Exclusive(void)
+{
+  return;
+}
+#else
+extern void CallbackProgress(void)
+{
+}
+#endif
 
 extern int
 main( int argc, char **argv ) {
 
-  char ch, *filename, *szPosID = NULL;
+  char *filename, *szPosID = NULL;
   int id = 0;
   bearoffcontext *pbc;
   char sz[ 4096 ];
   int n;
   int nUs;
   int nThem;
-  int anBoard[ 2 ][ 25 ];
-  int fIsPosID = FALSE;
+  TanBoard anBoard;
 
-  const struct option ao[] = {
-    { "index", required_argument, NULL, 'n' },
-    { "posid", required_argument, NULL, 'p' },
-    { "help", no_argument, NULL, 'h' },
-    { NULL, 0, NULL, 0 }
+  GOptionEntry ao[] = {
+    {"index", 'n', 0, G_OPTION_ARG_INT, &id,
+      "index", NULL},
+    {"posid", 'p', 0, G_OPTION_ARG_STRING, &szPosID,
+      "Position ID", NULL},
+    {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
+  GError *error = NULL;
+  GOptionContext *context;
 
-  if ( argc != 4 ) {
-    show_usage( argv[ 0 ] );
-    exit( 1 );
+  context = g_option_context_new("file");
+  g_option_context_add_main_entries(context, ao, PACKAGE);
+  g_option_context_parse(context, &argc, &argv, &error);
+  g_option_context_free(context);
+  if (error) {
+  	g_printerr("%s\n", error->message);
+  	exit(EXIT_FAILURE);
   }
 
+  if ((szPosID && id) || (!szPosID && !id))
+  {
+	  g_printerr("Either Position ID or index is required. Not Both.\n"
+                     "For more help try `bearoffdump --help'\n");
+	  exit(EXIT_FAILURE);
+  }
+
+  if (argc != 2)
+  {
+	  g_printerr("A bearoff database file should be given as an"
+	     	     "argument\nFor more help try `bearoffdump --help'\n");
+	  exit(EXIT_FAILURE);
+  }
   filename = argv[ 1 ];
 
-  while ( ( ch = getopt_long( argc, argv, "n:p:h", ao, NULL ) )
-          != (char) -1 )
-  switch( ch ) {
-  case 'n': /* position index in db */
-    id = atoi( optarg );
-    break;
-  case 'p': /* position index in db */
-    szPosID = optarg;
-    fIsPosID = TRUE;
-    break;
-  case 'h': /* help */
-    show_usage( argv[ 0 ] );
-    exit( 0 );
-    break;
-  default:
-    show_usage( argv[ 0 ] );
-    exit( 1 );
-}
-
   printf( "Bearoff database: %s\n", filename );
-  if ( fIsPosID ) {
+  if ( !id ) {
     printf( "Position ID     : %s\n", szPosID );
   }
   else {
     printf( "Position number : %d\n", id );
   }
 
-  if (  ! ( pbc = BearoffInit ( filename, NULL, BO_NONE, NULL ) ) ) {
+  if (  ! ( pbc = BearoffInit ( filename, BO_NONE, NULL ) ) ) {
     printf( "Failed to initialise bearoff database %s\n", filename );
     exit(-1);
   }
@@ -127,7 +121,7 @@ main( int argc, char **argv ) {
 
   memset( anBoard, 0, sizeof anBoard );
 
-  if ( fIsPosID ) {
+  if ( !id ) {
     printf( "\n"
             "Dump of position ID: %s\n\n", szPosID );
 
@@ -147,10 +141,10 @@ main( int argc, char **argv ) {
   /* dump req. position */
 
   *sz = 0;
-  BearoffDump( pbc, anBoard, sz );
+  BearoffDump( pbc, (ConstTanBoard)anBoard, sz );
   puts( sz );
 
-  BearoffClose( &pbc );
+  BearoffClose( pbc );
 
   return 0;
 
