@@ -1,6 +1,20 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 3 or later of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
 
-/* $Id: bearoffgammon.c,v 1.3 2005/02/21 23:23:07 jsegrave Exp $ */
+* $Id: bearoffgammon.c,v 1.10 2009/03/01 20:01:48 Superfly_Jon Exp $ */
 
+#include "config.h"
 #include "bearoffgammon.h"
 
 struct GroupInfo {
@@ -7194,45 +7208,47 @@ struct GroupInfo info[63] = {
 /*  63 111111 */
 { {0,0,0,36}, 0, 0, 0}, 
 };
-#include <assert.h>
+#include <glib.h>
 #include "positionid.h"
 
-extern struct GammonProbs*
-getBearoffGammonProbs(int board[6])
+extern struct GammonProbs* getBearoffGammonProbs(const unsigned int board[6])
 {
   int group = 0;
   int i;
   struct GroupInfo* in;
   
-  for(i = 5; i >= 0; --i) { 
-    group += (0x1 << i) * !!board[i];
+  for(i = 5; i >= 0; --i)
+  {
+	  if (board[i] > 0)
+		group += (0x1 << i);
   }
-
   
+  g_assert(group > 0);
   in = &info[group-1];
   if( ! in->info ) {
     return &in->gDefault;
   }
 
   {
-    int grpSize = 0;
-    int b1[6];
+    unsigned int grpSize = 0;
+	unsigned int b1[6] = {0};
     unsigned int k;
-    unsigned int index;
+    unsigned short index;
     
-    for(k = 0; k < 6; ++k) {
-      if( (group & (0x1 << k)) ) {
-	{                                               assert( board[k] > 0 ); }
-      
-	b1[grpSize] = board[k] - 1;
-	++grpSize;
+    for(k = 0; k < 6; ++k)
+	{
+      if( (group & (0x1 << k)) )
+	  {
+		g_assert( board[k] > 0 );
+		b1[grpSize] = board[k] - 1;
+		grpSize++;
       }
     }
 
     index = PositionIndex(grpSize, b1);
 
-    {             assert( index >= in->base && index - in->base < in->size ); }
-    {      assert( in->info[index - in->base] < sizeof(all)/sizeof(all[0]) ); }
+    g_assert( index >= in->base && index - in->base < in->size );
+    g_assert( in->info[index - in->base] < sizeof(all)/sizeof(all[0]) );
   
     return &all[in->info[index - in->base]];
   }
@@ -7798,45 +7814,43 @@ typedef long T[RBG_NPROBS];
 
 static T* y[] = {
   y1, y2, y3, y4, y5, y6, y7, y8, y9, y10, y11, y12, y13, y14, y15, y16,
-  y17, y18, y19, y20, y21, y22, y23, y24, y25, y26, y27, y28, y29, y30, y31, 
+  y17, y18, y19, y20, y21, y22, y23, y24, y25, y26, y27, y28, y29, y30, y31
 };
 
-
-CONST long*
-getRaceBGprobs(int board[6])
+extern long *getRaceBGprobs(const unsigned int board[6])
 {
   unsigned int tot = 0;
-  int group = 0;
   int i;
-  {                                                 assert( board[5] == 0 ); }
+  unsigned int group = 0;
+  g_assert( board[5] == 0 );
   
-  for(i = 4; i >= 0; --i) { 
-    group += (0x1 << i) * !!board[i];
+  for(i = 4; i >= 0; --i)
+  {
+	if (board[i])
+	  group += (0x1 << i);
     tot += board[i];
   }
 
-  if( tot > 6 ) {
+  g_assert( group < 32 );
+
+  if( tot > 6 )
     return 0;
-  }
 
   {
-    int grpSize = 0;
-    int b1[6];
+    unsigned int grpSize = 0;
+	unsigned int b1[6] = {0};
     unsigned int k;
 
-    for(k = 0; k < 5; ++k) {
-      if( (group & (0x1 << k)) ) {
-	{                                            assert( board[k] > 0 ); }
-      
-	b1[grpSize] = board[k] - 1;
-	++grpSize;
+    for(k = 0; k < 5; ++k)
+	{
+      if( (group & (0x1 << k)) )
+	  {
+		g_assert( board[k] > 0 );
+		b1[grpSize] = board[k] - 1;
+		++grpSize;
       }
     }
 
-    {
-      unsigned int const index = PositionIndex(grpSize, b1);
-
-      return y[group-1][index];
-    }
+    return y[group - 1][PositionIndex(grpSize, b1)];
   }
 }
