@@ -6,7 +6,7 @@
 *
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of version 2 of the GNU General Public License as
+* it under the terms of version 3 or later of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
@@ -18,24 +18,27 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: font3d.c,v 1.6.2.1 2006/11/10 11:34:40 c_anthon Exp $
+* $Id: font3d.c,v 1.27 2011/04/06 23:42:53 mdpetch Exp $
 */
 
 #include "config.h"
-
 #include "inc3d.h"
-#include "gtkboard.h"
-#include <assert.h>
-#include "path.h"
+#include "util.h"
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
-#define FONT_VERA "fonts/Vera.ttf"
-#define FONT_VERA_SERIF_BOLD "fonts/VeraSeBd.ttf"
-#define FONT_VERA_BOLD "fonts/VeraBd.ttf"
+#include <assert.h>
 
-#define FONT_SIZE (base_unit / 20.0f)
-#define CUBE_FONT_SIZE (base_unit / 24.0f)
+struct _OGLFont
+{
+	unsigned int glyphs;
+	int advance;
+	int kern[10][10];
+	float scale;
+	float heightRatio;
+	float height;
+	float length;
+};
 
 typedef struct _Point
 {
@@ -50,7 +53,7 @@ typedef struct _Contour
 typedef struct _Vectoriser
 {
 	GArray *contours;
-	int numPoints;
+	unsigned int numPoints;
 } Vectoriser;
 
 typedef struct _Tesselation
@@ -64,48 +67,146 @@ typedef struct _Mesh
 	GArray *tesselations;
 } Mesh;
 
-int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int pointSize, float scale);
-void PopulateVectoriser(Vectoriser* pVect, FT_Outline* pOutline);
-void TidyMemory(Vectoriser* pVect, Mesh* pMesh);
-void PopulateContour(Contour* pContour, FT_Vector* points, char* pointTags, int numberOfPoints);
-void PopulateMesh(Vectoriser* pVect, Mesh* pMesh);
-int MakeGlyph(FT_Outline* pOutline, int list);
+static int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int pointSize, float size, float heightRatio);
+static int RenderText(const char *text, FT_Library ftLib, OGLFont *pFont, const char *pPath, int pointSize, float scale, float heightRatio);
+static void PopulateVectoriser(Vectoriser* pVect, const FT_Outline* pOutline);
+static void TidyMemory(const Vectoriser* pVect, const Mesh* pMesh);
+static void PopulateContour(GArray *contour, const FT_Vector* points, const char* pointTags, int numberOfPoints);
+static void PopulateMesh(const Vectoriser* pVect, Mesh* pMesh);
+static int RenderGlyph(const FT_Outline* pOutline);
 
-int BuildFont3d(BoardData3d* bd3d)
+int CreateNumberFont(OGLFont **ppFont, const char *fontFile, int pitch, float size, float heightRatio)
 {
+	char *filename;
 	FT_Library ftLib;
-        char *file;
 	if (FT_Init_FreeType(&ftLib))
 		return 0;
 
-        file = PathSearch(FONT_VERA, szDataDirectory);
-        if (!CreateOGLFont(ftLib, &bd3d->numberFont, file, 24, FONT_SIZE))
-		return 0;
-        free(file);
-        file = PathSearch(FONT_VERA_SERIF_BOLD, szDataDirectory);
-        if (!CreateOGLFont(ftLib, &bd3d->cubeFont, file, 44, CUBE_FONT_SIZE))
-		return 0;
-        free(file);
+	free(*ppFont);
+	*ppFont = (OGLFont*)malloc(sizeof(OGLFont));
 
+	filename = BuildFilename(fontFile);
+	if (!CreateOGLFont(ftLib, *ppFont, filename, pitch, size, heightRatio))
+	{
+		outputerrf(_("Failed to create font from (%s)\n"), filename);
+		g_free(filename);
+		return 0;
+	}
+	g_free(filename);
 	return !FT_Done_FreeType(ftLib);
 }
 
-int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int pointSize, float scale)
+int CreateFontText(OGLFont **ppFont, const char *text, const char *fontFile, int pitch, float size, float heightRatio)
 {
-	int i, j;
+	char *filename;
+
+	FT_Library ftLib;
+	if (FT_Init_FreeType(&ftLib))
+		return 0;
+
+	free(*ppFont);
+	*ppFont = (OGLFont*)malloc(sizeof(OGLFont));
+
+	filename = BuildFilename(fontFile);
+	if (!RenderText(text, ftLib, *ppFont, filename, pitch, size, heightRatio))
+	{
+		outputerrf(_("Failed to create font from (%s)\n"), filename);
+		g_free(filename);
+		return 0;
+	}
+	g_free(filename);
+	return !FT_Done_FreeType(ftLib);
+}
+
+void FreeNumberFont(OGLFont *ppFont)
+{
+	glDeleteLists(ppFont->glyphs, 10);
+	ppFont->glyphs = 0;
+}
+
+void FreeFontText(OGLFont *ppFont)
+{
+	glDeleteLists(ppFont->glyphs, 1);
+	ppFont->glyphs = 0;
+}
+
+static int RenderText(const char *text, FT_Library ftLib, OGLFont *pFont, const char *pPath, int pointSize, float scale, float heightRatio)
+{
+	int len = 0;
+	FT_Face face;
+
+	if (FT_New_Face(ftLib, pPath, 0, &face))
+		return 0;
+
+	if (FT_Set_Char_Size(face, 0, pointSize * 64 /* 26.6 fractional points */, 0, 0))
+		return 0;
+
+	/* Create text glyphs */
+	pFont->glyphs = glGenLists(1);
+	glNewList(pFont->glyphs, GL_COMPILE);
+
+	pFont->scale = scale;
+	pFont->height = 0;
+
+	while (*text)
+	{
+		/* Draw character */
+		FT_ULong charCode = (FT_ULong)(int)(*text);
+		unsigned int glyphIndex = FT_Get_Char_Index(face, charCode);
+		if (!glyphIndex)
+			return 0;
+
+		if (FT_Load_Glyph(face, glyphIndex, FT_LOAD_NO_HINTING))
+			return 0;
+		assert(face->glyph->format == ft_glyph_format_outline);
+		if (pFont->height == 0)
+			pFont->height = (float)face->glyph->metrics.height * scale * heightRatio / 64;
+
+
+		if (!RenderGlyph(&face->glyph->outline))
+			return 0;
+
+		text++;
+		len += face->glyph->advance.x;
+		/* Move on to next place */
+		if (*text)
+		{
+			unsigned int nextGlyphIndex;
+			int kern = 0;
+			FT_Vector kernAdvance;
+			charCode = (FT_ULong)(int)(*text);
+			nextGlyphIndex = FT_Get_Char_Index(face, charCode);
+			if (nextGlyphIndex && !FT_Get_Kerning(face, glyphIndex, nextGlyphIndex, ft_kerning_unfitted, &kernAdvance))
+				kern = kernAdvance.x;
+				
+			glTranslatef((face->glyph->advance.x + kern) / 64.0f, 0.f, 0.f);
+			len += kern;
+		}
+	}
+	glEndList();
+
+	pFont->length = (len / 64.0f) * pFont->scale;
+
+	return !FT_Done_Face(face);
+}
+
+static int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int pointSize, float scale, float heightRatio)
+{
+	unsigned int i, j;
 	FT_Face face;
 
 	memset(pFont, 0, sizeof(OGLFont));
 	pFont->scale = scale;
+	pFont->heightRatio = heightRatio;
 
-        if (FT_New_Face(ftLib, pPath, 0, &face))
+	if (FT_New_Face(ftLib, pPath, 0, &face))
 		return 0;
 
 	if (FT_Set_Char_Size(face, 0, pointSize * 64 /* 26.6 fractional points */, 0, 0))
 		return 0;
 
 	{	/* Find height of "1" */
-		int glyphIndex = FT_Get_Char_Index(face, '1');
+		unsigned int glyphIndex = FT_Get_Char_Index(face, '1');
 		if (!glyphIndex)
 			return 0;
 
@@ -113,7 +214,7 @@ int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int point
 			(face->glyph->format != ft_glyph_format_outline))
 			return 0;
 
-		pFont->height = (float)face->glyph->metrics.height * scale / 64;
+		pFont->height = (float)face->glyph->metrics.height * scale * heightRatio / 64;
 		pFont->advance = face->glyph->advance.x;
 	}
 
@@ -122,7 +223,7 @@ int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int point
 
 	for (i = 0; i < 10; i++)
 	{
-		int glyphIndex = FT_Get_Char_Index(face, '0' + i);
+		unsigned int glyphIndex = FT_Get_Char_Index(face, '0' + i);
 		if (!glyphIndex)
 			return 0;
 
@@ -130,14 +231,16 @@ int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int point
 			return 0;
 		assert(face->glyph->format == ft_glyph_format_outline);
 
-		if (!MakeGlyph(&face->glyph->outline, pFont->glyphs + i))
+		glNewList(pFont->glyphs + i, GL_COMPILE);
+		if (!RenderGlyph(&face->glyph->outline))
 			return 0;
+		glEndList();
 
 		/* Calculate kerning table */
 		for (j = 0; j < 10; j++)
 		{
 			FT_Vector kernAdvance;
-			int nextGlyphIndex = FT_Get_Char_Index(face, '0' + j);
+			unsigned int nextGlyphIndex = FT_Get_Char_Index(face, '0' + j);
 			if (!nextGlyphIndex || FT_Get_Kerning(face, glyphIndex, nextGlyphIndex, ft_kerning_unfitted, &kernAdvance))
 				return 0;
 			pFont->kern[i][j] = kernAdvance.x;
@@ -147,19 +250,16 @@ int CreateOGLFont(FT_Library ftLib, OGLFont *pFont, const char *pPath, int point
 	return !FT_Done_Face(face);
 }
 
-int MakeGlyph(FT_Outline* pOutline, int list)
+static int RenderGlyph(const FT_Outline* pOutline)
 {
-	Vectoriser vect;
 	Mesh mesh;
 	unsigned int index, point;
 
+	Vectoriser vect;
 	PopulateVectoriser(&vect, pOutline);
-
 	if ((vect.contours->len < 1) || (vect.numPoints < 3))
 		return 0;
 
-	glNewList(list, GL_COMPILE);
-
 	/* Solid font */
 	PopulateMesh(&vect, &mesh);
 
@@ -197,14 +297,12 @@ int MakeGlyph(FT_Outline* pOutline, int list)
 	glDisable(GL_LINE_SMOOTH);
 	glDisable(GL_BLEND);
 
-	glEndList();
-
 	TidyMemory(&vect, &mesh);
 
 	return 1;
 }
 
-int GetKern(OGLFont *pFont, char cur, char next)
+static int GetKern(const OGLFont *pFont, char cur, char next)
 {
 	if (next)
 		return pFont->kern[cur - '0'][next - '0'];
@@ -212,7 +310,7 @@ int GetKern(OGLFont *pFont, char cur, char next)
 		return 0;
 }
 
-extern float getTextLen3d(OGLFont *pFont, const char* str)
+static float getTextLen3d(const OGLFont *pFont, const char* str)
 {
 	int len = 0;
 	while (*str)
@@ -220,29 +318,29 @@ extern float getTextLen3d(OGLFont *pFont, const char* str)
 		len += pFont->advance + GetKern(pFont, str[0], str[1]);
 		str++;
 	}
-	return len / 64.0f * pFont->scale;
+	return (len / 64.0f) * pFont->scale;
 }
 
-extern int RenderString3d(OGLFont *pFont, const char* str)
+static void RenderString3d(const OGLFont *pFont, const char* str)
 {
-	glScalef(pFont->scale, pFont->scale, 1);
+	glScalef(pFont->scale, pFont->scale * pFont->heightRatio, 1.f);
 
 	while (*str)
 	{
+		int offset = *str - '0';
 		/* Draw character */
-		glCallList(pFont->glyphs + (*str - '0'));
+		glCallList(pFont->glyphs + (unsigned int)offset);
 
 		/* Move on to next place */
-		glTranslatef((pFont->advance + GetKern(pFont, str[0], str[1])) / 64.0f, 0, 0);
+		glTranslatef((pFont->advance + GetKern(pFont, str[0], str[1])) / 64.0f, 0.f, 0.f);
 
 		str++;
 	}
-	return 1;
 }
 
-GList *combineList;
+static GList *combineList;
 
-void PopulateVectoriser(Vectoriser* pVect, FT_Outline* pOutline)
+static void PopulateVectoriser(Vectoriser* pVect, const FT_Outline* pOutline)
 {
 	int startIndex = 0;
 	int endIndex;
@@ -261,7 +359,8 @@ void PopulateVectoriser(Vectoriser* pVect, FT_Outline* pOutline)
 		endIndex = pOutline->contours[contourIndex];
 		contourLength = (endIndex - startIndex) + 1;
 
-		PopulateContour(&newContour, pointList, tagList, contourLength);
+		newContour.conPoints = g_array_new(FALSE, FALSE, sizeof(Point));
+		PopulateContour(newContour.conPoints, pointList, tagList, contourLength);
 		pVect->numPoints += newContour.conPoints->len;
 		g_array_append_val(pVect->contours, newContour);
 
@@ -269,12 +368,14 @@ void PopulateVectoriser(Vectoriser* pVect, FT_Outline* pOutline)
 	}
 }
 
-void AddPoint(Contour* pContour, double x, double y)
+static void AddPoint(GArray *points, double x, double y)
 {
-	if (pContour->conPoints->len > 0)
+	if (points->len > 0)
 	{	/* Ignore duplicate contour points */
-		Point* point = &g_array_index(pContour->conPoints, Point, pContour->conPoints->len - 1);
-		if (point->data[0] == x && point->data[1] == y)
+		Point* point = &g_array_index(points, Point, points->len - 1);
+		/* See if point is duplicated (often happens when tesselating)
+		   so suppress lint error 777 (testing floats for equality) */
+		if (/*lint --e(777)*/point->data[0] == x && point->data[1] == y)
 			return;
 	}
 
@@ -284,15 +385,15 @@ void AddPoint(Contour* pContour, double x, double y)
 		newPoint.data[1] = y;
 		newPoint.data[2] = 0;
 
-		g_array_append_val(pContour->conPoints, newPoint);
+		g_array_append_val(points, newPoint);
 	}
 }
 
 #define BEZIER_STEPS 5
 #define BEZIER_STEP_SIZE 0.2f
-double controlPoints[4][2]; /* 2D array storing values of de Casteljau algorithm */
+static double controlPoints[4][2]; /* 2D array storing values of de Casteljau algorithm */
 
-void evaluateQuadraticCurve(Contour* pContour)
+static void evaluateQuadraticCurve(GArray *points)
 {
 	int i;
 	for (i = 0; i <= BEZIER_STEPS; i++)
@@ -310,23 +411,21 @@ void evaluateQuadraticCurve(Contour* pContour)
 		bezierValues[0][0] = (1.0f - t) * bezierValues[0][0] + t * bezierValues[1][0];
 		bezierValues[0][1] = (1.0f - t) * bezierValues[0][1] + t * bezierValues[1][1];
 
-		AddPoint(pContour, bezierValues[0][0], bezierValues[0][1]);
+		AddPoint(points, bezierValues[0][0], bezierValues[0][1]);
 	}
 }
 
-void PopulateContour(Contour* pContour, FT_Vector* points, char* pointTags, int numberOfPoints)
+static void PopulateContour(GArray *contour, const FT_Vector* points, const char* pointTags, int numberOfPoints)
 {
 	int pointIndex;
 
-	pContour->conPoints = g_array_new(FALSE, FALSE, sizeof(Point));
-
 	for (pointIndex = 0; pointIndex < numberOfPoints; pointIndex++)
 	{
 		char pointTag = pointTags[pointIndex];
 
 		if (pointTag == FT_Curve_Tag_On || numberOfPoints < 2)
 		{
-			AddPoint(pContour, points[pointIndex].x, points[pointIndex].y);
+			AddPoint(contour, (double)points[pointIndex].x, (double)points[pointIndex].y);
 		}
 		else
 		{
@@ -351,7 +450,7 @@ void PopulateContour(Contour* pContour, FT_Vector* points, char* pointTags, int
 				controlPoints[1][0] = controlPoint.data[0];		controlPoints[1][1] = controlPoint.data[1];
 				controlPoints[2][0] = nextPoint.data[0];		controlPoints[2][1] = nextPoint.data[1];
 				
-				evaluateQuadraticCurve(pContour);
+				evaluateQuadraticCurve(contour);
 
 				pointIndex++;
 				previousPoint = nextPoint;
@@ -366,28 +465,35 @@ void PopulateContour(Contour* pContour, FT_Vector* points, char* pointTags, int
 			controlPoints[1][0] = controlPoint.data[0];		controlPoints[1][1] = controlPoint.data[1];
 			controlPoints[2][0] = nextPoint.data[0];		controlPoints[2][1] = nextPoint.data[1];
 			
-			evaluateQuadraticCurve(pContour);
+			evaluateQuadraticCurve(contour);
 		}
 	}
 }
 
 /* Unfortunately the glu library doesn't define this callback type precisely
  so it may well cause problems on different platforms / opengl implementations */
-#if WIN32
+#ifdef WIN32
 /* Need to set the callback calling convention for windows */
 #define TESS_CALLBACK APIENTRY
 #else
 #define TESS_CALLBACK
 #endif
 
-Tesselation curTess;
+#if defined(USE_APPLE_OPENGL)
+#define GLUFUN(X) X
+#elif defined(__GNUC__)
+#define GLUFUN(X) (_GLUfuncptr)X
+#else
+#define GLUFUN(X) X
+#endif
 
-void TESS_CALLBACK tcbError(GLenum errCode, Mesh* mesh)
-{
-	assert(0);	/* This is very unlikely to happen... */
+static Tesselation curTess;
+static void TESS_CALLBACK tcbError(GLenum error)
+{	/* This is very unlikely to happen... */
+	g_print("Tesselation error! (%d)\n", error);
 }
 
-void TESS_CALLBACK tcbVertex(void* data, Mesh* pMesh)
+static void TESS_CALLBACK tcbVertex(void* data, Mesh* UNUSED(pMesh))
 {
 	double* vertex = (double*)data;
 	Point newPoint;
@@ -399,28 +505,29 @@ void TESS_CALLBACK tcbVertex(void* data, Mesh* pMesh)
 	g_array_append_val(curTess.tessPoints, newPoint);
 }
 
-void TESS_CALLBACK tcbCombine(double coords[3], void* vertex_data[4], GLfloat weight[4], void** outData, Mesh* pMesh)
+static void TESS_CALLBACK tcbCombine(const double coords[3], const double *UNUSED(vertex_data[4]), GLfloat UNUSED(weight[4]), void** outData)
 {
 	/* Just return vertex position (colours etc. not required) */
 	Point *newEle = (Point*)malloc(sizeof(Point));
+	assert(newEle);
 	memcpy(newEle->data, coords, sizeof(double[3]));
 
 	combineList = g_list_append(combineList, newEle);
 	*outData = newEle;
 }
 
-void TESS_CALLBACK tcbBegin(GLenum type, Mesh* pMesh)
+static void TESS_CALLBACK tcbBegin(GLenum type, Mesh* UNUSED(pMesh))
 {
 	curTess.tessPoints = g_array_new(FALSE, FALSE, sizeof(Point));
 	curTess.meshType = type;
 }
 
-void TESS_CALLBACK tcbEnd(Mesh* pMesh)
+static void TESS_CALLBACK tcbEnd(/*lint -e{818}*/Mesh* pMesh)	/* Declaring pMesh as constant isn't useful as pointer member is modified (error 818) */
 {
 	g_array_append_val(pMesh->tesselations, curTess);
 }
 
-void PopulateMesh(Vectoriser* pVect, Mesh* pMesh)
+static void PopulateMesh(const Vectoriser* pVect, Mesh* pMesh)
 {
 	unsigned int c, p;
 	GLUtesselator* tobj = gluNewTess();
@@ -429,15 +536,15 @@ void PopulateMesh(Vectoriser* pVect, Mesh* pMesh)
 
 	pMesh->tesselations = g_array_new(FALSE, FALSE, sizeof(Tesselation));
 
-	gluTessCallback( tobj, GLU_TESS_BEGIN_DATA, tcbBegin);
-	gluTessCallback( tobj, GLU_TESS_VERTEX_DATA, tcbVertex);
-	gluTessCallback( tobj, GLU_TESS_COMBINE_DATA, tcbCombine);
-	gluTessCallback( tobj, GLU_TESS_END_DATA, tcbEnd);
-	gluTessCallback( tobj, GLU_TESS_ERROR_DATA, tcbError);
+	gluTessCallback( tobj, GLU_TESS_BEGIN_DATA, GLUFUN(tcbBegin));
+	gluTessCallback( tobj, GLU_TESS_VERTEX_DATA, GLUFUN(tcbVertex));
+	gluTessCallback( tobj, GLU_TESS_COMBINE, GLUFUN(tcbCombine));
+	gluTessCallback( tobj, GLU_TESS_END_DATA, GLUFUN(tcbEnd));
+	gluTessCallback( tobj, GLU_TESS_ERROR, GLUFUN(tcbError));
 
-	gluTessProperty( tobj, GLU_TESS_WINDING_RULE, GLU_TESS_WINDING_ODD);
+	gluTessProperty( tobj, GLU_TESS_WINDING_RULE, (double)GLU_TESS_WINDING_ODD);
 
-	gluTessNormal( tobj, 0, 0, 1);
+	gluTessNormal( tobj, 0.0, 0.0, 1.0);
 
 	gluTessBeginPolygon( tobj, pMesh);
 
@@ -461,7 +568,7 @@ void PopulateMesh(Vectoriser* pVect, Mesh* pMesh)
 	gluDeleteTess( tobj);
 }
 
-void TidyMemory(Vectoriser* pVect, Mesh* pMesh)
+static void TidyMemory(const Vectoriser* pVect, const Mesh* pMesh)
 {
 	GList *pl;
 	unsigned int c, i;
@@ -486,23 +593,46 @@ void TidyMemory(Vectoriser* pVect, Mesh* pMesh)
 	g_list_free(combineList);
 }
 
-extern void glPrintPointNumbers(BoardData3d* bd3d, const char *text)
+extern void glPrintPointNumbers(const OGLFont *numberFont, const char *text)
 {
 	/* Align horizontally */
-	glTranslatef(-getTextLen3d(&bd3d->numberFont, text) / 2.0f, 0, 0);
-	RenderString3d(&bd3d->numberFont, text);
+	glTranslatef(-getTextLen3d(numberFont, text) / 2.0f, 0.f, 0.f);
+	RenderString3d(numberFont, text);
 }
 
-extern void glPrintCube(BoardData3d* bd3d, const char *text)
+extern void glPrintCube(const OGLFont *cubeFont, const char *text)
 {
 	/* Align horizontally and vertically */
-	glTranslatef(-getTextLen3d(&bd3d->cubeFont, text) / 2.0f, -bd3d->cubeFont.height / 2.0f, 0);
-	RenderString3d(&bd3d->cubeFont, text);
+	float saveScale = 0;
+	float heightOffset = -cubeFont->height;
+	if (strlen(text) > 1)
+	{	/* Make font smaller for 2 digit cube numbers */
+		saveScale = cubeFont->scale;
+		((OGLFont *)cubeFont)->scale *= CUBE_TWODIGIT_FACTOR;
+		heightOffset *= CUBE_TWODIGIT_FACTOR;
+	}
+	glTranslatef(-getTextLen3d(cubeFont, text) / 2.0f, heightOffset / 2.0f, 0.f);
+	RenderString3d(cubeFont, text);
+
+	if (strlen(text) > 1)
+		((OGLFont *)cubeFont)->scale = saveScale;
 }
 
-extern void glPrintNumbersRA(BoardData3d* bd3d, const char *text)
+extern void glPrintNumbersRA(const OGLFont *numberFont, const char *text)
 {
 	/* Right align */
-	glTranslatef(-getTextLen3d(&bd3d->numberFont, text), 0, 0);
-	RenderString3d(&bd3d->numberFont, text);
+	glTranslatef(-getTextLen3d(numberFont, text), 0.f, 0.f);
+	RenderString3d(numberFont, text);
+}
+
+extern float GetFontHeight3d(const OGLFont *font)
+{
+	return font->height;
+}
+
+extern void glDrawText(const OGLFont *font)
+{
+	glTranslatef(-font->length / 2.f, -font->height / 2.f, 0.f);
+	glScalef(font->scale, font->scale, 1.f);
+	glCallList(font->glyphs);
 }
