@@ -6,7 +6,7 @@
 *
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of version 2 of the GNU General Public License as
+* it under the terms of version 3 or later of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
@@ -18,68 +18,56 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: graph.c,v 1.9 2004/04/30 17:35:40 Superfly_Jon Exp $
+* $Id: graph.c,v 1.29 2011/10/31 09:41:14 c_anthon Exp $
 */
 
-#include <config.h>
-#include <GL/gl.h>
-#include <GL/glu.h>
-#if !WIN32
-#include <GL/glx.h>
-#endif
-#if HAVE_GTKGLEXT
-#include <gtk/gtkgl.h>
-#include <gtk/gtkglwidget.h>
-#else
-#include <gtkgl/gtkglarea.h>
-#endif
-#include <gtk/gtk.h>
-#include <stdlib.h>
+
+#include "config.h"
 #include "inc3d.h"
+
 #include "renderprefs.h"
+#include "gtklocdefs.h"
 
-#if HAVE_GTKGLEXT
-extern GdkGLConfig *glconfig;
-#endif
+struct _GraphData
+{
+	float ***data;
+	unsigned int numGames;
+	float maxY;
+};
 
-#define BAR_WIDTH 5
+#define COL_WIDTH 5.f
 #define MID_GAP 1
 #define INTER_GAP 4
 #define TOTAL_GAP 5
-#define RES_WIDTH (2 * BAR_WIDTH + MID_GAP + INTER_GAP)
+#define RES_WIDTH (2 * COL_WIDTH + MID_GAP + INTER_GAP)
 #define NUM_WIDTH (modelWidth * NUM_WIDTH_PER)
 #define NUM_WIDTH_PER .1f
 #define NUM_HEIGHT (modelHeight * NUM_HEIGHT_PER)
 #define NUM_HEIGHT_PER .15f
-#define TOT_WIDTH (NUM_HEIGHT * 3)
+/*#define TOT_WIDTH (NUM_HEIGHT * 3)*/
 
-float modelWidth, modelHeight;
-BoardData fonts;
-Texture total;
+static float modelWidth, modelHeight;
+static OGLFont *numberFont = NULL, *totalText = NULL;
 
-static gboolean graph_button_press_event(GtkWidget *board, GdkEventButton *event, void* arg)
+static gboolean graph_button_press_event(void)
 {
 	return FALSE;
 }
 
-static gboolean configure_event(GtkWidget *widget, GdkEventConfigure *notused, GraphData* gd)
+static gboolean configure_event(GtkWidget *widget, GdkEventConfigure *UNUSED(eventDetails), const GraphData* gd)
 {
 	int width, height;
 	float maxY, maxX;
-#if HAVE_GTKGLEXT
-	/*** OpenGL BEGIN ***/
+	GtkAllocation allocation;
 	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
 
 	if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
 		return FALSE;
-#else
-	if (!gtk_gl_area_make_current(GTK_GL_AREA(widget)))
-		return FALSE;
-#endif
-	width = widget->allocation.width;
-	height = widget->allocation.height;
 
-	maxX = (float)(gd->numGames * RES_WIDTH + RES_WIDTH + TOTAL_GAP);
+	gtk_widget_get_allocation (widget, &allocation);
+	width = allocation.width;
+	height = allocation.height;
+	maxX = (float)gd->numGames * RES_WIDTH + RES_WIDTH + TOTAL_GAP;
 	modelWidth = maxX * (1 + NUM_WIDTH_PER);
 
 	maxY = gd->maxY * 1.05f + 1;
@@ -88,145 +76,134 @@ static gboolean configure_event(GtkWidget *widget, GdkEventConfigure *notused, G
 	glViewport(0, 0, width, height);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-	glOrtho(0, modelWidth, 0, modelHeight, -1, 0);
+	glOrtho(0.0, (double)modelWidth, 0.0, (double)modelHeight, -1.0, 0.0);
 	glMatrixMode(GL_MODELVIEW);
 	glLoadIdentity();
 
-#if HAVE_GTKGLEXT
 	gdk_gl_drawable_gl_end(gldrawable);
-	/*** OpenGL END ***/
-#endif
 
 	return TRUE;
 }
 
-static void realize(GtkWidget *widget, void* arg)
+static void realize(GtkWidget *widget, const GraphData* UNUSED(gd))
 {
-#if HAVE_GTKGLEXT
-	/*** OpenGL BEGIN ***/
 	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
 
 	if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
 		return;
-#else
-    if (!gtk_gl_area_make_current(GTK_GL_AREA(widget)))
-		return;
-#endif
 	/* Deep blue background colour */
-	glClearColor(.2f, .2f, .4f, 1);
+	glClearColor(.2f, .2f, .4f, 1.f);
 
-	BuildFont(&fonts);
+	if (!CreateNumberFont(&numberFont, FONT_VERA, FONT_PITCH, FONT_SIZE, FONT_HEIGHT_RATIO))
+		g_print("Error creating font\n");
 
-	total.texID = 0;
-	LoadTexture(&total, TEXTURE_PATH"total.bmp", TF_BMP);
+	if (!CreateFontText(&totalText, _("Totals"), FONT_VERA, FONT_PITCH, FONT_SIZE, FONT_HEIGHT_RATIO))
+		g_print("Error creating font\n");
+
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-#if HAVE_GTKGLEXT
 	gdk_gl_drawable_gl_end(gldrawable);
 	/*** OpenGL END ***/
-#endif
 }
 
-void DrawBar(float col[4], float x, float y, float first, float second)
+static void DrawBar(const float col[4], float x, float y, float first, float second)
 {
 	glPushMatrix();
-	glTranslatef(x, y, 0);
+	glTranslatef(x, y, 0.f);
 
 	glBegin(GL_QUADS);
-		glColor3f(0, 1, 0);
-		glVertex2f(0, 0);
-		glVertex2f(BAR_WIDTH, 0);
-		glVertex2f(BAR_WIDTH, first);
-		glVertex2f(0, first);
+		glColor3f(0.f, 1.f, 0.f);
+		glVertex2f(0.f, 0.f);
+		glVertex2f(COL_WIDTH, 0.f);
+		glVertex2f(COL_WIDTH, first);
+		glVertex2f(0.f, first);
 
-		glColor3f(0, 0, 1);
-		glVertex2f(0, first);
-		glVertex2f(BAR_WIDTH, first);
-		glVertex2f(BAR_WIDTH, first + second);
-		glVertex2f(0, first + second);
+		glColor3f(0.f, 0.f, 1.f);
+		glVertex2f(0.f, first);
+		glVertex2f(COL_WIDTH, first);
+		glVertex2f(COL_WIDTH, first + second);
+		glVertex2f(0.f, first + second);
 	glEnd();
 
-	glLineWidth(3);
+	glLineWidth(3.f);
 	glColor3f(col[0], col[1], col[2]);
 	glBegin(GL_LINE_STRIP);
-		glVertex2f(0, 0);
-		glVertex2f(0, first + second);
-		glVertex2f(BAR_WIDTH, first + second);
-		glVertex2f(BAR_WIDTH, 0);
+		glVertex2f(0.f, 0.f);
+		glVertex2f(0.f, first + second);
+		glVertex2f(COL_WIDTH, first + second);
+		glVertex2f(COL_WIDTH, 0.f);
 	glEnd();
-	glLineWidth(1);
+	glLineWidth(1.f);
 	glBegin(GL_POINTS);
-		glVertex2f(0, first + second);
-		glVertex2f(BAR_WIDTH, first + second);
+		glVertex2f(0.f, first + second);
+		glVertex2f(COL_WIDTH, first + second);
 	glEnd();
 
 	glPopMatrix();
 }
 
-static void DrawColourBar(int player, float x, float y, float first, float second)
+static void DrawColourBar(int playerCol, float x, float y, float first, float second)
 {
 	float col[4];
 	int i;
 	renderdata *prd = GetMainAppearance();
 	for (i = 0; i < 4; i++)
 	{
-		if (prd->fDisplayType == DT_2D)
-			col[i] = (float)prd->aarColour[player][i];
+		if (display_is_2d(prd))
+			col[i] = (float)prd->aarColour[playerCol][i];
 		else
-			col[i] = prd->ChequerMat[player].ambientColour[i];
+			col[i] = prd->ChequerMat[playerCol].ambientColour[i];
 	}
 	DrawBar(col, x, y, first, second);
 }
 
-static void DrawBars(int num, float **values, int total)
+static void DrawBars(unsigned int num, float * const *values, int totalBar)
 {
-	float x = NUM_WIDTH + RES_WIDTH * num;
-	if (total)
+	float x = NUM_WIDTH + RES_WIDTH * (float)num;
+	if (totalBar)
 		x += TOTAL_GAP;
 
 	DrawColourBar(0, x + INTER_GAP / 2.0f, NUM_HEIGHT, values[0][0], values[0][1]);
-	DrawColourBar(1, x + INTER_GAP / 2.0f + BAR_WIDTH + MID_GAP, NUM_HEIGHT, values[1][0], values[1][1]);
+	DrawColourBar(1, x + INTER_GAP / 2.0f + COL_WIDTH + MID_GAP, NUM_HEIGHT, values[1][0], values[1][1]);
 }
 
-void PrintBottomNumber(int num, float width, float height, float x, float y)
+static void PrintBottomNumber(unsigned int num, float width, float height, float x, float y)
 {
 	char numStr[10];
 	sprintf(numStr, "%d", num);
 
 	glPushMatrix();
-	glTranslatef(x, y, 0);
+	glTranslatef(x, y, 0.f);
 
-	glColor3f(1, 1, 1);
-	glScalef(width, height, 1);
-	glPushMatrix();
-	glPrintCube(&fonts, numStr, 0);
-	glPopMatrix();
-	glPrintCube(&fonts, numStr, 1);
+	glColor3f(1.f, 1.f, 1.f);
+	glScalef(width, height, 1.f);
+	glLineWidth(.5f);
+	glPrintCube(numberFont, numStr);
 	glPopMatrix();
 }
 
-void PrintSideNumber(int num, float width, float height, float x, float y)
+static void PrintSideNumber(int num, float width, float height, float x, float y)
 {
 	char numStr[10];
 	sprintf(numStr, "%d", num);
 
 	glPushMatrix();
-	glTranslatef(x, y, 0);
+	glTranslatef(x, y, 0.f);
+
+	glScalef(width, height, 1.f);
+	glLineWidth(.5f);
+	glPrintNumbersRA(numberFont, numStr);
 
-	glScalef(width, height, 1);
-	glPushMatrix();
-	glPrintNumbersRA(&fonts, numStr, 0);
-	glPopMatrix();
-	glPrintNumbersRA(&fonts, numStr, 1);
 	glPopMatrix();
 }
 
-void DrawLeftAxis(GraphData *pgd)
+static void DrawLeftAxis(const GraphData *pgd)
 {
 	int scale[] = {1, 5, 10, 20, 50, 100, 0};
 	int* pScale = scale;
 	int i, numPoints, pointInc;
 
-	while (pScale[1] && pgd->maxY > *pScale * 5)
+	while (pScale[1] && pgd->maxY > *pScale * 5.f)
 		pScale++;
 
 	pointInc = *pScale;
@@ -237,8 +214,8 @@ void DrawLeftAxis(GraphData *pgd)
 	for (i = 1; i <= numPoints; i++)
 	{
 		float y = NUM_HEIGHT;
-		y += i * pointInc;
-		glColor3f(1, 1, 1);
+		y += (float)i * pointInc;
+		glColor3f(1.f, 1.f, 1.f);
 		PrintSideNumber(i * pointInc, NUM_WIDTH * 10, NUM_HEIGHT * 10, NUM_WIDTH - 1, y);
 
 		glColor3f(.5f, .5f, .5f);
@@ -252,35 +229,27 @@ void DrawLeftAxis(GraphData *pgd)
 	}
 }
 
-void DrawGraph(GraphData *gd)
+static void DrawGraph(const GraphData *gd)
 {
-	int i;
+	unsigned int i;
 	float lastx = 0;
 
-	if (total.texID)
-	{
-		glEnable(GL_TEXTURE_2D);
-		glBindTexture(GL_TEXTURE_2D, total.texID);
+	glPushMatrix();
+	glTranslatef(NUM_WIDTH + RES_WIDTH * (float)gd->numGames + TOTAL_GAP + COL_WIDTH + (INTER_GAP + MID_GAP) / 2.0f,
+		NUM_HEIGHT / 2, 0.f);
 
-		glPushMatrix();
-		glTranslatef(NUM_WIDTH + RES_WIDTH * gd->numGames + TOTAL_GAP + (INTER_GAP + MID_GAP) / 2.0f,
-			NUM_HEIGHT / 2.0f - TOT_WIDTH / 6.0f, 0);
+	glColor3f(1.f, 1.f, 1.f);
+	glLineWidth(.5f);
+	glScalef(NUM_WIDTH * 10, NUM_HEIGHT * 10, 1.f);
+	glDrawText(totalText);
 
-		glBegin(GL_QUADS);
-			glTexCoord2f(0, 0); glVertex3f(0, 0, 0);
-			glTexCoord2f(1, 0); glVertex3f(BAR_WIDTH * 2, 0, 0);
-			glTexCoord2f(1, 1); glVertex3f(BAR_WIDTH * 2, TOT_WIDTH, 0);
-			glTexCoord2f(0, 1); glVertex3f(0, TOT_WIDTH, 0);
-		glEnd();
-		glDisable(GL_TEXTURE_2D);
-		glPopMatrix();
-	}
+	glPopMatrix();
 
 	DrawLeftAxis(gd);
 
 	for (i = 0; i < gd->numGames; i++)
 	{
-		float x = NUM_WIDTH + RES_WIDTH * i + BAR_WIDTH + (INTER_GAP + MID_GAP) / 2.0f;
+		float x = NUM_WIDTH + RES_WIDTH * (float)i + COL_WIDTH + (INTER_GAP + MID_GAP) / 2.0f;
 
 		DrawBars(i, gd->data[i], 0);
 
@@ -295,7 +264,7 @@ void DrawGraph(GraphData *gd)
 	DrawBars(i, gd->data[i], 1);
 
 	/* Axis */
-	glColor3f(1, 1, 1);
+	glColor3f(1.f, 1.f, 1.f);
 	glBegin(GL_LINES);
 		glVertex2f(NUM_WIDTH, NUM_HEIGHT);
 		glVertex2f(modelWidth - 1, NUM_HEIGHT);
@@ -307,55 +276,51 @@ void DrawGraph(GraphData *gd)
 	glEnd();
 }
 
-static gboolean expose_event(GtkWidget *widget, GdkEventExpose *event, GraphData* gd)
+static gboolean expose_event(GtkWidget *widget, GdkEventExpose *UNUSED(eventDetails), const GraphData* gd)
 {
-#if HAVE_GTKGLEXT
-	/*** OpenGL BEGIN ***/
 	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
 
 	if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
 		return TRUE;
-#else
-
-	/* OpenGL functions can be called only if make_current returns true */
-	if (!gtk_gl_area_make_current(GTK_GL_AREA(widget)))
-		return TRUE;
-#endif
 	CheckOpenglError();
 
 	glClear(GL_COLOR_BUFFER_BIT);
 
 	DrawGraph(gd);
 
-#if HAVE_GTKGLEXT
 	gdk_gl_drawable_swap_buffers(gldrawable);
 
 	gdk_gl_drawable_gl_end(gldrawable);
-	/*** OpenGL END ***/
-#else
-	gtk_gl_area_swapbuffers(GTK_GL_AREA(widget));
-#endif
 
 	return TRUE;
 }
 
+static void destroy_event(GtkWidget *UNUSED(widget), void *UNUSED(data))
+{
+	if (numberFont != NULL)
+	{
+		FreeNumberFont(numberFont);
+		numberFont = NULL;
+	}
+	if (totalText != NULL)
+	{
+		FreeFontText(totalText);
+		totalText = NULL;
+	}
+}
+
 GtkWidget* StatGraph(GraphData* pgd)
 {
 	float f1, f2;
 	GtkWidget* pw;
-#if HAVE_GTKGLEXT
+
 	/* Drawing area for OpenGL */
 	pw = gtk_drawing_area_new();
 	/* Set OpenGL-capability to the widget - no list sharing */
-	gtk_widget_set_gl_capability(pw, glconfig, NULL, TRUE, GDK_GL_RGBA_TYPE);
-#else
-	pw = gtk_gl_area_new_vargs(NULL, /* no sharing of lists */
-			 GDK_GL_RGBA, GDK_GL_DOUBLEBUFFER, GDK_GL_STENCIL_SIZE, 1, GDK_GL_NONE);
-#endif
-	if (pw == NULL)
+	if ((pw == NULL) || !gtk_widget_set_gl_capability(pw, getGlConfig(), NULL, TRUE, GDK_GL_RGBA_TYPE))
 	{
-		g_print("Can't create opengl drawing widget\n");
-		return 0;
+		g_print("Can't create opengl capable widget\n");
+		return NULL;
 	}
 
 	f1 = pgd->data[pgd->numGames][0][0] + pgd->data[pgd->numGames][0][1];
@@ -365,26 +330,33 @@ GtkWidget* StatGraph(GraphData* pgd)
 		pgd->maxY = .5f;
 
 	gtk_widget_set_events(pw, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
-	gtk_signal_connect(GTK_OBJECT(pw), "button_press_event", GTK_SIGNAL_FUNC(graph_button_press_event), pgd);
-	gtk_signal_connect(GTK_OBJECT(pw), "realize", GTK_SIGNAL_FUNC(realize), pgd);
-	gtk_signal_connect(GTK_OBJECT(pw), "configure_event", GTK_SIGNAL_FUNC(configure_event), pgd);
-	gtk_signal_connect(GTK_OBJECT(pw), "expose_event", GTK_SIGNAL_FUNC(expose_event), pgd);
+	g_signal_connect(G_OBJECT(pw), "button_press_event", G_CALLBACK(graph_button_press_event), pgd);
+	g_signal_connect(G_OBJECT(pw), "realize", G_CALLBACK(realize), pgd);
+	g_signal_connect(G_OBJECT(pw), "configure_event", G_CALLBACK(configure_event), pgd);
+	g_signal_connect(G_OBJECT(pw), "expose_event", G_CALLBACK(expose_event), pgd);
+	g_signal_connect(G_OBJECT(pw), "destroy", G_CALLBACK(destroy_event), NULL);
 
 	return pw;
 }
 
-void SetNumGames(GraphData* pgd, int numGames)
+void SetNumGames(GraphData* pgd, unsigned int numGames)
 {
 	pgd->numGames = numGames;
 	pgd->data = Alloc3d(numGames + 1, 2, 2);
 }
 
+extern GraphData *CreateGraphData(void)
+{
+	return (GraphData*)malloc(sizeof(GraphData));
+}
+
 void TidyGraphData(GraphData* pgd)
 {
 	Free3d(pgd->data, pgd->numGames, 2);
+	free(pgd);
 }
 
-void AddGameData(GraphData* pgd, int game, statcontext *psc)
+void AddGameData(/*lint -e{818}*/GraphData* pgd, int game, const statcontext *psc)
 {
 	float aaaar[3][2][2][2];
 	float f1, s1, f2, s2;
