@@ -6,7 +6,7 @@
 *
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of version 2 of the GNU General Public License as
+* it under the terms of version 3 or later of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
@@ -18,46 +18,31 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: gtkcolour3d.c,v 1.26 2006/09/11 22:59:40 Superfly_Jon Exp $
+* $Id: gtkcolour3d.c,v 1.42 2008/03/07 21:30:48 c_anthon Exp $
 */
 
 #include "config.h"
-#include <string.h>
 #include "inc3d.h"
-#include <gdk/gdkkeysyms.h>
-#include "gtkboard.h"
-#include <glib/gi18n.h>
-
-#include <gtk/gtkgl.h>
+#include "misc3d.h"
 
 #include "gtkcolour.h"
-#include "gtkwindows.h"
+#include "gtkprefs.h"
 
-extern void UpdatePreview(GtkWidget **ppw);
-static void RenderPreview(Material* pMat, unsigned char* buf);
-
-#define COLOUR_SEL_DIA( pcp ) GTK_COLOR_SELECTION_DIALOG( GTK_COLOUR_PICKER( \
-	pcp )->pwColourSel )
-#define COLOUR_SEL( pcp ) GTK_COLOR_SELECTION( COLOUR_SEL_DIA(pcp)->colorsel )
-
-GtkWidget *pcpAmbient, *pcpDiffuse, *pcpSpecular;
-GtkAdjustment *padjShine, *padjOpacity;
-GtkWidget *psOpacity, *pOpacitylabel, *pTexturelabel, *pwPreview, *textureCombo;
-Material col3d;
-Material cancelValue;
-GdkPixmap *xppm = 0;
-int useOpacity, useTexture;
-float opacityValue;
-int bUpdate = TRUE;
-GLUquadricObj *qobj;
-char lastTextureStr[NAME_SIZE + 1];
-GtkWidget* dialogParent;
-GdkWindow* refWind;
+static GtkWidget *pcpAmbient, *pcpDiffuse, *pcpSpecular;
+static GtkAdjustment *padjShine, *padjOpacity;
+static GtkWidget *psOpacity, *pOpacitylabel, *pTexturelabel, *pwPreview, *textureCombo;
+static Material col3d;
+static Material cancelValue;
+static int useOpacity, useTexture;
+static float opacityValue;
+static int bUpdate = TRUE;
+static GLUquadricObj *qobj;
+static char lastTextureStr[NAME_SIZE + 1];
 
 /* Store the previews details here */
 #define MAX_DETAILS 15
-UpdateDetails details[MAX_DETAILS];
-int curDetail;
+static UpdateDetails details[MAX_DETAILS];
+static int curDetail;
 
 /* Size of screen widget */
 #define PREVIEW_WIDTH 200
@@ -65,20 +50,21 @@ int curDetail;
 
 /* World sizes */
 #define STRIP_WIDTH 100
-#define STRIP_HEIGHT 10
+#define STRIP_HEIGHT 10.f
 
-unsigned char auch[PREVIEW_WIDTH * PREVIEW_HEIGHT * 3];
+static int previewLightLevels[3];
 
-extern GtkWidget *pwPrevBoard;
+static int IsFlagSet(int flags, int bit)
+{
+	return ((flags & bit) == bit) ? TRUE : FALSE;
+}
 
-int previewLightLevels[3];
-
-void SetPreviewLightLevel(int levels[3])
+void SetPreviewLightLevel(const int levels[3])
 {
 	memcpy(previewLightLevels, levels, sizeof(int[3]));
 }
 
-void SetupLight()
+static void SetupLight(void)
 {
 	float al[4], dl[4], sl[4];
 	float lp[4] = {PREVIEW_WIDTH / 2, PREVIEW_HEIGHT / 2, 50, 1};
@@ -110,7 +96,8 @@ static void Draw(Material* pMat)
 		char buf[100];
 		strcpy(buf, TEXTURE_PATH);
 		strcat(buf, pMat->textureInfo->file);
-		LoadTexture(&texture, buf);
+		if (!LoadTexture(&texture, buf))
+			return;
 
 		gluQuadricTexture(qobj, GL_TRUE);
 
@@ -120,12 +107,12 @@ static void Draw(Material* pMat)
 		glPushMatrix();
 		if (pMat->textureInfo->type == TT_HINGE)
 		{
-			glScalef(1, HINGE_SEGMENTS, 1);
+			glScalef(1.f, HINGE_SEGMENTS, 1.f);
 		}
 		else
 		{
-			glRotatef(-90, 0, 0, 1);
-			glScalef(TEXTURE_SCALE / texture.width, TEXTURE_SCALE / texture.height, 1);
+			glRotatef(-90.f, 0.f, 0.f, 1.f);
+			glScalef(TEXTURE_SCALE / texture.width, TEXTURE_SCALE / texture.height, 1.f);
 		}
 		glMatrixMode(GL_MODELVIEW);
 	}
@@ -143,10 +130,10 @@ static void Draw(Material* pMat)
 
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 	glPushMatrix();
-	glTranslatef(edge, STRIP_HEIGHT / 2, 0);
-	glRotatef(90, 0, 1, 0);
+	glTranslatef(edge, STRIP_HEIGHT / 2, 0.f);
+	glRotatef(90.f, 0.f, 1.f, 0.f);
 	/* -edge to give a black outline */
-	gluCylinder(qobj, STRIP_HEIGHT / 2 - edge, STRIP_HEIGHT / 2 - edge, STRIP_WIDTH - edge * 2, 36, 36);
+	gluCylinder(qobj, (double)STRIP_HEIGHT / 2 - edge, (double)STRIP_HEIGHT / 2 - edge, (double)STRIP_WIDTH - edge * 2, 36, 36);
 	glPopMatrix();
 
 	if (pMat->textureInfo)
@@ -160,21 +147,9 @@ static void Draw(Material* pMat)
 		glDisable(GL_BLEND);
 }
 
-static void UpdatePreviewBar(Material* pMat, GdkPixmap *pixmap)
-{
-	GdkGC *gc;
-	Material Copy = *pMat;
-
-	RenderPreview(&Copy, auch);
-
-	gc = gdk_gc_new(pixmap);
-	gdk_draw_rgb_image(pixmap, gc, 0, 0, PREVIEW_WIDTH, PREVIEW_HEIGHT, GDK_RGB_DITHER_MAX,
-					  auch, PREVIEW_WIDTH * 3 );
-	gdk_gc_unref( gc );
-}
-
-void UpdateColourPreview(void *arg)
+static void UpdateColourPreview(void *notused)
 {
+	TextureInfo* tempTexture;
 	double ambient[4], diffuse[4], specular[4];
 
 	if (!bUpdate)
@@ -187,20 +162,20 @@ void UpdateColourPreview(void *arg)
 	gtk_colour_picker_get_colour(GTK_COLOUR_PICKER(pcpDiffuse), diffuse);
 	gtk_colour_picker_get_colour(GTK_COLOUR_PICKER(pcpSpecular), specular);
 
+	tempTexture = col3d.textureInfo;	/* Remeber texture, as setupmat resets it */
 	SetupMat(&col3d, (float)ambient[0], (float)ambient[1], (float)ambient[2],
 		(float)diffuse[0], (float)diffuse[1], (float)diffuse[2],
 		(float)specular[0], (float)specular[1], (float)specular[2],
 		(int)padjShine->value, opacityValue);
-
-	UpdatePreviewBar(&col3d, xppm);
+	col3d.textureInfo = tempTexture;
 
 	gtk_widget_queue_draw(pwPreview);
 }
 
-void SetupColourPreview()
+static void SetupColourPreview(void)
 {
 	InitGL(0);
-	glClearColor(0,0,0,1);
+	glClearColor(0.f, 0.f, 0.f, 1.f);
 
 	qobj = gluNewQuadric();
 	gluQuadricDrawStyle(qobj, GLU_FILL);
@@ -209,49 +184,12 @@ void SetupColourPreview()
 	glViewport(0, 0, PREVIEW_WIDTH, PREVIEW_HEIGHT);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-	glOrtho(0, STRIP_WIDTH, 0, STRIP_HEIGHT, -STRIP_HEIGHT, 0);
+	glOrtho(0.0, (double)STRIP_WIDTH, 0.0, (double)STRIP_HEIGHT, (double)-STRIP_HEIGHT, 0.0);
 	glMatrixMode(GL_MODELVIEW);
 	glLoadIdentity();
 }
 
-GdkGLConfig *getglconfigSingle();
-
-GdkGLPixmap *glpixmap;
-static GdkGLContext *glPixmapContext = NULL;
-
-void CreatePreview()
-{
-	GdkGLDrawable *gldrawable;
-	glpixmap = gdk_pixmap_set_gl_capability(xppm, getglconfigSingle(), NULL);
-	gldrawable = GDK_GL_DRAWABLE(glpixmap);
-	glPixmapContext = gdk_gl_context_new(gldrawable, NULL, FALSE, GDK_GL_RGBA_TYPE);
-
-	if (!gdk_gl_drawable_gl_begin (gldrawable, glPixmapContext))
-		return;
-
-	SetupColourPreview();
-
-	gdk_gl_drawable_gl_end (gldrawable);
-}
-
-static void RenderPreview(Material* pMat, unsigned char* buf)
-{
-	/*** OpenGL BEGIN ***/
-	GdkGLDrawable *gldrawable = GDK_GL_DRAWABLE(glpixmap);
-
-	if (!gdk_gl_drawable_gl_begin (gldrawable, glPixmapContext))
-		return;
-
-	SetupLight();
-	Draw(pMat);
-
-	glReadPixels(0, 0, PREVIEW_WIDTH, PREVIEW_HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, buf);
-
-	gdk_gl_drawable_gl_end(gldrawable);
-	/*** OpenGL END ***/
-}
-
-void TextureChange(GtkList *list, gpointer user_data)
+static void TextureChange(void)
 {
 	char* current = (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(textureCombo)->entry));
 
@@ -268,7 +206,59 @@ void TextureChange(GtkList *list, gpointer user_data)
 	}
 }
 
-void AddWidgets(GdkWindow* pixWind, GtkWidget *window)
+static gboolean expose_event_preview3d(GtkWidget *widget, GdkEventExpose *notused, Material* pMat)
+{
+	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
+	if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
+		return TRUE;
+
+	CheckOpenglError();
+
+	SetupLight();
+	Draw(pMat);
+
+	gdk_gl_drawable_swap_buffers(gldrawable);
+
+	gdk_gl_drawable_gl_end(gldrawable);
+
+	return TRUE;
+}
+
+static void realize_preview3d(GtkWidget *widget, void* notused)
+{
+	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
+	if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
+		return;
+
+	SetupColourPreview();
+
+	gdk_gl_drawable_gl_end(gldrawable);
+}
+
+static GtkWidget *CreateGLPreviewWidget(Material* pMat)	// Rename this (and the one above to CreateGLBoardWidget)
+{
+	GtkWidget *p3dWidget = gtk_drawing_area_new();
+
+	/* Set OpenGL-capability to the widget - no list sharing */
+	if (!gtk_widget_set_gl_capability(p3dWidget, getGlConfig(), NULL, TRUE, GDK_GL_RGBA_TYPE))
+	{
+		g_print("Can't create opengl capable widget\n");
+		return NULL;
+	}
+
+	if (p3dWidget == NULL)
+	{
+		g_print("Can't create opengl drawing widget\n");
+		return NULL;
+	}
+
+	g_signal_connect(G_OBJECT(p3dWidget), "realize", G_CALLBACK(realize_preview3d), NULL);
+	g_signal_connect(G_OBJECT(p3dWidget), "expose_event", G_CALLBACK(expose_event_preview3d), pMat);
+
+	return p3dWidget;
+}
+
+static void AddWidgets(GtkWidget *window)
 {
 	GtkWidget *table, *label, *scale;
 	table = gtk_table_new(5, 4, TRUE);
@@ -291,22 +281,20 @@ void AddWidgets(GdkWindow* pixWind, GtkWidget *window)
 
 	label = gtk_label_new(_("Shine:"));
 	gtk_table_attach_defaults(GTK_TABLE (table), label, 2, 3, 1, 2);
-	padjShine = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 128, 1, 10, 0));
-	gtk_signal_connect_object( GTK_OBJECT( padjShine ),
+	padjShine = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 128.0, 1.0, 10.0, 0.0));
+	g_signal_connect( G_OBJECT( padjShine ),
 			       "value-changed",
-			       GTK_SIGNAL_FUNC( UpdateColourPreview ),
-			       0 );
+			       G_CALLBACK( UpdateColourPreview ), NULL);
 	scale = gtk_hscale_new(padjShine);
 	gtk_scale_set_digits( GTK_SCALE( scale ), 0 );
 	gtk_table_attach_defaults(GTK_TABLE (table), scale, 3, 4, 1, 2);
 
 	pOpacitylabel = gtk_label_new(_("Opacity:"));
 	gtk_table_attach_defaults(GTK_TABLE (table), pOpacitylabel, 0, 1, 2, 3);
-	padjOpacity = GTK_ADJUSTMENT(gtk_adjustment_new(0, 1, 100, 1, 10, 0));
-	gtk_signal_connect_object( GTK_OBJECT( padjOpacity ),
+	padjOpacity = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 1.0, 100.0, 1.0, 10.0, 0.0));
+	g_signal_connect( G_OBJECT( padjOpacity ),
 			       "value-changed",
-			       GTK_SIGNAL_FUNC( UpdateColourPreview ),
-			       0 );
+			       G_CALLBACK( UpdateColourPreview ), NULL);
 	psOpacity = gtk_hscale_new(padjOpacity);
 	gtk_scale_set_digits( GTK_SCALE( psOpacity ), 0 );
 	gtk_table_attach_defaults(GTK_TABLE (table), psOpacity, 1, 2, 2, 3);
@@ -316,24 +304,24 @@ void AddWidgets(GdkWindow* pixWind, GtkWidget *window)
 	textureCombo = gtk_combo_new();
 	gtk_combo_set_value_in_list(GTK_COMBO(textureCombo), TRUE, FALSE);
 	gtk_widget_set_sensitive(GTK_WIDGET(GTK_COMBO(textureCombo)->entry), FALSE);
-	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(textureCombo)->list), "selection-changed",
-							GTK_SIGNAL_FUNC(TextureChange), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO(textureCombo)->list), "selection-changed",
+							G_CALLBACK(TextureChange), 0);
 	gtk_table_attach_defaults(GTK_TABLE (table), textureCombo, 2, 4, 3, 4);
 
 	label = gtk_label_new(_("Preview:"));
 	gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 3, 4);
 
-	pwPreview = gtk_pixmap_new(xppm, NULL);
+	pwPreview  = CreateGLPreviewWidget(&col3d);
 	gtk_table_attach_defaults(GTK_TABLE (table), pwPreview, 0, 2, 4, 5);
 }
 
 static gboolean OkClicked(GtkWidget *pw, UpdateDetails* pDetails)
 {	/* Apply new settings */
 	char* texStr;
-	GdkGC *gc;
 
 	/* Copy new settings to preview material */
-	*pDetails->pMat = col3d;
+	pDetails->mat = col3d;
+	*pDetails->pBoardMat = col3d;
 
 	if (useTexture)
 	{
@@ -343,18 +331,13 @@ static gboolean OkClicked(GtkWidget *pw, UpdateDetails* pDetails)
 			col3d.textureInfo = 0;
 		else
 		{
-			BoardData *bd = BOARD(pwPrevBoard)->board_data;
-			ClearTextures(&bd->bd3d);
-			GetTextures(&bd->bd3d, bd->rd);
+			BoardData *bd = (BOARD(pwPrevBoard))->board_data;
+			ClearTextures(bd->bd3d);
+			GetTextures(bd->bd3d, bd->rd);
 		}
 	}
-	gc = gdk_gc_new(pDetails->pixmap);
-	gdk_draw_rgb_image(pDetails->pixmap, gc, 0, 0, PREVIEW_WIDTH, PREVIEW_HEIGHT, GDK_RGB_DITHER_MAX,
-						auch, PREVIEW_WIDTH * 3 );
-	gdk_gc_unref( gc );
-
-	gtk_widget_queue_draw(pDetails->preview);
 	UpdatePreview(0);
+	gtk_widget_queue_draw(pDetails->preview);
 
 	/* Close dialog */
     gtk_widget_destroy(gtk_widget_get_toplevel(pw));
@@ -362,7 +345,7 @@ static gboolean OkClicked(GtkWidget *pw, UpdateDetails* pDetails)
 	return TRUE;
 }
 
-void setCol(GtkColourPicker* pCP, float val[4])
+static void setCol(GtkColourPicker* pCP, const float val[4])
 {
 	double dval[4];
 	int i;
@@ -372,16 +355,15 @@ void setCol(GtkColourPicker* pCP, float val[4])
 	gtk_colour_picker_set_colour(pCP, dval);
 }
 
-static void UpdateColour3d(GtkWidget *pw, UpdateDetails* pDetails)
+static void UpdateColour3d(GtkWidget *notused, UpdateDetails* pDetails)
 {
 	GtkWidget* pwColourDialog3d = GTKCreateDialog(_("3d Colour selection"), DT_QUESTION, 
-		pDetails->preview, DIALOG_FLAG_MODAL, GTK_SIGNAL_FUNC(OkClicked), pDetails);
+		pDetails->preview, DIALOG_FLAG_MODAL, G_CALLBACK(OkClicked), pDetails);
 
-	gtk_signal_connect(GTK_OBJECT(pwColourDialog3d), "realize", GTK_SIGNAL_FUNC(UpdateColourPreview), 0 );
+	g_signal_connect(G_OBJECT(pwColourDialog3d), "realize", G_CALLBACK(UpdateColourPreview), 0 );
 
-	AddWidgets(pDetails->preview->window, DialogArea(pwColourDialog3d, DA_MAIN));
-
-	col3d = *pDetails->pMat;
+	col3d = pDetails->mat;
+	AddWidgets(DialogArea(pwColourDialog3d, DA_MAIN));
 
 	/* Avoid updating preview */
 	bUpdate = FALSE;
@@ -391,27 +373,27 @@ static void UpdateColour3d(GtkWidget *pw, UpdateDetails* pDetails)
 	setCol(GTK_COLOUR_PICKER(pcpDiffuse), col3d.diffuseColour);
 	setCol(GTK_COLOUR_PICKER(pcpSpecular), col3d.specularColour);
 
-	gtk_adjustment_set_value (padjShine, (float)col3d.shine);
-	if (IsSet(pDetails->opacity, DF_VARIABLE_OPACITY))
+	gtk_adjustment_set_value (padjShine, (double)col3d.shine);
+	if (IsFlagSet(pDetails->opacity, DF_VARIABLE_OPACITY))
 	{
 		useOpacity = 1;
 		gtk_adjustment_set_value(padjOpacity, 
-			col3d.alphaBlend ? (col3d.ambientColour[3] + .001f) * 100 : 100);
+			col3d.alphaBlend ? (col3d.ambientColour[3] + .001) * 100 : 100.0);
 	}
 	else
 	{
 		useOpacity = 0;
-		if (IsSet(pDetails->opacity, DF_FULL_ALPHA))
+		if (IsFlagSet(pDetails->opacity, DF_FULL_ALPHA))
 			opacityValue = 1;
 		else
 			opacityValue = 0;
 	}
 
-	if (IsSet(pDetails->texture, TT_NONE))
+	if (IsFlagSet(pDetails->textureType, TT_NONE))
 		useTexture = 0;
 	else
 	{
-		GList *glist = GetTextureList(pDetails->texture);
+		GList *glist = GetTextureList(pDetails->textureType);
 		*lastTextureStr = '\0';	/* Don't update values */
 		gtk_combo_set_popdown_strings(GTK_COMBO(textureCombo), glist);
 		g_list_free(glist);
@@ -424,8 +406,7 @@ static void UpdateColour3d(GtkWidget *pw, UpdateDetails* pDetails)
 		else
 			strcpy(lastTextureStr, NO_TEXTURE_STRING);
 		useTexture = 1;
-
-		gtk_widget_set_sensitive(GTK_WIDGET(textureCombo), !IsSet(pDetails->texture, TT_DISABLED));
+		gtk_widget_set_sensitive(GTK_WIDGET(textureCombo), !IsFlagSet(pDetails->textureType, TT_DISABLED));
 	}
 
 	/* Copy material - to reset if cancel pressed */
@@ -452,63 +433,52 @@ static void UpdateColour3d(GtkWidget *pw, UpdateDetails* pDetails)
 	gtk_main();
 }
 
-void ResetPreviews()
+extern void ResetPreviews(void)
 {
 	curDetail = 0;
 }
 
-int GetPreviewId()
+extern int GetPreviewId(void)
 {
 	return curDetail - 1;
 }
 
 void UpdateColPreview(int ID)
 {
-	UpdatePreviewBar(details[ID].pMat, details[ID].pixmap);
+	details[ID].mat = *details[ID].pBoardMat;
 	gtk_widget_queue_draw(details[ID].preview);
 }
 
-void UpdateColPreviews()
+extern void UpdateColPreviews(void)
 {
 	int i;
 	for (i = 0; i < curDetail; i++)
-		UpdatePreviewBar(details[i].pMat, details[i].pixmap);
+		UpdateColPreview(i);
 }
 
-extern void Setup3dColourPicker(GtkWidget* parent, GdkWindow* wind)
+GtkWidget* gtk_colour_picker_new3d(Material* pMat, int opacity, TextureType textureType)
 {
-	dialogParent = parent;
-	refWind = wind;
-	if (!xppm)
-		xppm = gdk_pixmap_new(refWind, PREVIEW_WIDTH, PREVIEW_HEIGHT, -1);
-	CreatePreview();
-}
-
-GtkWidget* gtk_colour_picker_new3d(Material* pMat, int opacity, int texture)
-{
-	GtkWidget *pixmapwid, *button;
-	GdkPixmap *pixmap;
-	pixmap = gdk_pixmap_new(refWind, PREVIEW_WIDTH, PREVIEW_HEIGHT, -1);
-
-	button = gtk_button_new();
-	pixmapwid = gtk_pixmap_new(pixmap, NULL);
-	gtk_container_add(GTK_CONTAINER(button), pixmapwid);
+	GtkWidget *previewWidget, *button = gtk_button_new();
 
 	if (curDetail == MAX_DETAILS)
 	{
 		g_print("Error: Too many 3d colour previews\n");
 		return 0;
 	}
-	details[curDetail].pMat = pMat;
-	details[curDetail].pixmap = pixmap;
+	details[curDetail].mat = *pMat;
+	previewWidget = CreateGLPreviewWidget(&details[curDetail].mat);
+	gtk_widget_set_size_request (previewWidget, PREVIEW_WIDTH, PREVIEW_HEIGHT);
+	gtk_container_add(GTK_CONTAINER(button), previewWidget);
+
+	details[curDetail].pBoardMat = pMat;	/* Material on board */
 	details[curDetail].preview = button;
 	details[curDetail].opacity = opacity;
-	details[curDetail].texture = texture;
+	details[curDetail].textureType = textureType;
 
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-				   GTK_SIGNAL_FUNC(UpdateColour3d), &details[curDetail]);
+	g_signal_connect(G_OBJECT(button), "clicked",
+				   G_CALLBACK(UpdateColour3d), &details[curDetail]);
 
-	UpdatePreviewBar(pMat, pixmap);
 	curDetail++;
+
 	return button;
 }
