@@ -6,7 +6,7 @@
 *
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of version 2 of the GNU General Public License as
+* it under the terms of version 3 or later of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
@@ -18,16 +18,12 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: matrix.c,v 1.7 2006/06/27 17:36:00 Superfly_Jon Exp $
+* $Id: matrix.c,v 1.21 2008/07/13 19:09:35 c_anthon Exp $
 */
 
 #include "config.h"
-#include <math.h>
-#include <stdio.h>
-#include <memory.h>
-
-#include "matrix.h"
-#include "common.h"
+#include "inc3d.h"
+#include <glib/gstdio.h>
 
 void setIdMatrix(float m[4][4])
 {
@@ -35,7 +31,7 @@ void setIdMatrix(float m[4][4])
 	copyMatrix(m, id);
 }
 
-void mult_matrix_vec(float mat[4][4], float src[4], float dst[4])
+void mult_matrix_vec(const float mat[4][4], const float src[4], float dst[4])
 {
     dst[0] = (src[0] * mat[0][0] + src[1] * mat[0][1] +
 		src[2] * mat[0][2] + src[3] * mat[0][3]);
@@ -50,7 +46,11 @@ void mult_matrix_vec(float mat[4][4], float src[4], float dst[4])
 		src[2] * mat[3][2] + src[3] * mat[3][3]);
 }
 
-void matrixmult(float m[4][4], float b[4][4])
+/*
+void matrixmult(float m[4][4], const float b[4][4])
+Causes compiler warnings on gcc/linux
+*/
+void matrixmult(float m[4][4], const float b[4][4])
 {
 	int i, j, c;
 	float a[4][4] = {{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}};
@@ -69,7 +69,7 @@ void matrixmult(float m[4][4], float b[4][4])
 	copyMatrix(m, a);
 }
 
-void makeInverseTransposeMatrix(float m[4][4], float v[3])
+void makeInverseTransposeMatrix(float m[4][4], const float v[3])
 {
 	setIdMatrix(m);
 	m[0][3] = -v[0];
@@ -80,9 +80,9 @@ void makeInverseTransposeMatrix(float m[4][4], float v[3])
 /* Simple rotation matrices */
 void makeInverseRotateMatrixX(float m[4][4], float degRot)
 {
-	float radRot = -(degRot * (float)PI) / 180.0f; 
-	float cosRot = (float)cos(radRot);
-	float sinRot = (float)sin(radRot);
+	float radRot = -(degRot * (float)G_PI) / 180.0f; 
+	float cosRot = cosf(radRot);
+	float sinRot = sinf(radRot);
 
 	setIdMatrix(m);
 	m[1][1] = cosRot;
@@ -93,9 +93,9 @@ void makeInverseRotateMatrixX(float m[4][4], float degRot)
 
 void makeInverseRotateMatrixY(float m[4][4], float degRot)
 {
-	float radRot = -(degRot * (float)PI) / 180.0f; 
-	float cosRot = (float)cos(radRot);
-	float sinRot = (float)sin(radRot);
+	float radRot = -(degRot * (float)G_PI) / 180.0f; 
+	float cosRot = cosf(radRot);
+	float sinRot = sinf(radRot);
 
 	setIdMatrix(m);
 	m[0][0] = cosRot;
@@ -106,9 +106,9 @@ void makeInverseRotateMatrixY(float m[4][4], float degRot)
 
 void makeInverseRotateMatrixZ(float m[4][4], float degRot)
 {
-	float radRot = -(degRot * (float)PI) / 180.0f; 
-	float cosRot = (float)cos(radRot);
-	float sinRot = (float)sin(radRot);
+	float radRot = -(degRot * (float)G_PI) / 180.0f; 
+	float cosRot = cosf(radRot);
+	float sinRot = sinf(radRot);
 
 	setIdMatrix(m);
 	m[0][0] = cosRot;
@@ -117,24 +117,26 @@ void makeInverseRotateMatrixZ(float m[4][4], float degRot)
 	m[1][0] = sinRot;
 }
 
-/* Generic rotation matrix - just used for testing */
+/* Test functions */
+#if 0
+/* Generic rotation matrix - for comparisions */
 void makeInverseRotateMatrix(float m[4][4], float degRot, float x, float y, float z)
 {
-	float radRot = -(degRot * (float)PI) / 180.0f; 
+	float radRot = -(degRot * (float)G_PI) / 180.0f; 
 	float sqnorm = x * x + y * y + z * z;
 	float sin_theta;
 	float q[4];
 
 	radRot *= 0.5f;
-	sin_theta = (float)sin(radRot);
+	sin_theta = sinf(radRot);
 
 	if (sqnorm != 1) 
-		sin_theta /= (float)sqrt(sqnorm);
+		sin_theta /= sqrtf(sqnorm);
 
 	q[0] = sin_theta * x;
 	q[1] = sin_theta * y;
 	q[2] = sin_theta * z;
-	q[3] = (float)cos(radRot);
+	q[3] = cosf(radRot);
 
 {
     float s, xs, ys, zs, wx, wy, wz, xx, xy, xz, yy, yz, zz;
@@ -179,8 +181,7 @@ void makeInverseRotateMatrix(float m[4][4], float degRot, float x, float y, floa
 
 }
 
-/* Test function */
-void dumpMatrix(float m[4][4])
+void dumpMatrix(const float m[4][4])
 {
 	static int create = 1;
 	int i, j;
@@ -189,12 +190,14 @@ void dumpMatrix(float m[4][4])
 	if (create)
 	{
 		create = 0;
-		fp = fopen("test.txt", "w");
+		fp = g_fopen("test.txt", "w");
 	}
 	else
 	{
-		fp = fopen("test.txt", "a");
+		fp = g_fopen("test.txt", "a");
 	}
+	if (!fp)
+		return;
 
 	for (i = 0; i < 4; i++)
 	{
@@ -205,3 +208,4 @@ void dumpMatrix(float m[4][4])
 	fprintf(fp, "\n");
 	fclose(fp);
 }
+#endif
