@@ -6,7 +6,7 @@
 *
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of version 2 of the GNU General Public License as
+* it under the terms of version 3 or later of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
@@ -18,27 +18,27 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: shadow.c,v 1.9 2006/09/21 22:24:45 Superfly_Jon Exp $
+* $Id: shadow.c,v 1.18 2007/12/29 14:32:32 Superfly_Jon Exp $
 */
 
 #include "config.h"
+#include "inc3d.h"
 
-#include <assert.h>
-#include <stdio.h>
-#include <math.h>
-#include <stdlib.h>
+static int midStencilVal;
 
-#include "shadow.h"
-#include "model.h"
-#include "matrix.h"
-
-int midStencilVal;
+extern int ShadowsInitilised(const BoardData3d *bd3d)
+{
+	return (bd3d != NULL && bd3d->shadowsInitialised);
+}
 
 void shadowInit(BoardData3d *bd3d, renderdata *prd)
 {
 	int i;
 	int stencilBits;
 
+	if (bd3d->shadowsInitialised)
+		return;
+
 	/* Darkness as percentage of ambient light */
 	prd->dimness = ((prd->lightLevels[1] / 100.0f) * (100 - prd->shadowDarkness)) / 100;
 
@@ -52,11 +52,14 @@ void shadowInit(BoardData3d *bd3d, renderdata *prd)
 		g_print("No stencil buffer - no shadows\n");
 		return;
 	}
-	midStencilVal = (int)pow(2, stencilBits - 1);
+	midStencilVal = powi(2, stencilBits - 1);
 	glClearStencil(midStencilVal);
-}
 
-void draw_shadow_volume_edges(Occluder* pOcc)
+	bd3d->shadowsInitialised = TRUE;
+}
+#ifdef TEST_HARNESS
+void GenerateShadowEdges(const Occluder* pOcc);
+extern void draw_shadow_volume_edges(Occluder* pOcc)
 {
 	if (pOcc->show)
 	{
@@ -66,27 +69,51 @@ void draw_shadow_volume_edges(Occluder* pOcc)
 		glPopMatrix();
 	}
 }
-
-void draw_shadow_volume_extruded_edges(Occluder* pOcc, float light_position[4], int prim)
+#endif
+extern void draw_shadow_volume_extruded_edges(/*lint -e{818}*/Occluder* pOcc, const float light_position[4], unsigned int prim)
 {
 	if (pOcc->show)
 	{
 		float olight[4];
-		mult_matrix_vec(pOcc->invMat, light_position, olight);
+		mult_matrix_vec((ConstMatrix)pOcc->invMat, light_position, olight);
 
-		glPushMatrix();
-		moveToOcc(pOcc);
-		glBegin(prim);
-		GenerateShadowVolume(pOcc, olight);
-		glEnd();
-		glPopMatrix();
+		glNewList(pOcc->shadow_list, GL_COMPILE);
+			glPushMatrix();
+			moveToOcc(pOcc);
+			glBegin(prim);
+			GenerateShadowVolume(pOcc, olight);
+			glEnd();
+			glPopMatrix();
+		glEndList();
 	}
 }
 
-void draw_shadow_volume_to_stencil(BoardData3d* bd3d)
+int renderingBase = FALSE;
+
+static void DrawShadows(const BoardData3d* bd3d)
 {
 	int i;
+	if (renderingBase)
+	{
+		glCallList(bd3d->Occluders[OCC_BOARD].shadow_list);
+		if (bd3d->Occluders[OCC_HINGE1].show)
+		{
+			glCallList(bd3d->Occluders[OCC_HINGE1].shadow_list);
+			glCallList(bd3d->Occluders[OCC_HINGE2].shadow_list);
+		}
+	}
+	else
+	{
+		for (i = 0; i < NUM_OCC; i++)
+		{
+			if (bd3d->Occluders[i].show)
+				glCallList(bd3d->Occluders[i].shadow_list);
+		}
+	}
+}
 
+static void draw_shadow_volume_to_stencil(const BoardData3d* bd3d)
+{
 	/* First clear the stencil buffer */
 	glClear(GL_STENCIL_BUFFER_BIT);
 
@@ -101,14 +128,12 @@ void draw_shadow_volume_to_stencil(BoardData3d* bd3d)
 	glCullFace(GL_FRONT);
 	glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
 
-	for (i = 0; i < NUM_OCC; i++)
-		draw_shadow_volume_extruded_edges(&bd3d->Occluders[i], bd3d->shadow_light_position, GL_QUADS);
+	DrawShadows(bd3d);
 
 	glCullFace(GL_BACK);
 	glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
 
-	for (i = 0; i < NUM_OCC; i++)
-		draw_shadow_volume_extruded_edges(&bd3d->Occluders[i], bd3d->shadow_light_position, GL_QUADS);
+	DrawShadows(bd3d);
 
 	/* Enable colour buffer, lighting and depth buffer */
 	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
@@ -116,14 +141,23 @@ void draw_shadow_volume_to_stencil(BoardData3d* bd3d)
 	glDepthMask(GL_TRUE);
 }
 
-void shadowDisplay(void (*drawScene)(BoardData *, BoardData3d *, renderdata *), BoardData* bd)
+void shadowDisplay(void (*drawScene)(const BoardData *, const BoardData3d *, const renderdata *), const BoardData* bd, const BoardData3d *bd3d, const renderdata *prd)
 {
 	/* Pass 1: Draw model, ambient light only (some diffuse to vary shadow darkness) */
 	float zero[4] = {0,0,0,0};
 	float d1[4];
 	float specular[4];
 	float diffuse[4];
-	renderdata *prd = bd->rd;
+
+	drawScene(bd, bd3d, prd);
+
+	/* Create shadow volume in stencil buffer */
+	glEnable(GL_STENCIL_TEST);
+	draw_shadow_volume_to_stencil(bd3d);
+
+	/* Pass 2: Redraw model, full light in non-shadowed areas */
+	glStencilFunc(GL_NOTEQUAL, midStencilVal, (GLuint)~0);
+	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
 
 	glGetLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
 	d1[0] = d1[1] = d1[2] = d1[3] = prd->dimness;
@@ -132,20 +166,13 @@ void shadowDisplay(void (*drawScene)(BoardData *, BoardData3d *, renderdata *),
 	glGetLightfv(GL_LIGHT0, GL_SPECULAR, specular);
 	glLightfv(GL_LIGHT0, GL_SPECULAR, zero);
 
-	drawScene(bd, &bd->bd3d, prd);
+	if (renderingBase)
+		drawScene(bd, bd3d, prd);
+	else
+		drawBoard(bd, bd3d, prd);
 
 	glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
 	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
 
-	/* Create shadow volume in stencil buffer */
-	glEnable(GL_STENCIL_TEST);
-	draw_shadow_volume_to_stencil(&bd->bd3d);
-
-	/* Pass 2: Redraw model, full light in non-shadowed areas */
-	glStencilFunc(GL_EQUAL, midStencilVal, (GLuint)~0);
-	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
-
-	drawScene(bd, &bd->bd3d, prd);
-
 	glDisable(GL_STENCIL_TEST);
 }
