@@ -4,7 +4,7 @@
  * by Holger Bochnig <hbgg@gmx.net>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,22 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: boarddim.h,v 1.7 2004/10/05 16:13:00 Superfly_Jon Exp $
+ * $Id: boarddim.h,v 1.13 2009/03/01 20:01:48 Superfly_Jon Exp $
  */
 
 #ifndef _BOARDDIM_H_
 #define _BOARDDIM_H_
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
 /* fundamental constants */
 #define CHEQUER_WIDTH     6
 #define DIE_WIDTH         7
-#define CUBE_WIDTH        8
-#define ARROW_WIDTH       5
+#define CUBE_WIDTH        6
+#define ARROW_WIDTH       3
 #define HINGE_HEIGHT     12
 #define HINGE_WIDTH       2
 
@@ -123,14 +119,14 @@ extern "C" {
 /* x and y coordinates of cube when not available/doubling/owned/centred */
 #define NO_CUBE -32768
 /* width of board between tray and bar */
-#define PLAY_WIDTH   ((BOARD_WIDTH - 2 * BEAROFF_WIDTH - BAR_WIDTH) / 2)
+#define PLAY_WIDTH   (((BOARD_WIDTH - (2 * BEAROFF_WIDTH)) - BAR_WIDTH) / 2)
 
 #define CUBE_RIGHT_X  (BEAROFF_WIDTH + (PLAY_WIDTH - CUBE_WIDTH) / 2)
-#define CUBE_LEFT_X   (BOARD_WIDTH - CUBE_RIGHT_X)
-#define CUBE_CENTRE_X ((BOARD_WIDTH - CUBE_WIDTH) / 2)
+#define CUBE_LEFT_X   (BOARD_WIDTH - (CUBE_RIGHT_X))
+#define CUBE_TRAY_X (BEAROFF_LEFT_X + (BEAROFF_INSIDE - CUBE_WIDTH) / 2)
 #define PLAY_HEIGHT   (BOARD_HEIGHT - 2 * BORDER_HEIGHT )
 #define CUBE_CENTRE_Y (BORDER_HEIGHT + (PLAY_HEIGHT - CUBE_HEIGHT) / 2)
-#define CUBE_OWN_1_Y  (BOARD_HEIGHT - CUBE_HEIGHT - BORDER_HEIGHT)
+#define CUBE_OWN_1_Y  (BOARD_HEIGHT - (CUBE_HEIGHT + BORDER_HEIGHT))
 #define CUBE_OWN_0_Y  (BORDER_HEIGHT)
 #define CUBE_RESIGN_LEFT_X   (CUBE_LEFT_X - CHEQUER_WIDTH)
 #define CUBE_RESIGN_RIGHT_X  (CUBE_RIGHT_X + CHEQUER_WIDTH)
@@ -140,10 +136,4 @@ extern "C" {
 #define HINGE_TOP_Y ((BOARD_HEIGHT + CUBE_HEIGHT) / 2 + \
                      2 * CHEQUER_HEIGHT + 1)
 
-extern int positions[ 2 ][ 30 ][ 3 ];
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
 #endif /* _BOARDDIM_H_ */
