@@ -4,7 +4,7 @@
  * by Jrn Thyssen <jth@gnubg.org>, 2003.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,19 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: boardpos.c,v 1.7 2006/06/17 17:58:27 oysteijo Exp $
+ * $Id: boardpos.c,v 1.15 2008/03/08 15:10:57 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
-#include <assert.h>
+#include <glib.h>
 #include <stdlib.h>
+#include <glib/gi18n.h>
 
 #include "boarddim.h"
 #include "boardpos.h"
-#include <glib/gi18n.h>
+#include "common.h"
 
 extern void
 ChequerPosition( const int clockwise, 
@@ -55,7 +54,7 @@ PointArea( const int fClockwise, const int nSize,
            int *px, int *py, int *pcx, int *pcy ) {
 
     /* max chequer in column */
-    int c_chequer = ( !n || n == 25 ) ? 3 : 5;
+    int c_chequer = ( n == 0 || n == 25 ) ? 3 : 5;
     
     *px = positions[ fClockwise ][ n ][ 0 ] * nSize;
     *py = positions[ fClockwise ][ n ][ 1 ] * nSize;
@@ -76,25 +75,39 @@ PointArea( const int fClockwise, const int nSize,
 }
 
 
-extern void
-CubePosition( const int crawford_game, const int cube_use,
-              const int doubled, const int cube_owner,
-              int *px, int *py, int *porient ) {
-
-    if( crawford_game || !cube_use ) {
-	/* no cube */
-	if( px ) *px = NO_CUBE;
-	if( py ) *py = NO_CUBE;
-	if( porient ) *porient = -1;
-    } else if( doubled ) {
-      if( px ) *px = (doubled > 0) ? CUBE_RIGHT_X: CUBE_LEFT_X;
-	if( py ) *py = CUBE_CENTRE_Y;
-	if( porient ) *porient = doubled;
-    } else {
-	if( px ) *px = CUBE_CENTRE_X;
-	if( py ) *py = (cube_owner < 0)  ? CUBE_OWN_1_Y :
-	               (cube_owner == 0) ? CUBE_CENTRE_Y :  CUBE_OWN_0_Y;
-	if( porient ) *porient = cube_owner;
+/* Determine the position and rotation of the cube; *px and *py return the
+   position (in board units -- multiply by nSize to get
+   pixels) and *porient returns the rotation (1 = facing the top, 0 = facing
+   the side, -1 = facing the bottom). */
+extern void CubePosition( const int crawford_game, const int cube_use,
+              const int doubled, const int cube_owner, int fClockwise,
+              int *px, int *py, int *porient )
+{
+    if( crawford_game || !cube_use )
+	{
+		/* no cube */
+		if( px ) *px = NO_CUBE;
+		if( py ) *py = NO_CUBE;
+		if( porient ) *porient = -1;
+    }
+	else if( doubled )
+	{
+		if( px ) *px = (doubled > 0) ? CUBE_RIGHT_X: CUBE_LEFT_X;
+		if( py ) *py = CUBE_CENTRE_Y;
+		if( porient ) *porient = doubled;
+    }
+	else
+	{
+		if( px )
+		{
+			if (fClockwise)
+				*px = BOARD_WIDTH - BEAROFF_INSIDE - CUBE_TRAY_X;
+			else
+				*px = CUBE_TRAY_X;
+		}
+		if( py ) *py = (cube_owner < 0)  ? CUBE_OWN_1_Y :
+						(cube_owner == 0) ? CUBE_CENTRE_Y :  CUBE_OWN_0_Y;
+		if( porient ) *porient = cube_owner;
     }
 
 }
@@ -104,10 +117,10 @@ extern void
 ResignPosition( const int resigned, int *px, int *py, int *porient ) {
 
   if( resigned ) {
-    if ( px ) *px = (resigned / abs ( resigned ) < 0) ? 
+    if ( px ) *px = (SGN(resigned) < 0) ? 
 		CUBE_RESIGN_LEFT_X : CUBE_RESIGN_RIGHT_X;
     if ( py ) *py = CUBE_CENTRE_Y;
-    if( porient ) *porient = - resigned / abs ( resigned );
+    if( porient ) *porient = - SGN(resigned);
   }
   else {
     /* no resignation */
@@ -118,29 +131,34 @@ ResignPosition( const int resigned, int *px, int *py, int *porient ) {
 
 }
 
-extern void
-ArrowPosition( const int clockwise, const int nSize, int *px, int *py ) {
-
+extern void ArrowPosition( const int clockwise, int turn, const int nSize, int *px, int *py )
+{
   /* calculate the position of the arrow to indicate
      player on turn and direction of play;  *px and *py
      return the position of the upper left corner in pixels,
      NOT board units */
 
-    int Point28_x, Point28_y, Point28_dx, Point28_dy;
-    int Point29_x, Point29_y, Point29_dx, Point29_dy;
-
-    PointArea( clockwise, nSize, POINT_UNUSED0, 
-               &Point28_x, &Point28_y, &Point28_dx, &Point28_dy );
-    PointArea( clockwise, nSize, POINT_UNUSED1, 
-               &Point29_x, &Point29_y, &Point29_dx, &Point29_dy );
-
-    assert( Point28_x == Point29_x );
-    assert( Point28_dx == Point29_dx );
-    assert( Point28_dy == Point29_dy );
-
-    if ( px ) *px = Point29_x + Point29_dx / 2
-			- nSize * ARROW_SIZE / 2;
-    if ( py ) *py = (nSize * (BOARD_HEIGHT - ARROW_HEIGHT) / 2);
+    int Point_x, Point_y, Point_dx, Point_dy;
 
+	PointArea( clockwise, nSize, (turn == 1) ? 26 : 27, 
+               &Point_x, &Point_y, &Point_dx, &Point_dy );
 
+    if ( px )
+	{
+		if (clockwise)
+			*px = Point_x + (nSize * BEAROFF_INSIDE);
+		else
+			*px = Point_x - (nSize * BORDER_WIDTH);
+		/* Center arrow in border */
+		*px += (nSize * (BORDER_WIDTH - ARROW_WIDTH)) / 2;
+	}
+    if ( py )
+	{
+		if (turn == 1)
+			*py = Point_y + Point_dy;
+		else
+			*py = Point_y - (nSize * BORDER_WIDTH);//- 2 * nSize * ARROW_HEIGHT;
+		/* Center arrow in border */
+		*py += (nSize * (BORDER_WIDTH - ARROW_HEIGHT)) / 2;
+	}
 }
