@@ -5,7 +5,7 @@
 # by Gary Wong <gtw@gnu.org>, 1998, 1999, 2000, 2001, 2002, 2003, 2004
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of version 2 of the GNU General Public License as
+# it under the terms of version 3 or later of the GNU General Public License as
 # published by the Free Software Foundation.
 #
 # This program is distributed in the hope that it will be useful,
@@ -17,11 +17,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: credits.sh,v 1.120 2006/05/01 20:22:20 oysteijo Exp $
+# $Id: credits.sh,v 1.141 2012/01/08 22:47:10 plm Exp $
 # 
 
-column < /dev/null || exit 0
-
 authors=/tmp/auth.$$
 contributors=/tmp/cont.$$
 support=/tmp/supp.$$
@@ -37,6 +35,8 @@ David Montgomery
 Jim Segrave
 Joern Thyssen
 Gary Wong
+Christian Anthon
+Michael Petch
 EOF
 
 cat > $contributors <<EOF
@@ -46,8 +46,9 @@ Nis Joergensen
 Petr Kadlec
 Kaoru Takahashi
 Stein Kulseth
-Mike Petch
+Philippe Michel
 Rod Roark
+Ingo Macherius
 EOF
 
 cat > $support <<EOF
@@ -65,13 +66,13 @@ Achim Mueller,German
 Hlynur Sigurgislason,Icelandic
 Renzo Campagna,Italian
 Yoshito Takeuchi,Japanese
+Mihai Varzaru,Romanian
+Dmitri I Gouliaev,Russian
 Akif Dinc,Turkish
-Dmitri I GOULIAEV,Russian
 EOF
 
 cat > $credit <<EOF
 Misja Alma
-Christian Anthon
 Kazuaki Asai
 Eric Augustine
 Erik Barfoed
@@ -82,6 +83,7 @@ Chuck Bower
 Adrian Bunk
 Nick Bratby
 Craig Campbell
+Timothy Chow
 John Chuang
 Tristan Colgate
 Olivier Croisille
@@ -93,6 +95,7 @@ Ian Dunstan
 Max Durbano
 Peter Eberhard
 Robert Eberlein
+Guido Flohr
 Fotis
 Kennedy Fraser
 Ric Gerace
@@ -124,6 +127,7 @@ Massimiliano Maini
 Giulio De Marco
 John Marttila
 Alix Martin
+Tom Martin
 William Maslen
 Joachim Matussek
 Thomas Meyer
@@ -174,6 +178,7 @@ Frank Worrell
 Christopher D. Yep
 Anders Zachrison
 Douglas Zare
+Louis Zulli
 EOF
 
 cat > $extra <<EOF
@@ -191,9 +196,22 @@ EOF
 
 cat > credits.h <<EOF
 /* Do not modify this file!  It is created automatically by credits.sh.
-   Modify credits.sh instead. */
+   Modify credits.sh instead. 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 3 or later of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
 
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 typedef struct _credEntry {
 	char* Name;
@@ -201,7 +219,7 @@ typedef struct _credEntry {
 } credEntry;
 
 typedef struct _credits {
-	char* Title;
+	const char* Title;
 	credEntry *Entry;
 } credits;
 
@@ -213,21 +231,34 @@ extern credEntry ceCredits[];
 
 extern credits creditList[];
 
-extern char aszAUTHORS[];
+extern const char aszAUTHORS[];
 
-extern char aszCOPYRIGHT[];
+extern const char aszCOPYRIGHT[];
 
 EOF
-
 cat > credits.c <<EOF
 /* Do not modify this file!  It is created automatically by credits.sh.
-   Modify credits.sh instead. */
+   Modify credits.sh instead. 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 3 or later of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
 
-#include "i18n.h"
+#include "config.h"
+#include <glib/gi18n.h>
 #include "credits.h"
 
-char aszCOPYRIGHT[] = N_("Copyright 1999, 2000, 2001, 2002, 2003, 2004 "
-			"by Gary Wong.");
+const char aszCOPYRIGHT[] = N_("Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 by Gary Wong.\n"
+                               "Copyright (C) 2012 by Gary Wong and the AUTHORS; for details type \`show version'.");
 
 credEntry ceAuthors[] = {
 EOF
@@ -303,7 +334,7 @@ cat >> credits.c <<EOF
 
 credits creditList[] =
 {
-	{N_("Developers"), ceAuthors},
+	{N_("Authors"), ceAuthors},
 	{N_("Code Contributors"), ceContrib},
 	{N_("Translations"), ceTranslations},
 	{N_("Support"), ceSupport},
@@ -321,14 +352,14 @@ cat > AUTHORS <<EOF
 
 EOF
 
-column -c 72 < $authors | expand | sed 's/^/    /' >> AUTHORS
+pr -3 -t < $authors | expand | sed 's/^/    /' >> AUTHORS
 
 cat >> AUTHORS <<EOF
  
                                    Support by:
 
 EOF
-cat $support | sed -e 's/^\(.*\),\(.*\)$/ \1 (\2)/g' | column -c 72 | expand | sed 's/^/    /' >> AUTHORS
+cat $support | sed -e 's/^\(.*\),\(.*\)$/ \1 (\2)/g' | pr -2 -t | expand | sed 's/^/    /' >> AUTHORS
 
 cat >> AUTHORS <<EOF
 
@@ -336,7 +367,7 @@ cat >> AUTHORS <<EOF
 
 EOF
 
-column -c 72 < $contributors | expand | sed 's/^/    /' >> AUTHORS
+pr -3 -t < $contributors | expand | sed 's/^/    /' >> AUTHORS
 
 cat >> AUTHORS <<EOF
 
@@ -344,7 +375,7 @@ cat >> AUTHORS <<EOF
 
 EOF
 
-cat $translations | sed -e 's/^\(.*\),\(.*\)$/ \1 (\2)/g' | column -c 72 | expand | sed 's/^/    /' >> AUTHORS
+cat $translations | sed -e 's/^\(.*\),\(.*\)$/ \1 (\2)/g' | pr -1 -t | expand | sed 's/^/    /' >> AUTHORS
 
 
 cat >> AUTHORS <<EOF
@@ -353,7 +384,7 @@ cat >> AUTHORS <<EOF
 
 EOF
 
-column -c 72 < $credit | expand | sed 's/^/    /' >> AUTHORS
+pr -3 -t < $credit | expand | sed 's/^/    /' >> AUTHORS
 
 cat >> AUTHORS <<'EOF'
 
@@ -364,7 +395,7 @@ cat >> AUTHORS <<'EOF'
 
 EOF
 
-column -c 72 < $extra | expand | sed 's/^/    /' >> AUTHORS
+pr -4 -t < $extra | expand | sed 's/^/    /' >> AUTHORS
 
 cat >> AUTHORS <<'EOF'                            
 
@@ -377,7 +408,7 @@ cat >> AUTHORS <<'EOF'
   Library code from the following authors has been included in GNU Backgammon:
 
      Ulrich Drepper (an implementation of the public domain MD5 algorithm)
-             Bob Jenkins (the ISAAC pseudo random number generator)
+   Bob Jenkins (the ISAAC pseudo random number generator, hashing functions)
        Takuji Nishimura and Makoto Matsumoto (the Mersenne Twister PRNG)
                 Gerry Tesauro (the "pubeval" position evaluator)
              Claes Tornberg (the mec match equity table generator)
@@ -395,7 +426,7 @@ EOF
 # 
 
 cat >> credits.c <<EOF
-char aszAUTHORS[] =
+const char aszAUTHORS[] =
 EOF
 
 sed -e 's/"/\\"/g' AUTHORS | sed -e 's/.*/"&\\n"/g' >> credits.c
@@ -403,3 +434,5 @@ sed -e 's/"/\\"/g' AUTHORS | sed -e 's/.*/"&\\n"/g' >> credits.c
 cat >> credits.c <<EOF
 ;
 EOF
+
+rm -f $authors $contributors $support $translations $credit $extra
