@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 1999, 2000, 2001, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,21 +16,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: drawboard.c,v 1.37.2.1 2006/11/06 23:29:34 c_anthon Exp $
+ * $Id: drawboard.c,v 1.57 2012/04/26 17:02:09 mdpetch Exp $
  */
 
 #include "config.h"
 
-#include <assert.h>
+#include <glib.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <glib/gi18n.h>
 
 #include "drawboard.h"
 #include "positionid.h"
-#include <glib/gi18n.h>
+#include "matchequity.h"
 
 int fClockwise = FALSE; /* Player 1 moves clockwise */
 
@@ -53,12 +54,14 @@ int fClockwise = FALSE; /* Player 1 moves clockwise */
  *
  */
 
-static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
+static char *DrawBoardStd( char *sz, const TanBoard anBoard, int fRoll,
                            char *asz[], char *szMatchID, 
                            int nChequers ) {
 
     char *pch = sz, *pchIn;
-    int x, y, an[ 2 ][ 25 ], cOffO = nChequers, cOffX = nChequers;
+    unsigned int x, y;
+	unsigned int cOffO = nChequers, cOffX = nChequers;
+	TanBoard an;
     static char achX[ 17 ] = "     X6789ABCDEF",
         achO[ 17 ] = "     O6789ABCDEF";
 
@@ -67,19 +70,17 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
         cOffX -= anBoard[ 1 ][ x ];
     }
     
-    strcpy( pch, _ (" GNU Backgammon  Position ID: ") );
-
-    pch = strchr ( pch, 0 );
+	pch += sprintf(pch, " %-15s %s: ", _("GNU Backgammon"), _("Position ID"));
 
     if( fRoll )
-        strcpy( pch, PositionID( anBoard ) );
+        strcpy( pch, PositionID( (ConstTanBoard)anBoard ) );
     else {
         for( x = 0; x < 25; x++ ) {
             an[ 0 ][ x ] = anBoard[ 1 ][ x ];
             an[ 1 ][ x ] = anBoard[ 0 ][ x ];
         }
         
-        strcpy( pch, PositionID( an ) );
+        strcpy( pch, PositionID( (ConstTanBoard)an ) );
     }
     
     pch += 14;
@@ -88,8 +89,7 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
     /* match id */
 
     if ( szMatchID && *szMatchID ) {
-      sprintf ( pch, _("                 Match ID   : %s\n"), szMatchID );
-      pch = strchr ( pch, 0 );
+      pch += sprintf(pch, "                 %s   : %s\n", _("Match ID"), szMatchID);
     }
             
     strcpy( pch, fRoll ? " +13-14-15-16-17-18------19-20-21-22-23-24-+     " :
@@ -137,12 +137,6 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
         if( y < 2 && asz[ y + 1 ] )
             for( pchIn = asz[ y + 1 ]; *pchIn; pchIn++ )
                 *pch++ = *pchIn;
-#if USE_TIMECONTROL
-	if ((y==3) && asz[7])
-            for( pchIn = asz[ 7 ]; *pchIn; pchIn++ )
-                *pch++ = *pchIn;
-#endif
-
         *pch++ = '\n';
     }
 
@@ -179,7 +173,7 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
     *pch++ = '\n';
     
     *pch++ = fRoll ? 'v' : '^';
-    strcpy( pch, _("|                  |BAR|                  |     ") );
+    strcpy( pch, "|                  |BAR|                  |     " );
     pch = strchr ( pch, 0 );
     
     if( asz[ 3 ] )
@@ -204,7 +198,7 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
     *pch++ = ' ';
     *pch++ = '|';
         
-    for( ; x >= 0; x-- ) {
+    for( ; x < UINT_MAX; x-- ) {
         *pch++ = ' ';
         *pch++ = anBoard[ 1 ][ x ] ? achX[ anBoard[ 1 ][ x ] ] :
                 achO[ anBoard[ 0 ][ 23 - x ] ];
@@ -220,7 +214,7 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
 
     *pch++ = '\n';
     
-    for( y = 3; y >= 0; y-- ) {
+    for( y = 3; y < UINT_MAX; y-- ) {
         *pch++ = ' ';
         *pch++ = '|';
 
@@ -237,7 +231,7 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
         *pch++ = ' ';
         *pch++ = '|';
         
-        for( ; x >= 0; x-- ) {
+        for( ; x < UINT_MAX; x-- ) {
             *pch++ = ' ';
             *pch++ = anBoard[ 1 ][ x ] > y ? 'X' :
                 anBoard[ 0 ][ 23 - x ] > y ? 'O' : ' ';
@@ -256,11 +250,6 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
             for( pchIn = asz[ 5 - y ]; *pchIn; pchIn++ )
                 *pch++ = *pchIn;
         
-#if USE_TIMECONTROL
-	if ((y==3) && asz[8])
-            for( pchIn = asz[ 8 ]; *pchIn; pchIn++ )
-                *pch++ = *pchIn;
-#endif
         *pch++ = '\n';
     }
 
@@ -298,12 +287,13 @@ static char *DrawBoardStd( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
  *
  */
 
-static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
+static char *DrawBoardCls( char *sz, const TanBoard anBoard, int fRoll,
                            char *asz[], char *szMatchID,
                            int nChequers) {
 
     char *pch = sz, *pchIn;
-    int x, y, an[ 2 ][ 25 ], cOffO = nChequers, cOffX = nChequers;
+    unsigned int x, y, cOffO = nChequers, cOffX = nChequers;
+	TanBoard an;
     static char achX[ 17 ] = "     X6789ABCDEF",
         achO[ 17 ] = "     O6789ABCDEF";
 
@@ -312,19 +302,17 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
         cOffX -= anBoard[ 1 ][ x ];
     }
     
-    strcpy( pch, _("    GNU Backgammon  Position ID: ") );
-
-    pch = strchr ( pch, 0 );
+	pch += sprintf(pch, "%18s  %s: ", _("GNU Backgammon"), _("Position ID"));
 
     if( fRoll )
-        strcpy( pch, PositionID( anBoard ) );
+        strcpy( pch, PositionID( (ConstTanBoard)anBoard ) );
     else {
         for( x = 0; x < 25; x++ ) {
             an[ 0 ][ x ] = anBoard[ 1 ][ x ];
             an[ 1 ][ x ] = anBoard[ 0 ][ x ];
         }
         
-        strcpy( pch, PositionID( an ) );
+        strcpy( pch, PositionID( (ConstTanBoard)an ) );
     }
     
     pch += 14;
@@ -333,8 +321,7 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
     /* match id */
 
     if ( szMatchID && *szMatchID ) {
-      sprintf ( pch, _("                    Match ID   : %s\n"), szMatchID );
-      pch = strchr ( pch, 0 );
+      pch += sprintf(pch, "                    %s   : %s\n", _("Match ID"), szMatchID);
     }
             
     strcpy( pch, fRoll ? "    +24-23-22-21-20-19------18-17-16-15-14-13-+  " :
@@ -349,7 +336,7 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
 
     for( y = 0; y < 4; y++ ) {
     
-        for( x = 2; x >= 0; x-- )
+        for( x = 2; x < UINT_MAX; x-- )
             *pch++ = ( cOffO > 5 * x + y ) ? 'O' : ' ';
 
         *pch++ = ' ';
@@ -385,7 +372,7 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
         *pch++ = '\n';
     }
 
-    for( x = 2; x >= 0; x-- )
+    for( x = 2; x < UINT_MAX; x-- )
         *pch++ = ( cOffO > 5 * x + 4 ) ? 'O' : ' ';
 
     *pch++ = ' ';
@@ -418,7 +405,7 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
 
     *pch++ = '\n';
     
-    strcpy( pch, _("    |                  |BAR|                  |") );
+    strcpy( pch, "    |                  |BAR|                  |" );
     pch = strchr ( pch, 0 );
     *pch++ = fRoll ? 'v' : '^';
     *pch++ = ' ';   
@@ -429,7 +416,7 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
 
     *pch++ = '\n';
 
-    for( x = 2; x >= 0; x-- )
+    for( x = 2; x < UINT_MAX; x-- )
         *pch++ = ( cOffX > 5 * x + 4 ) ? 'X' : ' ';
 
     *pch++ = ' ';
@@ -462,9 +449,9 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
 
     *pch++ = '\n';
     
-    for( y = 3; y >= 0; y-- ) {
+    for( y = 3; y < UINT_MAX; y-- ) {
 
-        for( x = 2; x >= 0; x-- )
+        for( x = 2; x < UINT_MAX; x-- )
             *pch++ = ( cOffX > 5 * x + y ) ? 'X' : ' ';
 
         *pch++ = ' ';
@@ -514,7 +501,7 @@ static char *DrawBoardCls( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
     return sz;
 }
 
-extern char *DrawBoard( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
+extern char *DrawBoard( char *sz, const TanBoard anBoard, int fRoll,
                         char *asz[], char *szMatchID, int nChequers ) {
     if( fClockwise == FALSE ) 
         return ( DrawBoardStd( sz, anBoard, fRoll, asz, 
@@ -526,7 +513,7 @@ extern char *DrawBoard( char *sz, int anBoard[ 2 ][ 25 ], int fRoll,
 
 static char *FormatPoint( char *pch, int n ) {
 
-    assert( n >= 0 );
+    g_assert( n >= 0 );
     
     /*don't translate 'off' and 'bar' as these may be used in UserCommand at a later
      * point */
@@ -546,7 +533,7 @@ static char *FormatPoint( char *pch, int n ) {
 
 static char *FormatPointPlain( char *pch, int n ) {
 
-    assert( n >= 0 );
+    g_assert( n >= 0 );
     
     if( n > 9 )
         *pch++ = n / 10 + '0';
@@ -556,7 +543,7 @@ static char *FormatPointPlain( char *pch, int n ) {
     return pch;
 }
 
-extern char *FormatMovePlain( char *sz, int anBoard[ 2 ][ 25 ],
+extern char *FormatMovePlain( char *sz, TanBoard anBoard,
                               int anMove[ 8 ] ) {
 
     char *pch = sz;
@@ -586,7 +573,7 @@ extern char *FormatMovePlain( char *sz, int anBoard[ 2 ][ 25 ],
     return sz;
 }
 
-static int CompareMoves( const void *p0, const void *p1 ) {
+static int CompareMovesSimple( const void *p0, const void *p1 ) {
 
     int n0 = *( (int *) p0 ), n1 = *( (int *) p1 );
 
@@ -603,10 +590,10 @@ extern void CanonicalMoveOrder( int an[] ) {
     for( i = 0; i < 4 && an[ 2 * i ] > -1; i++ )
 	;
     
-    qsort( an, i, sizeof( int ) << 1, CompareMoves );
+    qsort( an, i, sizeof( int ) << 1, CompareMovesSimple );
 }
 
-extern char *FormatMove( char *sz, int anBoard[ 2 ][ 25 ], int anMove[ 8 ] ) {
+extern char *FormatMove( char *sz, const TanBoard anBoard, int anMove[ 8 ] ) {
 
     char *pch = sz;
     int aanMove[ 4 ][ 4 ], *pnSource[ 4 ], *pnDest[ 4 ], i, j;
@@ -627,7 +614,7 @@ extern char *FormatMove( char *sz, int anBoard[ 2 ][ 25 ], int anMove[ 8 ] ) {
     }
     
     /* Order the moves in decreasing order of source point. */
-    qsort( aanMove, 4, 4 * sizeof( int ), CompareMoves );
+    qsort( aanMove, 4, 4 * sizeof( int ), CompareMovesSimple );
 
     /* Combine moves of a single chequer. */
     for( i = 0; i < 4; i++ )
@@ -934,8 +921,8 @@ extern int ParseMove( char *pch, int an[ 8 ] ) {
  *
  */
 
-extern char *FIBSBoard( char *pch, int anBoard[ 2 ][ 25 ], int fRoll,
-			char *szPlayer, char *szOpp, int nMatchTo,
+extern char *FIBSBoard( char *pch, TanBoard anBoard, int fRoll,
+			const char *szPlayer, const char *szOpp, int nMatchTo,
 			int nScore, int nOpponent, int nDice0, int nDice1,
 			int nCube, int fCubeOwner, int fDoubled, int fTurn,
 			int fCrawford, int nChequers ) {
@@ -954,12 +941,15 @@ extern char *FIBSBoard( char *pch, int anBoard[ 2 ][ 25 ], int fRoll,
     sprintf( sz, ":%d:%d:%d:", nMatchTo, nScore, nOpponent );
 
     /* Opponent on bar */
-    sprintf( strchr( sz, 0 ), "%d:", -anBoard[ 0 ][ 24 ] );
+    sprintf( strchr( sz, 0 ), "%d:", -(int)anBoard[ 0 ][ 24 ] );
 
     /* Board */
     for( i = 0; i < 24; i++ )
-	sprintf( strchr( sz, 0 ), "%d:", anBoard[ 0 ][ 23 - i ] ?
-		 -anBoard[ 0 ][ 23 - i ] : anBoard[ 1 ][ i ] );
+	{
+		int point = (int)anBoard[ 0 ][ 23 - i ];
+	sprintf( strchr( sz, 0 ), "%d:", (point > 0) ?
+		 -point : (int)anBoard[ 1 ][ i ] );
+	}
 
     /* Player on bar */
     sprintf( strchr( sz, 0 ), "%d:", anBoard[ 1 ][ 24 ] );
@@ -982,13 +972,14 @@ extern char *FIBSBoard( char *pch, int anBoard[ 2 ][ 25 ], int fRoll,
     return pch;
 }
 
-extern int ParseFIBSBoard( char *pch, int anBoard[ 2 ][ 25 ],
+extern int ParseFIBSBoard( char *pch, TanBoard anBoard,
 			   char *szPlayer, char *szOpp, int *pnMatchTo,
 			   int *pnScore, int *pnScoreOpponent,
 			   int anDice[ 2 ], int *pnCube, int *pfCubeOwner,
 			   int *pfDoubled, int *pfTurn, int *pfCrawford ) {
     
     int i, c, n, fCanDouble, fOppCanDouble, anOppDice[ 2 ];
+    int fNonCrawford, fPostCrawford;
 
     for( i = 0; i < 25; i++ )
 	anBoard[ 0 ][ i ] = anBoard[ 1 ][ i ] = 0;
@@ -1001,6 +992,35 @@ extern int ParseFIBSBoard( char *pch, int anBoard[ 2 ][ 25 ],
 	return -1;
     pch += c;
 
+    /* FIBS has a maximum match length of 99.  Unlimited matches are 
+     * encoded with a match length of 9999.
+     *
+     * FIXME! Is 0 correct for money sessions here?
+     */
+    if ( *pnMatchTo == 9999 )
+        *pnMatchTo = 0;
+
+    if ( *pnMatchTo
+         && ( *pnMatchTo <= *pnScore || *pnMatchTo <= *pnScoreOpponent ))
+        return -1;
+
+    /* If the match length exceeds MAXSCORE we correct the match length
+     * to MAXSCORE and the scores to the closest equivalents.
+     */
+    if ( *pnMatchTo > MAXSCORE ) {
+        if ( *pnMatchTo - *pnScore > MAXSCORE )
+                *pnScore = 0;
+        else
+                *pnScore -= *pnMatchTo - MAXSCORE;
+
+        if ( *pnMatchTo - *pnScoreOpponent > MAXSCORE )
+                *pnScoreOpponent = 0;
+        else
+                *pnScoreOpponent -= *pnMatchTo - MAXSCORE;
+
+        *pnMatchTo = MAXSCORE;
+    }
+
     /* Opponent on bar */
     c = -1;
     sscanf( pch, "%d:%n", &n, &c );
@@ -1032,9 +1052,9 @@ extern int ParseFIBSBoard( char *pch, int anBoard[ 2 ][ 25 ],
 
     c = -1;
     sscanf( pch, "%d:%d:%d:%d:%d:%d:%d:%d:%d:%*d:%*d:%*d:%*d:%*d:%*d:%*d:%*d:"
-	    "%*d:%*d:%d:%n", pfTurn, anDice, anDice + 1, anOppDice,
+	    "%*d:%d:%d:%n", pfTurn, anDice, anDice + 1, anOppDice,
 	    anOppDice + 1, pnCube, &fCanDouble, &fOppCanDouble,
-	    pfDoubled, pfCrawford, &c );
+	    pfDoubled, &fNonCrawford, &fPostCrawford, &c );
     if( c < 0 )
 	return -1;
 
@@ -1045,6 +1065,47 @@ extern int ParseFIBSBoard( char *pch, int anBoard[ 2 ][ 25 ],
 	anDice[ 1 ] = anOppDice[ 1 ];
     }
 
+    /*
+     * Crawford detection.  This is rather tricky with FIBS board states
+     * because FIBS sets both may-double flags to 1 in the Crawford game.
+     *
+     * We have to inspect the last and second last field for that.
+     * The last field is the post-Crawford flag, the second last
+     * field is the non-Crawford flag.
+     *
+     * Until at least one of the players is 1-away, you cannot deduce whether
+     * the Crawford rule is in use or not.  Once one of the players
+     * is 1-away, the non-Crawford flag is set to 3 if the Crawford rule
+     * is not in use; otherwise everything is still 0.
+     *
+     * Once the Crawford game is finished, the post-Crawford flag is set
+     * to 1.
+     *
+     * Since we cannot find out whether the Crawford rule is in use or not
+     * in the pre-Crawford games, the cubeful evaluation can be slightly
+     * biased.  But since we the vast majority of matches on FIBS is played
+     * with the Crawford rule and we assume usage of the Crawford rule
+     * as a default, the bias is negligible.
+     */
+    if ( !*pnMatchTo ) {
+        *pfCrawford = 0;
+    } else {
+        *pfCrawford = 1;
+        if ( *pnMatchTo - *pnScore == 1 || *pnMatchTo - *pnScore == 1 ) {
+            if ( fNonCrawford ) {
+                *pfCrawford = 0;
+            } else if ( !fPostCrawford ) {
+                if ( *pnMatchTo - *pnScore == 1 ) {
+                    fCanDouble = 0;
+                    fOppCanDouble = 1;
+                } else {
+                    fCanDouble = 1;
+                    fOppCanDouble = 0;
+                }
+            }
+        }
+    }
+
     *pfCubeOwner = fCanDouble != fOppCanDouble ? fCanDouble : -1;
 
     return 0;
