@@ -4,7 +4,7 @@
  * by Gary Wong <gary@cs.arizona.edu>, 1998-2001.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: eval.h,v 1.133 2006/09/21 22:24:05 Superfly_Jon Exp $
+ * $Id: eval.h,v 1.166 2009/09/01 19:13:55 c_anthon Exp $
  */
 
 #ifndef _EVAL_H_
@@ -24,14 +24,13 @@
 
 #include "dice.h"
 #include "bearoff.h"
+#include "neuralnet.h"
 
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#ifndef TRUE
-#define TRUE 1
-#endif
+#define EXP_LOCK_FUN(ret, name, ...) \
+	typedef ret (*f_##name)( __VA_ARGS__); \
+	extern f_##name name; \
+	extern ret name##NoLocking( __VA_ARGS__); \
+	extern ret name##WithLocking( __VA_ARGS__);
 
 #define WEIGHTS_VERSION "0.15"
 #define WEIGHTS_VERSION_BINARY 0.15f
@@ -64,11 +63,6 @@ typedef enum {
   N_CUBEFUL_OUTPUTS
 } CubefulOutputs;
 
-#define GNUBG_WEIGHTS "gnubg.weights"
-#define GNUBG_WEIGHTS_BINARY "gnubg.wd"
-#define GNUBG_BEAROFF "gnubg.bd"
-#define GNUBG_BEAROFF_OS "gnubg_os.bd"
-
 /* A trivial upper bound on the number of (complete or incomplete)
  * legal moves of a single roll: if all 15 chequers are spread out,
  * then there are 18 C 4 + 17 C 3 + 16 C 2 + 15 C 1 = 3875
@@ -102,12 +96,7 @@ typedef struct {
        classes */
     unsigned int fCubeful : 1; /* cubeful evaluation */
     unsigned int nPlies   : 3;
-#if defined( REDUCTION_CODE )
-    unsigned int nReduced : 3; /* this will need to be expanded if we add
-				  support for nReduced != 3 */
-#else
     unsigned int fUsePrune : 1;
-#endif
     unsigned int fDeterministic : 1;
     float        rNoise;       /* standard deviation */
 } evalcontext;
@@ -122,11 +111,7 @@ typedef struct {
    data is present in the file based on the version number
 */
 
-#if defined( REDUCTION_CODE )
-#define SGF_FORMAT_VER 2
-#else
 #define SGF_FORMAT_VER 3
-#endif
 
 typedef struct {
 
@@ -152,12 +137,12 @@ typedef struct {
   unsigned int   nTrials; /* number of rollouts */
   unsigned short nLate; /* switch evaluations on move nLate of game */
   rng rngRollout;
-  int nSeed;
+  unsigned long nSeed;
   unsigned int nMinimumGames; /* but always do at least this many */
   double       rStdLimit;     /* stop when abs( value / std ) < this */
   unsigned int nMinimumJsdGames;
   double       rJsdLimit;
-  int          nGamesDone;
+  unsigned int nGamesDone;
   int          nSkip;
 } rolloutcontext;
 
@@ -177,21 +162,12 @@ typedef enum {
 /* enumeration of variations of backgammon
    (starting position and/or special rules) */
 
-typedef enum {
-  VARIATION_STANDARD,      /* standard backgammon */
-  VARIATION_NACKGAMMON,    /* standard backgammon with nackgammon starting
-                            position */
-  VARIATION_HYPERGAMMON_1, /* 1-chequer hypergammon */
-  VARIATION_HYPERGAMMON_2, /* 2-chequer hypergammon */
-  VARIATION_HYPERGAMMON_3, /* 3-chequer hypergammon */
-  NUM_VARIATIONS
-} bgvariation;
 
 extern bgvariation bgvDefault;
 
 extern int anChequers[ NUM_VARIATIONS ];
-extern char *aszVariations[ NUM_VARIATIONS ];
-extern char *aszVariationCommands[ NUM_VARIATIONS ];
+extern const char *aszVariations[ NUM_VARIATIONS ];
+extern const char *aszVariationCommands[ NUM_VARIATIONS ];
 
 
 /*
@@ -228,20 +204,28 @@ typedef struct {
 
 } evalsetup;
 
-typedef enum  {
-  DOUBLE_TAKE, DOUBLE_PASS, NODOUBLE_TAKE, TOOGOOD_TAKE, TOOGOOD_PASS,
-  DOUBLE_BEAVER, NODOUBLE_BEAVER,
-  REDOUBLE_TAKE, REDOUBLE_PASS, NO_REDOUBLE_TAKE,
-  TOOGOODRE_TAKE, TOOGOODRE_PASS,
-  NO_REDOUBLE_BEAVER,
-  NODOUBLE_DEADCUBE,     /* cube is dead (match play only) */
-  NO_REDOUBLE_DEADCUBE,     /* cube is dead (match play only) */
-  NOT_AVAILABLE, /* Cube not available */
-  OPTIONAL_DOUBLE_TAKE,
-  OPTIONAL_REDOUBLE_TAKE,
-  OPTIONAL_DOUBLE_BEAVER,
-  OPTIONAL_DOUBLE_PASS,
-  OPTIONAL_REDOUBLE_PASS
+typedef enum {
+	DOUBLE_TAKE,
+	DOUBLE_PASS,
+	NODOUBLE_TAKE,
+	TOOGOOD_TAKE,
+	TOOGOOD_PASS,
+	DOUBLE_BEAVER,
+	NODOUBLE_BEAVER,
+	REDOUBLE_TAKE,
+	REDOUBLE_PASS,
+	NO_REDOUBLE_TAKE,
+	TOOGOODRE_TAKE,
+	TOOGOODRE_PASS,
+	NO_REDOUBLE_BEAVER,
+	NODOUBLE_DEADCUBE,	/* cube is dead (match play only) */
+	NO_REDOUBLE_DEADCUBE,	/* cube is dead (match play only) */
+	NOT_AVAILABLE,		/* Cube not available */
+	OPTIONAL_DOUBLE_TAKE,
+	OPTIONAL_REDOUBLE_TAKE,
+	OPTIONAL_DOUBLE_BEAVER,
+	OPTIONAL_DOUBLE_PASS,
+	OPTIONAL_REDOUBLE_PASS
 } cubedecision;
 
 typedef enum {
@@ -257,7 +241,7 @@ typedef enum {
   TT_BEAVER
 } taketype;
 
-extern char* aszDoubleTypes[ NUM_DOUBLE_TYPES ];
+extern const char* aszDoubleTypes[ NUM_DOUBLE_TYPES ];
 
 
 /*
@@ -284,38 +268,38 @@ extern const char *aszSettings[ NUM_SETTINGS ];
 extern const char *aszMoveFilterSettings[ NUM_MOVEFILTER_SETTINGS ];
 extern movefilter aaamfMoveFilterSettings[ NUM_MOVEFILTER_SETTINGS ][ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ];
 
+typedef enum
+{
+	CMARK_NONE,
+	CMARK_ROLLOUT
+} CMark;
+
 typedef struct {
   int anMove[ 8 ];
   unsigned char auch[ 10 ];
-  int cMoves, cPips;
+  unsigned int cMoves, cPips;
   /* scores for this move */
   float rScore, rScore2; 
   /* evaluation for this move */
   float arEvalMove[ NUM_ROLLOUT_OUTPUTS ];
   float arEvalStdDev[ NUM_ROLLOUT_OUTPUTS ];
   evalsetup esMove;
+  CMark cmark;
 } move;
 
-
-extern volatile int fInterrupt, fAction;
-extern void ( *fnAction )( void );
-extern void ( *fnTick )( void );
+extern int fInterrupt;
 extern cubeinfo ciCubeless;
-extern char *aszEvalType[ EVAL_ROLLOUT + 1 ];
-extern int fEgyptian;
-extern int fUse15x15;
+extern const char *aszEvalType[ (int)EVAL_ROLLOUT + 1 ];
 
 extern bearoffcontext *pbc1;
 extern bearoffcontext *pbc2;
 extern bearoffcontext *pbcOS;
 extern bearoffcontext *pbcTS;
 extern bearoffcontext *apbcHyper[ 3 ];
-extern bearoffcontext *pbc15x15;
-extern bearoffcontext *pbc15x15_dvd;
 
 typedef struct {
-    int cMoves; /* and current move when building list */
-    int cMaxMoves, cMaxPips;
+    unsigned int cMoves; /* and current move when building list */
+    unsigned int cMaxMoves, cMaxPips;
     int iMoveBest;
     float rBestScore;
     move* amMoves;
@@ -340,7 +324,6 @@ typedef enum  {
     CLASS_HYPERGAMMON3, /* hypergammon with 3 chequers */
     CLASS_BEAROFF2,     /* Two-sided bearoff database (in memory) */
     CLASS_BEAROFF_TS,   /* Two-sided bearoff database (on disk) */
-    CLASS_BEAROFF_15x15,/* Hugh Sconyers complete bearoff database */
     CLASS_BEAROFF1,     /* One-sided bearoff database (in memory) */
     CLASS_BEAROFF_OS,   /* One-sided bearoff database /on disk) */
     CLASS_RACE,         /* Race neural network */
@@ -350,7 +333,7 @@ typedef enum  {
 
 #define N_CLASSES (CLASS_CONTACT + 1)
 
-#define CLASS_PERFECT CLASS_BEAROFF_15x15 
+#define CLASS_PERFECT CLASS_BEAROFF_TS
 
 #define CFMONEY(arEquity,pci) \
    ( ( (pci)->fCubeOwner == -1 ) ? arEquity[ 2 ] : \
@@ -361,30 +344,22 @@ typedef enum  {
      ( ( (pci)->fJacoby ) ? arEquity[ 2 ] : arEquity[ 1 ] ) : \
      ( ( (pci)->fCubeOwner == (pci)->fMove ) ? arEquity[ 0 ] : arEquity[ 3 ] ) )
 
-extern int
-EvalInitialise( char *szWeights, char *szWeightsBinary,
-		int fNoBearoff, 
-                char *szDir, int nSize,
-		void (*pfProgress)( int ) );
+extern void EvalInitialise( char *szWeights, char *szWeightsBinary,
+		int fNoBearoff, void (*pfProgress)( unsigned int ) );
 
-extern int
-EvalShutdown( void );
+extern int EvalShutdown( void );
 
 extern void EvalStatus( char *szOutput );
 
 extern int EvalNewWeights( int nSize );
 
 extern int 
-EvalSave( char *szWeights );
+EvalSave( const char *szWeights );
 
-extern int 
-EvaluatePosition( int anBoard[ 2 ][ 25 ], float arOutput[],
+
+EXP_LOCK_FUN(int, EvaluatePosition, NNState *nnStates, const TanBoard anBoard, float arOutput[],
                   const cubeinfo* pci, const evalcontext* pec );
 
-extern int
-EvaluatePerfectCubeful ( int anBoard[ 2 ][ 25 ], float arEquity[],
-                         const bgvariation bgv );
-
 extern void
 InvertEvaluationR ( float ar[ NUM_ROLLOUT_OUTPUTS], const cubeinfo* pci);
 
@@ -394,46 +369,36 @@ InvertEvaluation( float ar[ NUM_OUTPUTS ] );
 extern void 
 InvertEvaluationCf( float ar[ 4 ] );
 
-extern 
-int FindBestMove( int anMove[ 8 ], int nDice0, int nDice1,
-                  int anBoard[ 2 ][ 25 ], cubeinfo *pci,
+EXP_LOCK_FUN(int, FindBestMove, int anMove[ 8 ], int nDice0, int nDice1,
+                  TanBoard anBoard, cubeinfo *pci,
                   evalcontext *pec,
                   movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ] ); 
 
-extern int 
-FindnSaveBestMoves( movelist *pml,
-                    int nDice0, int nDice1, int anBoard[ 2 ][ 25 ],
+EXP_LOCK_FUN(int, FindnSaveBestMoves, movelist *pml,
+                    int nDice0, int nDice1, const TanBoard anBoard,
                     unsigned char *auchMove, const float rThr,
                     const cubeinfo* pci, const evalcontext* pec,
                     movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ] );
 
-extern int 
-FindPubevalMove( int nDice0, int nDice1, int anBoard[ 2 ][ 25 ],
-		 int anMove[ 8 ], const bgvariation bgv );
+extern void
+PipCount( const TanBoard anBoard, unsigned int anPips[ 2 ] );
 
 extern int 
-TrainPosition( int anBoard[ 2 ][ 25 ], float arDesired[], float rAlpha,
-	       float rAnneal, const bgvariation bgv );
-
-extern int 
-PipCount( int anBoard[ 2 ][ 25 ], int anPips[ 2 ] );
-
-extern int 
-ThorpCount( int anBoard[ 2 ][ 25 ], int *pnLeader, int *pnTrailer );
+ThorpCount( const TanBoard anBoard, int *pnLeader, float *adjusted, int *pnTrailer );
 
 extern int
-KeithCount(int anBoard[2][25], int *pnL, int *pnT);
+KeithCount( const TanBoard anBoard, int pn[2]);
 
 extern int 
-DumpPosition( int anBoard[ 2 ][ 25 ], char *szOutput,
+DumpPosition( const TanBoard anBoard, char *szOutput,
               const evalcontext* pec, cubeinfo* pci, int fOutputMWC,
 	      int fOutputWinPC, int fOutputInvert, const char *szMatchID );
 
 extern void 
-SwapSides( int anBoard[ 2 ][ 25 ] );
+SwapSides( TanBoard anBoard );
 
 extern int 
-GameStatus( int anBoard[ 2 ][ 25 ], const bgvariation bgv );
+GameStatus( const TanBoard anBoard, const bgvariation bgv );
 
 extern void
 EvalCacheFlush(void);
@@ -442,21 +407,28 @@ extern int
 EvalCacheResize( unsigned int cNew );
 
 extern int 
-EvalCacheStats( int *pcUsed, int *pcSize, int *pcLookup, int *pcHit );
+EvalCacheStats( unsigned int *pcUsed, unsigned int *pcLookup, unsigned int *pcHit );
+
+extern double GetEvalCacheSize(void);
+void SetEvalCacheSize(unsigned int size);
+extern unsigned int GetEvalCacheEntries(void);
+extern int GetCacheMB(double size);
 
 extern int 
-GenerateMoves( movelist *pml, int anBoard[ 2 ][ 25 ],
+GenerateMoves( movelist *pml, const TanBoard anBoard,
                int n0, int n1, int fPartial );
 
-extern int 
-ApplyMove( int anBoard[ 2 ][ 25 ], const int anMove[ 8 ],
-           const int fCheckLegal );
+extern int ApplySubMove(TanBoard anBoard, const int iSrc, const int nRoll,
+			const int fCheckLegal);
+
+extern int ApplyMove(TanBoard anBoard, const int anMove[8],
+		     const int fCheckLegal);
 
 extern positionclass 
-ClassifyPosition( int anBoard[ 2 ][ 25 ], const bgvariation bgv );
+ClassifyPosition( const TanBoard anBoard, const bgvariation bgv );
 
 /* internal use only */
-extern int EvalBearoff1Full( int anBoard[ 2 ][ 25 ],
+extern int EvalBearoff1Full( const TanBoard anBoard,
                              float arOutput[] );
 
 extern float
@@ -482,27 +454,25 @@ SetCubeInfo ( cubeinfo *pci, const int nCube, const int fCubeOwner,
               const int fCrawford, const int fJacoby, const int fBeavers, 
               const bgvariation bgv );
  
+extern void 
+swap_us( unsigned int *p0, unsigned int *p1 );
+
 extern void 
 swap( int *p0, int *p1 );
 
 extern void 
-SanityCheck( int anBoard[ 2 ][ 25 ], float arOutput[] );
+SanityCheck( const TanBoard anBoard, float arOutput[] );
 
 extern int
-EvalBearoff1( int anBoard[ 2 ][ 25 ], float arOutput[], 
-              const bgvariation bgv );
+EvalBearoff1( const TanBoard anBoard, float arOutput[], 
+              const bgvariation bgv, NNState *nnStates );
 
 extern int
-EvalOver( int anBoard[ 2 ][ 25 ], float arOutput[], const bgvariation bgv );
+EvalOver( const TanBoard anBoard, float arOutput[], const bgvariation bgv, NNState *nnStates );
 
-extern float 
+extern float
 KleinmanCount (int nPipOnRoll, int nPipNotOnRoll);
 
-extern int 
-EvaluatePositionCubeful( int anBoard[ 2 ][ 25 ], float arCfOutput[],
-                         float arClOutput[ NUM_OUTPUTS ],
-                         cubeinfo *pci, evalcontext *pec, int nPlies );
-
 extern int
 GetDPEq ( int *pfCube, float *prDPEq, const cubeinfo *pci );
 
@@ -521,47 +491,21 @@ se_eq2mwc ( const float rEq, const cubeinfo *ci );
 extern char 
 *FormatEval ( char *sz, evalsetup *pes );
 
-/* For some reason, gcc complains when 2d non-const array is passed as
-   const array
-*/
-#define GCCCONSTAHACK (const float (*)[NUM_ROLLOUT_OUTPUTS])
-
 extern cubedecision
 FindCubeDecision ( float arDouble[],
-                   const float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
+                   float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 		   const cubeinfo *pci );
 
-extern int
-GeneralCubeDecisionE ( float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                       int anBoard[ 2 ][ 25 ],
+EXP_LOCK_FUN(int, GeneralCubeDecisionE, float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+                       const TanBoard anBoard,
                        const cubeinfo* pci, const evalcontext* pec,
 		       const evalsetup* pes );
 
-extern int
-GeneralEvaluationE ( float arOutput [ NUM_ROLLOUT_OUTPUTS ],
-                     int anBoard[ 2 ][ 25 ],
+EXP_LOCK_FUN(int, GeneralEvaluationE, float arOutput [ NUM_ROLLOUT_OUTPUTS ],
+                     const TanBoard anBoard,
                      const cubeinfo* pci, const evalcontext* pec );
 
 extern int
-GeneralEvaluationEPlied ( float arOutput [ NUM_ROLLOUT_OUTPUTS ],
-                          int anBoard[ 2 ][ 25 ],
-                          const cubeinfo* pci, const evalcontext* pec,
-			  int nPlies );
-
-extern int 
-EvaluatePositionCubeful3( int anBoard[ 2 ][ 25 ],
-                          float arOutput[ NUM_OUTPUTS ],
-                          float arCubeful[],
-                          const cubeinfo aciCubePos[], int cci, 
-                          const cubeinfo* pciMove, const evalcontext* pec,
-			  int nPlies, int fTop );
-
-extern int 
-GeneralEvaluationEPliedCubeful ( float arOutput [ NUM_ROLLOUT_OUTPUTS ],
-                                 int anBoard[ 2 ][ 25 ],
-                                 const cubeinfo* pci, const evalcontext* pec,
-                                 int nPlies );
-extern int
 cmp_evalsetup ( const evalsetup *pes1, const evalsetup *pes2 );
 
 extern int
@@ -575,15 +519,15 @@ extern char
 
 extern cubedecision
 FindBestCubeDecision ( float arDouble[], 
-                       const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
+                       float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
                        const cubeinfo *pci );
 
 extern int
 getCurrentGammonRates ( float aarRates[ 2 ][ 2 ], 
                         float arOutput[], 
-                        int anBoard[ 2 ][ 25 ],
+                        const TanBoard anBoard,
                         cubeinfo *pci,
-                        evalcontext *pec );
+                        const evalcontext *pec );
 
 extern void
 calculate_gammon_rates( float aarRates[ 2 ][ 2 ],
@@ -599,13 +543,13 @@ extern void
 getMatchPoints ( float aaarPoints[ 2 ][ 4 ][ 2 ],
                  int afAutoRedouble[ 2 ],
                  int afDead[ 2 ],
-                 cubeinfo *pci,
+                 const cubeinfo *pci,
                  float aarRates[ 2 ][ 2 ] );
 
 extern void
 getCubeDecisionOrdering ( int aiOrder[ 3 ],
                           float arDouble[ 4 ], 
-                          const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+                          float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
                           const cubeinfo* pci );
 
 extern float
@@ -615,8 +559,7 @@ getPercent ( const cubedecision cd,
 extern void
 RefreshMoveList ( movelist *pml, int *ai );
 
-extern int 
-ScoreMove( move *pm, const cubeinfo* pci, const evalcontext* pec, int nPlies );
+EXP_LOCK_FUN(int, ScoreMove, NNState *nnStates, move *pm, const cubeinfo* pci, const evalcontext* pec, int nPlies );
 
 extern void
 CopyMoveList ( movelist *pmlDest, const movelist *pmlSrc );
@@ -626,16 +569,16 @@ isCloseCubedecision ( const float arDouble[] );
 
 extern int
 isMissedDouble ( float arDouble[], 
-                 const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
+                 float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
                  const int fDouble, 
                  const cubeinfo *pci );
 
-extern int
-locateMove ( int anBoard[ 2 ][ 25 ], 
+extern unsigned int
+locateMove ( const TanBoard anBoard, 
              const int anMove[ 8 ], const movelist *pml );
 
 extern int
-MoveKey ( int anBoard[ 2 ][ 25 ], const int anMove[ 8 ], 
+MoveKey ( const TanBoard anBoard, const int anMove[ 8 ], 
           unsigned char auch[ 10 ] );
 
 extern int
@@ -653,12 +596,23 @@ DoubleType ( const int fDoubled, const int fMove, const int fTurn );
 
 extern int
 PerfectCubeful ( bearoffcontext *pbc, 
-                 int anBoard[ 2 ][ 25 ], float arEquity[] );
+                 const TanBoard anBoard, float arEquity[] );
 
 extern void
-baseInputs(int anBoard[2][25], float arInput[]);
+baseInputs(const TanBoard anBoard, float arInput[]);
 
 extern void 
-CalculateRaceInputs(int anBoard[2][25], float inputs[]);
+CalculateRaceInputs(const TanBoard anBoard, float inputs[]);
 
+
+extern float Noise( const evalcontext* pec, const TanBoard anBoard, int iOutput );
+extern int CompareMoves(const move *pm0, const move *pm1);
+extern float EvalEfficiency( const TanBoard anBoard, positionclass pc );
+extern float Cl2CfMoney ( float arOutput [ NUM_OUTPUTS ], cubeinfo *pci, float rCubeX );
+extern float Cl2CfMatch ( float arOutput [ NUM_OUTPUTS ], cubeinfo *pci, float rCubeX );
+extern float Noise( const evalcontext* pec, const TanBoard anBoard, int iOutput );
+extern int EvalKey ( const evalcontext *pec, const int nPlies, const cubeinfo *pci, int fCubefulEquity );
+extern void MakeCubePos( const cubeinfo aciCubePos[], const int cci, const int fTop, cubeinfo aci[], const int fInvert );
+extern void GetECF3 ( float arCubeful[], int cci, float arCf[], cubeinfo aci[] );
+extern int EvaluatePerfectCubeful ( const TanBoard anBoard, float arEquity[], const bgvariation bgv );
 #endif
