@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 2001, 2002.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: external.c,v 1.51 2006/09/21 22:24:05 Superfly_Jon Exp $
+ * $Id: external.c,v 1.70 2012/04/26 01:15:19 mdpetch Exp $
  */
 
 #include "config.h"
@@ -25,6 +25,7 @@
 
 #include <signal.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
@@ -32,11 +33,9 @@
 
 #ifndef WIN32
 
-#include <assert.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glib.h>
 
 #if HAVE_SYS_SOCKET_H
 #include <sys/types.h>
@@ -50,6 +49,7 @@
 #include <stdio.h>
 
 #else /* #ifndef WIN32 */
+#include <stdlib.h>
 #include <winsock2.h>
 #include <ws2tcpip.h>
 
@@ -104,7 +104,6 @@
 #include "drawboard.h"
 #include "external.h"
 #include "rollout.h"
-#include <glib/gi18n.h>
 
 #if HAVE_SOCKETS
 /* Stuff for the yacc/lex parser */
@@ -124,20 +123,24 @@ extern void ExtStartParse( const char *s );
 void OutputWin32SocketError(const char* action)
 {
 	LPVOID lpMsgBuf;
-	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+	if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
 		FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS ,
 		NULL, WSAGetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-		(LPTSTR) &lpMsgBuf, 0, NULL);
-
-	outputerrf(_("Windows socket error (%s):\n%s"), action, (LPCTSTR)lpMsgBuf);
-	LocalFree(lpMsgBuf);
+		(LPTSTR) &lpMsgBuf, 0, NULL) != 0)
+	{
+		outputerrf("%s (%s):\n%s", _("Windows socket error"), action, (LPCTSTR)lpMsgBuf);
+		if (LocalFree(lpMsgBuf) != NULL)
+			g_print("LocalFree() failed\n");
+	}
 }
 #endif
 
 extern int ExternalSocket( struct sockaddr **ppsa, int *pcb, char *sz ) {
 
     int sock, f;
+#ifndef WIN32
     struct sockaddr_un *psun;
+#endif
     struct sockaddr_in *psin;
     struct hostent *phe;
     char *pch;
@@ -172,7 +175,7 @@ extern int ExternalSocket( struct sockaddr **ppsa, int *pcb, char *sz ) {
 #else
 	else if( !inet_aton( sz, &psin->sin_addr ) ) {
 #endif /* WIN32 */
-	    if( !( phe = gethostbyname( sz ) ) ) {
+	    if( ( phe = gethostbyname( sz ) ) == 0 ) {
 		*pch = ':';
 		errno = EINVAL;
 		free( psin );
@@ -189,10 +192,11 @@ extern int ExternalSocket( struct sockaddr **ppsa, int *pcb, char *sz ) {
 
 	*pch++ = ':';
 	
-	psin->sin_port = htons( atoi( pch ) );
+	psin->sin_port = htons( (u_short)atoi( pch ) );
 	
 	*ppsa = (struct sockaddr *) psin;
     } else {
+#ifndef WIN32
 	/* Local domain socket. */
 	if( ( sock = socket( PF_LOCAL, SOCK_STREAM, 0 ) ) < 0 )
 	    return -1;
@@ -201,23 +205,16 @@ extern int ExternalSocket( struct sockaddr **ppsa, int *pcb, char *sz ) {
 	   sockaddr_un size, but this is a conservative estimate */
 	psun = malloc( *pcb = 16 + strlen( sz ) );
 	
-#ifndef WIN32
 	psun->sun_family = AF_LOCAL;
 	strcpy( psun->sun_path, sz );
+	*ppsa = (struct sockaddr *) psun;
 #else /* #ifndef WIN32 */
 	/* FIXME: what will we do on Windows? */
 	return -1;
 #endif /* #ifndef WIN32 */
-
-	*ppsa = (struct sockaddr *) psun;
     }
 
     return sock;
-
-#if 0 
-    assert( FALSE );
-#endif /* 0 */
-
 }
 #endif /* HAVE_SOCKETS */
 
@@ -228,12 +225,12 @@ static void ExternalUnbind( char *sz ) {
 	/* it was a TCP socket; no cleanup necessary */
 	return;
 
-    unlink( sz );
+    g_unlink( sz );
 }
 #endif /* HAVE_SOCKETS */
 
 #if HAVE_SOCKETS
-extern int ExternalRead( int h, char *pch, int cch ) {
+extern int ExternalRead( int h, char *pch, size_t cch ) {
 
     char *p = pch, *pEnd;
     int n;
@@ -241,9 +238,9 @@ extern int ExternalRead( int h, char *pch, int cch ) {
     psighandler sh;
 #endif
     
-    while( cch ) {
-	if( fAction )
-	    fnAction();
+    while( cch )
+	{
+		ProcessEvents();
 
 	if( fInterrupt )
 	    return -1;
@@ -287,14 +284,11 @@ extern int ExternalRead( int h, char *pch, int cch ) {
 
     p[ cch - 1 ] = 0;
     return 0;
-#if 0
-    assert( FALSE );
-#endif
 }
 #endif /* HAVE_SOCKETS */
 
 #if HAVE_SOCKETS
-extern int ExternalWrite( int h, char *pch, int cch ) {
+extern int ExternalWrite( int h, char *pch, size_t cch ) {
 
     char *p = pch;
     int n;
@@ -302,12 +296,12 @@ extern int ExternalWrite( int h, char *pch, int cch ) {
     psighandler sh;
 #endif
 
-    while( cch ) {
-	if( fAction )
-	    fnAction();
+    while( cch )
+	{
+		ProcessEvents();
 
-	if( fInterrupt )
-	    return -1;
+		if( fInterrupt )
+			return -1;
 
 #ifndef WIN32
 	PortableSignal( SIGPIPE, SIG_IGN, &sh, FALSE );
@@ -340,9 +334,6 @@ extern int ExternalWrite( int h, char *pch, int cch ) {
     }
 
     return 0;
-#if 0
-    assert( FALSE );
-#endif
 }
 #endif /* HAVE_SOCKETS */
 
@@ -376,8 +367,9 @@ static char *
 ExtEvaluation( extcmd *pec ) {
 
   char szName[ MAX_NAME_LEN ], szOpp[ MAX_NAME_LEN ];
-  int anBoard[ 2 ][ 25 ], nMatchTo, anScore[ 2 ],
-    anDice[ 2 ], nCube, fCubeOwner, fDoubled, fTurn, fCrawford;
+  int nMatchTo, anScore[ 2 ],
+    anDice[ 2 ], nCube, fCubeOwner, fDoubled, fTurn, fCrawford, fJacoby;
+  TanBoard anBoard;
   float arOutput[ NUM_ROLLOUT_OUTPUTS ];
   cubeinfo ci;
   int nScore, nScoreOpponent;
@@ -396,24 +388,27 @@ ExtEvaluation( extcmd *pec ) {
     
     anScore[ 0 ] = fTurn ? nScoreOpponent : nScore;
     anScore[ 1 ] = fTurn ? nScore : nScoreOpponent;
+	/* If the session isn't using Crawford rule, set crawford flag to false */
+	fCrawford = pec->fCrawfordRule ? fCrawford : FALSE;
+	/* Set the Jacoby flag appropriately from the external interface settings */
+	fJacoby   = pec->fJacobyRule;
+
+	/* printf ("Jacoby Setting: %d\n", fJacoby);*/
+	/* printf ("Crawford Setting: %d\n", fCrawford); */
     
     SetCubeInfo ( &ci, nCube, fCubeOwner, fTurn, nMatchTo, anScore,
                   fCrawford, fJacoby, nBeavers, bgvDefault ); 
 
     ec.fCubeful = pec->fCubeful;
     ec.nPlies = pec->nPlies;
-#if defined( REDUCTION_CODE )
-    ec.nReduced = pec->nReduced;
-#else
     ec.fUsePrune = pec->fUsePrune;
-#endif
     ec.fDeterministic = pec->fDeterministic;
     ec.rNoise = pec->rNoise;
-    
+
     if ( !fTurn )
       SwapSides( anBoard );
 
-    if ( GeneralEvaluationE( arOutput, anBoard, &ci, &ec ) )
+    if ( GeneralEvaluationE( arOutput, (ConstTanBoard)anBoard, &ci, &ec ) )
       return NULL;
 
     if ( nMatchTo ) {
@@ -442,9 +437,10 @@ static char *
 ExtFIBSBoard( extcmd *pec ) {
 
   char szName[ MAX_NAME_LEN ], szOpp[ MAX_NAME_LEN ];
-  int anBoard[ 2 ][ 25 ], anBoardOrig[ 2 ][ 25 ], nMatchTo, anScore[ 2 ],
-    anDice[ 2 ], nCube, fCubeOwner, fDoubled, fTurn, fCrawford,
+  int nMatchTo, anScore[ 2 ],
+    anDice[ 2 ], nCube, fCubeOwner, fDoubled, fTurn, fCrawford, fJacoby,
     anMove[ 8 ];
+  TanBoard anBoard, anBoardOrig;
   float arDouble[ NUM_CUBEFUL_OUTPUTS ],
     aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
     aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ];
@@ -464,7 +460,15 @@ ExtFIBSBoard( extcmd *pec ) {
     
     anScore[ 0 ] = fTurn ? nScoreOpponent : nScore;
     anScore[ 1 ] = fTurn ? nScore : nScoreOpponent;
-    
+
+	/* If the session isn't using Crawford rule, set crawford flag to false */
+	fCrawford = pec->fCrawfordRule ? fCrawford : FALSE;
+	/* Set the Jacoby flag appropriately from the external interface settings */
+	fJacoby   = pec->fJacobyRule;
+
+	/* printf ("Crawford Setting: %d\n", fCrawford); */
+	/* printf ("Jacoby Setting: %d\n", fJacoby); */
+	    
     SetCubeInfo ( &ci, nCube, fCubeOwner, fTurn, nMatchTo, anScore,
                   fCrawford, fJacoby, nBeavers, bgvDefault ); 
     
@@ -492,11 +496,11 @@ ExtFIBSBoard( extcmd *pec ) {
 
       /* take decision */
       if( GeneralCubeDecision( aarOutput, aarStdDev,
-                               aarsStatistics, anBoard, &ci,
-                               &esEvalCube, NULL, NULL ) < 0 )
+                               aarsStatistics, (ConstTanBoard)anBoard, &ci,
+                               GetEvalCube(), NULL, NULL ) < 0 )
         return NULL;
 	  
-      switch( FindCubeDecision( arDouble, GCCCONSTAHACK aarOutput, &ci )) {
+      switch( FindCubeDecision( arDouble,  aarOutput, &ci )) {
       case DOUBLE_PASS:
       case TOOGOOD_PASS:
       case REDOUBLE_PASS:
@@ -541,8 +545,8 @@ ExtFIBSBoard( extcmd *pec ) {
       /* move */
       char szMove[ 64 ];
       if( FindBestMove( anMove, anDice[ 0 ], anDice[ 1 ],
-                        anBoard, &ci, &esEvalChequer.ec,
-                        aamfEval ) < 0 )
+                        anBoard, &ci, &GetEvalChequer()->ec,
+                        *GetEvalMoveFilter() ) < 0 )
         return NULL;
 
       FormatMovePlain( szMove, anBoardOrig, anMove );
@@ -550,11 +554,11 @@ ExtFIBSBoard( extcmd *pec ) {
     } else {
       /* double decision */
       if( GeneralCubeDecision( aarOutput, aarStdDev,
-                               aarsStatistics, anBoard, &ci,
-                               &esEvalCube, NULL, NULL ) < 0 )
+                               aarsStatistics, (ConstTanBoard)anBoard, &ci,
+                               GetEvalCube(), NULL, NULL ) < 0 )
         return NULL;
 		
-      switch( FindCubeDecision( arDouble, GCCCONSTAHACK aarOutput, &ci )) {
+      switch( FindCubeDecision( arDouble,  aarOutput, &ci )) {
       case DOUBLE_TAKE:
       case DOUBLE_PASS:
       case DOUBLE_BEAVER:
@@ -592,8 +596,7 @@ extern void CommandExternal( char *sz ) {
     sz = NextToken( &sz );
     
     if( !sz || !*sz ) {
-	outputl( _("You must specify the name of the socket to the external\n"
-		 "controller -- try `help external'.") );
+	outputl( _("You must specify the name of the socket to the external controller.") );
 	return;
     }
 
@@ -615,19 +618,22 @@ listenloop:
       free( psa );
       
       if( listen( h, 1 ) < 0 ) {
-	SockErr( _("listen") );
+	SockErr( "listen" );
 	closesocket( h );
 	ExternalUnbind( sz );
 	return;
       }
       outputf( _("Waiting for a connection from %s...\n"), sz);
+      outputx();
+      ProcessEvents();
 
       /* Must set length when using windows */
       saLen = sizeof(struct sockaddr);
-      while( ( hPeer = accept( h, (struct sockaddr*)&saRemote, &saLen ) ) < 0 ) {
-	if( errno == EINTR ) {
-          if( fAction )
-            fnAction();
+      while( ( hPeer = accept( h, (struct sockaddr*)&saRemote, &saLen ) ) < 0 )
+	  {
+		if( errno == EINTR )
+		{
+			ProcessEvents();
 
           if( fInterrupt ) {
             closesocket( h );
@@ -638,7 +644,7 @@ listenloop:
           continue;
 	}
 	
-	SockErr( _("accept") );
+	SockErr( "accept" );
 	closesocket( h );
 	ExternalUnbind( sz );
 	return;
@@ -649,12 +655,14 @@ listenloop:
 
       /* print info about remove client */
 
-      outputf( _("Accepted connection from %s.\n"), 
+      outputf(_("Accepted connection from %s.\n"),
                  inet_ntoa( saRemote.sin_addr ) );
+      outputx();
+      ProcessEvents();
 
       while( !ExternalRead( hPeer, szCommand, sizeof( szCommand ) ) ) {
 
-        if ( ! ( pec = ExtParse( szCommand ) ) ) {
+        if ( ( pec = ExtParse( szCommand ) ) == 0 ) {
           /* parse error */
           szResponse = szError;
         }
