@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 2001.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: external.h,v 1.8 2004/03/18 23:29:14 oysteijo Exp $
+ * $Id: external.h,v 1.13 2007/12/18 21:48:03 Superfly_Jon Exp $
  */
 
 #ifndef _EXTERNAL_H_
@@ -26,6 +26,8 @@
 
 #ifndef WIN32
 
+#define closesocket close
+
 #if HAVE_SYS_SOCKET_H
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -51,14 +53,21 @@ typedef struct _extcmd {
   int fDeterministic;
   int fCubeful;
   int nReduced;
+  int fUsePrune;
 
   /* fibs board */
   char *szFIBSBoard;
 } extcmd;
 
 extern int ExternalSocket( struct sockaddr **ppsa, int *pcb, char *sz );
-extern int ExternalRead( int h, char *pch, int cch );
-extern int ExternalWrite( int h, char *pch, int cch );
+extern int ExternalRead( int h, char *pch, size_t cch );
+extern int ExternalWrite( int h, char *pch, size_t cch );
+#ifdef WIN32
+extern void OutputWin32SocketError(const char* action);
+#define SockErr OutputWin32SocketError
+#else
+#define SockErr outputerr
+#endif
 
 /* Parser functions */
 
