@@ -4,7 +4,7 @@
  * by Joern Thyssen  <jth@gnubg.org>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,36 +16,38 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format.c,v 1.16 2006/03/28 22:35:45 c_anthon Exp $
+ * $Id: format.c,v 1.43 2011/10/31 09:41:12 c_anthon Exp $
  */
 
 #include "config.h"
+#include "backgammon.h"
 
-#include <stdio.h>
+#include <glib.h>
 #include <string.h>
 
-#define USES_badSkill
-#include "backgammon.h"
 #include "eval.h"
 #include "format.h"
 
-#include "i18n.h"
 #include "export.h"
+#include "positionid.h"
 
 
 int fOutputMWC = TRUE;
 int fOutputWinPC = FALSE;
 int fOutputMatchPC = TRUE;
-int fOutputDigits = 3;
+unsigned int fOutputDigits = 3;
 float rErrorRateFactor = 1000.0f;
 
+typedef int ( *classdumpfunc )( const TanBoard anBoard, char *szOutput,
+                                 const bgvariation bgv );
 
 extern char *
 OutputRolloutResult( const char *szIndent,
 		     char asz[][ 1024 ],
-		     const  float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
-		     const float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
+		     float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
+		     float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
 		     const cubeinfo aci[],
+		     const int alt,
 		     const int cci,
 		     const int fCubeful ) {
 
@@ -74,7 +76,7 @@ OutputRolloutResult( const char *szIndent,
     strcat ( sz, OutputPercents ( aarOutput[ ici ], TRUE ) );
     strcat ( sz, " CL " );
     strcat ( sz, OutputEquityScale ( aarOutput[ ici ][ OUTPUT_EQUITY ], 
-                                     &aci[ ici ], &aci[ 0 ], TRUE ) );
+                                     &aci[ alt + ici ], &aci[ 0 ], TRUE ) );
 
     if ( fCubeful ) {
       strcat ( sz, " CF " );
@@ -93,7 +95,7 @@ OutputRolloutResult( const char *szIndent,
     strcat ( sz, OutputPercents ( aarStdDev[ ici ], FALSE ) );
     strcat ( sz, " CL " );
     strcat ( sz, OutputEquityScale ( aarStdDev[ ici ][ OUTPUT_EQUITY ], 
-                                     &aci[ ici ], &aci[ 0 ], FALSE ) );
+                                     &aci[ alt + ici ], &aci[ 0 ], FALSE ) );
 
     if ( fCubeful ) {
       strcat ( sz, " CF " );
@@ -117,20 +119,13 @@ OutputEvalContext ( const evalcontext *pec, const int fChequer ) {
   char *pc;
   int i;
   
-  sprintf ( sz, _("%d-ply %s"), 
-            pec->nPlies, 
+  sprintf ( sz, "%d-%s %s", 
+            pec->nPlies, _("ply"),
             ( ! fChequer || pec->fCubeful ) ? _("cubeful") : _("cubeless") );
 
-#if defined( REDUCTION_CODE )
-  if ( pec->nPlies == 2 ) 
-    sprintf ( pc = strchr ( sz, 0 ),
-              " %d%% speed",
-              (pec->nReduced) ? 100 / pec->nReduced : 100 );
-#else
   if( pec->fUsePrune ) {
     sprintf( pc = strchr ( sz, 0 ), " prune" );
   }
-#endif
 	    
   if ( fChequer && pec->nPlies ) {
     /* FIXME: movefilters!!! */
@@ -147,7 +142,7 @@ OutputEvalContext ( const evalcontext *pec, const int fChequer ) {
     if ( ! cmp_evalcontext ( &aecSettings[ i ], pec ) ) {
       sprintf ( pc = strchr ( sz, 0 ),
                 " [%s]",
-                gettext ( aszSettings[ i ] ) );
+                Q_ ( aszSettings[ i ] ) );
       break;
     }
 
@@ -161,7 +156,7 @@ OutputEvalContext ( const evalcontext *pec, const int fChequer ) {
            n = index of predefined setting if it's a match or
                if the eval context matches and it's 0 ply
  */
-extern int
+static int
 GetPredefinedChequerplaySetting( const evalcontext *pec,
                                  movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ] ) {
 
@@ -225,7 +220,7 @@ GetPredefinedChequerplaySetting( const evalcontext *pec,
   return -1;
 }
 
-extern char *
+static char *
 OutputMoveFilterPly( const char *szIndent, 
                      const int nPlies,
                      movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ] ) {
@@ -245,28 +240,25 @@ OutputMoveFilterPly( const char *szIndent,
     if ( szIndent && *szIndent )
       strcat( sz, szIndent );
 
-    if ( pmf->Accept < 0 ) {
-      sprintf( strchr( sz, 0 ), 
-               _("Skip pruning for %d-ply moves.\n"),
-               i );
+    if ( pmf->Accept < 0 )
+	{
+      sprintf( strchr( sz, 0 ), _("Skip pruning for %d-ply moves."), i );
+	  strcat(sz, "\n");
       continue;
     }
 
     if ( pmf->Accept == 1 )
-      sprintf( strchr( sz, 0 ),
-               _("keep the best %d-ply move"), i );
+      sprintf( strchr( sz, 0 ), _("keep the best %d-ply move"), i );
     else
-      sprintf( strchr( sz, 0 ),
-               _("keep the first %d %d-ply moves"),
+      sprintf( strchr( sz, 0 ), _("keep the first %d %d-ply moves"),
                pmf->Accept, i );
 
     if ( pmf->Extra )
       sprintf( strchr( sz, 0 ),
-               _(" and up to %d more moves within equity %0.3g\n"),
+               _(" and up to %d more moves within equity %0.3g"),
                pmf->Extra, pmf->Threshold );
-    else
-      strcat( sz, "\n" );
 
+	strcat( sz, "\n" );
   }
 
   return sz;
@@ -297,7 +289,7 @@ OutputEvalContextsForRollout( char *sz, const char *szIndent,
       if ( szIndent && *szIndent )
         strcat ( sz, szIndent );
 
-      sprintf ( strchr ( sz, 0 ), _("Player %d:\n" ), i );
+      sprintf ( strchr ( sz, 0 ), "%s %d:\n", _("Player"), i );
     }
 
     /* chequer play */
@@ -310,8 +302,8 @@ OutputEvalContextsForRollout( char *sz, const char *szIndent,
     if ( aecChequer[ i ].nPlies ) {
 
       sprintf( strchr( sz, 0 ),
-               _("Play: %s "), 
-               ( j < 0 ) ? "" : gettext ( aszSettings[ j ] ) );
+               "%s: %s ", _("Play"),
+               ( j < 0 ) ? "" : Q_ ( aszSettings[ j ] ) );
 
       strcat( sz, OutputEvalContext ( &aecChequer[ i ], TRUE ) );
       strcat( sz, "\n" );
@@ -320,7 +312,7 @@ OutputEvalContextsForRollout( char *sz, const char *szIndent,
 
     }
     else {
-      strcat ( sz, _("Play: ") );
+      sprintf ( strchr( sz, 0 ), "%s: ", _("Play") );
       strcat ( sz, OutputEvalContext ( &aecChequer[ i ], FALSE ) );
       strcat ( sz, "\n" );
     }
@@ -328,7 +320,7 @@ OutputEvalContextsForRollout( char *sz, const char *szIndent,
     if ( szIndent && *szIndent )
       strcat ( sz, szIndent );
 
-    strcat ( sz, _("Cube: ") );
+    sprintf ( strchr( sz, 0 ), "%s: ", _("Cube") );
     strcat ( sz, OutputEvalContext ( &aecCube[ i ], FALSE ) );
     strcat ( sz, "\n" );
 
@@ -339,9 +331,8 @@ OutputEvalContextsForRollout( char *sz, const char *szIndent,
 
 
 extern char *
-OutputRolloutContext ( const char *szIndent, const evalsetup *pes ) {
+OutputRolloutContext ( const char *szIndent, const rolloutcontext *prc ) {
 
-  const rolloutcontext *prc = &pes->rc;
   static char sz[ 1024 ];
 
   strcpy ( sz, "" );
@@ -362,12 +353,12 @@ OutputRolloutContext ( const char *szIndent, const evalsetup *pes ) {
               _("Full cubeless rollout") );
 
   if ( prc->fTruncBearoffOS && ! prc->fCubeful )
-    strcat ( sz, _(" (trunc. at one-sided bearoff)") );
+    sprintf( strchr ( sz, 0 ), " (%s)", _("trunc. at one-sided bearoff") );
   else if ( prc->fTruncBearoff2 && ! prc->fCubeful )
-    strcat ( sz, _(" (trunc. at exact bearoff)") );
+    sprintf( strchr ( sz, 0 ), " (%s)", _("trunc. at exact bearoff") );
 
-  sprintf ( strchr ( sz, 0 ),
-            prc->fVarRedn ? _(" with var.redn.") : _(" without var.redn.") );
+  sprintf ( strchr ( sz, 0 ), " %s",
+            prc->fVarRedn ? _("with var.redn.") : _("without var.redn.") );
 
   strcat ( sz, "\n" );
   
@@ -381,35 +372,44 @@ OutputRolloutContext ( const char *szIndent, const evalsetup *pes ) {
   if ( prc->fInitial )
     strcat ( sz, ", rollout as initial position" );
 
+  strcat(sz, ", ");
   if ( prc->fRotate ) 
     sprintf ( strchr ( sz, 0 ),
-              _(", %s dice gen. with seed %d and quasi-random dice\n"),
+              _("%s dice gen. with seed %lu and quasi-random dice"),
               gettext( aszRNG[ prc->rngRollout ] ),
-              prc->nSeed );
+              prc->nSeed ); /* seed may be unsigned long int */
   else
     sprintf ( strchr ( sz, 0 ),
-              _(", %s dice generator with seed %d\n"),
+              _("%s dice generator with seed %lu"),
               gettext( aszRNG[ prc->rngRollout ] ),
-              prc->nSeed );
+              prc->nSeed ); /* seed may be unsigned long int */
 
-   if ( ( prc->fStopOnJsd || prc->fStopMoveOnJsd || prc->fStopOnSTD )
+  strcat ( sz, "\n" );
+
+   if ( ( prc->fStopOnJsd || prc->fStopOnSTD )
         && szIndent && *szIndent )
      strcat ( sz, szIndent );
 
     /* stop on std.err */
 
-   if ( prc->fStopOnSTD && !prc->fStopMoveOnJsd )
+   if ( prc->fStopOnSTD && !prc->fStopOnJsd )
+   {
     sprintf( strchr( sz, 0 ),
              _("Stop when std.errs. are small enough: ratio "
-               "%.4g (min. %d games)\n"),
+               "%.4g (min. %d games)"),
              prc->rStdLimit, prc->nMinimumGames );
+    strcat ( sz, "\n" );
+   }
 
    /* stop on JSD */
-   if ( prc->fStopOnJsd || prc->fStopMoveOnJsd )
+   if ( prc->fStopOnJsd )
+   {
      sprintf( strchr( sz, 0 ),
               _("Stop when best play is enough JSDs ahead: limit "
-                "%.4g (min. %d games)\n"),
+                "%.4g (min. %d games)"),
               prc->rJsdLimit, prc->nMinimumJsdGames );
+    strcat ( sz, "\n" );
+   }
 
   /* first play */
 
@@ -424,9 +424,8 @@ OutputRolloutContext ( const char *szIndent, const evalsetup *pes ) {
     if ( szIndent && *szIndent )
       strcat ( sz, szIndent );
     
-    sprintf( strchr( sz, 0 ), 
-             _("Different evaluations after %d plies:\n"),
-             prc->nLate );
+    sprintf( strchr( sz, 0 ), _("Different evaluations after %d plies:"), prc->nLate );
+    strcat ( sz, "\n" );
 
     OutputEvalContextsForRollout( sz, szIndent, 
                                   prc->aecCubeLate, prc->aecChequerLate,
@@ -456,25 +455,20 @@ extern char *
 OutputEquity ( const float r, const cubeinfo *pci, const int f ) {
 
   static char sz[ 9 ];
-  char fmt[ 32 ];
 
   if ( !pci->nMatchTo || ( pci->nMatchTo && ! fOutputMWC ) ) {
-    /* fmt: "%+7.3f" or "%7.3f" */
-    sprintf( fmt, f ? "%%+%d.%df" : "%%%d.%df", 
-             fOutputDigits + 4, fOutputDigits );
-    sprintf ( sz, fmt, r );
+    if (f)
+	    sprintf ( sz, "%+*.*f", fOutputDigits + 4, fOutputDigits, r );
+    else
+	    sprintf ( sz, "%*.*f", fOutputDigits + 4, fOutputDigits, r );
   }
   else {
     if ( fOutputMatchPC ) {
-      /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
-      sprintf ( sz, fmt,
+      sprintf ( sz, "%*.*f%%",fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
                 100.0f * ( f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) ) );
     }
     else {
-      /* fmt: "%6.4f" */
-      sprintf( fmt, "%%+%d.%df", fOutputDigits + 3, fOutputDigits + 1 );
-      sprintf ( sz, fmt, f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) );
+      sprintf ( sz, "%*.*f",fOutputDigits + 3, fOutputDigits + 1, f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) );
     }
   }
 
@@ -483,22 +477,20 @@ OutputEquity ( const float r, const cubeinfo *pci, const int f ) {
 }
 
 
-extern char *
-OutputMoneyEquity ( const float ar[], const int f ) {
+extern char *OutputMoneyEquity(const float ar[], const int f)
+{
 
-  static char sz[ 9 ];
-  char fmt[ 32 ];
+	static char sz[9];
+	float eq = 2.0f * ar[OUTPUT_WIN] - 1.0f + ar[OUTPUT_WINGAMMON] + ar[OUTPUT_WINBACKGAMMON] -
+	    ar[OUTPUT_LOSEGAMMON] - ar[OUTPUT_LOSEBACKGAMMON];
 
-  /* fmt: "%+7.3f" or "%7.3f" */
-  sprintf( fmt, f ? "%%+%d.%df" : "%%%d.%df", 
-           fOutputDigits + 4, fOutputDigits );
+	if (f)
+		sprintf(sz, "%+*.*f", fOutputDigits + 4, fOutputDigits, eq);
+	else
+		sprintf(sz, "%*.*f", fOutputDigits + 4, fOutputDigits, eq);
 
-  sprintf ( sz, fmt,
-            2.0 * ar[ OUTPUT_WIN ] - 1.0
-            + ar[ OUTPUT_WINGAMMON ] + ar[ OUTPUT_WINBACKGAMMON] 
-            - ar[ OUTPUT_LOSEGAMMON ] - ar[ OUTPUT_LOSEBACKGAMMON] );
 
-  return sz;
+	return sz;
 
 }
 
@@ -522,38 +514,30 @@ OutputEquityScale ( const float r, const cubeinfo *pci,
                     const cubeinfo *pciBase, const int f ) {
 
   static char sz[ 9 ];
-  char fmt[ 32 ];
 
   if ( ! pci->nMatchTo ) {
-    /* fmt: "%+7.3f" or "%7.3f" */
-    sprintf( fmt, f ? "%%+%d.%df" : "%%%d.%df", 
-             fOutputDigits + 4, fOutputDigits );
-    sprintf ( sz, fmt, pci->nCube / pciBase->nCube * r );
+	  if (f)
+		  sprintf(sz, "%+*.*f", fOutputDigits + 4, fOutputDigits, pci->nCube / pciBase->nCube * r);
+	  else
+		  sprintf(sz, "%*.*f", fOutputDigits + 4, fOutputDigits,pci->nCube / pciBase->nCube * r);
   }
   else {
 
     if ( fOutputMWC ) {
 
       if ( fOutputMatchPC ) {
-        /* fmt: "%6.2f%%" */
-        sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
-        sprintf ( sz, fmt,
-                  100.0f * ( f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) ) );
+        sprintf ( sz, "%*.*f%%",fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0, 100.0f * ( f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) ) );
       }
       else {
-        /* fmt: "%6.4f" */
-        sprintf( fmt, "%%+%d.%df", fOutputDigits + 3, fOutputDigits + 1 );
-        sprintf ( sz, fmt, f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) );
+        sprintf( sz, "%*.*f", fOutputDigits + 3, fOutputDigits + 1, f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) );
       }
 
     }
     else {
-      /* fmt: "%+7.3f" or "%7.3f" */
-      sprintf( fmt, f ? "%%+%d.%df" : "%%%d.%df", 
-               fOutputDigits + 4, fOutputDigits );
-      sprintf ( sz, fmt, f ? 
-                mwc2eq ( eq2mwc ( r, pci ),pciBase ) : 
-                se_mwc2eq( se_eq2mwc( r, pci ), pciBase ) );
+	    if (f)
+		    sprintf ( sz, "%+*.*f",fOutputDigits + 4, fOutputDigits, mwc2eq ( eq2mwc ( r, pci ),pciBase ));
+	    else
+		    sprintf ( sz, "%*.*f",fOutputDigits + 4, fOutputDigits, se_mwc2eq( se_eq2mwc( r, pci ), pciBase ) );
     }
       
 
@@ -581,70 +565,17 @@ extern char *
 OutputEquityDiff ( const float r1, const float r2, const cubeinfo *pci ) {
 
   static char sz[ 9 ];
-  char fmt[ 32 ];
 
   if ( !pci->nMatchTo || ( pci->nMatchTo && ! fOutputMWC ) ) {
-    /* fmt: "%+7.3f" */
-    sprintf( fmt, "%%+%d.%df", fOutputDigits + 4, fOutputDigits );
-    sprintf ( sz, fmt, r1 - r2 );
+    sprintf ( sz, "%+*.*f", fOutputDigits + 4, fOutputDigits, r1 - r2 );
   }
   else {
     if ( fOutputMatchPC ) {
-      /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
-      sprintf ( sz, fmt,
+      sprintf ( sz, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0, 
                 100.0f * eq2mwc ( r1, pci ) - 100.0f * eq2mwc ( r2, pci ) );
     }
     else {
-      /* fmt: "%6.4f" */
-      sprintf( fmt, "%%+%d.%df", fOutputDigits + 3, fOutputDigits + 1 );
-      sprintf ( sz, fmt, eq2mwc ( r1, pci ) - eq2mwc ( r2, pci ) );
-    }
-  }
-
-  return sz;
-
-}
-
-
-/*
- * Return formatted string with equity or MWC for an equity difference.
- *
- * Input:
- *    r: equity (either money equity or MWC for match play
- *    pci: cubeinfo
- *    f: indicates equity (TRUE) or std. error (FALSE)
- *    
- *
- * Important: function is not re-entrant. Caller must save output
- * if needed.
- */
-
-extern char *
-OutputMWCDiff ( const float r1, const float r2, const cubeinfo *pci ) {
-
-  static char sz[ 9 ];
-  char fmt[ 32 ];
-
-  if ( !pci->nMatchTo || ( pci->nMatchTo && ! fOutputMWC ) ) {
-    /* fmt: "%+7.3f" */
-    sprintf( fmt, "%%+%d.%df", fOutputDigits + 4, fOutputDigits );
-    if ( pci->nMatchTo ) 
-      sprintf ( sz, fmt, r1 - r2 );
-    else
-      sprintf ( sz, fmt, mwc2eq( r1, pci ) - mwc2eq( r2, pci ) );
-  }
-  else {
-    if ( fOutputMatchPC ) {
-      /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
-      sprintf ( sz, fmt,
-                100.0f * r1 - 100.0f * r2 );
-    }
-    else {
-      /* fmt: "%6.4f" */
-      sprintf( fmt, "%%+%d.%df", fOutputDigits + 3, fOutputDigits + 1 );
-      sprintf ( sz, fmt, r1 - r2 );
+      sprintf ( sz, "%*.*f",  fOutputDigits + 3, fOutputDigits + 1, eq2mwc ( r1, pci ) - eq2mwc ( r2, pci ) );
     }
   }
 
@@ -666,33 +597,29 @@ extern char *
 OutputMWC ( const float r, const cubeinfo *pci, const int f ) {
 
   static char sz[ 9 ];
-  char fmt[ 32 ];
 
   if ( ! pci->nMatchTo ) {
-    /* fmt: "%+7.3f" */
-    sprintf( fmt, f ? "%%+%d.%df" : "%%%d.%df", 
-             fOutputDigits + 4, fOutputDigits );
-    sprintf ( sz, fmt, r );
+	  if (f)
+		  sprintf ( sz, "%+*.*f", fOutputDigits + 4, fOutputDigits, r );
+	  else
+		  sprintf ( sz, "%*.*f", fOutputDigits + 4, fOutputDigits, r );
   }
   else {
     
     if ( ! fOutputMWC ) {
-      /* fmt: "%+7.3f" */
-      sprintf( fmt, f ? "%%+%d.%df" : "%%%d.%df", 
-               fOutputDigits + 4, fOutputDigits );
-      sprintf ( sz, fmt, 
-                f ? mwc2eq ( r, pci ) : se_mwc2eq ( r, pci ) );
+      if (f)
+	      sprintf ( sz, "%+*.*f", fOutputDigits + 4, fOutputDigits, mwc2eq ( r, pci ));
+      else
+	      sprintf ( sz, "%*.*f", fOutputDigits + 4, fOutputDigits, se_mwc2eq ( r, pci ) );
     }
     else if ( fOutputMatchPC ) {
-      /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
-      sprintf ( sz, fmt, 100.0f * r );
+      sprintf ( sz, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 , 100.0f * r );
     }
     else {
-      /* fmt: "%6.4f" */
-      sprintf( fmt, f? "%%+%d.%df" : "%%%d.%df", 
-               fOutputDigits + 3, fOutputDigits + 1 );
-      sprintf ( sz, fmt, r );
+	    if (f)
+		    sprintf ( sz, "%+*.*f", fOutputDigits + 3, fOutputDigits + 1 , r );
+	    else
+		    sprintf ( sz, "%*.*f", fOutputDigits + 3, fOutputDigits + 1 , r );
     }
   }
 
@@ -705,17 +632,12 @@ extern char *
 OutputPercent ( const float r ) {
 
   static char sz[ 9 ];
-  char fmt[ 32 ];
 
   if ( fOutputWinPC ) {
-    /* fmt: "%5.1f" */
-	  sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 2, fOutputDigits > 2 ? fOutputDigits - 2 : 0 );
-    sprintf ( sz, fmt, 100.0 * r );
+    sprintf ( sz, "%*.*f", fOutputDigits + 2, fOutputDigits > 2 ? fOutputDigits - 2 : 0, 100.0 * r );
   }
   else {
-    /* fmt: "%5.3f" */
-    sprintf( fmt, "%%%d.%df", fOutputDigits + 2, fOutputDigits  );
-    sprintf ( sz, fmt, r );
+    sprintf ( sz, "%*.*f", fOutputDigits + 2, fOutputDigits, r );
   }
 
   return sz;
@@ -765,8 +687,8 @@ OutputPercents ( const float ar[], const int f ) {
  */
 
 extern char *
-OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                         const float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+OutputCubeAnalysisFull ( float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+                         float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
                          const evalsetup* pes, const cubeinfo* pci,
                          int fDouble, int fTake,
                          skilltype stDouble,
@@ -805,7 +727,7 @@ OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
     /* missed double */
 
-    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!",
+    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!\n",
               _("Alert: missed double"),
               OutputEquityDiff ( arDouble[ OUTPUT_NODOUBLE ], 
                                  ( arDouble[ OUTPUT_TAKE ] > 
@@ -828,7 +750,7 @@ OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
     /* wrong take */
 
-    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!",
+    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!\n",
               _("Alert: wrong take"),
               OutputEquityDiff ( arDouble[ OUTPUT_DROP ],
                                  arDouble[ OUTPUT_TAKE ],
@@ -848,7 +770,7 @@ OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
     /* wrong pass */
 
-    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!",
+    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!\n",
               _("Alert: wrong pass"),
               OutputEquityDiff ( arDouble[ OUTPUT_TAKE ],
                                  arDouble[ OUTPUT_DROP ],
@@ -872,7 +794,7 @@ OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
     /* wrong double */
 
-    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!",
+    sprintf ( pc = strchr ( sz, 0 ), "%s (%s)!\n",
               _("Alert: wrong double"),
               OutputEquityDiff ( ( arDouble[ OUTPUT_TAKE ] > 
                                    arDouble[ OUTPUT_DROP ] ) ? 
@@ -910,8 +832,8 @@ OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 }
 
 extern char *
-OutputCubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                    const float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+OutputCubeAnalysis( float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+                    float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
                     const evalsetup* pes, const cubeinfo* pci ) {
 
   static char sz[ 4096 ];
@@ -932,7 +854,7 @@ OutputCubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
   /* header */
 
-  strcpy ( sz, _("\nCube analysis\n") );
+  sprintf(sz, "\n%s\n", _("Cube analysis"));
 
   /* ply & cubeless equity */
 
@@ -942,8 +864,8 @@ OutputCubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
     break;
   case EVAL_EVAL:
     sprintf ( strchr ( sz, 0 ), 
-              _("%d-ply"), 
-              pes->ec.nPlies );
+              "%d-%s", 
+              pes->ec.nPlies, _("ply"));
     break;
   case EVAL_ROLLOUT:
     strcat ( sz, _("Rollout") );
@@ -958,7 +880,7 @@ OutputCubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
               _("Money"), 
               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
   else
-    sprintf ( strchr ( sz, 0 ), _(" cubeless equity %s\n"),
+    sprintf ( strchr ( sz, 0 ), " %s %s\n", _("cubeless equity"),
               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
 
 
@@ -977,7 +899,8 @@ OutputCubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
   /* equities */
 
-  strcat( sz, _("Cubeful equities:\n") );
+  strcat( sz, _("Cubeful equities") );
+  strcat( sz, ":\n" );
   if ( pes->et == EVAL_EVAL && exsExport.afCubeParameters[ 0 ] ) {
     strcat( sz, "  " );
     strcat( sz, OutputEvalContext( &pes->ec, FALSE ) );
@@ -1008,8 +931,8 @@ OutputCubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
   cd = FindBestCubeDecision ( arDouble, aarOutput, pci );
 
   sprintf ( strchr ( sz, 0 ),
-            "%s %s",
-            _("Proper cube action:"),
+            "%s: %s",
+            _("Proper cube action"),
             GetCubeRecommendation ( cd ) );
 
   if ( ( r = getPercent ( cd, arDouble ) ) >= 0.0 )
@@ -1037,17 +960,18 @@ OutputCubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
     }
 
-    strcat ( sz, _("Rollout details:\n") );
+	sprintf(strchr ( sz, 0 ), "\n%s:\n", _("Rollout details"));
 
-    strcat ( sz, 
+    strcat ( strchr ( sz, 0 ),
              OutputRolloutResult ( NULL, asz, aarOutput, aarStdDev,
-                                   aci, 2, pes->rc.fCubeful ) );
+                                   aci, 0, 2, pes->rc.fCubeful ) );
              
 
   }
 
   if ( pes->et == EVAL_ROLLOUT && exsExport.afCubeParameters[ 1 ] )
-    strcat ( sz, OutputRolloutContext ( NULL, pes ) );
+    strcat ( strchr ( sz, 0 ),
+    	OutputRolloutContext ( NULL, &pes->rc ) );
     
   return sz;
 }
@@ -1084,3 +1008,263 @@ extern char *FormatCubePosition ( char *sz, cubeinfo *pci ) {
 
 }
 
+static int DumpOver(const TanBoard anBoard, char *pchOutput,
+		    const bgvariation bgv)
+{
+
+	float ar[NUM_OUTPUTS] = { 0, 0, 0, 0, 0 };	/* NUM_OUTPUTS is 5 */
+
+	if (EvalOver(anBoard, ar, bgv, NULL))
+		return -1;
+
+	if (ar[OUTPUT_WIN] > 0.0)
+		strcpy(pchOutput, _("Win"));
+	else
+		strcpy(pchOutput, _("Loss"));
+	strcat(pchOutput, " ");
+
+	if (ar[OUTPUT_WINBACKGAMMON] > 0.0 ||
+	    ar[OUTPUT_LOSEBACKGAMMON] > 0.0)
+		sprintf(pchOutput, "(%s)\n", _("backgammon"));
+	else if (ar[OUTPUT_WINGAMMON] > 0.0 || ar[OUTPUT_LOSEGAMMON] > 0.0)
+		sprintf(pchOutput, "(%s)\n", _("gammon"));
+	else
+		sprintf(pchOutput, "(%s)\n", _("single"));
+
+	return 0;
+
+}
+
+
+static int
+DumpBearoff1(const TanBoard anBoard, char *szOutput, const bgvariation UNUSED(bgv))
+{
+
+	g_assert(pbc1);
+	return BearoffDump(pbc1, anBoard, szOutput);
+
+}
+
+static int
+DumpBearoff2(const TanBoard anBoard, char *szOutput, const bgvariation UNUSED(bgv))
+{
+
+	g_assert(pbc2);
+
+	if (BearoffDump(pbc2, anBoard, szOutput))
+		return -1;
+
+	if (pbc1)
+		if (BearoffDump(pbc1, anBoard, szOutput))
+			return -1;
+
+	return 0;
+
+}
+
+
+static int
+DumpBearoffOS(const TanBoard anBoard, char *szOutput, const bgvariation UNUSED(bgv))
+{
+
+	g_assert(pbcOS);
+	return BearoffDump(pbcOS, anBoard, szOutput);
+
+}
+
+
+static int
+DumpBearoffTS(const TanBoard anBoard, char *szOutput, const bgvariation UNUSED(bgv))
+{
+
+	g_assert(pbcTS);
+	return BearoffDump(pbcTS, anBoard, szOutput);
+
+}
+
+
+static int DumpRace(const TanBoard UNUSED(anBoard), char *UNUSED(szOutput),
+		    const bgvariation UNUSED(bgv))
+{
+
+	/* no-op -- nothing much we can say, really (pip count?) */
+	return 0;
+
+}
+
+static void
+DumpAnyContact(const TanBoard UNUSED(anBoard), char *UNUSED(szOutput),
+	       const bgvariation UNUSED(bgv), int UNUSED(isCrashed))
+{
+	return;
+}
+
+static int
+DumpContact(const TanBoard anBoard, char *szOutput, const bgvariation bgv)
+{
+	DumpAnyContact(anBoard, szOutput, bgv, 0);
+	return 0;
+}
+
+static int
+DumpCrashed(const TanBoard anBoard, char *szOutput, const bgvariation bgv)
+{
+	DumpAnyContact(anBoard, szOutput, bgv, 1);
+	return 0;
+}
+
+static int
+DumpHypergammon1(const TanBoard anBoard, char *szOutput, const bgvariation UNUSED(bgv))
+{
+
+	g_assert(apbcHyper[0]);
+	return BearoffDump(apbcHyper[0], anBoard, szOutput);
+
+}
+
+static int
+DumpHypergammon2(const TanBoard anBoard, char *szOutput, const bgvariation UNUSED(bgv))
+{
+
+	g_assert(apbcHyper[1]);
+	return BearoffDump(apbcHyper[1], anBoard, szOutput);
+
+}
+
+static int
+DumpHypergammon3(const TanBoard anBoard, char *szOutput, const bgvariation UNUSED(bgv))
+{
+
+	g_assert(apbcHyper[2]);
+	return BearoffDump(apbcHyper[2], anBoard, szOutput);
+
+}
+
+static classdumpfunc acdf[N_CLASSES] = {
+	DumpOver,
+	DumpHypergammon1, DumpHypergammon2, DumpHypergammon3,
+	DumpBearoff2, DumpBearoffTS,
+	DumpBearoff1, DumpBearoffOS,
+	DumpRace, DumpCrashed, DumpContact
+};
+
+extern int
+DumpPosition(const TanBoard anBoard, char *szOutput,
+	     const evalcontext * pec, cubeinfo * pci, int fOutputMWC,
+	     int UNUSED(fOutputWinPC), int fOutputInvert, const char *szMatchID)
+{
+
+	float aarOutput[2][NUM_ROLLOUT_OUTPUTS];
+	positionclass pc = ClassifyPosition(anBoard, pci->bgv);
+	int i, nPlies;
+	int j;
+	evalcontext ec;
+	static const char *aszEvaluator[] = {
+		N_("Over"),
+		N_("Hypergammon-1"),
+		N_("Hypergammon-2"),
+		N_("Hypergammon-3"),
+		N_("Bearoff2"),
+		N_("Bearoff-TS"),
+		N_("Bearoff1"),
+		N_("Bearoff-OS"),
+		N_("Race"),
+		N_("Crashed"),
+		N_("Contact")
+	};
+
+	strcpy(szOutput, "");
+
+	sprintf(strchr(szOutput, 0), "%s:\t", _("Position ID"));
+	strcat(szOutput, PositionID(anBoard));
+	strcat(szOutput, "\n");
+	if (szMatchID) {
+		sprintf(strchr(szOutput, 0), "%s:\t", _("Match ID"));
+		strcat(szOutput, szMatchID);
+		strcat(szOutput, "\n");
+	}
+	strcat(szOutput, "\n");
+
+	sprintf(strchr(szOutput, 0), "%s: \t", _("Evaluator"));
+	strcat(szOutput, gettext(aszEvaluator[pc]));
+	strcat(szOutput, "\n\n");
+	acdf[pc] (anBoard, strchr(szOutput, 0), pci->bgv);
+	szOutput = strchr(szOutput, 0);
+
+	sprintf(strchr(szOutput, 0),
+		"\n"
+		"        %-7s %-7s %-7s %-7s %-7s %-9s %-9s\n",
+		_("Win"), _("W(g)"), _("W(bg)"), _("L(g)"), _("L(bg)"),
+		(!pci->nMatchTo || (pci->nMatchTo && !fOutputMWC)) ?
+		_("Equity") : _("MWC"), _("Cubeful"));
+
+	nPlies = pec->nPlies > 9 ? 9 : pec->nPlies;
+
+	memcpy(&ec, pec, sizeof(evalcontext));
+
+	for (i = 0; i <= nPlies; i++) {
+		szOutput = strchr(szOutput, 0);
+
+
+		ec.nPlies = i;
+
+		if (GeneralCubeDecisionE(aarOutput, anBoard, pci, &ec, 0) <
+		    0)
+			return -1;
+
+		if (!i)
+			strcpy(szOutput, _("static"));
+		else
+			sprintf(szOutput, "%2d %s", i, _("ply"));
+
+		szOutput = strchr(szOutput, 0);
+
+		if (fOutputInvert) {
+			InvertEvaluationR(aarOutput[0], pci);
+			InvertEvaluationR(aarOutput[1], pci);
+			pci->fMove = !pci->fMove;
+		}
+
+		/* Print %'s and equities */
+
+		strcat(szOutput, ": ");
+
+		for (j = 0; j < 5; ++j) {
+			sprintf(strchr(szOutput, 0), "%-7s ",
+				OutputPercent(aarOutput[0][j]));
+		}
+
+		if (pci->nMatchTo)
+			sprintf(strchr(szOutput, 0), "%-9s ",
+				OutputEquity(Utility(aarOutput[0], pci),
+					     pci, TRUE));
+		else
+			sprintf(strchr(szOutput, 0), "%-9s ",
+				OutputMoneyEquity(aarOutput[0], TRUE));
+
+		sprintf(strchr(szOutput, 0), "%-9s ",
+			OutputMWC(aarOutput[0][6], pci, TRUE));
+
+		strcat(szOutput, "\n");
+
+		if (fOutputInvert) {
+			pci->fMove = !pci->fMove;
+		}
+	}
+
+	/* if cube is available, output cube action */
+	if (GetDPEq(NULL, NULL, pci)) {
+
+		evalsetup es;
+
+		es.et = EVAL_EVAL;
+		es.ec = *pec;
+
+		strcat(szOutput, "\n\n");
+		strcat(szOutput,
+		       OutputCubeAnalysis(aarOutput, NULL, &es, pci));
+
+	}
+
+	return 0;
+}
