@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,13 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format.h,v 1.5 2004/01/01 19:31:10 uid65656 Exp $
+ * $Id: format.h,v 1.8 2007/07/02 12:43:38 ace Exp $
  */
 
 #ifndef _FORMAT_H_
 #define _FORMAT_H_
 
-extern int fOutputDigits, fOutputMWC, fOutputWinPC, fOutputMatchPC;
+extern int fOutputMWC, fOutputWinPC, fOutputMatchPC;
+extern unsigned int fOutputDigits;
 extern float rErrorRateFactor;
 
 /* misc. output routines used by text and HTML export */
@@ -58,23 +59,23 @@ OutputEquityScale ( const float r, const cubeinfo *pci,
 extern char *
 OutputRolloutResult ( const char *szIndent,
                       char asz[][ 1024 ],
-                      const float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
-                      const float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
+                      float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
+                      float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
                       const cubeinfo aci[],
                       const int cci,
                       const int fCubeful );
 
 extern char *
-OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                         const float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+OutputCubeAnalysisFull ( float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+                         float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
                          const evalsetup* pes, const cubeinfo* pci,
                          int fDouble, int fTake,
                          skilltype stDouble,
                          skilltype stTake );
 
 extern char *
-OutputCubeAnalysis ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                     const float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+OutputCubeAnalysis ( float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+                     float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
                      const evalsetup* pes, const cubeinfo* pci );
 
 extern char *
