@@ -4,7 +4,7 @@
  * by Joern Thyssen  <jth@gnubg.org>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,24 +16,32 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: formatgs.c,v 1.13 2006/10/04 12:28:39 c_anthon Exp $
+ * $Id: formatgs.c,v 1.23 2008/03/18 01:30:32 c_anthon Exp $
  */
 
 #include "config.h"
-
-#include <stdio.h>
-#include <string.h>
-#include <glib.h>
-#include <math.h>
-
 #include "backgammon.h"
+
+#include <glib.h>
+#include <glib/gi18n.h>
+
 #include "formatgs.h"
 #include "format.h"
 #include "analysis.h"
 
-#include <glib/gi18n.h>
 #include "export.h"
 
+
+static char *total_text(int nMatchTo)
+{
+    return nMatchTo ? _("EMG (MWC)") : _("EMG (Points)");
+}
+
+static char *rate_text(int nMatchTo)
+{
+    return nMatchTo ? _("mEMG (MWC)") : _("mEMG (Points)");
+}
+
 static char **
 numberEntry( const char *sz, const int n0, const int n1 ) {
 
@@ -49,47 +57,33 @@ numberEntry( const char *sz, const int n0, const int n1 ) {
 
 }
 
-static char *
-errorRate( const float rn, const float ru, const int nMatchTo) {
-
-  if ( rn != 0.0f ) {
-
-    if ( nMatchTo ) 
-      return g_strdup_printf( "%+6.3f (%+7.3f%%)",
-                              rn, ru * 100.0f );
-    else
-      return g_strdup_printf( "%+6.3f (%+7.3f)",
-                              rn, ru );
-
-  }
-  else
-    return g_strdup_printf( "%+6.3f", 0.0f );
+static char *errorRate(const float rn, const float ru, const int nMatchTo)
+{
+    int n = fOutputDigits - (int) (log10(rErrorRateFactor) - 0.5);
+    n = MAX(n, 0);
 
+    if (nMatchTo) {
+	return g_strdup_printf("     %+*.*f (%+7.3f%%)", n + 5, n + 2, rn,
+			       ru * 100.0f);
+    } else {
+	return g_strdup_printf("     %+*.*f (%+7.3f)", n + 5, n + 2, rn, ru);
+    }
 }
 
-static char *
-errorRateMP( const float rn, const float ru, const int nMatchTo ) {
+static char *errorRateMP(const float rn, const float ru,
+			 const int nMatchTo)
+{
 
-  int n = fOutputDigits - (int)( log10( rErrorRateFactor ) - 0.5 );
-
-  if ( n < 0 )
-    n = 0;
-
-  if ( rn != 0.0f && ru != 0.0f ) {
-
-    if ( nMatchTo ) 
-      return g_strdup_printf( "%+*.*f (%+7.3f%%)",
-                              n + 5, n, 
-                              rErrorRateFactor * rn, ru * 100.0f );
-    else
-      return g_strdup_printf( "%+*.*f (%+7.3f)",
-                              n + 5, n, 
-                              rErrorRateFactor * rn, ru );
-
-  }
-  else
-    return g_strdup_printf( "%+*.*f", n + 5, n, rErrorRateFactor * rn );
+    int n = fOutputDigits - (int) (log10(rErrorRateFactor) - 0.5);
+    n = MAX(n, 0);
 
+    if (nMatchTo) {
+	return g_strdup_printf("   %+*.*f   (%+7.3f%%)", n + 5, n,
+			       rErrorRateFactor * rn, ru * 100.0f);
+    } else {
+	return g_strdup_printf("   %+*.*f   (%+7.3f)", n + 5, n,
+			       rErrorRateFactor * rn, ru);
+    }
 }
 
 static char *
@@ -147,11 +141,10 @@ formatGS( const statcontext *psc, const int nMatchTo,
   case FORMATGS_CHEQUER: 
     {
 
-      static int ai[ 5 ] = { SKILL_NONE, SKILL_GOOD, SKILL_DOUBTFUL,
+      static int ai[ 4 ] = { SKILL_NONE, SKILL_DOUBTFUL,
                              SKILL_BAD, SKILL_VERYBAD };
-      static char *asz[] = {
+      static char *asz[4] = {
         N_("Unmarked moves"),
-        N_("Moves marked good"),
         N_("Moves marked doubtful"),
         N_("Moves marked bad"),
         N_("Moves marked very bad") };
@@ -166,7 +159,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
       list = g_list_append( list, numberEntry( _("Unforced moves"),
                                                psc->anUnforcedMoves[ 0 ],
                                                psc->anUnforcedMoves[ 1 ] ) );
-      for ( i = 0; i < 5; ++i )
+      for ( i = 0; i < 4; ++i )
         list = g_list_append( list, 
                               numberEntry( gettext( asz[ i ] ),
                                            psc->anMoves[ 0 ][ ai[ i ] ],
@@ -176,7 +169,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-      aasz[ 0 ] = g_strdup( _("Error rate (total)") );
+      aasz[ 0 ] = g_strdup_printf( _("Error total %s"), total_text(nMatchTo) );
 
       for ( i = 0; i < 2; ++i )
         aasz[ i + 1 ] = errorRate( 
@@ -190,7 +183,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-      aasz[ 0 ] = g_strdup( _("Error rate (per move)") );
+      aasz[ 0 ] = g_strdup_printf( _("Error rate %s"), rate_text(nMatchTo) );
 
       for ( i = 0; i < 2; ++i )
         aasz[ i + 1 ] = errorRateMP( 
@@ -227,39 +220,46 @@ formatGS( const statcontext *psc, const int nMatchTo,
         N_("Doubles"),
         N_("Takes"),
         N_("Passes") };
-      const int *ai[ 5 ] = { psc->anTotalCube, psc->anCloseCube,
-                             psc->anDouble, psc->anTake, psc->anPass };
 
       static char *asz2[] = {
-        N_("Missed doubles (below CP)"),
-        N_("Missed doubles (above CP)"),
-        N_("Wrong doubles (below DP)"),
-        N_("Wrong doubles (above TG)"),
+        N_("Missed doubles below CP"),
+        N_("Missed doubles above CP"),
+        N_("Wrong doubles below DP"),
+        N_("Wrong doubles above TG"),
         N_("Wrong takes"),
         N_("Wrong passes") };
 
-      const int *ai2[ 6 ] = { psc->anCubeMissedDoubleDP,
-                              psc->anCubeMissedDoubleTG,
-                              psc->anCubeWrongDoubleDP,
-                              psc->anCubeWrongDoubleTG,
-                              psc->anCubeWrongTake,
-                              psc->anCubeWrongPass };
-      const float *af2[ 2 ][ 6 ] = {
-        { psc->arErrorMissedDoubleDP[ 0 ],
-          psc->arErrorMissedDoubleTG[ 0 ],
-          psc->arErrorWrongDoubleDP[ 0 ],
-          psc->arErrorWrongDoubleTG[ 0 ],
-          psc->arErrorWrongTake[ 0 ],
-          psc->arErrorWrongPass[ 0 ] },
-        { psc->arErrorMissedDoubleDP[ 1 ],
-          psc->arErrorMissedDoubleTG[ 1 ],
-          psc->arErrorWrongDoubleDP[ 1 ],
-          psc->arErrorWrongDoubleTG[ 1 ],
-          psc->arErrorWrongTake[ 1 ],
-          psc->arErrorWrongPass[ 1 ] } };
-
       int i, j;
 
+      const int *ai[ 5 ], *ai2[ 6 ];
+      const float *af2[ 2 ][ 6 ];
+
+		ai[ 0 ] = psc->anTotalCube;
+		ai[ 1 ] = psc->anCloseCube;
+		ai[ 2 ] = psc->anDouble;
+		ai[ 3 ] = psc->anTake;
+		ai[ 4 ] = psc->anPass;
+		
+		ai2[ 0 ] = psc->anCubeMissedDoubleDP;
+		ai2[ 1 ] = psc->anCubeMissedDoubleTG;
+		ai2[ 2 ] = psc->anCubeWrongDoubleDP;
+		ai2[ 3 ] = psc->anCubeWrongDoubleTG;
+		ai2[ 4 ] = psc->anCubeWrongTake;
+		ai2[ 5 ] = psc->anCubeWrongPass;
+
+		af2[0][0] = psc->arErrorMissedDoubleDP[ 0 ];
+		af2[0][1] = psc->arErrorMissedDoubleTG[ 0 ];
+		af2[0][2] = psc->arErrorWrongDoubleDP[ 0 ];
+		af2[0][3] = psc->arErrorWrongDoubleTG[ 0 ];
+		af2[0][4] = psc->arErrorWrongTake[ 0 ];
+		af2[0][5] = psc->arErrorWrongPass[ 0 ];
+		af2[1][0] = psc->arErrorMissedDoubleDP[ 1 ];
+		af2[1][1] = psc->arErrorMissedDoubleTG[ 1 ];
+		af2[1][2] = psc->arErrorWrongDoubleDP[ 1 ];
+		af2[1][3] = psc->arErrorWrongDoubleTG[ 1 ];
+		af2[1][4] = psc->arErrorWrongTake[ 1 ];
+		af2[1][5] = psc->arErrorWrongPass[ 1 ];
+
       for ( i = 0; i < 5; ++i )
         list = g_list_append( list, 
                               numberEntry( gettext( asz[ i ] ),
@@ -268,7 +268,8 @@ formatGS( const statcontext *psc, const int nMatchTo,
       for ( i = 0; i < 6; ++i ) {
         aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-        aasz[ 0 ] = g_strdup( gettext( asz2[ i ] ) );
+        aasz[ 0 ] = g_strdup_printf( "%s (%s)", gettext( asz2[ i ] ),
+			total_text(nMatchTo) );
 
         for ( j = 0; j < 2; ++j ) 
           aasz[ j + 1 ] = cubeEntry( ai2[ i ][ j ],
@@ -284,7 +285,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-      aasz[ 0 ] = g_strdup( _("Error rate (total)") );
+      aasz[ 0 ] = g_strdup_printf( _("Error total %s"), total_text(nMatchTo) );
 
       for ( i = 0; i < 2; ++i )
         aasz[ i + 1 ] = errorRate( 
@@ -298,7 +299,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-      aasz[ 0 ] = g_strdup( _("Error rate (per cube decision)") );
+      aasz[ 0 ] = g_strdup_printf( _("Error rate %s"), rate_text(nMatchTo) );
 
       for ( i = 0; i < 2; ++i )
         aasz[ i + 1 ] = errorRateMP( 
@@ -350,7 +351,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-      aasz[ 0 ] = g_strdup( _("Luck rate (total)") );
+      aasz[ 0 ] = g_strdup_printf( _("Luck total %s"), total_text(nMatchTo) );
 
       for ( i = 0; i < 2; ++i )
         aasz[ i + 1 ] = errorRate( psc->arLuck[ i ][ 0 ],
@@ -362,7 +363,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-      aasz[ 0 ] = g_strdup( _("Luck rate (per move)") );
+      aasz[ 0 ] = g_strdup_printf( _("Luck rate %s"), rate_text(nMatchTo) );
 
       for ( i = 0; i < 2; ++i )
         if ( psc->anTotalMoves[ i ] ) 
@@ -405,7 +406,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       if ( psc->fCube || psc->fMoves ) {
 
-        aasz[ 0 ] = g_strdup( _("Error rate (total)") );
+        aasz[ 0 ] = g_strdup_printf( _("Error total %s"), total_text(nMatchTo) );
 
         for ( i = 0; i < 2; ++i )
           aasz[ i + 1 ] = errorRate( 
@@ -419,7 +420,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
         aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-        aasz[ 0 ] = g_strdup( _("Error rate (per decision)") );
+        aasz[ 0 ] = g_strdup_printf( _("Error rate %s"), rate_text(nMatchTo) );
 
         for ( i = 0; i < 2; ++i )
           aasz[ i + 1 ] = errorRateMP( 
@@ -433,7 +434,7 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
         aasz = g_malloc( 3 * sizeof ( *aasz ) );
 
-        aasz[ 0 ] = g_strdup( _("Equiv. Snowie error rate") );
+        aasz[ 0 ] = g_strdup( _("Snowie error rate") );
 
         for ( i = 0; i < 2; ++i )
           if ( ( n = psc->anTotalMoves[ 0 ] + psc->anTotalMoves[ 1 ] ) )
@@ -570,10 +571,12 @@ formatGS( const statcontext *psc, const int nMatchTo,
             /* xgettext: no-c-format */
             N_("95% confidence interval (ppg)") }
 	};
-        const float *af[ 2 ][ 2 ] = { 
-          { psc->arActualResult, psc->arVarianceActual },
-          { psc->arLuckAdj, psc->arVarianceLuckAdj } };
         int i, j;
+        const float *af[ 2 ][ 2 ];
+			af[0][0] = psc->arActualResult;
+			af[0][1] = psc->arVarianceActual;
+			af[1][0] = psc->arLuckAdj;
+			af[1][1] = psc->arVarianceLuckAdj;
 
         for ( i = 0; i < 2; ++i ) {
 
@@ -608,45 +611,6 @@ formatGS( const statcontext *psc, const int nMatchTo,
 
       }
 
-      /* time penalties */
-
-#if USE_TIMECONTROL
-
-      if ( psc->anTimePenalties[ 0 ] || psc->anTimePenalties[ 1 ] ) {
-        
-        int j;
-
-        /* show time penalty statistics */
-
-        /* number of penalties */
-
-        aasz = g_malloc( 3 * sizeof ( *aasz ) );
-        aasz[ 0 ] = g_strdup( _("Number of time penalties" ) );
-      
-        for ( j = 0; j < 2; ++j ) 
-          aasz[ j + 1 ] = 
-            g_strdup_printf( "%d", psc->anTimePenalties[ j ] );
-        
-        list = g_list_append( list, aasz );
-
-        /* loss */
-
-        aasz = g_malloc( 3 * sizeof ( *aasz ) );
-
-        aasz[ 0 ] = g_strdup( _("Loss from time pen. (total)") );
-
-        for ( i = 0; i < 2; ++i )
-          aasz[ i + 1 ] = errorRate( -psc->aarTimeLoss[ i ][ 0 ],
-                                     -psc->aarTimeLoss[ i ][ 1 ], nMatchTo );
-        
-        list = g_list_append( list, aasz );
-
-
-      }
-
-#endif /* TIME_CONTROL */      
-
-
     }
     
     break;
@@ -683,5 +647,6 @@ freeGS( GList *list ) {
   g_list_foreach( list, _freeGS, NULL );
 
   g_list_free( list );
-
 }
+
+
