@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2003.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,40 +16,28 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gnubgmodule.c,v 1.53.2.2 2006/11/10 11:34:40 c_anthon Exp $
+ * $Id: gnubgmodule.c,v 1.99 2009/09/21 07:43:43 c_anthon Exp $
  */
 
-#if HAVE_CONFIG_H
 #include "config.h"
-#endif
-#if USE_PYTHON
+
 #include "gnubgmodule.h"
-
-#include <signal.h>
-#include <assert.h>
-#include <string.h>
-#include <math.h>
-
-#if HAVE_LIBREADLINE
-#include <readline/history.h>
-#include <readline/readline.h>
-#endif
-
-#include <stdio.h>
-#include <glib.h>
 #include "backgammon.h"
+#include <signal.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include "eval.h"
 #include "matchequity.h"
-#include "path.h"
 #include "positionid.h"
-#include "analysis.h"
-#include "md5.h"
-#include <glib/gi18n.h>
+#include "util.h"
+
+#if USE_PYTHON
 
 #define UNUSED_PARAM __attribute__ ((unused))
 
 static PyObject *
-BoardToPy( int anBoard[ 2 ][ 25 ] )
+BoardToPy( const TanBoard anBoard )
 {
   PyObject* b = PyTuple_New(2);
   PyObject* b0 = PyTuple_New(25);
@@ -68,7 +56,7 @@ BoardToPy( int anBoard[ 2 ][ 25 ] )
 }
 
 static PyObject *
-Board1ToPy( int anBoard [ 25 ] ) {
+Board1ToPy( unsigned int anBoard [ 25 ] ) {
   unsigned int k;
   PyObject* b = PyTuple_New(25);
   
@@ -81,7 +69,7 @@ Board1ToPy( int anBoard [ 25 ] ) {
 
 
 static int
-PyToBoard1( PyObject* p, int anBoard[ 25 ] )
+PyToBoard1( PyObject* p, unsigned int anBoard[ 25 ] )
 {
   if( PySequence_Check(p) && PySequence_Size(p) == 25 ) {
     int j;
@@ -98,7 +86,7 @@ PyToBoard1( PyObject* p, int anBoard[ 25 ] )
 }
 
 static int
-PyToBoard(PyObject* p, int anBoard[ 2 ][ 25 ])
+PyToBoard(PyObject* p, TanBoard anBoard)
 {
   if( PySequence_Check(p) && PySequence_Size(p) == 2 ) {
     int i;
@@ -158,20 +146,27 @@ static int
 PyToCubeInfo( PyObject *p, cubeinfo *pci ) {
 
   PyObject *pyKey, *pyValue;
-  int iPos = 0;
+  Py_ssize_t iPos = 0;
   char *pchKey;
-  static char *aszKeys[] = {
+  static const char *aszKeys[] = {
     "jacoby", "crawford", "move", "beavers", "cube", "matchto",
     "bgv", "cubeowner", "score", "gammonprice", NULL };
   int iKey;
-  int i;
-  void *ap[] = { &pci->fJacoby, &pci->fCrawford, &pci->fMove,
-                 &pci->fBeavers, &pci->nCube, &pci->nMatchTo,
-                 &pci->bgv, &pci->fCubeOwner, pci->anScore,
-                 pci->arGammonPrice };
+  void *ap[10];
   int *pi;
   float *pf;
-  
+  int i = 0;
+  ap[i++] = &pci->fJacoby;
+  ap[i++] = &pci->fCrawford;
+  ap[i++] = &pci->fMove;
+  ap[i++] = &pci->fBeavers;
+  ap[i++] = &pci->nCube;
+  ap[i++] = &pci->nMatchTo;
+  ap[i++] = &pci->bgv;
+  ap[i++] = &pci->fCubeOwner;
+  ap[i++] = pci->anScore;
+  ap[i++] = pci->arGammonPrice;
+
   while( PyDict_Next( p, &iPos, &pyKey, &pyValue ) ) {
 
     if ( ! ( pchKey = PyString_AsString( pyKey ) ) )
@@ -230,7 +225,7 @@ PyToCubeInfo( PyObject *p, cubeinfo *pci ) {
       
     default:
 
-      assert( FALSE );
+      g_assert( FALSE );
 
     }
 
@@ -244,16 +239,9 @@ PyToCubeInfo( PyObject *p, cubeinfo *pci ) {
 static PyObject*
 EvalContextToPy( const evalcontext* pec)
 {
-  return Py_BuildValue( "{s:i,s:i"
-#if defined( REDUCTION_CODE )
-			",s:i"
-#endif
-			",s:i,s:f}",
+  return Py_BuildValue( "{s:i,s:i,s:i,s:f}",
                         "cubeful", pec->fCubeful,
                         "plies", pec->nPlies,
-#if defined( REDUCTION_CODE )
-                        "reduced", pec->nReduced,
-#endif
                         "deterministic", pec->fDeterministic,
                         "noise", pec->rNoise );
 }
@@ -263,9 +251,9 @@ static int
 PyToEvalContext( PyObject *p, evalcontext *pec ) {
 
   PyObject *pyKey, *pyValue;
-  int iPos = 0;
+  Py_ssize_t iPos = 0;
   char *pchKey;
-  static char *aszKeys[] = {
+  static const char *aszKeys[] = {
     "cubeful", "plies", "reduced", "deterministic", "noise", NULL };
   int iKey;
   int i;
@@ -310,9 +298,6 @@ PyToEvalContext( PyObject *p, evalcontext *pec ) {
       else if ( iKey == 1 )
         pec->nPlies = ( i < 8 ) ? i : 7;
       else if ( iKey == 2 ) {
-#if defined( REDUCTION_CODE )
-        pec->nReduced = ( i < 8 ) ? i : 7;
-#endif
       } else 
         pec->fDeterministic = i ? 1 : 0;
 
@@ -334,7 +319,7 @@ PyToEvalContext( PyObject *p, evalcontext *pec ) {
 
     default:
 
-      assert( FALSE );
+      g_assert( FALSE );
 
     }
 
@@ -407,44 +392,45 @@ PythonEvalContext( PyObject* self UNUSED_PARAM, PyObject *args ) {
 
   ec.fCubeful = fCubeful ? 1 : 0;
   ec.nPlies = ( nPlies < 8 ) ? nPlies : 7;
-#if defined( REDUCTION_CODE )
-  ec.nReduced = ( nReduced < 8 ) ? nReduced : 7;
-#endif
   ec.fDeterministic = fDeterministic ? 1 : 0;
   ec.rNoise = rNoise;
                            
   return EvalContextToPy( &ec );
 }
 
-static PyObject *
-PythonCommand( PyObject* self UNUSED_PARAM, PyObject *args ) {
+static PyObject *PythonCommand(PyObject * self UNUSED_PARAM, PyObject * args)
+{
 
-  char *pch;
-  char *sz;
-  psighandler sh;
+	char *pch;
+	char *sz;
+	psighandler sh;
 
-  if ( ! PyArg_ParseTuple( args, "s:command", &pch ) )
-    return NULL;
+	if (!PyArg_ParseTuple(args, "s:command", &pch))
+		return NULL;
 
-  sz = strdup( pch );
+	sz = g_strdup(pch);
 
-  PortableSignal( SIGINT, HandleInterrupt, &sh, FALSE );
-  HandleCommand( sz, acTop );
+	PortableSignal(SIGINT, HandleInterrupt, &sh, FALSE);
+	HandleCommand(sz, acTop);
 
-   if( ms.gs != GAME_NONE ) {  /* HACK, no idea if this is right or not */
-  PythonNextTurn(0, 0); 
-   }
+	/* this is what the cl interface does. Let's try that. */
+	while (fNextTurn)
+		NextTurn(TRUE);
+	/* before we had 
+	 * if( ms.gs != GAME_NONE )
+	 * PythonNextTurn(0, 0);
+	 * which always run NextTurn(TRUE), and that isn't right */
 
-  outputx();
-  free( sz );
-  PortableSignalRestore( SIGINT, &sh );
-  if( fInterrupt ) {
-    raise( SIGINT );
-    fInterrupt = FALSE;
-  }
-  
-  Py_INCREF(Py_None);
-  return Py_None;
+	outputx();
+	free(sz);
+	PortableSignalRestore(SIGINT, &sh);
+	if (fInterrupt) {
+		raise(SIGINT);
+		fInterrupt = FALSE;
+	}
+
+	Py_INCREF(Py_None);
+	return Py_None;
 }
 
 
@@ -460,7 +446,7 @@ PythonBoard( PyObject* self UNUSED_PARAM, PyObject *args ) {
     return Py_None;
   }
 
-  return BoardToPy( ms.anBoard );
+  return BoardToPy( msBoard() );
 }
 
 static PyObject *
@@ -492,12 +478,12 @@ PythonEvaluate( PyObject* self UNUSED_PARAM, PyObject *args ) {
   PyObject *pyCubeInfo = NULL;
   PyObject *pyEvalContext = NULL;
 
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   cubeinfo ci;
   evalcontext ec = { 0, 0, 0, 1, 0.0f };
   float arOutput[ 7 ];
 
-  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  memcpy( anBoard, msBoard(), sizeof(TanBoard) );
   GetMatchStateCubeInfo( &ci, &ms );
 
   if ( ! PyArg_ParseTuple( args, "|OOO", 
@@ -513,9 +499,9 @@ PythonEvaluate( PyObject* self UNUSED_PARAM, PyObject *args ) {
   if ( pyEvalContext && PyToEvalContext( pyEvalContext, &ec ) )
     return NULL;
 
-  if ( GeneralEvaluationE( arOutput, anBoard, &ci, &ec ) ) {
+  if ( GeneralEvaluationE( arOutput, (ConstTanBoard)anBoard, &ci, &ec ) ) {
     PyErr_SetString( PyExc_StandardError, 
-                     _("interupted/errro in GeneralEvaluateE") );
+                     _("interupted/errno in GeneralEvaluateE") );
     return NULL;
   }
 
@@ -536,13 +522,13 @@ PythonEvaluateCubeful( PyObject* self UNUSED_PARAM, PyObject *args ) {
   PyObject *pyCubeInfo = NULL;
   PyObject *pyEvalContext = NULL;
 
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], arCube[ NUM_CUBEFUL_OUTPUTS ];
   cubeinfo ci;
   evalcontext ec = { 0, 0, 0, 1, 0.0f };
   int cp;
 
-  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  memcpy( anBoard, msBoard(), sizeof(TanBoard) );
   
   GetMatchStateCubeInfo( &ci, &ms );
 
@@ -559,13 +545,13 @@ PythonEvaluateCubeful( PyObject* self UNUSED_PARAM, PyObject *args ) {
   if ( pyEvalContext && PyToEvalContext( pyEvalContext, &ec ) )
     return NULL;
 
-  if ( GeneralCubeDecisionE ( aarOutput, anBoard, &ci, &ec, 0 ) < 0 ){
+  if ( GeneralCubeDecisionE ( aarOutput, (ConstTanBoard)anBoard, &ci, &ec, 0 ) < 0 ){
     PyErr_SetString( PyExc_StandardError, 
                      _("interupted/errno in GeneralCubeDecisionE") );
     return NULL;
   }
 
-  cp = FindCubeDecision ( arCube, GCCCONSTAHACK aarOutput, &ci );
+  cp = FindCubeDecision ( arCube,  aarOutput, &ci );
 
   {
     PyObject* p = PyTuple_New( NUM_CUBEFUL_OUTPUTS + 2 );
@@ -588,13 +574,13 @@ PythonFindBestMove( PyObject* self UNUSED_PARAM, PyObject *args ) {
   PyObject *pyCubeInfo = NULL;
   PyObject *pyEvalContext = NULL;
 
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   int anDice[ 2 ];
   int anMove[ 8 ];
   cubeinfo ci;
   evalcontext ec = { 0, 0, 0, 1, 0.0f };
 
-  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  memcpy( anBoard, msBoard(), sizeof(TanBoard) );
   memcpy( anDice, ms.anDice, sizeof anDice );
   GetMatchStateCubeInfo( &ci, &ms );
   if ( ! PyArg_ParseTuple( args, "|OOOO", 
@@ -604,7 +590,8 @@ PythonFindBestMove( PyObject* self UNUSED_PARAM, PyObject *args ) {
   if ( pyDice && !PyToDice( pyDice, anDice ) )
     return NULL;
 
-  if (!anDice){
+  if (anDice[0] == 0)
+  {
 	  printf("What? No dice?\n");
 	  return NULL;
   }
@@ -747,82 +734,9 @@ PythonEq2mwc( PyObject* self UNUSED_PARAM, PyObject *args ) {
 
 }
 
-#define STRBUF_SIZE 1024
-
-void AddString(list* buffers, char* str)
+static PyObject *PythonMatchChecksum( PyObject* self UNUSED_PARAM, PyObject *args )
 {
-	list* pCurrent = buffers->plPrev;
-	if (!pCurrent->p || strlen(str) + strlen(pCurrent->p) > STRBUF_SIZE)
-	{
-		char* newBuf = malloc(STRBUF_SIZE + 1);
-		*newBuf = '\0';
-		pCurrent = ListInsert(buffers, newBuf);
-	}
-	strcat(pCurrent->p, str);
-}
-
-extern char* GameAsString()
-{
-	char *ret;
-	int size;
-	char buf[1024];
-	list *pList, *plGame, *plMove;
-	list buffers;
-	ListCreate(&buffers);
-
-	sprintf(buf, "%s vs %s (%d)", ap[0].szName, ap[1].szName, ms.nMatchTo);
-	AddString(&buffers, buf);
-
-	for (plGame = lMatch.plNext; plGame->p; plGame = plGame->plNext)
-	{
-		list* plStart = plGame->p;
-		int move = 1;
-		for (plMove = plStart->plNext; plMove->p; plMove = plMove->plNext)
-		{
-			char playerStr[4] = ".AB";
-			int player;
-			moverecord* pmr = plMove->p;
-			char* moveString = GetMoveString(pmr, &player);
-			if (moveString)
-			{
-				sprintf(buf, " %d%c %s", move, playerStr[player + 1], moveString);
-				AddString(&buffers, buf);
-				if (player == 1)
-					move++;
-			}
-		}
-	}
-
-	/* Create single string from buffers */
-	size = 0;
-	for (pList = buffers.plNext; pList->p; pList = pList->plNext)
-		size += strlen(pList->p);
-
-	ret = malloc(size + 1);
-	*ret = '\0';
-	for (pList = buffers.plNext; pList->p; pList = pList->plNext)
-		strcat(ret, pList->p);
-
-	ListDeleteAll(&buffers);
-
-	return ret;
-}
-
-extern PyObject *
-PythonMatchChecksum( PyObject* self UNUSED_PARAM, PyObject *args )
-{
-	unsigned char auch[16];
-	char auchHex[33];
-	int i;
-	// Work out md5 checksum
-	char* gameStr = GameAsString();
-	md5_buffer(gameStr, strlen(gameStr), auch);
-	free(gameStr);
-	/* Convert to hex so stores easily in database - is there a better way? */
-	for (i = 0; i < 16; i++)
-		sprintf(auchHex + (i * 2), "%02x", auch[i]);
-
-	return PyString_FromString(auchHex);
+	return PyString_FromString(GetMatchCheckSum());
 }
 
 static PyObject *
@@ -843,14 +757,13 @@ PythonMwc2eq( PyObject* self UNUSED_PARAM, PyObject *args ) {
   return PyFloat_FromDouble( mwc2eq( r, &ci ) );
 }
 
-
 static PyObject *
 PythonPositionID( PyObject* self UNUSED_PARAM, PyObject *args ) {
 
   PyObject *pyBoard = NULL;
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
 
-  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  memcpy( anBoard, msBoard(), sizeof(TanBoard) );
 
   if ( ! PyArg_ParseTuple( args, "|O:positionid", &pyBoard ) )
     return NULL;
@@ -858,7 +771,7 @@ PythonPositionID( PyObject* self UNUSED_PARAM, PyObject *args ) {
   if ( pyBoard && !PyToBoard( pyBoard, anBoard ) )
     return NULL;
 
-  return PyString_FromString( PositionID( anBoard ) );
+  return PyString_FromString( PositionID( (ConstTanBoard)anBoard ) );
 
 }
 
@@ -866,7 +779,7 @@ static PyObject *
 PythonPositionFromID( PyObject* self UNUSED_PARAM, PyObject *args )
 {
   char* sz = NULL;
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
 
   if( ! PyArg_ParseTuple( args, "|s:positionfromid", &sz ) ) {
     return NULL;
@@ -879,10 +792,10 @@ PythonPositionFromID( PyObject* self UNUSED_PARAM, PyObject *args )
       return NULL;
     }
   } else {
-    memcpy( anBoard, ms.anBoard, sizeof anBoard );
+    memcpy( anBoard, msBoard(), sizeof(TanBoard) );
   }
 
-  return BoardToPy( anBoard );
+  return BoardToPy( (ConstTanBoard)anBoard );
 }
 
 
@@ -890,10 +803,10 @@ static PyObject *
 PythonPositionKey( PyObject* self UNUSED_PARAM, PyObject *args ) {
 
   PyObject *pyBoard = NULL;
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   unsigned char auch[ 10 ];
 
-  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  memcpy( anBoard, msBoard(), sizeof(TanBoard) );
 
   if ( ! PyArg_ParseTuple( args, "|O!:positionkey", &PyList_Type, &pyBoard ) )
     return NULL;
@@ -901,7 +814,7 @@ PythonPositionKey( PyObject* self UNUSED_PARAM, PyObject *args ) {
   if ( pyBoard && !PyToBoard( pyBoard, anBoard ) )
     return NULL;
 
-  PositionKey( anBoard, auch );
+  PositionKey( (ConstTanBoard)anBoard, auch );
 
   {
     PyObject* a = PyTuple_New(10);
@@ -916,7 +829,7 @@ PythonPositionKey( PyObject* self UNUSED_PARAM, PyObject *args ) {
 static PyObject *
 PythonPositionFromKey( PyObject* self UNUSED_PARAM, PyObject *args ) {
 
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   int i;
   PyObject *pyKey = NULL;
   PyObject *py;
@@ -944,7 +857,7 @@ PythonPositionFromKey( PyObject* self UNUSED_PARAM, PyObject *args ) {
 
   PositionFromKey( anBoard, auch );
 
-  return BoardToPy( anBoard );
+  return BoardToPy( (ConstTanBoard)anBoard );
 }
 
 
@@ -954,9 +867,9 @@ PythonPositionBearoff( PyObject* self UNUSED_PARAM, PyObject *args )
   PyObject *pyBoard = NULL;
   int nChequers = 15;
   int nPoints = 6;
-  int anBoard[ 25 ];
+  unsigned int anBoard[ 25 ];
 
-  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  memcpy( anBoard, msBoard(), sizeof(anBoard) );
 
   if ( ! PyArg_ParseTuple( args, "|Oii:positionbearoff", 
                            &pyBoard, &nPoints, &nChequers ) )
@@ -968,10 +881,9 @@ PythonPositionBearoff( PyObject* self UNUSED_PARAM, PyObject *args )
   return PyInt_FromLong( PositionBearoff( anBoard, nPoints, nChequers ) );
 }
 
-static PyObject *
-PythonPositionFromBearoff( PyObject* self UNUSED_PARAM, PyObject *args ) {
-
-  int anBoard[ 25 ];
+static PyObject *PythonPositionFromBearoff( PyObject* self UNUSED_PARAM, PyObject *args )
+{
+  unsigned int anBoard[ 25 ];
   int iPos = 0;
   int nChequers = 15;
   int nPoints = 6;
@@ -995,7 +907,7 @@ PythonPositionFromBearoff( PyObject* self UNUSED_PARAM, PyObject *args ) {
     return NULL;
   }
 
-  memset( anBoard, 0, sizeof anBoard );
+  memset( anBoard, 0, sizeof(anBoard) );
   PositionFromBearoff( anBoard, iPos, nPoints, nChequers );
 
   return Board1ToPy( anBoard );
@@ -1012,12 +924,13 @@ PythonPositionFromBearoff( PyObject* self UNUSED_PARAM, PyObject *args ) {
 #define CHARP_HACK
 #endif
 
-static inline void
-DictSetItemSteal(PyObject* dict, const char* key, PyObject* val)
+static void DictSetItemSteal(PyObject * dict, const char *key, PyObject * val)
 {
-  int const s = PyDict_SetItemString(dict, CHARP_HACK key, val);  
-  {                                                         assert( s == 0 ); }
-  Py_DECREF(val);
+	int const s = PyDict_SetItemString(dict, CHARP_HACK key, val);
+	{
+		g_assert(s == 0);
+	}
+	Py_DECREF(val);
 }
 
 typedef struct {
@@ -1050,12 +963,6 @@ diffContext(const evalcontext* c, PyMatchState* ms)
       DictSetItemSteal(context, "plies", PyInt_FromLong(c->nPlies));
     }
 
-#if defined( REDUCTION_CODE )
-    if( c->nReduced != s->nReduced ) {
-      DictSetItemSteal(context, "reduced", PyInt_FromLong(c->nReduced));
-    }
-#endif
-
     if( c->fDeterministic != s->fDeterministic ) {
       DictSetItemSteal(context, "deterministic",
 		       PyInt_FromLong(c->fDeterministic));
@@ -1195,7 +1102,7 @@ PyMove(const int move[8])
   }
 
   if( i < 4 ) {
-    int s = _PyTuple_Resize(&moveTuple, i);                  assert(s != -1);
+    int s = _PyTuple_Resize(&moveTuple, i);                  g_assert(s != -1);
   }
   
   return moveTuple;
@@ -1211,7 +1118,7 @@ luckString(lucktype const lt, int const ignoreNone)
     case LUCK_GOOD: return "good";
     case LUCK_VERYGOOD: return "verygood";
   }
-  assert(0);
+  g_assert(0);
   return 0;
 }
     
@@ -1223,9 +1130,8 @@ skillString(skilltype const st, int const ignoreNone)
     case SKILL_BAD:         return "bad";
     case SKILL_DOUBTFUL:    return "doubtful";
     case SKILL_NONE:        return ignoreNone ? 0 : "unmarked";
-    case SKILL_GOOD:        return "good";
   }
-  assert(0);
+  g_assert(0);
   return 0;
 }
 
@@ -1261,8 +1167,7 @@ addLuck(PyObject* dict, float const rLuck, lucktype const lt)
 static PyObject*
 PyMoveAnalysis(const movelist* pml, PyMatchState* ms)
 {
-  int i;
-
+  unsigned int i;
   unsigned int n = 0;
   
   for(i = 0; i < pml->cMoves; i++) {
@@ -1347,7 +1252,7 @@ PyMoveAnalysis(const movelist* pml, PyMatchState* ms)
 
         case EVAL_NONE: break;
 
-        default: assert( 0 );
+        default: g_assert( 0 );
       }
 
       if( v ) {
@@ -1361,8 +1266,8 @@ PyMoveAnalysis(const movelist* pml, PyMatchState* ms)
 
 static PyObject*
 PyDoubleAnalysis(const evalsetup* pes,
-		 const float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
-		 const float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
+		 float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
+		 float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
 		 PyMatchState* ms,
 		 int const verbose)
 {
@@ -1437,7 +1342,7 @@ PyDoubleAnalysis(const evalsetup* pes,
       break;
     }
     default:
-      assert( 0 );
+      g_assert( 0 );
   }
 
   return dict;
@@ -1585,21 +1490,6 @@ PyGameStats(const statcontext* sc, const int fIsMatch, const int nMatchTo)
     }
   }
 
-#if USE_TIMECONTROL
-  {
-     int side;
-     for ( side = 0; side < 2; ++side ) {
-        PyObject *d =
-            Py_BuildValue( "{s:i,s:f,s:f}",
-                           "time-penalty", sc->anTimePenalties[ side ],
-                           "time-penalty-skill", sc->aarTimeLoss[ side ][ 0 ],
-                           "time-penalty-cost", sc->aarTimeLoss[ side ][ 1 ] );
-
-         DictSetItemSteal( p[ side ], "time", d );
-      }
-  }
-#endif /* USE_TIMECONTROL */
-
   /* advantage */
 
   if ( sc->fDice && fIsMatch && !nMatchTo && sc->nGames > 1 ) {
@@ -1674,21 +1564,21 @@ PyGameStats(const statcontext* sc, const int fIsMatch, const int nMatchTo)
  */
 
 static PyObject*
-PythonGame(const list*    plGame,
+PythonGame(const listOLD*    plGame,
 	   int const      doAnalysis,
 	   int const      verbose,
 	   statcontext*   scMatch,
 	   int const      includeBoards,
 	   PyMatchState*  ms)
 {
-  const list* pl = plGame->plNext;
+  const listOLD* pl = plGame->plNext;
   const moverecord* pmr = pl->p;            
   const xmovegameinfo* g = &pmr->g;
 
   PyObject* gameDict = PyDict_New();
   PyObject* gameInfoDict = PyDict_New();
 
-  {                                       assert( pmr->mt == MOVE_GAMEINFO ); }
+  g_assert( pmr->mt == MOVE_GAMEINFO );
 
   if( ! (gameDict && gameInfoDict) ) {
     PyErr_SetString(PyExc_MemoryError, "");
@@ -1743,11 +1633,11 @@ PythonGame(const list*    plGame,
   
   {
     int nRecords = 0;
-    int anBoard[2][25];
+    TanBoard anBoard;
     PyObject* gameTuple;
 
     {
-      list* t;
+      listOLD* t;
       for( t = pl->plNext; t != plGame; t = t->plNext ) {
 	++nRecords;
       }
@@ -1787,7 +1677,7 @@ PythonGame(const list*    plGame,
 
 	  if( includeBoards ) {
 	    DictSetItemSteal(recordDict, "board",
-			     PyString_FromString(PositionID(anBoard)));
+			     PyString_FromString(PositionID((ConstTanBoard)anBoard)));
 	    
 	    ApplyMove(anBoard, pmr->n.anMove, 0);
 	    SwapSides(anBoard);
@@ -1797,11 +1687,11 @@ PythonGame(const list*    plGame,
 	    if( pmr->CubeDecPtr->esDouble.et != EVAL_NONE ) {
 	      PyObject* d =
 		PyDoubleAnalysis(&pmr->CubeDecPtr->esDouble,   
-                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput, 
-                                 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+                                  (float (*)[ NUM_ROLLOUT_OUTPUTS ])pmr->CubeDecPtr->aarOutput, 
+                                  (float (*)[ NUM_ROLLOUT_OUTPUTS ])pmr->CubeDecPtr->aarStdDev,
 				 ms, verbose);
 	      {
-		int s = PyDict_Merge(analysis, d, 1);     assert( s != -1 );
+		int s = PyDict_Merge(analysis, d, 1);     g_assert( s != -1 );
 	      }
 	      Py_DECREF(d);
 	    }
@@ -1831,16 +1721,16 @@ PythonGame(const list*    plGame,
 
 	  if( includeBoards ) {
 	    DictSetItemSteal(recordDict, "board",
-			     PyString_FromString(PositionID(anBoard)));
+			     PyString_FromString(PositionID((ConstTanBoard)anBoard)));
 	  }
 	    
 	  if( analysis ) {
 	    const cubedecisiondata* c = pmr->CubeDecPtr;
 	    if( c->esDouble.et != EVAL_NONE ) {
-	      PyObject* d = PyDoubleAnalysis(&c->esDouble, c->aarOutput,
-					     c->aarStdDev, ms, verbose);
+	      PyObject* d = PyDoubleAnalysis(&c->esDouble, (float (*)[ NUM_ROLLOUT_OUTPUTS ])c->aarOutput,
+					     (float (*)[ NUM_ROLLOUT_OUTPUTS ])c->aarStdDev, ms, verbose);
 	      {
-		int s = PyDict_Merge(analysis, d, 1);     assert( s != -1 );
+		int s = PyDict_Merge(analysis, d, 1);     g_assert( s != -1 );
 	      }
 	      Py_DECREF(d);
 	    }
@@ -1931,7 +1821,7 @@ PythonGame(const list*    plGame,
 
 	default:
 	{
-	  assert(0);
+	  g_assert(0);
 	}
       }
 
@@ -1982,7 +1872,7 @@ static PyObject*
 PythonMatch(PyObject* self UNUSED_PARAM, PyObject* args, PyObject* keywds)
 {
   /* take match info from first game */
-  const list* firstGame = lMatch.plNext->p;
+  const listOLD* firstGame = lMatch.plNext->p;
   const moverecord* pmr;
   const xmovegameinfo* g;
   int includeAnalysis = 1;
@@ -2001,7 +1891,7 @@ PythonMatch(PyObject* self UNUSED_PARAM, PyObject* args, PyObject* keywds)
   }
   
   pmr = firstGame->plNext->p;
-  assert( pmr->mt == MOVE_GAMEINFO );
+  g_assert( pmr->mt == MOVE_GAMEINFO );
   g = &pmr->g;
 
   if( !PyArg_ParseTupleAndKeywords(args, keywds, "|iiii", kwlist,
@@ -2023,7 +1913,7 @@ PythonMatch(PyObject* self UNUSED_PARAM, PyObject* args, PyObject* keywds)
       PyErr_SetString(PyExc_StandardError, "First game missing from match");
 	  return 0;
   }
-  assert( g->i == 0 );
+  g_assert( g->i == 0 );
 
   /* W,X,0 
      B,O,1 */
@@ -2066,7 +1956,7 @@ PythonMatch(PyObject* self UNUSED_PARAM, PyObject* args, PyObject* keywds)
   }
 
   {
-    char* v[] = { "Standard", "Nackgammon", "Hypergammon1", "Hypergammon2",
+    const char* v[] = { "Standard", "Nackgammon", "Hypergammon1", "Hypergammon2",
 		  "Hypergammon3" };
 
     addProperty(matchInfoDict, "variation", v[g->bgv]);
@@ -2099,7 +1989,7 @@ PythonMatch(PyObject* self UNUSED_PARAM, PyObject* args, PyObject* keywds)
   
   {
     int nGames = 0;
-    const list* pl;
+    const listOLD* pl;
     PyObject *matchTuple;
     statcontext scMatch;
 
@@ -2190,12 +2080,12 @@ PythonNavigate(PyObject* self UNUSED_PARAM, PyObject* args, PyObject* keywds)
     }
   
     if( nextGame != INT_MIN && nextGame != 0 ) {
-      list* pl= lMatch.plNext;
+      listOLD* pl= lMatch.plNext;
 
       for( ; pl->p != plGame && pl != &lMatch; pl = pl->plNext)
 	;	
 
-      {                                            assert( pl->p == plGame ); }
+      {                                            g_assert( pl->p == plGame ); }
       {
 	int n = nextGame;
 	if( n > 0 ) {
@@ -2216,7 +2106,7 @@ PythonNavigate(PyObject* self UNUSED_PARAM, PyObject* args, PyObject* keywds)
     }
   
     if( nextRecord != INT_MIN ) {
-      recordsDiff = nextRecord - InternalCommandNext(0, nextRecord);
+      recordsDiff = nextRecord - InternalCommandNext(0, 0, nextRecord);
     }
 
     /* (HACK)
@@ -2426,79 +2316,86 @@ PyMethodDef gnubgMethods[] = {
 
 };
 
-#if HAVE_LIBREADLINE
+extern void PythonInitialise(char *argv0)
+{
 
-#if 0
-static char *
-PythonReadline( char *p ) {
-
-  char *pch;
-  int l;
-
-  pch = (char *) readline( p );
-
-  l = pch ? strlen( pch ) : 0;
-
-  pch = (char *) realloc( pch, l + 1 );
-
-  if( l )
-    strcat( pch, "\n" );
-  else
-    strcpy( pch, "" );
-
-  return pch;
-
-}
-#endif
-
-#endif
-
-extern void
-PythonInitialise( const char *szDir ) {
-
-  char *pch;
-  static char workingDir[BIG_PATH];
-  getcwd(workingDir, BIG_PATH);
-
-#if WIN32
+#ifdef WIN32
 {	/* Setup python to look in the pythonlib directory if present */
-	char pythonDir[BIG_PATH];
-	strcpy(pythonDir, workingDir);
-	strcat(pythonDir, "/PythonLib");
-	if (!access(pythonDir, F_OK))
+	char *working_dir = g_get_current_dir();
+	char *python_dir = g_build_filename(working_dir, "/PythonLib", NULL);
+	if (access(python_dir, F_OK) == 0)
 	{	/* Set Pyton to use this directory */
-		char buf[BIG_PATH + 100];
-		sprintf(buf, "PYTHONPATH=%s", pythonDir);
-		_putenv(buf);
-		sprintf(buf, "PYTHONROOT=%s", pythonDir);
-		_putenv(buf);
+		char *buf;
+		buf = g_strdup_printf("PYTHONPATH=%s", python_dir);
+		putenv(buf);
+		g_free(buf);
+		buf = g_strdup_printf("PYTHONROOT=%s", python_dir);
+		putenv(buf);
+		g_free(buf);
 	}
+	g_free(python_dir);
+	g_free(working_dir);
 }
-#endif
 
-  Py_SetProgramName(workingDir);
+#endif
+  Py_SetProgramName(argv0);
   Py_Initialize();
 
   /* ensure that python know about our gnubg module */
   Py_InitModule( "gnubg", gnubgMethods );
   PyRun_SimpleString( "import gnubg\n" );
 
-#if HAVE_LIBREADLINE
-  /* FIXME: implement switching of readline contexts 
-     PyOS_ReadlineFunctionPointer = PythonReadline; */
-#endif
-
   /* run gnubg.py start up script */
-  pch = g_strdup( "gnubg.py" );
-  CommandLoadPython( pch );
-  g_free( pch );
+  LoadPythonFile("gnubg.py");
 }
 
-extern void
-PythonShutdown( void ) {
-
+extern void PythonShutdown( void )
+{
   Py_Finalize();
-
 }
 
-#endif /* USE_PYTHON */
+extern void PythonRun(const char *sz)
+{
+	if (*sz) {
+		PyRun_SimpleString(sz);
+	} else {
+		PyRun_SimpleString("import sys\n"
+				   "print 'Python', sys.version\n");
+		while (PyRun_SimpleString(
+				   "while 1:\n"
+				   "    print '>>> ',\n"
+				   "    line = sys.stdin.readline()\n"
+				   "    if not line:\n"
+				   "        break\n"
+				   "    exec(line)\n"))
+		{};
+	}
+}
+extern int LoadPythonFile(const char *sz)
+{
+	char *path = NULL;
+	char *cmd = NULL;
+	int ret = FALSE;
+
+	if (g_file_test(sz, G_FILE_TEST_EXISTS))
+		path = g_strdup(sz);
+	else {
+		path = BuildFilename2("/scripts", sz);
+		if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
+			g_free(path);
+			path = g_build_filename("scripts", sz, NULL);
+		}
+	}
+	if (!g_file_test(path, G_FILE_TEST_IS_REGULAR)) {
+		g_free(path);
+		outputerrf("Python file (%s) not found\n", sz);
+		return FALSE;
+	}
+	cmd = g_strdup_printf("execfile('%s')", path);
+	PyRun_SimpleString(cmd);
+	g_free(path);
+	g_free(cmd);
+
+	return ret;
+}
+#endif
