@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,32 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gnubgmodule.h,v 1.10 2005/02/10 10:31:12 Superfly_Jon Exp $
+ * $Id: gnubgmodule.h,v 1.15 2008/02/06 22:47:57 Superfly_Jon Exp $
  */
 
 #ifndef _PYTHONMODULE_H_
 #define _PYTHONMODULE_H_
 
-#if USE_PYTHON
-
-/* Python.h defines HAVE_FSTAT (2.3) and HAVE_SETVBUF (2.4) so ignore and redefine below */
-#undef HAVE_SETVBUF
-#undef HAVE_LIMITS_H
-#undef HAVE_FSTAT
-#include <Python.h>
-#undef HAVE_SETVBUF
-#undef HAVE_LIMITS_H
-#undef HAVE_FSTAT
-#include "config.h"
-
-extern void
-PythonInitialise( const char *szDir );
-
-extern void
-PythonShutdown();
-
-extern PyObject *PythonMatchChecksum( PyObject* self, PyObject *args );
-
-#endif /* USE_PYTHON */
+extern void PythonInitialise(void);
+extern void PythonShutdown(void);
+extern void PythonRun(const char *sz);
+extern int LoadPythonFile(const char *sz);
 
 #endif /* _PYTHONMODULE_H_ */
