@@ -4,7 +4,7 @@
  * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,22 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkchequer.c,v 1.72 2006/09/13 21:55:07 c_anthon Exp $
+ * $Id: gtkchequer.c,v 1.89 2008/03/14 13:07:53 c_anthon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
+#include <glib.h>
 
 #include "backgammon.h"
 #include "drawboard.h"
@@ -46,19 +41,12 @@
 #include "gtkwindows.h"
 #include "progress.h"
 #include "format.h"
+#if USE_BOARD3D
+#include "fun3d.h"
+#endif
 
-extern moverecord *pmrCurAnn;
 int showMoveListDetail = 1;
-extern void MoveListCreate(hintdata *phd);
-extern void MoveListUpdate(const hintdata *phd);
-extern GList* MoveListGetSelectionList(const hintdata *phd);
-extern void MoveListFreeSelectionList(GList *pl);
-extern move *MoveListGetMove(const hintdata *phd, GList *pl);
-extern gint MoveListClearSelection( GtkWidget *pw, GdkEventSelection *pes, hintdata *phd );
-extern void MoveListShowToggledClicked(GtkWidget *pw, hintdata *phd);
-extern int MoveListGetSelectionCount(const hintdata *phd);
-
-static void MoveListCopy(GtkWidget *pw, hintdata *phd);
+moverecord *pmrCurAnn;
 
 static void MoveListRolloutClicked(GtkWidget *pw, hintdata *phd)
 {
@@ -77,31 +65,28 @@ static void MoveListRolloutClicked(GtkWidget *pw, hintdata *phd)
 
   /* setup rollout dialog */
   {
-	move **ppm = malloc( c * sizeof (move *));
-	cubeinfo** ppci = malloc( c * sizeof (cubeinfo *));
-	char ( *asz )[ 40 ] = malloc( 40 * c );
+	move **ppm = (move**)malloc( c * sizeof (move *));
+	cubeinfo** ppci = (cubeinfo**)malloc( c * sizeof (cubeinfo *));
+	char (*asz)[40] = (char(*)[40])malloc(40 * c);
 
   for( i =  0, pl = plSelList; i < c; pl = pl->next, i++ )
   {
     m = ppm[ i ] = MoveListGetMove(phd, pl);
     ppci[ i ] = &ci;
-    FormatMove ( asz[ i ], ms.anBoard, m->anMove );
+    FormatMove ( asz[ i ], msBoard(), m->anMove );
   }
 	MoveListFreeSelectionList(plSelList);
 
 	GTKSetCurrentParent(pw);
   RolloutProgressStart( &ci, c, NULL, &rcRollout, asz, &p );
 
-  if ( fAction )
-    HandleXAction();
-
   res = ScoreMoveRollout ( ppm, (const cubeinfo**)ppci, c, RolloutProgress, p );
 
   RolloutProgressEnd( &p );
 
+	free(asz);
 	free(ppm);
 	free(ppci);
-	free(asz);
 
 	if (res < 0)
 		return;
@@ -124,10 +109,10 @@ static void MoveListRolloutClicked(GtkWidget *pw, hintdata *phd)
   MoveListUpdate ( phd );
 }
 
-void ShowMove ( hintdata *phd, const int f )
+extern void ShowMove ( hintdata *phd, const int f )
 {
   char *sz;
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
 	if ( f )
 	{
 		move *pm;
@@ -140,7 +125,7 @@ void ShowMove ( hintdata *phd, const int f )
 
 		MoveListFreeSelectionList(plSelList);
 
-		memcpy ( anBoard, ms.anBoard, sizeof ( anBoard ) );
+		memcpy ( anBoard, msBoard(), sizeof(TanBoard) );
 		ApplyMove ( anBoard, pm->anMove, FALSE );
 
 		UpdateMove( ( BOARD( pwBoard ) )->board_data, anBoard );
@@ -182,7 +167,7 @@ static void MoveListTempMapClicked( GtkWidget *pw, hintdata *phd )
 
     memcpy( &ams[ i ], &ms, sizeof ( matchstate ) );
 
-    FormatMove( szMove, ams[ i ].anBoard, m->anMove );
+    FormatMove( szMove, (ConstTanBoard)ams[ i ].anBoard, m->anMove );
     ApplyMove( ams[ i ].anBoard, m->anMove, FALSE );
 
     /* Swap sides */
@@ -236,24 +221,25 @@ EvalMoves ( hintdata *phd, evalcontext *pec )
 	if (!plSelList)
 		return;
 
-  GetMatchStateCubeInfo( &ci, &ms );
+	GetMatchStateCubeInfo( &ci, &ms );
   
-  ProgressStart( _("Evaluating positions...") );
-
 	for(pl = plSelList; pl; pl = pl->next)
 	{
-    if ( ScoreMove (MoveListGetMove(phd, pl), &ci, pec, pec->nPlies ) < 0 ) {
-      ProgressEnd ();
-		MoveListFreeSelectionList(plSelList);
-      return;
-    }
+		scoreData sd;
+		sd.pm = MoveListGetMove(phd, pl);
+		sd.pci = &ci;
+		sd.pec = pec;
 
-    /* Calling RefreshMoveList here requires some extra work, as
-       it may reorder moves */
+		if (RunAsyncProcess((AsyncFun)asyncScoreMove, &sd, _("Evaluating positions...")) != ASR_OK)
+		{
+			MoveListFreeSelectionList(plSelList);
+			return;
+		}
 
-    MoveListUpdate ( phd );
-
-  }
+		/* Calling RefreshMoveList here requires some extra work, as
+		it may reorder moves */
+		MoveListUpdate ( phd );
+	}
 	MoveListFreeSelectionList(plSelList);
 
   MoveListClearSelection(0, 0, phd);
@@ -268,7 +254,6 @@ EvalMoves ( hintdata *phd, evalcontext *pec )
 
   MoveListUpdate ( phd );
 
-  ProgressEnd ();
 }
 
 static void
@@ -280,7 +265,7 @@ MoveListEval ( GtkWidget *pw, hintdata *phd )
 static void
 MoveListEvalPly ( GtkWidget *pw, hintdata *phd )
 {
-  char *szPly = gtk_object_get_data ( GTK_OBJECT ( pw ), "user_data" );
+  char *szPly = (char*)gtk_object_get_data ( GTK_OBJECT ( pw ), "user_data" );
 #if defined (REDUCTION_CODE)
   evalcontext ec = { TRUE, 0, 0, TRUE, 0.0 };
 #else
@@ -314,6 +299,33 @@ MoveListRolloutSettings ( GtkWidget *pw, void *unused )
   gtk_window_present ( GTK_WINDOW ( gtk_widget_get_toplevel( pw ) ) );
 }
 
+static void MoveListRolloutPresets(GtkWidget * pw, hintdata * phd)
+{
+	const gchar *preset;
+	gchar *file=NULL;
+	gchar *path=NULL;
+	gchar *command=NULL;
+
+	preset = (const gchar *) gtk_object_get_data(GTK_OBJECT(pw), "user_data");
+	file = g_strdup_printf("%s.rol", preset);
+	path = g_build_filename(szHomeDirectory, "rol", file, NULL);
+	if (g_file_test(path, G_FILE_TEST_IS_REGULAR)) {
+		command = g_strdup_printf("load commands \"%s\"", path);
+		outputoff();
+		UserCommand(command);
+		outputon();
+		MoveListRolloutClicked(pw, phd);
+	}
+	else
+	{
+		outputerrf(_("You need to save a preset as \"%s\""), file);
+		MoveListRolloutSettings(pw, NULL);
+	}
+        g_free(file);
+        g_free(path);
+	g_free(command);
+}
+
 typedef int ( *cfunc )( const void *, const void * );
 
 static int CompareInts( int *p0, int *p1 ) {
@@ -330,8 +342,8 @@ static char *MoveListCopyData ( hintdata *phd )
 	GList *plSelList = MoveListGetSelectionList(phd);
 	c = g_list_length(plSelList);
 
-	an = malloc( c * sizeof( an[ 0 ] ) );
-	sz = malloc( c * 9 * 80 );
+	an = (int*)malloc( c * sizeof( an[ 0 ] ) );
+	sz = (char*)malloc( c * 9 * 80 );
 
 	*sz = 0;
 
@@ -362,7 +374,7 @@ MoveListMove ( GtkWidget *pw, hintdata *phd )
 	move m;
 	move *pm;
 	char szMove[ 40 ];
-	int anBoard[ 2 ][ 25 ];
+	TanBoard anBoard;
 	GList *plSelList = MoveListGetSelectionList(phd);
 	if (!plSelList)
 		return;
@@ -372,20 +384,20 @@ MoveListMove ( GtkWidget *pw, hintdata *phd )
 
 	memcpy(&m, pm, sizeof(move));
 
-	memcpy ( anBoard, ms.anBoard, sizeof ( anBoard ) );
+	memcpy ( anBoard, msBoard(), sizeof(TanBoard) );
 	ApplyMove ( anBoard, m.anMove, FALSE );
 
 	if ( ! ms.fMove )
 	SwapSides ( anBoard );
 
-	sprintf ( szMove, "show fullboard %s", PositionID ( anBoard ) );
+	sprintf ( szMove, "show fullboard %s", PositionID ( (ConstTanBoard)anBoard ) );
 	UserCommand ( szMove );
 
 	if ( phd->fDestroyOnMove )
 	/* Destroy widget on exit */
 		gtk_widget_destroy( gtk_widget_get_toplevel( pw ) );
 
-	FormatMove( szMove, ms.anBoard, m.anMove );
+	FormatMove( szMove, msBoard(), m.anMove );
 	UserCommand( szMove );
 
 #if USE_BOARD3D
@@ -403,6 +415,13 @@ MoveListDetailsClicked( GtkWidget *pw, hintdata *phd )
 
 GtkWidget *pwDetails;
 
+static void MoveListCopy(GtkWidget *pw, hintdata *phd)
+{
+  char *pc = MoveListCopyData(phd);
+  if ( pc )
+    TextToClipboard( pc );
+}
+
 static GtkWidget *
 CreateMoveListTools ( hintdata *phd )
 {
@@ -457,8 +476,8 @@ CreateMoveListTools ( hintdata *phd )
 
     gtk_box_pack_start ( GTK_BOX ( phd->pwEvalPly ), pwply, TRUE, TRUE, 0 );
 
-    gtk_signal_connect( GTK_OBJECT( pwply ), "clicked",
-                        GTK_SIGNAL_FUNC( MoveListEvalPly ), phd );
+    g_signal_connect( G_OBJECT( pwply ), "clicked",
+                        G_CALLBACK( MoveListEvalPly ), phd );
 
     gtk_object_set_data_full ( GTK_OBJECT ( pwply ), "user_data", sz, g_free );
 
@@ -468,27 +487,56 @@ CreateMoveListTools ( hintdata *phd )
 
   }
 
-  gtk_table_attach (GTK_TABLE (pwTools), pwRollout, 3, 4, 0, 1,
+  gtk_table_attach (GTK_TABLE (pwTools), pwShow, 3, 4, 0, 1,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  
+  gtk_table_attach (GTK_TABLE (pwTools), pwMWC, 4, 5, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
-  gtk_table_attach (GTK_TABLE (pwTools), pwRolloutSettings, 4, 5, 0, 1,
-                    (GtkAttachOptions) (GTK_FILL),
-                    (GtkAttachOptions) (0), 0, 0);
-  
-  gtk_table_attach (GTK_TABLE (pwTools), pwMWC, 5, 6, 0, 1,
+  if ( !phd->fDetails ) 
+    gtk_table_attach (GTK_TABLE (pwTools), pwDetails, 5, 6, 0, 1,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL), 0, 0);
+
+  gtk_table_attach (GTK_TABLE (pwTools), pwRollout, 0, 1, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
-  gtk_table_attach (GTK_TABLE (pwTools), pwCopy, 0, 2, 1, 2,
+  gtk_table_attach (GTK_TABLE (pwTools), pwRolloutSettings, 1, 2, 1, 2,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+
+  phd->pwRolloutPresets = gtk_hbox_new ( FALSE, 0 );
+  gtk_table_attach (GTK_TABLE (pwTools), phd->pwRolloutPresets, 2, 3, 1, 2, 
+                   (GtkAttachOptions) (GTK_FILL),
+                  (GtkAttachOptions) (0), 0, 0);
+
+  for ( i = 0; i < 5; ++i ) {
+	  GtkWidget *ro_preset;
+	  sz = g_strdup_printf ( "%c", i+'a' ); /* string is freed by set_data_full */
+	  ro_preset = gtk_button_new_with_label ( sz );
+
+	  gtk_box_pack_start ( GTK_BOX ( phd->pwRolloutPresets ), ro_preset, TRUE, TRUE, 0 );
+
+	  g_signal_connect( G_OBJECT( ro_preset ), "clicked",
+			  G_CALLBACK( MoveListRolloutPresets ), phd );
+
+	  gtk_object_set_data_full ( GTK_OBJECT ( ro_preset ), "user_data", sz, g_free );
+
+	  sz = g_strdup_printf ( _("Rollout preset %c"), i+'a' );
+	  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), ro_preset, sz, sz );
+	  g_free ( sz );
+
+  }
+
+  gtk_table_attach (GTK_TABLE (pwTools), pwMove, 3, 4, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   
-  gtk_table_attach (GTK_TABLE (pwTools), pwShow, 2, 3, 1, 2,
-                    (GtkAttachOptions) (GTK_FILL),
-                    (GtkAttachOptions) (0), 0, 0);
-  
-  gtk_table_attach (GTK_TABLE (pwTools), pwMove, 3, 5, 1, 2,
+
+  gtk_table_attach (GTK_TABLE (pwTools), pwCopy, 4, 5, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   
@@ -496,10 +544,6 @@ CreateMoveListTools ( hintdata *phd )
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   
-  if ( !phd->fDetails ) 
-    gtk_table_attach (GTK_TABLE (pwTools), pwDetails, 6, 7, 0, 2,
-                      (GtkAttachOptions) (GTK_FILL),
-                      (GtkAttachOptions) (GTK_FILL), 0, 0);
   
   gtk_widget_set_sensitive( pwMWC, ms.nMatchTo );
   gtk_widget_set_sensitive( pwMove, FALSE );
@@ -513,27 +557,27 @@ CreateMoveListTools ( hintdata *phd )
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pwDetails), showMoveListDetail);
   /* signals */
 
-  gtk_signal_connect( GTK_OBJECT( pwRollout ), "clicked",
-                      GTK_SIGNAL_FUNC( MoveListRolloutClicked ), phd );
-  gtk_signal_connect( GTK_OBJECT( pwEval ), "clicked",
-                      GTK_SIGNAL_FUNC( MoveListEval ), phd );
-  gtk_signal_connect( GTK_OBJECT( pwEvalSettings ), "clicked",
-                      GTK_SIGNAL_FUNC( MoveListEvalSettings ), NULL );
-  gtk_signal_connect( GTK_OBJECT( pwRolloutSettings ), "clicked",
-                      GTK_SIGNAL_FUNC( MoveListRolloutSettings ), NULL );
-  gtk_signal_connect( GTK_OBJECT( pwMWC ), "toggled",
-                      GTK_SIGNAL_FUNC( MoveListMWC ), phd );
-  gtk_signal_connect( GTK_OBJECT( pwMove ), "clicked",
-                      GTK_SIGNAL_FUNC( MoveListMove ), phd );
-  gtk_signal_connect( GTK_OBJECT( pwShow ), "toggled",
-                      GTK_SIGNAL_FUNC( MoveListShowToggledClicked ), phd );
-  gtk_signal_connect( GTK_OBJECT( pwCopy ), "clicked",
-                      GTK_SIGNAL_FUNC( MoveListCopy ), phd );
-  gtk_signal_connect( GTK_OBJECT( pwTempMap ), "clicked",
-                      GTK_SIGNAL_FUNC( MoveListTempMapClicked ), phd );
+  g_signal_connect( G_OBJECT( pwRollout ), "clicked",
+                      G_CALLBACK( MoveListRolloutClicked ), phd );
+  g_signal_connect( G_OBJECT( pwEval ), "clicked",
+                      G_CALLBACK( MoveListEval ), phd );
+  g_signal_connect( G_OBJECT( pwEvalSettings ), "clicked",
+                      G_CALLBACK( MoveListEvalSettings ), NULL );
+  g_signal_connect( G_OBJECT( pwRolloutSettings ), "clicked",
+                      G_CALLBACK( MoveListRolloutSettings ), NULL );
+  g_signal_connect( G_OBJECT( pwMWC ), "toggled",
+                      G_CALLBACK( MoveListMWC ), phd );
+  g_signal_connect( G_OBJECT( pwMove ), "clicked",
+                      G_CALLBACK( MoveListMove ), phd );
+  g_signal_connect( G_OBJECT( pwShow ), "toggled",
+                      G_CALLBACK( MoveListShowToggledClicked ), phd );
+  g_signal_connect( G_OBJECT( pwCopy ), "clicked",
+                      G_CALLBACK( MoveListCopy ), phd );
+  g_signal_connect( G_OBJECT( pwTempMap ), "clicked",
+                      G_CALLBACK( MoveListTempMapClicked ), phd );
   if ( !phd->fDetails )
-    gtk_signal_connect( GTK_OBJECT( pwDetails ), "clicked",
-                        GTK_SIGNAL_FUNC( MoveListDetailsClicked ), phd );
+    g_signal_connect( G_OBJECT( pwDetails ), "clicked",
+                        G_CALLBACK( MoveListDetailsClicked ), phd );
 
   /* tool tips */
 
@@ -574,7 +618,7 @@ CreateMoveListTools ( hintdata *phd )
   return pwTools;
 }
 
-void HintDoubleClick(GtkTreeView        *treeview,
+extern void HintDoubleClick(GtkTreeView        *treeview,
                        GtkTreePath        *path,
                        GtkTreeViewColumn  *col,
                        hintdata *phd)
@@ -582,7 +626,7 @@ void HintDoubleClick(GtkTreeView        *treeview,
 	gtk_button_clicked( GTK_BUTTON( phd->pwMove ) );
 }
 
-void HintSelect(GtkTreeSelection *selection, hintdata *phd)
+extern void HintSelect(GtkTreeSelection *selection, hintdata *phd)
 {
 	CheckHintButtons( phd );
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(phd->pwShow)))
@@ -602,13 +646,6 @@ void HintSelect(GtkTreeSelection *selection, hintdata *phd)
 	}
 }
 
-static void MoveListCopy(GtkWidget *pw, hintdata *phd)
-{
-  char *pc = MoveListCopyData(phd);
-  if ( pc )
-    TextToClipboard( pc );
-}
-
 extern int 
 CheckHintButtons( hintdata *phd )
 {
@@ -618,6 +655,7 @@ CheckHintButtons( hintdata *phd )
     gtk_widget_set_sensitive( phd->pwCopy, c && phd->fButtonsValid );
     gtk_widget_set_sensitive( phd->pwTempMap, c && phd->fButtonsValid );
     gtk_widget_set_sensitive( phd->pwRollout, c && phd->fButtonsValid );
+    gtk_widget_set_sensitive( phd->pwRolloutPresets, c && phd->fButtonsValid );
     gtk_widget_set_sensitive( phd->pwEval, c && phd->fButtonsValid );
     gtk_widget_set_sensitive( phd->pwEvalPly, c && phd->fButtonsValid );
 
@@ -625,7 +663,7 @@ CheckHintButtons( hintdata *phd )
 }
 
 extern GtkWidget *
-CreateMoveList( movelist *pml, int *piHighlight, const int fButtonsValid,
+CreateMoveList( movelist *pml, unsigned int *piHighlight, const int fButtonsValid,
                 const int fDestroyOnMove, const int fDetails )
 {
     GtkWidget *pw;
@@ -635,7 +673,7 @@ CreateMoveList( movelist *pml, int *piHighlight, const int fButtonsValid,
 
     /* This function should only be called when the game state matches
        the move list. */
-    assert( ms.fMove == 0 || ms.fMove == 1 );
+    g_assert( ms.fMove == 0 || ms.fMove == 1 );
 
     phd->piHighlight = piHighlight;
     phd->pml = pml;
