@@ -4,7 +4,7 @@
  * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkchequer.h,v 1.8 2005/08/29 14:56:06 Superfly_Jon Exp $
+ * $Id: gtkchequer.h,v 1.14 2007/12/12 23:08:16 Superfly_Jon Exp $
  */
 
 #ifndef _GTKCHEQUER_H_
@@ -29,24 +29,25 @@ typedef struct _hintdata {
   GtkWidget *pwMove; /* move button */
   GtkWidget *pwCopy; /* copy button */
   GtkWidget *pwEvalPly; /* predefined eval buttons */
+  GtkWidget *pwRolloutPresets; /* predefined Rollout buttons */
   GtkWidget *pwShow; /* button for showing moves */
   GtkWidget *pwTempMap; /* button for showing temperature map */
   movelist *pml;
   int fButtonsValid;
   int fDestroyOnMove;
-  int *piHighlight;
+  unsigned int *piHighlight;
   int fDetails;
 } hintdata;
 
 extern GtkWidget *
-CreateMoveList( movelist *pml, int *piHighlight, 
+CreateMoveList( movelist *pml, unsigned int *piHighlight, 
                 const int fButtonsValid, const int fDestroyOnMove,
                 const int fDetails );
 
 extern int 
 CheckHintButtons( hintdata *phd );
 
-extern void MoveListRefreshSize();
+extern void MoveListRefreshSize(void);
 extern GtkWidget* pwDetails;
 extern int showMoveListDetail;
 #endif
