@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 1997-2002.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,12 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkcolour.c,v 1.15 2006/09/21 22:24:05 Superfly_Jon Exp $
+ * $Id: gtkcolour.c,v 1.19 2007/12/12 23:08:16 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include <gtk/gtk.h>
 #include <string.h>
@@ -32,9 +30,7 @@
 #define COLOUR_SEL_DIA( pcp ) GTK_COLOR_SELECTION_DIALOG( (pcp)->pwColourSel )
 #define COLOUR_SEL( pcp ) GTK_COLOR_SELECTION( COLOUR_SEL_DIA(pcp)->colorsel )
 
-static gpointer parent_class = NULL;
-
-static void set_gc_colour( GdkGC *gc, GdkColormap *pcm, GdkColor *col ) {
+static void set_gc_colour( GdkGC *gc, GdkColormap *notused, const GdkColor *col ) {
 
     gdk_gc_set_rgb_fg_color( gc, col );
 }
@@ -43,7 +39,7 @@ extern GtkWidget *gtk_colour_picker_new(ColorPickerFunc	func, void *data) {
 
     GtkColourPicker *pcp;
 
-    pcp = gtk_type_new( GTK_TYPE_COLOUR_PICKER );
+    pcp = GTK_COLOUR_PICKER(gtk_type_new( GTK_TYPE_COLOUR_PICKER ));
     pcp->func = func;
     pcp->data = data;
 
@@ -57,7 +53,7 @@ static void render_pixmap( GtkColourPicker *pcp ) {
     double opac;
     
     if( !pcp->ppm )
-	return;
+		return;
 
     /* FIXME different appearance if insensitive */
     
@@ -94,9 +90,8 @@ extern void gtk_colour_picker_set_has_opacity_control(GtkColourPicker *pcp, gboo
 	pcp->hasOpacity = f;
 }
 
-extern void gtk_colour_picker_set_colour( GtkColourPicker *pcp,
-					  gdouble *ar ) {
-
+extern void gtk_colour_picker_set_colour( GtkColourPicker *pcp, const gdouble *ar )
+{
     memcpy(pcp->arColour, ar, sizeof (pcp->arColour));
 
     render_pixmap( pcp );
@@ -110,30 +105,30 @@ extern void gtk_colour_picker_get_colour( GtkColourPicker *pcp,
     render_pixmap( pcp );
 }
 
-static void realize( GtkWidget *pwDraw, GtkColourPicker *pcp ) {
-
+static void realize( const GtkWidget *pwDraw, GtkColourPicker *pcp )
+{
     pcp->ppm = gdk_pixmap_new( pwDraw->window, 16, 16, -1 );
     gdk_window_set_back_pixmap( pwDraw->window, pcp->ppm, FALSE );
 
     render_pixmap( pcp );
 }
 
-static void colour_changed( GtkWidget *pw, GtkColourPicker *pcp ) {
-
+static void colour_changed( GtkWidget *notused, GtkColourPicker *pcp )
+{
     gtk_color_selection_get_color( COLOUR_SEL( pcp ), pcp->arColour );
     render_pixmap( pcp );
 }
 
-static void ok( GtkWidget *pw, GtkColourPicker *pcp )
+static void ok( GtkWidget *notused, GtkColourPicker *pcp )
 {
 	gtk_window_set_modal( GTK_WINDOW( COLOUR_SEL_DIA( pcp ) ), FALSE );
 	gtk_color_selection_get_color( COLOUR_SEL( pcp ), pcp->arColour );
 	gtk_widget_destroy( GTK_WIDGET( COLOUR_SEL_DIA( pcp ) ) );
 	pcp->pwColourSel = NULL;
-	(pcp->func)(pcp->data);
+	(pcp->func)(GTK_WIDGET(pcp->data));
 }
 
-static void cancel( GtkWidget *pw, GtkColourPicker *pcp )
+static void cancel( GtkWidget *notused, GtkColourPicker *pcp )
 {
 	/* Restore original colour */
 	memcpy(pcp->arColour, pcp->arOrig, sizeof (pcp->arColour));
@@ -144,14 +139,12 @@ static void cancel( GtkWidget *pw, GtkColourPicker *pcp )
 	pcp->pwColourSel = NULL;
 }
 
-static gboolean delete_event( GtkWidget *pw, GdkEvent *pev,
-			      GtkColourPicker *pcp ) {
-
+static gboolean delete_event( GtkWidget *pw, GdkEvent *notused, GtkColourPicker *pcp )
+{
     cancel(pw, pcp);
     return TRUE;
 }
 
-
 static void gtk_colour_picker_init( GtkColourPicker *pcp ) {
 
     pcp->ppm = NULL;
@@ -161,8 +154,8 @@ static void gtk_colour_picker_init( GtkColourPicker *pcp ) {
     pcp->pwDraw = gtk_drawing_area_new();
     gtk_drawing_area_size( GTK_DRAWING_AREA( pcp->pwDraw ), 32, 16 );
 
-    gtk_signal_connect( GTK_OBJECT( pcp->pwDraw ), "realize",
-			GTK_SIGNAL_FUNC( realize ), pcp );    
+    g_signal_connect( G_OBJECT( pcp->pwDraw ), "realize",
+			G_CALLBACK( realize ), pcp );    
     gtk_container_add( GTK_CONTAINER( pcp ), pcp->pwDraw );
     gtk_widget_show( pcp->pwDraw );
 }	
@@ -174,14 +167,14 @@ static void gtk_colour_picker_new_dialog( GtkColourPicker *pcp ) {
     gtk_color_selection_set_has_opacity_control( COLOUR_SEL( pcp ), pcp->hasOpacity);
 	gtk_color_selection_set_color( COLOUR_SEL( pcp ), pcp->arColour );
 
-    gtk_signal_connect( GTK_OBJECT( COLOUR_SEL( pcp ) ), "color-changed",
-			GTK_SIGNAL_FUNC( colour_changed ), pcp );    
-    gtk_signal_connect( GTK_OBJECT( COLOUR_SEL_DIA( pcp ) ), "delete-event",
-			GTK_SIGNAL_FUNC( delete_event ), pcp );
-    gtk_signal_connect( GTK_OBJECT( COLOUR_SEL_DIA( pcp )->ok_button ),
-			"clicked", GTK_SIGNAL_FUNC( ok ), pcp );
-    gtk_signal_connect( GTK_OBJECT( COLOUR_SEL_DIA( pcp )->cancel_button ),
-			"clicked", GTK_SIGNAL_FUNC( cancel ), pcp );
+    g_signal_connect( G_OBJECT( COLOUR_SEL( pcp ) ), "color-changed",
+			G_CALLBACK( colour_changed ), pcp );    
+    g_signal_connect( G_OBJECT( COLOUR_SEL_DIA( pcp ) ), "delete-event",
+			G_CALLBACK( delete_event ), pcp );
+    g_signal_connect( G_OBJECT( COLOUR_SEL_DIA( pcp )->ok_button ),
+			"clicked", G_CALLBACK( ok ), pcp );
+    g_signal_connect( G_OBJECT( COLOUR_SEL_DIA( pcp )->cancel_button ),
+			"clicked", G_CALLBACK( cancel ), pcp );
 }
 
 static void clicked( GtkButton *pb ) {
@@ -204,21 +197,19 @@ static void clicked( GtkButton *pb ) {
     
     /* If there is a grabbed window, set new dialog as modal */
     if( gtk_grab_get_current() )
-	gtk_window_set_modal( GTK_WINDOW( pwDia ), TRUE );
+		gtk_window_set_modal( GTK_WINDOW( pwDia ), TRUE );
 }
 
-static void gtk_colour_picker_class_init( GtkColourPickerClass *pc ) {
-
-    parent_class = gtk_type_class( GTK_TYPE_BUTTON );
-
+static void gtk_colour_picker_class_init( GtkColourPickerClass *pc )
+{
     GTK_BUTTON_CLASS( pc )->clicked = clicked;
 }
 
 extern GtkType gtk_colour_picker_get_type( void ) {
 
-    static GtkType t;
+    static GtkType t = 0;
     static const GtkTypeInfo ti = {
-	"GtkColourPicker",
+	(char*)"GtkColourPicker",
 	sizeof( GtkColourPicker ),
 	sizeof( GtkColourPickerClass ),
 	(GtkClassInitFunc) gtk_colour_picker_class_init,
@@ -226,8 +217,8 @@ extern GtkType gtk_colour_picker_get_type( void ) {
 	NULL, NULL, NULL
     };
     
-    if( !t )
-	t = gtk_type_unique( GTK_TYPE_BUTTON, &ti );
+    if (t == 0)
+		t = gtk_type_unique( GTK_TYPE_BUTTON, &ti );
 
     return t;
 }
