@@ -1,10 +1,11 @@
+
 /*
  * gtkcube.c
  *
  * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,22 +17,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkcube.c,v 1.47 2006/09/13 21:55:07 c_anthon Exp $
+ * $Id: gtkcube.c,v 1.82 2011/09/20 00:16:32 mdpetch Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
+#include "config.h"
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
+#include <glib.h>
 
 #include "backgammon.h"
 #include "eval.h"
@@ -40,21 +35,20 @@
 #include "gtkgame.h"
 #include "gtkcube.h"
 #include "gtkwindows.h"
-#include <glib/gi18n.h>
 #include "progress.h"
 #include "format.h"
-
+#include "gtklocdefs.h"
 
 
 typedef struct _cubehintdata {
   GtkWidget *pwFrame;   /* the table */
   GtkWidget *pw;        /* the box */
   GtkWidget *pwTools;   /* the tools */
-  float (*aarOutput)[ NUM_ROLLOUT_OUTPUTS ];
-  float (*aarStdDev)[ NUM_ROLLOUT_OUTPUTS ];
-  evalsetup *pes;
-  movetype mt;
+  moverecord *pmr;
   matchstate ms;
+  int did_double;
+  int did_take;
+  int hist;
 } cubehintdata;
 
 
@@ -65,7 +59,7 @@ OutputPercentsTable( const float ar[] )
     GtkWidget *pw;
     int i;
     static gchar *headings[] = { N_("Win"), N_("W(g)"), N_("W(bg)"),
-                                 N_("-"), N_("Lose"), N_("L(g)"), N_("L(bg)") };
+                                 "-", N_("Lose"), N_("L(g)"), N_("L(bg)") };
     pwTable = gtk_table_new ( 2, 7, FALSE );
     
     for (i = 0; i < 7; i++)
@@ -99,38 +93,31 @@ OutputPercentsTable( const float ar[] )
 
 
 static GtkWidget*
-TakeAnalysis( const movetype mt, 
-	      const float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
-	      const float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
-	      const evalsetup* pes,
-	      const matchstate* pms ) {
-
+TakeAnalysis(cubehintdata *pchd) {
     cubeinfo ci;
-
     GtkWidget *pw;
     GtkWidget *pwTable;
     GtkWidget *pwFrame;
-
     int iRow;
     int i;
     cubedecision cd;
-
     int ai[ 2 ];
     const char *aszCube[] = {
       NULL, NULL, 
       N_("Take"), 
       N_("Pass") };
-
     float arDouble[ 4 ];
     gchar *sz;
+    cubedecisiondata *cdec = pchd->pmr->CubeDecPtr;
+    const evalsetup *pes = &cdec->esDouble;
 
 
     if( pes->et == EVAL_NONE )
 	return NULL;
 
-    GetMatchStateCubeInfo( &ci, pms );
+    GetMatchStateCubeInfo( &ci, &pchd->ms );
 
-    cd = FindCubeDecision ( arDouble, aarOutput, &ci );
+    cd = FindCubeDecision ( arDouble, cdec->aarOutput, &ci );
     
     /* header */
 
@@ -150,13 +137,13 @@ TakeAnalysis( const movetype mt,
         sz = g_strdup_printf ( _("Cubeless %d-ply %s: %s (Money: %s)"),
                                pes->ec.nPlies,
                                fOutputMWC ? _("MWC") : _("equity"),
-                               OutputEquity ( aarOutput[ 0 ][ OUTPUT_EQUITY ],
+                               OutputEquity ( cdec->aarOutput[ 0 ][ OUTPUT_EQUITY ],
                                               &ci, TRUE ),
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
       else
         sz = g_strdup_printf ( _("Cubeless %d-ply equity: %s"),
                                pes->ec.nPlies,
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
 
       break;
 
@@ -164,12 +151,12 @@ TakeAnalysis( const movetype mt,
       if ( ci.nMatchTo )
         sz = g_strdup_printf ( _("Cubeless rollout %s: %s (Money: %s)"),
                                fOutputMWC ? _("MWC") : _("equity"),
-                               OutputEquity ( aarOutput[ 0 ][ OUTPUT_EQUITY ],
+                               OutputEquity ( cdec->aarOutput[ 0 ][ OUTPUT_EQUITY ],
                                               &ci, TRUE ),
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
       else
         sz = g_strdup_printf ( _("Cubeless rollout equity: %s"),
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
 
       break;
 
@@ -196,7 +183,7 @@ TakeAnalysis( const movetype mt,
 
     case EVAL_EVAL:
 
-      pw = OutputPercentsTable( aarOutput[ 0 ] );
+      pw = OutputPercentsTable( cdec->aarOutput[ 0 ] );
       gtk_table_attach ( GTK_TABLE ( pwTable ), pw,
                          0, 4, iRow, iRow + 1, 
                          GTK_EXPAND | GTK_FILL, 
@@ -208,13 +195,19 @@ TakeAnalysis( const movetype mt,
 
     case EVAL_ROLLOUT:
 
-      /* FIXME: */
+      pw = OutputPercentsTable( cdec->aarOutput[ 1 ] );
+      gtk_table_attach ( GTK_TABLE ( pwTable ), pw,
+                         0, 4, iRow, iRow + 1, 
+                         GTK_EXPAND | GTK_FILL, 
+                         GTK_EXPAND | GTK_FILL, 
+                         8, 4 );
+      iRow++;
 
       break;
 
     default:
 
-      assert ( FALSE );
+      g_assert ( FALSE );
       break;
 
     }
@@ -389,24 +382,15 @@ TakeAnalysis( const movetype mt,
  *
  */
 
-static GtkWidget*
-CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-	      const float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-	      const evalsetup *pes,
-	      const matchstate* pms,
-	      const int fDouble ) {
-
+static GtkWidget* CubeAnalysis(cubehintdata *pchd) {
     cubeinfo ci;
-
     GtkWidget *pw;
     GtkWidget *pwTable;
     GtkWidget *pwFrame;
-
     int iRow;
     int i;
     cubedecision cd;
     float r;
-
     int ai[ 3 ];
     const char *aszCube[] = {
       NULL, 
@@ -414,19 +398,20 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
       N_("Double, take"), 
       N_("Double, pass") 
     };
-
     float arDouble[ 4 ];
     gchar *sz;
+    cubedecisiondata *cdec = pchd->pmr->CubeDecPtr;
+    const evalsetup *pes = &cdec->esDouble;
 
 
-    if( pes->et == EVAL_NONE )
+    if(pes->et == EVAL_NONE )
 	return NULL;
 
-    GetMatchStateCubeInfo( &ci, pms );
+    GetMatchStateCubeInfo( &ci, &pchd->ms );
 
-    cd = FindCubeDecision ( arDouble, aarOutput, &ci );
+    cd = FindCubeDecision ( arDouble, cdec->aarOutput, &ci );
     
-    if( !GetDPEq( NULL, NULL, &ci ) && ! fDouble )
+    if( !GetDPEq( NULL, NULL, &ci ) && ! pchd->pmr->mt == MOVE_DOUBLE)
 	/* No cube action possible */
 	return NULL;
 
@@ -452,13 +437,13 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
         sz = g_strdup_printf ( _("Cubeless %d-ply %s: %s (Money: %s)"),
                                pes->ec.nPlies,
                                fOutputMWC ? _("MWC") : _("equity"),
-                               OutputEquity ( aarOutput[ 0 ][ OUTPUT_EQUITY ],
+                               OutputEquity ( cdec->aarOutput[ 0 ][ OUTPUT_EQUITY ],
                                               &ci, TRUE ),
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
       else
         sz = g_strdup_printf ( _("Cubeless %d-ply equity: %s"),
                                pes->ec.nPlies,
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
 
       break;
 
@@ -466,12 +451,12 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
       if ( ci.nMatchTo )
          sz = g_strdup_printf ( _("Cubeless rollout %s: %s (Money: %s)"),
                                fOutputMWC ? _("MWC") : _("equity"),
-                               OutputEquity ( aarOutput[ 0 ][ OUTPUT_EQUITY ],
+                               OutputEquity ( cdec->aarOutput[ 0 ][ OUTPUT_EQUITY ],
                                               &ci, TRUE ),
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
       else
         sz = g_strdup_printf ( _("Cubeless rollout equity: %s"),
-                               OutputMoneyEquity ( aarOutput[ 0 ], TRUE ) );
+                               OutputMoneyEquity ( cdec->aarOutput[ 0 ], TRUE ) );
 
       break;
 
@@ -498,7 +483,7 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
     if ( pes->et == EVAL_EVAL ) {
 
-      pw = OutputPercentsTable(aarOutput[ 0 ] );
+      pw = OutputPercentsTable(cdec->aarOutput[ 0 ] );
 
       gtk_table_attach ( GTK_TABLE ( pwTable ), pw,
                          0, 4, iRow, iRow + 1, 
@@ -520,7 +505,7 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
     iRow++;
     
 
-    getCubeDecisionOrdering ( ai, arDouble, aarOutput, &ci );
+    getCubeDecisionOrdering ( ai, arDouble, cdec->aarOutput, &ci );
 
     for ( i = 0; i < 3; i++ ) {
 
@@ -587,7 +572,7 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
         /* FIXME: output cubeless euqities and percentages for rollout */
         /*        probably along with rollout details */
         
-        pw = gtk_label_new ( OutputPercents ( aarOutput[ ai [ i ] - 1 ], TRUE ) );
+        pw = gtk_label_new ( OutputPercents ( cdec->aarOutput[ ai [ i ] - 1 ], TRUE ) );
         gtk_misc_set_alignment( GTK_MISC( pw ), 0.5, 0.5 );
         
         gtk_table_attach ( GTK_TABLE ( pwTable ), pw,
@@ -628,6 +613,7 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
       sz = g_strdup_printf ( "(%.1f%%)", 100.0 * r );
       pw = gtk_label_new ( sz );
+      g_free(sz);
       gtk_misc_set_alignment( GTK_MISC( pw ), 1, 0.5 );
 
       gtk_table_attach ( GTK_TABLE ( pwTable ), pw,
@@ -641,54 +627,48 @@ CubeAnalysis( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
     return pwFrame;
 }
 
-static void
-UpdateCubeAnalysis ( cubehintdata *pchd ) {
+static void UpdateCubeAnalysis(cubehintdata * pchd)
+{
 
-  GtkWidget *pw = 0;
+	GtkWidget *pw = 0;
+	doubletype dt = DoubleType(ms.fDoubled, ms.fMove, ms.fTurn);
 
-  switch ( pchd->mt ) {
-  case MOVE_NORMAL:
-  case MOVE_DOUBLE:
-    pw = CubeAnalysis ( GCCCONSTAHACK pchd->aarOutput,
-                        GCCCONSTAHACK pchd->aarStdDev,
-                        pchd->pes,
-                        &pchd->ms,
-                        pchd->mt );
-    break;
+	find_skills(pchd->pmr, &ms, pchd->did_double, pchd->did_take);
 
-  case MOVE_DROP:
-  case MOVE_TAKE:
-    pw = TakeAnalysis ( pchd->mt,
-                        GCCCONSTAHACK pchd->aarOutput,
-                        GCCCONSTAHACK pchd->aarStdDev,
-                        pchd->pes,
-                        &pchd->ms );
-    break;
+	switch (pchd->pmr->mt) {
+	case MOVE_NORMAL:
+	case MOVE_SETDICE:
+	case MOVE_SETCUBEPOS:
+	case MOVE_SETCUBEVAL:
+		/* See CreateCubeAnalysis() for these  MOVE_SET* cases */
+	case MOVE_DOUBLE:
+		if (dt == DT_NORMAL)
+			pw = CubeAnalysis(pchd);
+		else if (dt == DT_BEAVER)
+			pw = TakeAnalysis(pchd);
+		break;
 
-  default:
+	case MOVE_DROP:
+	case MOVE_TAKE:
+		pw = TakeAnalysis(pchd);
+		break;
 
-    assert ( FALSE );
-    break;
+	default:
+		g_assert(FALSE);
+		break;
 
-  }
+	}
 
-  /* update cache */
+	/* destroy current analysis */
 
-  UpdateStoredCube ( pchd->aarOutput, pchd->aarStdDev, pchd->pes, &pchd->ms ),
+	gtk_container_remove(GTK_CONTAINER(pchd->pw), pchd->pwFrame);
 
-  /* destroy current analysis */
+	/* re-create analysis + tools */
 
-  gtk_container_remove ( GTK_CONTAINER ( pchd->pw ),
-                         pchd->pwFrame );
+	gtk_box_pack_start(GTK_BOX(pchd->pw), pchd->pwFrame = pw,
+			   FALSE, FALSE, 0);
 
-  /* re-create analysis + tools */
-
-  gtk_box_pack_start ( GTK_BOX ( pchd->pw ), pchd->pwFrame = pw, 
-                       FALSE, FALSE, 0 );
-
-
-  gtk_widget_show_all ( pchd->pw );
-  
+	gtk_widget_show_all(pchd->pw);
 
 }
 
@@ -701,8 +681,9 @@ CubeAnalysisRollout ( GtkWidget *pw, cubehintdata *pchd ) {
   cubeinfo ci;
   float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ];
   float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ];
+  cubedecisiondata *cdec = pchd->pmr->CubeDecPtr;
   rolloutstat aarsStatistics[ 2 ][ 2 ];
-  evalsetup *pes = pchd->pes;
+  evalsetup *pes = &cdec->esDouble;
   char asz[ 2 ][ 40 ];
   void *p;
 
@@ -715,9 +696,9 @@ CubeAnalysisRollout ( GtkWidget *pw, cubehintdata *pchd ) {
     pes->rc.fStopOnSTD = rcRollout.fStopOnSTD;
     pes->rc.nMinimumGames = rcRollout.nMinimumGames;
     pes->rc.rStdLimit = rcRollout.rStdLimit;
-	memcpy (aarOutput, pchd->aarOutput, 
+	memcpy (aarOutput, cdec->aarOutput, 
 			2 * NUM_ROLLOUT_OUTPUTS * sizeof ( float ) );
-	memcpy (aarStdDev, pchd->aarStdDev,
+	memcpy (aarStdDev, cdec->aarStdDev,
 			2 * NUM_ROLLOUT_OUTPUTS * sizeof ( float ) );
   }
 
@@ -725,73 +706,76 @@ CubeAnalysisRollout ( GtkWidget *pw, cubehintdata *pchd ) {
 
   FormatCubePositions( &ci, asz );
   GTKSetCurrentParent(pw);
-  RolloutProgressStart( &ci, 2, aarsStatistics, &pes->rc, asz, &p );
+  RolloutProgressStart( &ci, 2, aarsStatistics, &pes->rc, asz, FALSE, &p );
 
   if ( GeneralCubeDecisionR ( aarOutput, aarStdDev, aarsStatistics,
-                              pchd->ms.anBoard, &ci, 
+                              (ConstTanBoard)pchd->ms.anBoard, &ci, 
 			      &pes->rc, pes,
                               RolloutProgress, p ) < 0 ) {
-    RolloutProgressEnd( &p );
+    RolloutProgressEnd( &p, FALSE );
     return;
   }
 
-  RolloutProgressEnd( &p );
+  RolloutProgressEnd( &p, FALSE );
   
-  memcpy ( pchd->aarOutput, aarOutput, 
+  memcpy ( cdec->aarOutput, aarOutput, 
            2 * NUM_ROLLOUT_OUTPUTS * sizeof ( float ) );
-  memcpy ( pchd->aarStdDev, aarStdDev, 
+  memcpy ( cdec->aarStdDev, aarStdDev, 
            2 * NUM_ROLLOUT_OUTPUTS * sizeof ( float ) );
 
   if (pes->et != EVAL_ROLLOUT)
-	memcpy ( &pchd->pes->rc, &rcRollout, sizeof ( rcRollout ) );
+	memcpy ( &pes->rc, &rcRollout, sizeof ( rcRollout ) );
 
-  pchd->pes->et = EVAL_ROLLOUT;
+  pes->et = EVAL_ROLLOUT;
+
+  /* If the source widget parent has been destroyed do not attempt
+     to update the hint window */
+  if ( ! GDK_IS_WINDOW (gtk_widget_get_parent_window( pw ) ) )
+    return;
 
   UpdateCubeAnalysis ( pchd );
-
+  if (pchd->hist)
+	  ChangeGame(NULL);
 }
 
-static void
-EvalCube ( cubehintdata *pchd, evalcontext *pec ) {
+static void EvalCube ( cubehintdata *pchd, evalcontext *pec )
+{
+	cubeinfo ci;
+	decisionData dd;
+	cubedecisiondata *cdec = pchd->pmr->CubeDecPtr;
+	evalsetup *pes = &cdec->esDouble;
 
-  cubeinfo ci;
-  float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ];
+	GetMatchStateCubeInfo( &ci, &pchd->ms );
 
-  GetMatchStateCubeInfo( &ci, &pchd->ms );
-  
-  ProgressStart( _("Considering cube action...") );
-
-  if ( GeneralCubeDecisionE ( aarOutput, pchd->ms.anBoard, &ci, 
-                              pec, 0 ) < 0 ) {
-    ProgressEnd();
-    return;
-  }
-  ProgressEnd();
+	dd.pboard = (ConstTanBoard)pchd->ms.anBoard;
+	dd.pci = &ci;
+	dd.pec = pec;
+	dd.pes = NULL;
+	if (RunAsyncProcess((AsyncFun)asyncCubeDecisionE, &dd, _("Considering cube action...")) != 0)
+		return;
 
   /* save evaluation */
 
-  memcpy ( pchd->aarOutput, aarOutput, 
+
+  memcpy ( cdec->aarOutput, dd.aarOutput, 
            2 * NUM_ROLLOUT_OUTPUTS * sizeof ( float ) );
 
-  pchd->pes->et = EVAL_EVAL;
-  memcpy ( &pchd->pes->ec, pec, sizeof ( evalcontext ) );
+  pes->et = EVAL_EVAL;
+  memcpy ( &pes->ec, dd.pec, sizeof ( evalcontext ) );
 
   UpdateCubeAnalysis ( pchd );
-
+  if (pchd->hist)
+	  ChangeGame(NULL);
 }
 
 static void
 CubeAnalysisEvalPly ( GtkWidget *pw, cubehintdata *pchd ) {
 
-  char *szPly = gtk_object_get_data ( GTK_OBJECT ( pw ), "user_data" );
+  char *szPly = (char*)g_object_get_data ( G_OBJECT ( pw ), "ply" );
   evalcontext ec = {0, 0, 0, TRUE, 0.0};
 
   ec.fCubeful = esAnalysisCube.ec.fCubeful;
-#if defined( REDUCTION_CODE )
-  ec.nReduced = esAnalysisCube.ec.nReduced;
-#else
   ec.fUsePrune = esAnalysisCube.ec.fUsePrune;
-#endif
   ec.nPlies = atoi ( szPly );
 
   EvalCube ( pchd, &ec );
@@ -799,17 +783,17 @@ CubeAnalysisEvalPly ( GtkWidget *pw, cubehintdata *pchd ) {
 }
 
 static void
-CubeAnalysisEval ( GtkWidget *pw, cubehintdata *pchd ) {
+CubeAnalysisEval ( GtkWidget *UNUSED(pw), cubehintdata *pchd ) {
 
-  EvalCube ( pchd, &esEvalCube.ec );
+  EvalCube ( pchd, &GetEvalCube()->ec );
 
 }
 
 static void
-CubeAnalysisEvalSettings ( GtkWidget *pw, void *unused )
+CubeAnalysisEvalSettings ( GtkWidget *pw, void *UNUSED(data) )
 {
 	GTKSetCurrentParent(pw);
-	SetEvaluation ( NULL, 0, NULL );
+	SetAnalysis ( NULL, 0, NULL );
 
   /* bring the dialog holding this button to the top */
   gtk_window_present ( GTK_WINDOW ( gtk_widget_get_toplevel( pw ) ) );
@@ -817,7 +801,7 @@ CubeAnalysisEvalSettings ( GtkWidget *pw, void *unused )
 }
 
 static void
-CubeAnalysisRolloutSettings ( GtkWidget *pw, void *unused )
+CubeAnalysisRolloutSettings ( GtkWidget *pw, void *UNUSED(data) )
 {
 	GTKSetCurrentParent(pw);
 	SetRollouts ( NULL, 0, NULL );
@@ -827,6 +811,34 @@ CubeAnalysisRolloutSettings ( GtkWidget *pw, void *unused )
 
 }
 
+
+static void CubeRolloutPresets(GtkWidget * pw, cubehintdata * pchd)
+{
+	const gchar *preset;
+	gchar *file=NULL;
+	gchar *path=NULL;
+	gchar *command=NULL;
+
+	preset = (const gchar *) g_object_get_data(G_OBJECT(pw), "user_data");
+	file = g_strdup_printf("%s.rol", preset);
+	path = g_build_filename(szHomeDirectory, "rol", file, NULL);
+	if (g_file_test(path, G_FILE_TEST_IS_REGULAR)) {
+		command = g_strdup_printf("load commands \"%s\"", path);
+		outputoff();
+		UserCommand(command);
+		outputon();
+		CubeAnalysisRollout(pw, pchd);
+	}
+	else
+	{
+		outputerrf(_("You need to save a preset as \"%s\""), file);
+		CubeAnalysisRolloutSettings(pw, NULL);
+	}
+        g_free(file);
+        g_free(path);
+	g_free(command);
+}
+
 static void
 CubeAnalysisMWC ( GtkWidget *pw, cubehintdata *pchd ) {
 
@@ -837,6 +849,7 @@ CubeAnalysisMWC ( GtkWidget *pw, cubehintdata *pchd ) {
     sprintf ( sz, "set output mwc %s", fOutputMWC ? "off" : "on" );
     
     UserCommand ( sz );
+    UserCommand("save settings");
   }
 
   gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( pw ), fOutputMWC );
@@ -845,37 +858,31 @@ CubeAnalysisMWC ( GtkWidget *pw, cubehintdata *pchd ) {
 
 }
 
-
-static char *
-GetContent ( cubehintdata *pchd ) {
-
+static char *GetContent ( cubehintdata *pchd )
+{
   static char *pc;
   cubeinfo ci;
+  cubedecisiondata *cdec = pchd->pmr->CubeDecPtr;
 
   GetMatchStateCubeInfo ( &ci, &pchd->ms );
-
-  pc = OutputCubeAnalysis ( GCCCONSTAHACK pchd->aarOutput,
-                            GCCCONSTAHACK pchd->aarStdDev,
-                            pchd->pes,
-                            &ci );
+  pc = OutputCubeAnalysis (  cdec->aarOutput, cdec->aarStdDev,
+                            &cdec->esDouble, &ci );
 
   return pc;
-
 }
 
-
 static void
-CubeAnalysisCopy ( GtkWidget *pw, cubehintdata *pchd ) {
+CubeAnalysisCopy ( GtkWidget *UNUSED(pw), cubehintdata *pchd ) {
 
   char *pc =  GetContent ( pchd );
 
   if ( pc )
-    TextToClipboard( pc );
+	  GTKTextWindow(pc, _("Cube details"), DT_INFO, NULL);
 
 }
 
 static void
-CubeAnalysisTempMap ( GtkWidget *pw, cubehintdata *pchd ) {
+CubeAnalysisTempMap ( GtkWidget *UNUSED(pw), cubehintdata *UNUSED(pchd) ) {
 
   char *sz = g_strdup( "show temperaturemap =cube" );
   UserCommand( sz );
@@ -883,6 +890,11 @@ CubeAnalysisTempMap ( GtkWidget *pw, cubehintdata *pchd ) {
 
 }
 
+static void CubeAnalysisCmark(GtkWidget * pw, cubehintdata * pchd)
+{
+	pchd->pmr->CubeDecPtr->cmark = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pw));
+}
+
 static GtkWidget *
 CreateCubeAnalysisTools ( cubehintdata *pchd ) {
 
@@ -892,16 +904,16 @@ CreateCubeAnalysisTools ( cubehintdata *pchd ) {
   GtkWidget *pwply;
   GtkWidget *pwEvalSettings = gtk_button_new_with_label ( _("...") );
   GtkWidget *pwRollout = gtk_button_new_with_label( _("Rollout") );
+  GtkWidget *pwRolloutPresets = gtk_hbox_new ( FALSE, 0 );
   GtkWidget *pwRolloutSettings = gtk_button_new_with_label ( _("...") );
   GtkWidget *pwMWC = gtk_toggle_button_new_with_label( _("MWC") );
   GtkWidget *pwCopy = gtk_button_new_with_label ( _("Copy") );
   GtkWidget *pwTempMap = gtk_button_new_with_label( _("Temp. Map" ) );
+  GtkWidget *pwCmark = gtk_toggle_button_new_with_label( _("Cmark" ) );
   GtkWidget *pw;
   int i;
   char *sz;
 
-  GtkTooltips *pt = gtk_tooltips_new ();
-
   /* toolbox on the left with buttons for eval, rollout and more */
   
   pchd->pwTools = pwTools = gtk_table_new (2, 5, FALSE);
@@ -926,163 +938,181 @@ CreateCubeAnalysisTools ( cubehintdata *pchd ) {
 
     gtk_box_pack_start ( GTK_BOX ( pw ), pwply, TRUE, TRUE, 0 );
 
-    gtk_signal_connect( GTK_OBJECT( pwply ), "clicked",
-                        GTK_SIGNAL_FUNC( CubeAnalysisEvalPly ), pchd );
+    g_signal_connect( G_OBJECT( pwply ), "clicked",
+                        G_CALLBACK( CubeAnalysisEvalPly ), pchd );
 
-    gtk_object_set_data_full ( GTK_OBJECT ( pwply ), "user_data", sz, g_free );
+    g_object_set_data_full ( G_OBJECT ( pwply ), "ply", sz, g_free );
 
     sz = g_strdup_printf ( _("Evaluate play on cubeful %d-ply"), i );
-    gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwply, sz, sz );
+    gtk_widget_set_tooltip_text( pwply, sz);
     g_free ( sz );
 
   }
 
-  gtk_table_attach (GTK_TABLE (pwTools), pwRollout, 3, 4, 0, 1,
+  gtk_table_attach (GTK_TABLE (pwTools), pwMWC, 3, 4, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
-  gtk_table_attach (GTK_TABLE (pwTools), pwRolloutSettings, 4, 5, 0, 1,
-                    (GtkAttachOptions) (GTK_FILL),
-                    (GtkAttachOptions) (0), 0, 0);
-  
-  gtk_table_attach (GTK_TABLE (pwTools), pwMWC, 0, 2, 1, 2,
-                    (GtkAttachOptions) (GTK_FILL),
-                    (GtkAttachOptions) (0), 0, 0);
-  
-  gtk_table_attach (GTK_TABLE (pwTools), pwCopy, 2, 3, 1, 2,
+  gtk_table_attach (GTK_TABLE (pwTools), pwTempMap, 4, 5, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
-  gtk_table_attach (GTK_TABLE (pwTools), pwTempMap, 3, 5, 1, 2,
+  gtk_table_attach (GTK_TABLE (pwTools), pwCmark, 4, 5, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   
+  
+  gtk_table_attach (GTK_TABLE (pwTools), pwRollout, 0, 1, 1, 2,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+
+  gtk_table_attach (GTK_TABLE (pwTools), pwRolloutSettings, 1, 2, 1, 2,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  
+  gtk_table_attach (GTK_TABLE (pwTools), pwRolloutPresets, 2, 3, 1, 2,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  
+  for ( i = 0; i < 5; ++i ) {
+	  GtkWidget *ro_preset;
+	  sz = g_strdup_printf ( "%c", i+'a' ); /* string is freed by set_data_full */
+	  ro_preset = gtk_button_new_with_label ( sz );
+
+	  gtk_box_pack_start ( GTK_BOX ( pwRolloutPresets ), ro_preset, TRUE, TRUE, 0 );
+
+	  g_signal_connect( G_OBJECT( ro_preset ), "clicked",
+			  G_CALLBACK( CubeRolloutPresets ), pchd );
+
+	  g_object_set_data_full ( G_OBJECT ( ro_preset ), "user_data", sz, g_free );
+
+	  sz = g_strdup_printf ( _("Rollout preset %c"), i+'a' );
+	  gtk_widget_set_tooltip_text( ro_preset, sz);
+	  g_free ( sz );
+
+  }
+
+  gtk_table_attach (GTK_TABLE (pwTools), pwCopy, 3, 4, 1, 2,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+
   gtk_widget_set_sensitive( pwMWC, pchd->ms.nMatchTo );
   
+  gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( pwCmark ),
+                                 pchd->pmr->CubeDecPtr->cmark );
+
   gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( pwMWC ),
                                  fOutputMWC );
 
   /* signals */
 
-  gtk_signal_connect( GTK_OBJECT( pwRollout ), "clicked",
-                      GTK_SIGNAL_FUNC( CubeAnalysisRollout ), pchd );
-  gtk_signal_connect( GTK_OBJECT( pwEval ), "clicked",
-                      GTK_SIGNAL_FUNC( CubeAnalysisEval ), pchd );
-  gtk_signal_connect( GTK_OBJECT( pwEvalSettings ), "clicked",
-                      GTK_SIGNAL_FUNC( CubeAnalysisEvalSettings ), NULL );
-  gtk_signal_connect( GTK_OBJECT( pwRolloutSettings ), "clicked",
-                      GTK_SIGNAL_FUNC( CubeAnalysisRolloutSettings ), NULL );
-  gtk_signal_connect( GTK_OBJECT( pwMWC ), "toggled",
-                      GTK_SIGNAL_FUNC( CubeAnalysisMWC ), pchd );
-  gtk_signal_connect( GTK_OBJECT( pwCopy ), "clicked",
-                      GTK_SIGNAL_FUNC( CubeAnalysisCopy ), pchd );
-  gtk_signal_connect( GTK_OBJECT( pwTempMap ), "clicked",
-                      GTK_SIGNAL_FUNC( CubeAnalysisTempMap ), pchd );
+  g_signal_connect( G_OBJECT( pwRollout ), "clicked",
+                      G_CALLBACK( CubeAnalysisRollout ), pchd );
+  g_signal_connect( G_OBJECT( pwEval ), "clicked",
+                      G_CALLBACK( CubeAnalysisEval ), pchd );
+  g_signal_connect( G_OBJECT( pwEvalSettings ), "clicked",
+                      G_CALLBACK( CubeAnalysisEvalSettings ), NULL );
+  g_signal_connect( G_OBJECT( pwRolloutSettings ), "clicked",
+                      G_CALLBACK( CubeAnalysisRolloutSettings ), NULL );
+  g_signal_connect( G_OBJECT( pwMWC ), "toggled",
+                      G_CALLBACK( CubeAnalysisMWC ), pchd );
+  g_signal_connect( G_OBJECT( pwCopy ), "clicked",
+                      G_CALLBACK( CubeAnalysisCopy ), pchd );
+  g_signal_connect( G_OBJECT( pwTempMap ), "clicked",
+                      G_CALLBACK( CubeAnalysisTempMap ), pchd );
+  g_signal_connect( G_OBJECT( pwCmark ), "toggled",
+                      G_CALLBACK( CubeAnalysisCmark ), pchd );
 
   /* tool tips */
 
-  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwRollout,
-                         _("Rollout cube decision with current settings"),
+  gtk_widget_set_tooltip_text( pwRollout,
                          _("Rollout cube decision with current settings") );
 
-  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwEval,
-                         _("Evaluate cube decision with current settings"),
+  gtk_widget_set_tooltip_text( pwEval,
                          _("Evaluate cube decision with current settings") );
 
-  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwRolloutSettings,
-                         _("Modify rollout settings"),
+  gtk_widget_set_tooltip_text( pwRolloutSettings,
                          _("Modify rollout settings") );
 
-  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwEvalSettings,
-                         _("Modify evaluation settings"),
+  gtk_widget_set_tooltip_text( pwEvalSettings,
                          _("Modify evaluation settings") );
 
-  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwMWC,
-                         _("Toggle output as MWC or equity"),
+  gtk_widget_set_tooltip_text( pwMWC,
                          _("Toggle output as MWC or equity") );
 
-  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwCopy,
-                         _("Copy"),
+  gtk_widget_set_tooltip_text( pwCopy,
                          _("Copy") );
 
-  gtk_tooltips_set_tip ( GTK_TOOLTIPS ( pt ), pwTempMap,
+  gtk_widget_set_tooltip_text( pwTempMap,
                          _("Show Sho Sengoku Temperature Map of position "
-                           "after selected move"),
-                         "" );
+                           "after selected move"));
 
   return pwTools;
   
 }
 
 
-extern GtkWidget *
-CreateCubeAnalysis ( float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                     float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                     evalsetup *pes,
-                     const movetype mt ) {
+extern GtkWidget *CreateCubeAnalysis(moverecord *pmr, const matchstate *pms, int did_double, int did_take, int hist)
+{
+	doubletype dt = DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
+	cubehintdata *pchd = g_new0(cubehintdata, 1);
 
-  cubehintdata *pchd = (cubehintdata *) malloc ( sizeof ( cubehintdata ) );
+	GtkWidget *pw, *pwhb, *pwx;
 
-  GtkWidget *pw, *pwhb, *pwx;
+	pchd->pmr = pmr;
+	pchd->ms = *pms;
+	pchd->did_double = did_double;
+	pchd->did_take = did_take;
+	pchd->hist = hist;
+	pchd->pw = pw = gtk_hbox_new(FALSE, 2);
+	switch (pmr->mt) {
 
-  pchd->aarOutput = aarOutput;
-  pchd->aarStdDev = aarStdDev;
-  pchd->pes = pes;
-  pchd->mt = mt;
-  pchd->ms = ms;
-  
-  pchd->pw = pw = gtk_hbox_new ( FALSE, 2 );
+	case MOVE_NORMAL:
+	case MOVE_SETDICE:
+	  /*
+	    We're stepping back in the game record after rolling and
+	    asking for a hint on the missed cube decision, for instance
+	  */
+	case MOVE_SETCUBEPOS:
+	case MOVE_SETCUBEVAL:
+	  /*
+	    These two happen in cube decisions loaded from a .sgf file
+	  */
+	case MOVE_DOUBLE:
+		if (dt == DT_NORMAL)
+			pchd->pwFrame = CubeAnalysis(pchd);
+		else if (dt == DT_BEAVER)
+			pchd->pwFrame = TakeAnalysis(pchd);
+		break;
+	case MOVE_TAKE:
+	case MOVE_DROP:
+		pchd->pwFrame = TakeAnalysis(pchd);
+		break;
+	default:
+		g_assert(FALSE);
+		break;
 
-  switch ( mt ) {
+	}
 
-  case MOVE_NORMAL:
-  case MOVE_DOUBLE:
+	if (!pchd->pwFrame)
+		return NULL;
 
-    pchd->pwFrame = CubeAnalysis ( GCCCONSTAHACK aarOutput,
-				   GCCCONSTAHACK aarStdDev, pes, 
-                                   &pchd->ms,
-                                   mt == MOVE_DOUBLE );
-    break;
+	gtk_box_pack_start(GTK_BOX(pw), pchd->pwFrame, FALSE, FALSE, 0);
 
-  case MOVE_TAKE:
-  case MOVE_DROP:
+	pwx = gtk_vbox_new(FALSE, 0);
 
-    pchd->pwFrame = TakeAnalysis ( mt, 
-                                   GCCCONSTAHACK aarOutput,
-                                   GCCCONSTAHACK aarStdDev,
-                                   pes,
-                                   &pchd->ms );
-    break;
+	gtk_box_pack_start(GTK_BOX(pwx), pw, FALSE, FALSE, 0);
 
-  default:
+	pwhb = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(pwhb), CreateCubeAnalysisTools(pchd), FALSE,
+			   FALSE, 8);
 
-    assert ( FALSE );
-    break;
+	gtk_box_pack_start(GTK_BOX(pwx), pwhb, FALSE, FALSE, 0);
 
-  }
-
-  if ( ! pchd->pwFrame )
-    return NULL;
-
-  gtk_box_pack_start ( GTK_BOX ( pw ), pchd->pwFrame, FALSE, FALSE, 0 );
-
-
-  pwx = gtk_vbox_new ( FALSE, 0 );
-
-  gtk_box_pack_start ( GTK_BOX ( pwx ), pw, FALSE, FALSE, 0 );
-
-  pwhb = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(pwhb), CreateCubeAnalysisTools ( pchd ), FALSE, FALSE, 8 );
-
-  gtk_box_pack_start ( GTK_BOX ( pwx ), pwhb,
-                     FALSE, FALSE, 0 );
-
-  gtk_object_set_data_full( GTK_OBJECT( pw ), "user_data", 
-                            pchd, free );
-
-
-  return pwx;
+	/* hook to free cubehintdata on widget destruction */
+	g_object_set_data_full(G_OBJECT(pw), "cubehintdata", pchd, g_free);
 
+	return pwx;
 
 }
 
