@@ -4,7 +4,7 @@
  * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,19 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkexport.c,v 1.24 2003/12/29 11:28:33 uid68519 Exp $
+ * $Id: gtkexport.c,v 1.42 2008/03/13 21:28:18 c_anthon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
+#include "config.h"
 #include <gtk/gtk.h>
 
 #include <stdio.h>
+#include <glib/gstdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -37,9 +32,9 @@
 #include "gtkgame.h"
 #include "export.h"
 #include "gtkexport.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "boarddim.h"
-
+#include "gtkwindows.h"
 
 static char *aszInclude[] = {
     N_("Annotations"), 
@@ -55,9 +50,6 @@ static char *aszMovesDisplay[] = {
     N_("Show for moves marked 'bad'"),
     N_("Show for moves marked 'doubtful'"),
     N_("Show for unmarked moves"),
-/*     N_("Show for moves marked 'interesting'"), */
-/*     N_("Show for moves marked 'good'"), */
-/*     N_("Show for moves marked 'very good'")  */
   };
 
 #define NUM_MOVES (sizeof(aszMovesDisplay)/sizeof(aszMovesDisplay[0]))
@@ -67,9 +59,6 @@ static char *aszCubeDisplay[] = {
     N_("Show for cube decisions marked 'bad'"),
     N_("Show for cube decisions marked 'doubtful'"),
     N_("Show for unmarked cube decisions"),
-/*     N_("Show for cube decisions marked 'interesting'"), */
-/*     N_("Show for cube decisions marked 'good'"), */
-/*     N_("Show for cube decisions marked 'very good'"), */
     N_("Show for actual cube decisions"),
     N_("Show for missed doubles"),
     N_("Show for close cube decisions") };
@@ -123,7 +112,7 @@ typedef struct _exportwidget {
 static void
 ExportGetValues ( exportwidget *pew, exportsetup *pexs ) {
 
-  int i;
+  unsigned int i;
 
   /* include */
   
@@ -145,7 +134,7 @@ ExportGetValues ( exportwidget *pew, exportsetup *pexs ) {
 
   /* board */
 
-  pexs->fDisplayBoard = pew->padjDisplayBoard->value;
+  pexs->fDisplayBoard = (int)pew->padjDisplayBoard->value;
 
   pexs->fSide = 0;
   for ( i = 0; i < 2; i++ ) 
@@ -154,7 +143,7 @@ ExportGetValues ( exportwidget *pew, exportsetup *pexs ) {
 
   /* moves */
 
-  pexs->nMoves = pew->padjMoves->value;
+  pexs->nMoves = (int)pew->padjMoves->value;
 
   pexs->fMovesDetailProb = 
     gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( 
@@ -181,24 +170,27 @@ ExportGetValues ( exportwidget *pew, exportsetup *pexs ) {
       gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( 
                                        pew->apwCubeParameters[ i ] ) );
 
-  for ( i = 0; i < NUM_CUBES ; i++ )
-    pexs->afCubeDisplay[ i ] =
-      gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( 
-                                       pew->apwCubeDisplay[ i ] ) );
+  /* skip unused entries */
+  for ( i = 0; i < NUM_CUBES ; i++ ) {	
+    if (aszCubeDisplay[ i ]) {
+      pexs->afCubeDisplay[ i ] =
+	gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( 
+				 pew->apwCubeDisplay[ i ] ) );
+    }
+  }
 
   /* html */
 
   pexs->szHTMLPictureURL = 
-    strdup ( gtk_entry_get_text( GTK_ENTRY( pew->pwHTMLPictureURL ) ) );
+    g_strdup ( gtk_entry_get_text( GTK_ENTRY( pew->pwHTMLPictureURL ) ) );
 
-  pexs->het = gtk_option_menu_get_history (GTK_OPTION_MENU (pew->pwHTMLType));
+  pexs->het = (htmlexporttype)gtk_combo_box_get_active (GTK_COMBO_BOX (pew->pwHTMLType));
 
-  pexs->hecss = 
-    gtk_option_menu_get_history (GTK_OPTION_MENU (pew->pwHTMLCSS));
+  pexs->hecss = (htmlexportcss)gtk_combo_box_get_active (GTK_COMBO_BOX (pew->pwHTMLCSS));
 
   /* sizes */
-  pexs->nPNGSize = pew->adjPNGSize->value;
-  pexs->nHtmlSize = pew->adjHtmlSize->value;
+  pexs->nPNGSize = (int)pew->adjPNGSize->value;
+  pexs->nHtmlSize = (int)pew->adjHtmlSize->value;
 }
 
 #define CHECKVALUE(orig,new,flag,text,format) \
@@ -355,7 +347,7 @@ ExportOK ( GtkWidget *pw, exportwidget *pew ) {
   /* set new values */
 
   SetExportCommands ( pexs, &exsNew );
-  free ( exsNew.szHTMLPictureURL ),
+  g_free ( exsNew.szHTMLPictureURL );
 
   gtk_widget_destroy( gtk_widget_get_toplevel( pw ) );
 
@@ -365,7 +357,7 @@ static void
 ExportSet ( exportwidget *pew ) {
 
   exportsetup *pexs = pew->pexs;
-  int i;
+  unsigned int i;
 
   /* include */
 
@@ -410,17 +402,20 @@ ExportSet ( exportwidget *pew ) {
   /* cube */
   
   gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( pew->pwCubeDetailProb ),
-                                pexs->fMovesDetailProb );
+                                pexs->fCubeDetailProb );
 
   for ( i = 0; i < 2; i++ )
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( 
                                      pew->apwCubeParameters[ i ] ),
                                   pexs->afCubeParameters[ i ] );
 
-  for ( i = 0; i < NUM_CUBES ; i++ )
-    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( 
-                                     pew->apwCubeDisplay[ i ] ),
+  for ( i = 0; i < NUM_CUBES ; i++ ) {
+    if (aszCubeDisplay[i]) {
+      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( 
+				  pew->apwCubeDisplay[ i ] ),
                                   pexs->afCubeDisplay[ i ] );
+    }
+  }
 
   /* html */
 
@@ -428,10 +423,10 @@ ExportSet ( exportwidget *pew ) {
     gtk_entry_set_text( GTK_ENTRY( pew->pwHTMLPictureURL ), 
                         pexs->szHTMLPictureURL );
 
-  gtk_option_menu_set_history ( GTK_OPTION_MENU (pew->pwHTMLType), 
+  gtk_combo_box_set_active ( GTK_COMBO_BOX (pew->pwHTMLType), 
                                 pexs->het );
 
-  gtk_option_menu_set_history ( GTK_OPTION_MENU (pew->pwHTMLCSS), 
+  gtk_combo_box_set_active ( GTK_COMBO_BOX (pew->pwHTMLCSS), 
                                 pexs->hecss );
 
   /* Sizes */
@@ -456,35 +451,104 @@ SizeChanged ( GtkAdjustment *adj, GtkWidget *pwSize ) {
 
 }
 
+static void ExportHTMLImages(void)
+{
+	GtkWidget *fc;
+	gchar *message, *expfolder, *folder, *command;
+	gint ok = FALSE;
+	fc = gtk_file_chooser_dialog_new(_
+					 ("Select top folder for html export"),
+					 NULL,
+					 GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+					 GTK_STOCK_CANCEL,
+					 GTK_RESPONSE_CANCEL,
+					 GTK_STOCK_OPEN,
+					 GTK_RESPONSE_ACCEPT, NULL);
+	gtk_window_set_modal(GTK_WINDOW(fc), TRUE);
+	gtk_window_set_transient_for(GTK_WINDOW(fc), GTK_WINDOW(pwMain));
 
-extern void
-GTKShowExport ( exportsetup *pexs ) {
+	while (!ok) {
+		if (gtk_dialog_run(GTK_DIALOG(fc)) == GTK_RESPONSE_CANCEL) {
+			gtk_widget_destroy(fc);
+			return;
+		}
+		folder = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fc));
+		if (folder) {
+			char *name = g_path_get_basename(folder);
+			if (!StrCaseCmp(name, "html-images"))
+				expfolder = g_strdup(folder);
+			else
+				expfolder = g_build_filename(folder, "html-images", NULL);
+			g_free(name);
+			if (g_file_test(expfolder, G_FILE_TEST_IS_DIR)) {
+				message =
+				    g_strdup_printf(_
+						    ("Folder html-images exists\nin %s\nOK to overwrite images?"),
+						    folder);
+				ok = GTKGetInputYN(message);
+				g_free(message);
+			}
+			else if (g_mkdir(expfolder, 0777) == 0)
+			{
+				ok = TRUE;
+			}
+			else
+			{
+				message =
+				    g_strdup_printf(_
+						    ("Folder html-images can't be created\nin %s"),
+						    folder);
+				GTKMessage(message, DT_ERROR);
+				g_free(message);
+			}
+			if (ok) {
+				command =
+				    g_strconcat("export htmlimages \"",
+						expfolder, "\"", NULL);
+				UserCommand(command);
+				g_free(command);
+			}
+			g_free(expfolder);
+			g_free(folder);
+		}
+	}
+	gtk_widget_destroy(fc);
+}
 
+static void GenHtmlImages(GtkWidget *widget, gpointer data)
+{	/* Temporariarly set html size and create images */
+	int temp = exsExport.nHtmlSize;
+	exsExport.nHtmlSize = (int)gtk_adjustment_get_value(GTK_ADJUSTMENT(data));
+	ExportHTMLImages();
+	exsExport.nHtmlSize = temp;
+}
+
+extern void GTKShowExport ( exportsetup *pexs )
+{
   GtkWidget *pwDialog;
 
   GtkWidget *pwVBox;
   GtkWidget *pwFrame;
   GtkWidget *pwTable;
   GtkWidget *pwTableX;
-  GtkWidget *pwType_menu;
-  GtkWidget *glade_menuitem;
   GtkWidget *pwHBox;
   GtkWidget *pwHScale;
+  GtkWidget *genHtml;
   
   GtkWidget *pw;
 
-  int i;
+  unsigned int i;
 
   exportwidget *pew;
 
-  pew = malloc ( sizeof ( exportwidget ) );
+  pew = (exportwidget*)malloc ( sizeof ( exportwidget ) );
 
   pew->pexs = pexs;
 
   /* create dialog */
 
-  pwDialog = GTKCreateDialog ( _("GNU Backgammon - Export Settings"),
-			       DT_QUESTION, GTK_SIGNAL_FUNC ( ExportOK ), pew );
+  pwDialog = GTKCreateDialog ( _("GNU Backgammon - Export Settings"), DT_QUESTION,
+	  NULL, DIALOG_FLAG_MODAL, G_CALLBACK ( ExportOK ), pew );
 
   pwTable = gtk_table_new ( 3, 2, FALSE );
   gtk_container_add ( GTK_CONTAINER ( DialogArea ( pwDialog, DA_MAIN ) ),
@@ -676,14 +740,16 @@ GTKShowExport ( exportsetup *pexs ) {
                        TRUE, TRUE, 0 );
 
 
-  for ( i = 0; i < NUM_CUBES ; i++ )
-
-    gtk_box_pack_start ( GTK_BOX ( pwVBox ),
-                         pew->apwCubeDisplay[ i ] =
-                         gtk_check_button_new_with_label ( 
-                            gettext ( aszCubeDisplay[ i ] ) ), 
+  for ( i = 0; i < NUM_CUBES ; i++ ) {
+    if (aszCubeDisplay[i]) {
+      gtk_box_pack_start ( GTK_BOX ( pwVBox ),
+			   pew->apwCubeDisplay[ i ] =
+			   gtk_check_button_new_with_label ( 
+			       gettext ( aszCubeDisplay[ i ] ) ), 
                          TRUE, TRUE, 0 );
-                    
+    }
+  }
+                 
 
   /* html */
 
@@ -716,20 +782,12 @@ GTKShowExport ( exportsetup *pexs ) {
                        gtk_label_new ( _("HTML board type:") ),
                        TRUE, TRUE, 0 );
   
-  pew->pwHTMLType = gtk_option_menu_new ();
+  pew->pwHTMLType = gtk_combo_box_new_text ();
   gtk_box_pack_start (GTK_BOX (pwHBox), pew->pwHTMLType, FALSE, FALSE, 0);
-  pwType_menu = gtk_menu_new ();
-  glade_menuitem = gtk_menu_item_new_with_label (_("GNU Backgammon"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (pwType_menu), glade_menuitem);
-  glade_menuitem = gtk_menu_item_new_with_label (_("BBS"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (pwType_menu), glade_menuitem);
-  glade_menuitem = gtk_menu_item_new_with_label (_("fibs2html"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (pwType_menu), glade_menuitem);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (pew->pwHTMLType), pwType_menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (pew->pwHTMLType), 0);
+  gtk_combo_box_append_text(GTK_COMBO_BOX(pew->pwHTMLType), _("GNU Backgammon"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(pew->pwHTMLType), _("BBS"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(pew->pwHTMLType), _("fibs2html"));
+  gtk_combo_box_set_active(GTK_COMBO_BOX (pew->pwHTMLType), 0);
 
   gtk_container_set_border_width (GTK_CONTAINER (pwHBox), 4);
   gtk_box_pack_start (GTK_BOX (pwVBox), pwHBox, FALSE, FALSE, 0);
@@ -741,20 +799,12 @@ GTKShowExport ( exportsetup *pexs ) {
                        gtk_label_new ( _("CSS Style sheet:") ),
                        TRUE, TRUE, 0 );
   
-  pew->pwHTMLCSS = gtk_option_menu_new ();
+  pew->pwHTMLCSS = gtk_combo_box_new_text();
   gtk_box_pack_start (GTK_BOX (pwHBox), pew->pwHTMLCSS, FALSE, FALSE, 0);
-  pwType_menu = gtk_menu_new ();
-  glade_menuitem = gtk_menu_item_new_with_label (_("In <head>"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (pwType_menu), glade_menuitem);
-  glade_menuitem = gtk_menu_item_new_with_label (_("Inline (in tags)"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (pwType_menu), glade_menuitem);
-  glade_menuitem = gtk_menu_item_new_with_label (_("External file"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (pwType_menu), glade_menuitem);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (pew->pwHTMLCSS), pwType_menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (pew->pwHTMLCSS), 0);
+  gtk_combo_box_append_text(GTK_COMBO_BOX(pew->pwHTMLCSS), _("In <head>"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(pew->pwHTMLCSS), _("Inline (in tags)"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(pew->pwHTMLCSS), _("External file"));
+  gtk_combo_box_set_active(GTK_COMBO_BOX (pew->pwHTMLCSS), 0);
 
   gtk_container_set_border_width (GTK_CONTAINER (pwHBox), 4);
   gtk_box_pack_start (GTK_BOX (pwVBox), pwHBox, FALSE, FALSE, 0);
@@ -793,8 +843,8 @@ GTKShowExport ( exportsetup *pexs ) {
 
   gtk_box_pack_start (GTK_BOX (pwVBox), pwHScale, FALSE, FALSE, 0);
 
-  gtk_signal_connect ( GTK_OBJECT ( pew->adjPNGSize ), "value-changed",
-                       GTK_SIGNAL_FUNC ( SizeChanged ), pew->pwPNGSize );
+  g_signal_connect( G_OBJECT ( pew->adjPNGSize ), "value-changed",
+                       G_CALLBACK ( SizeChanged ), pew->pwPNGSize );
 
   /* Html size */
   pwHBox = gtk_hbox_new ( FALSE, 0 );
@@ -814,25 +864,18 @@ GTKShowExport ( exportsetup *pexs ) {
   gtk_scale_set_digits( GTK_SCALE( pwHScale ), 0 );
   gtk_box_pack_start (GTK_BOX (pwVBox), pwHScale, FALSE, FALSE, 0);
 
-  gtk_signal_connect ( GTK_OBJECT ( pew->adjHtmlSize ), "value-changed",
-                       GTK_SIGNAL_FUNC ( SizeChanged ), pew->pwHtmlSize );
+  genHtml = gtk_button_new_with_label(_("Generate Html images..."));
+  pwHBox = gtk_hbox_new ( FALSE, 0 );
+  gtk_box_pack_start ( GTK_BOX ( pwHBox ), genHtml, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (pwVBox), pwHBox, FALSE, FALSE, 0);
+
+  g_signal_connect( G_OBJECT( genHtml ), "clicked", G_CALLBACK( GenHtmlImages ), pew->adjHtmlSize );
+  g_signal_connect( G_OBJECT ( pew->adjHtmlSize ), "value-changed",
+                       G_CALLBACK ( SizeChanged ), pew->pwHtmlSize );
 
   /* show dialog */
 
-  gtk_window_set_modal( GTK_WINDOW( pwDialog ), TRUE );
-  gtk_window_set_transient_for( GTK_WINDOW( pwDialog ),
-                                GTK_WINDOW( pwMain ) );
-  gtk_signal_connect( GTK_OBJECT( pwDialog ), "destroy",
-                      GTK_SIGNAL_FUNC( gtk_main_quit ), NULL );
-
-  gtk_widget_show_all( pwDialog );
-
   ExportSet ( pew );
 
-  GTKDisallowStdin();
-  gtk_main();
-  GTKAllowStdin();
-
+  GTKRunDialog(pwDialog);
 }
-
-
