@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@freeshell.org>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,27 +16,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkmet.c,v 1.13 2006/09/11 22:59:40 Superfly_Jon Exp $
+ * $Id: gtkmet.c,v 1.26 2009/05/01 09:28:34 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-#include <gtk/gtk.h>
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
-
-#include "backgammon.h"
-#include "eval.h"
 #include "gtkgame.h"
-#include <glib/gi18n.h>
 #include "matchequity.h"
 #include "gtkmet.h"
 #include "gtkwindows.h"
@@ -52,20 +37,20 @@ typedef struct _mettable {
 typedef struct _metwidget {
   GtkWidget *apwPostCrawford[ 2 ];
   GtkWidget *pwPreCrawford;
-  int nMatchTo;
-  int anAway[ 2 ];
+  unsigned int nMatchTo;
+  unsigned int anAway[ 2 ];
 } metwidget;
 
 
-static void
-UpdateTable ( mettable *pmt, 
-              float aafMET[ MAXSCORE ][ MAXSCORE ],
-              metinfo *pmi,
-              const int nRows, const int nCols, const int fInvert ) {
+static void UpdateTable ( const mettable *pmt, 
+              /*lint -e{818}*/float met[ MAXSCORE ][ MAXSCORE ],
+              const metinfo *pmi,
+              const unsigned int nRows, const unsigned int nCols, const int fInvert ) {
 
 #define STRINGLENGTH 64
 
-  int i, j, nBytes;
+  unsigned int i, j;
+  int nBytes;
   char sz[ STRINGLENGTH ];
 
   /* set labels */
@@ -80,11 +65,11 @@ UpdateTable ( mettable *pmt,
     for( j = 0; j < nCols; j++ ) {
 
       if ( fInvert )
-        nBytes = sprintf( sz, "%8.4f", GET_MET( j, i, aafMET ) * 100.0f );
+        nBytes = sprintf( sz, "%8.4f", met[j][i] * 100.0f );
       else
-        nBytes = sprintf( sz, "%8.4f", GET_MET( i, j, aafMET ) * 100.0f );
+        nBytes = sprintf( sz, "%8.4f", met[i][j] * 100.0f );
 
-      assert( nBytes < STRINGLENGTH );
+      g_assert( nBytes < STRINGLENGTH );
 
       gtk_label_set_text ( GTK_LABEL ( pmt->aapwLabel[ i ][ j ] ), sz );
 
@@ -94,31 +79,25 @@ UpdateTable ( mettable *pmt,
 }
 
 
-static void
-UpdateAllTables ( metwidget *pmw ) {
-
-  mettable *pmt;
+static void UpdateAllTables(const metwidget *pmw)
+{
+  const mettable *pmt;
   int i;
 
-  pmt = gtk_object_get_user_data ( GTK_OBJECT ( pmw->pwPreCrawford ) );
+  pmt = (const mettable*)g_object_get_data ( G_OBJECT ( pmw->pwPreCrawford ),
+		  "mettable" );
   UpdateTable ( pmt, aafMET, &miCurrent, pmw->nMatchTo, pmw->nMatchTo, FALSE );
 
   for ( i = 0; i < 2; ++i ) {
-    pmt = 
-      gtk_object_get_user_data ( GTK_OBJECT ( pmw->apwPostCrawford[ i ] ) );
-    UpdateTable ( pmt, (float (*)[ MAXSCORE ]) aafMETPostCrawford[ i ], 
+    pmt = (const mettable*)g_object_get_data ( G_OBJECT ( pmw->apwPostCrawford[ i ] ), "mettable" );
+    UpdateTable ( pmt, (float (*)[ MAXSCORE ])(void*)aafMETPostCrawford[ i ], 
                   &miCurrent, pmw->nMatchTo, 1, TRUE );
   }
-
-
 }
 
-
-static GtkWidget 
-*GTKWriteMET ( const int nRows, const int nCols,
-               const int nAway0, const int nAway1 ) {
-
-  int i, j;
+static GtkWidget *GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned int nAway0, const unsigned int nAway1)
+{
+  unsigned int i, j;
   char sz[ 16 ];
   GtkWidget *pwScrolledWindow = gtk_scrolled_window_new( NULL, NULL );
   GtkWidget *pwTable = gtk_table_new( nRows + 1, nCols + 1, TRUE );
@@ -199,13 +178,13 @@ static GtkWidget
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC );
 
-  gtk_object_set_data_full ( GTK_OBJECT ( pwBox ), "user_data", pmt, g_free );
+  g_object_set_data_full ( G_OBJECT ( pwBox ), "mettable", pmt, g_free );
 
   return pwBox;
 
 }
 
-static void invertMETlocal( GtkWidget *pw, metwidget *pmw ){
+static void invertMETlocal( GtkWidget *UNUSED(widget), const metwidget *pmw ){
 
   if(fInvertMET)
     UserCommand( "set invert met off" );
@@ -213,11 +192,10 @@ static void invertMETlocal( GtkWidget *pw, metwidget *pmw ){
     UserCommand( "set invert met on" );
 
   UpdateAllTables ( pmw );
-
 }
 
 
-static void loadMET ( GtkWidget *pw, metwidget *pmw ) {
+static void loadMET ( GtkWidget *UNUSED(widget), const metwidget *pmw ) {
 
   SetMET ( NULL, NULL );
 
@@ -226,9 +204,8 @@ static void loadMET ( GtkWidget *pw, metwidget *pmw ) {
 }
 
 
-extern void GTKShowMatchEquityTable( const int nMatchTo,
-                                     const int anScore[ 2 ] ) {
-
+extern void GTKShowMatchEquityTable( const unsigned int nMatchTo, const int anScore[ 2 ] )
+{
   /* FIXME: Widget should update after 'Invert' or 'Load ...' */  
   int i;
   char sz[ 50 ];
@@ -242,8 +219,8 @@ extern void GTKShowMatchEquityTable( const int nMatchTo,
   metwidget mw;
 
   mw.nMatchTo = nMatchTo;
-  mw.anAway[ 0 ] = nMatchTo - anScore[ 0 ] - 1;
-  mw.anAway[ 1 ] = nMatchTo - anScore[ 1 ] - 1;
+  mw.anAway[ 0 ] = (nMatchTo - (unsigned)anScore[ 0 ]) - 1;
+  mw.anAway[ 1 ] = (nMatchTo - (unsigned)anScore[ 1 ]) - 1;
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pwInvertButton),
                                fInvertMET); 
@@ -257,8 +234,7 @@ extern void GTKShowMatchEquityTable( const int nMatchTo,
   gtk_container_add( GTK_CONTAINER( DialogArea( pwDialog, DA_BUTTONS ) ),
                      pwLoad );
 
-  mw.pwPreCrawford = GTKWriteMET ( mw.nMatchTo, mw.nMatchTo,
-                                   mw.anAway[ 0 ], mw.anAway[ 1 ] );
+  mw.pwPreCrawford = GTKWriteMET ( (unsigned)mw.nMatchTo, (unsigned)mw.nMatchTo, mw.anAway[ 0 ], mw.anAway[ 1 ] );
   gtk_notebook_append_page ( GTK_NOTEBOOK ( pwNotebook ),
                              mw.pwPreCrawford, 
                              gtk_label_new ( _("Pre-Crawford") ) );
@@ -267,24 +243,19 @@ extern void GTKShowMatchEquityTable( const int nMatchTo,
       
     sprintf ( sz, _("Post-Crawford for player %s"), ap[ i ].szName );
 
-    mw.apwPostCrawford[ i ] = GTKWriteMET ( nMatchTo , 1, 
-                                            mw.anAway[ i ], mw.anAway[ !i ] );
+    mw.apwPostCrawford[ i ] = GTKWriteMET ( nMatchTo , 1, mw.anAway[ i ], mw.anAway[ !i ] );
     gtk_notebook_append_page ( GTK_NOTEBOOK ( pwNotebook ),
                                mw.apwPostCrawford[ i ],
                                gtk_label_new ( sz ) );
   }
 
   gtk_window_set_default_size( GTK_WINDOW( pwDialog ), 500, 300 );
-  gtk_signal_connect( GTK_OBJECT( pwInvertButton ), "toggled",
-                      GTK_SIGNAL_FUNC( invertMETlocal ), &mw );
-  gtk_signal_connect( GTK_OBJECT( pwLoad ), "clicked",
-                      GTK_SIGNAL_FUNC ( loadMET ), &mw );
+  g_signal_connect( G_OBJECT( pwInvertButton ), "toggled",
+                      G_CALLBACK( invertMETlocal ), &mw );
+  g_signal_connect( G_OBJECT( pwLoad ), "clicked",
+                      G_CALLBACK ( loadMET ), &mw );
 
   UpdateAllTables ( &mw );
     
-  gtk_widget_show_all( pwDialog );
-
-  GTKDisallowStdin();
-  gtk_main();
-  GTKAllowStdin();
+  GTKRunDialog(pwDialog);
 }
