@@ -4,7 +4,7 @@
  * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,27 +16,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkmovefilter.c,v 1.13 2006/09/21 22:24:05 Superfly_Jon Exp $
+ * $Id: gtkmovefilter.c,v 1.26 2009/05/01 09:28:34 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-#include <gtk/gtk.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "backgammon.h"
-#include "eval.h"
 #include "gtkgame.h"
-#include <glib/gi18n.h>
-#include "matchequity.h"
 #include "gtkmovefilter.h"
 #include "gtkwindows.h"
 
@@ -51,7 +40,7 @@ typedef struct _movefilterwidget {
 
   /* callback for the parent */
 
-  GtkSignalFunc pfChanged;
+  GCallback pfChanged;
   gpointer userdata;
 
 } movefilterwidget;
@@ -76,7 +65,7 @@ typedef struct _movefiltersetupwidget {
 
 static void
 MoveFilterSetupSetValues ( movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ],
-                           movefiltersetupwidget *pmfsw ) {
+                           const movefiltersetupwidget *pmfsw ) {
 
   int i, j;
   int f;
@@ -111,7 +100,7 @@ MoveFilterSetupSetValues ( movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES
 
 
 static void
-MoveFilterSetupGetValues ( movefilter *pmf, movefiltersetupwidget *pmfsw ) {
+MoveFilterSetupGetValues ( movefilter *pmf, const movefiltersetupwidget *pmfsw ) {
 
   int i, j;
   movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ];
@@ -139,10 +128,10 @@ MoveFilterSetupGetValues ( movefilter *pmf, movefiltersetupwidget *pmfsw ) {
 
 
 static void
-AcceptChanged ( GtkWidget *pw, movefiltersetupwidget *pmfsw ) {
+AcceptChanged ( GtkWidget *UNUSED(pw), movefiltersetupwidget *pmfsw ) {
 
   int fFound;
-  int i;
+  unsigned int i;
   movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ];
 
   /* see if current settings match a predefined one */
@@ -150,13 +139,12 @@ AcceptChanged ( GtkWidget *pw, movefiltersetupwidget *pmfsw ) {
   fFound = FALSE;
 
   
-  MoveFilterSetupGetValues ( (movefilter *) aamf, pmfsw );
+  MoveFilterSetupGetValues ( aamf[0], pmfsw );
 
   for ( i = 0; i < NUM_MOVEFILTER_SETTINGS; ++i ) 
     if ( equal_movefilters ( aamf, 
                              aaamfMoveFilterSettings[ i ] ) ) {
-      gtk_option_menu_set_history ( GTK_OPTION_MENU ( pmfsw->pwOptionMenu ),
-                                    i );
+      gtk_option_menu_set_history ( GTK_OPTION_MENU ( pmfsw->pwOptionMenu ), i );
       fFound = TRUE;
       break;
     }
@@ -169,13 +157,14 @@ AcceptChanged ( GtkWidget *pw, movefiltersetupwidget *pmfsw ) {
 
 
 static void
-EnableToggled ( GtkWidget *pw, movefiltersetupwidget *pmfsw ) {
-
+EnableToggled ( GtkWidget *pw, movefiltersetupwidget *pmfsw )
+{
   int f;
   int i, j;
 
   for ( i = 0; i < MAX_FILTER_PLIES; ++i ) 
-    for ( j = 0; j <= i; ++j ) {
+    for ( j = 0; j <= i; ++j )
+	{
 
       f = gtk_toggle_button_get_active ( 
              GTK_TOGGLE_BUTTON ( pmfsw->aapwEnable[ i ][ j ] ) );
@@ -183,10 +172,9 @@ EnableToggled ( GtkWidget *pw, movefiltersetupwidget *pmfsw ) {
       gtk_widget_set_sensitive ( GTK_WIDGET ( pmfsw->aapwA[ i ][ j ] ), f );
       gtk_widget_set_sensitive ( GTK_WIDGET ( pmfsw->aapwET[ i ][ j ] ), f );
       gtk_widget_set_sensitive ( GTK_WIDGET ( pmfsw->aapwT[ i ][ j ] ), f );
-  }
+    }
 
   AcceptChanged ( pw, pmfsw );
-
 }
 
 
@@ -194,11 +182,9 @@ EnableToggled ( GtkWidget *pw, movefiltersetupwidget *pmfsw ) {
 
 static void
 SetupSettingsMenuActivate ( GtkWidget *pwItem,
-                            movefiltersetupwidget *pfmsw ) {
+                            const movefiltersetupwidget *pfmsw ) {
 
-  int *piSelected;
-  
-  piSelected = gtk_object_get_data ( GTK_OBJECT ( pwItem ), "user_data" );
+  int *piSelected = (int*)g_object_get_data ( G_OBJECT ( pwItem ), "user_data" );
 
   if ( *piSelected == NUM_MOVEFILTER_SETTINGS )
     return; /* user defined */
@@ -213,7 +199,7 @@ SetupSettingsMenuActivate ( GtkWidget *pwItem,
 
 static GtkWidget *
 MoveFilterPage ( const int i, const int j,
-                 movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ],
+                 movefilter UNUSED(aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ]),
                  movefiltersetupwidget *pmfsw ) {
 
   GtkWidget *pwPage;
@@ -229,9 +215,9 @@ MoveFilterPage ( const int i, const int j,
   gtk_box_pack_start ( GTK_BOX ( pwPage ), pmfsw->aapwEnable[ i ][ j ], 
                        FALSE, FALSE, 0 );
 
-  gtk_signal_connect ( GTK_OBJECT ( pmfsw->aapwEnable[ i ][ j ] ),
+  g_signal_connect( G_OBJECT ( pmfsw->aapwEnable[ i ][ j ] ),
                        "toggled", 
-                       GTK_SIGNAL_FUNC ( EnableToggled ), pmfsw );
+                       G_CALLBACK ( EnableToggled ), pmfsw );
 
   /* accept */
 
@@ -242,7 +228,6 @@ MoveFilterPage ( const int i, const int j,
   gtk_box_pack_start ( GTK_BOX ( pwhbox ),
                        gtk_label_new ( _("Always accept: ") ),
                        FALSE, FALSE, 0 );
-
   pmfsw->aapadjAccept[ i ][ j ] = 
     GTK_ADJUSTMENT ( gtk_adjustment_new ( 0, 0, 1000,
                                           1, 5, 5 ) );
@@ -258,9 +243,9 @@ MoveFilterPage ( const int i, const int j,
                        gtk_label_new ( _("moves.") ),
                        FALSE, FALSE, 0 );
 
-  gtk_signal_connect( GTK_OBJECT( pmfsw->aapadjAccept[ i ][ j ] ), 
+  g_signal_connect( G_OBJECT( pmfsw->aapadjAccept[ i ][ j ] ), 
                       "value-changed",
-                      GTK_SIGNAL_FUNC( AcceptChanged ), pmfsw );
+                      G_CALLBACK( AcceptChanged ), pmfsw );
 
   /* extra */
 
@@ -283,9 +268,9 @@ MoveFilterPage ( const int i, const int j,
 
   gtk_box_pack_start ( GTK_BOX ( pwhbox ), pw, FALSE, FALSE, 0 );
 
-  gtk_signal_connect( GTK_OBJECT( pmfsw->aapadjExtra[ i ][ j ] ), 
+  g_signal_connect( G_OBJECT( pmfsw->aapadjExtra[ i ][ j ] ), 
                       "value-changed",
-                      GTK_SIGNAL_FUNC( AcceptChanged ), pmfsw );
+                      G_CALLBACK( AcceptChanged ), pmfsw );
 
   /* threshold */
 
@@ -306,9 +291,9 @@ MoveFilterPage ( const int i, const int j,
 
   gtk_box_pack_start ( GTK_BOX ( pwhbox ), pw, TRUE, TRUE, 0 );
 
-  gtk_signal_connect( GTK_OBJECT( pmfsw->aapadjThreshold[ i ][ j ] ), 
+  g_signal_connect( G_OBJECT( pmfsw->aapadjThreshold[ i ][ j ] ), 
                       "value-changed",
-                      GTK_SIGNAL_FUNC( AcceptChanged ), pmfsw );
+                      G_CALLBACK( AcceptChanged ), pmfsw );
 
   
 
@@ -352,19 +337,19 @@ MoveFilterSetup ( movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ],
     if ( i < NUM_MOVEFILTER_SETTINGS )
       gtk_menu_append ( GTK_MENU ( pwMenu ),
                         pwItem = gtk_menu_item_new_with_label ( 
-                                    gettext ( aszMoveFilterSettings[ i ] ) ) );
+                                    Q_ ( aszMoveFilterSettings[ i ] ) ) );
     else
       gtk_menu_append ( GTK_MENU ( pwMenu ),
                         pwItem = gtk_menu_item_new_with_label (
                                     _("user defined") ) );
 
-    pi = g_malloc ( sizeof ( int ) );
+    pi = (int*)g_malloc ( sizeof ( int ) );
     *pi = i;
-    gtk_object_set_data_full( GTK_OBJECT( pwItem ), "user_data", 
+    g_object_set_data_full( G_OBJECT( pwItem ), "user_data", 
                               pi, g_free );
 
-    gtk_signal_connect ( GTK_OBJECT ( pwItem ), "activate",
-                         GTK_SIGNAL_FUNC ( SetupSettingsMenuActivate ),
+    g_signal_connect( G_OBJECT ( pwItem ), "activate",
+                         G_CALLBACK ( SetupSettingsMenuActivate ),
                          (void *) pmfsw );
 
     }
@@ -404,11 +389,11 @@ MoveFilterSetup ( movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ],
 
   }
 
-  gtk_object_set_data_full( GTK_OBJECT( pwSetup ), "user_data", 
+  g_object_set_data_full( G_OBJECT( pwSetup ), "user_data", 
                             pmfsw, g_free );
 
   pmfsw->pfOK = pfOK;
-  pmfsw->pmf = (movefilter *) aamf;
+  pmfsw->pmf = aamf[0];
 
   MoveFilterSetupSetValues ( aamf, pmfsw );
 
@@ -420,8 +405,8 @@ MoveFilterSetup ( movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ],
 static void
 MoveFilterSetupOK ( GtkWidget *pw, GtkWidget *pwMoveFilterSetup ) {
 
-  movefiltersetupwidget *pmfsw = 
-    gtk_object_get_user_data ( GTK_OBJECT ( pwMoveFilterSetup ) );
+  movefiltersetupwidget *pmfsw = (movefiltersetupwidget *)
+    g_object_get_data ( G_OBJECT ( pwMoveFilterSetup ), "user_data" );
 
                                
   if ( pmfsw->pfOK )
@@ -438,9 +423,9 @@ MoveFilterSetupOK ( GtkWidget *pw, GtkWidget *pwMoveFilterSetup ) {
 typedef void (*changed) ( GtkWidget *pw, gpointer p );
 
 static void
-MoveFilterChanged ( movefilterwidget *pmfw ) {
+MoveFilterChanged ( const movefilterwidget *pmfw ) {
 
-  int i;
+  unsigned int i;
   int fFound = FALSE;
   movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ];
   
@@ -472,9 +457,7 @@ static void
 SettingsMenuActivate ( GtkWidget *pwItem,
                        movefilterwidget *pmfw ) {
 
-  int *piSelected;
-  
-  piSelected = gtk_object_get_data ( GTK_OBJECT ( pwItem ), "user_data" );
+  int *piSelected = (int*)g_object_get_data ( G_OBJECT ( pwItem ), "user_data" );
 
   if ( *piSelected == NUM_MOVEFILTER_SETTINGS )
     return; /* user defined */
@@ -488,7 +471,7 @@ SettingsMenuActivate ( GtkWidget *pwItem,
 
 
 static void
-ClickButton ( GtkWidget *pw, movefilterwidget *pmfw ) {
+ModifyClickButton ( GtkWidget *pw, movefilterwidget *pmfw ) {
 
   int fOK;
   GtkWidget *pwDialog;
@@ -500,16 +483,12 @@ ClickButton ( GtkWidget *pw, movefilterwidget *pmfw ) {
 
   pwDialog = GTKCreateDialog( _("GNU Backgammon - Move filter setup"), 
 			DT_QUESTION, pw, DIALOG_FLAG_MODAL,
-            GTK_SIGNAL_FUNC( MoveFilterSetupOK ), pwMoveFilterSetup );
+            G_CALLBACK( MoveFilterSetupOK ), pwMoveFilterSetup );
 
   gtk_container_add( GTK_CONTAINER( DialogArea( pwDialog, DA_MAIN ) ),
                      pwMoveFilterSetup );
   
-  gtk_widget_show_all( pwDialog );
-  
-  GTKDisallowStdin();
-  gtk_main();
-  GTKAllowStdin();
+  GTKRunDialog(pwDialog);
 
   if( fOK ) {
     memcpy ( pmfw->pmf, aamf, sizeof ( aamf ) );
@@ -520,8 +499,8 @@ ClickButton ( GtkWidget *pw, movefilterwidget *pmfw ) {
 
 
 extern GtkWidget *
-MoveFilterWidget ( movefilter *pmf, int *pfOK,
-                   GtkSignalFunc pfChanged, gpointer userdata ) {
+MoveFilterWidget ( movefilter *pmf, int *UNUSED(pfOK),
+                   GCallback pfChanged, gpointer userdata ) {
 
   GtkWidget *pwFrame;
   movefilterwidget *pmfw;
@@ -550,19 +529,19 @@ MoveFilterWidget ( movefilter *pmf, int *pfOK,
     if ( i < NUM_MOVEFILTER_SETTINGS )
       gtk_menu_append ( GTK_MENU ( pwMenu ),
                         pwItem = gtk_menu_item_new_with_label ( 
-                                    gettext ( aszMoveFilterSettings[ i ] ) ) );
+                                    Q_( aszMoveFilterSettings[ i ] ) ) );
     else
       gtk_menu_append ( GTK_MENU ( pwMenu ),
                         pwItem = gtk_menu_item_new_with_label (
                                     _("user defined") ) );
 
-    pi = g_malloc ( sizeof ( int ) );
+    pi = (int*)g_malloc ( sizeof ( int ) );
     *pi = i;
-    gtk_object_set_data_full( GTK_OBJECT( pwItem ), "user_data", 
+    g_object_set_data_full( G_OBJECT( pwItem ), "user_data", 
                               pi, g_free );
 
-    gtk_signal_connect ( GTK_OBJECT ( pwItem ), "activate",
-                         GTK_SIGNAL_FUNC ( SettingsMenuActivate ),
+    g_signal_connect( G_OBJECT ( pwItem ), "activate",
+                         G_CALLBACK ( SettingsMenuActivate ),
                          (void *) pmfw );
 
     }
@@ -578,12 +557,12 @@ MoveFilterWidget ( movefilter *pmf, int *pfOK,
   gtk_box_pack_end ( GTK_BOX ( pw ), pwButton, FALSE, FALSE, 0 );
 
   
-  gtk_signal_connect ( GTK_OBJECT ( pwButton ), "clicked",
-                       GTK_SIGNAL_FUNC ( ClickButton ), pmfw );
+  g_signal_connect( G_OBJECT ( pwButton ), "clicked",
+                       G_CALLBACK ( ModifyClickButton ), pmfw );
 
   /* save movefilterwidget */
 
-  gtk_object_set_data_full( GTK_OBJECT( pwFrame ), "user_data", pmfw, g_free );
+  g_object_set_data_full( G_OBJECT( pwFrame ), "user_data", pmfw, g_free );
 
   MoveFilterChanged ( pmfw );
 
@@ -607,15 +586,13 @@ SetMovefilterCommands ( const char *sz,
 
   for ( i = 0; i < MAX_FILTER_PLIES; ++i )
     for ( j = 0; j <= i; ++j ) {
-      if ( aamfNew[ i ][ j ].Accept != aamfOld[ i ][ j ].Accept ||
-           aamfNew[ i ][ j ].Extra != aamfOld[ i ][ j ].Extra ||
-           aamfNew[ i ][ j ].Threshold != aamfOld[ i ][ j ].Threshold ) {
+      if (memcmp(&aamfNew[i][j], &aamfOld[i][j], sizeof(movefilter)) != 0)
+	  {
         szCmd = g_strdup_printf ( "%s %d %d %d %d %s",
                                   sz, i + 1, j,
                                   aamfNew[ i ][ j ].Accept,
                                   aamfNew[ i ][ j ].Extra,
-                                  g_ascii_formatd( buf, G_ASCII_DTOSTR_BUF_SIZE,
-			           "%0.3f", aamfNew[ i ][ j ].Threshold ));
+                                  g_ascii_formatd( buf, G_ASCII_DTOSTR_BUF_SIZE, "%0.3f", (double)aamfNew[ i ][ j ].Threshold ));
         UserCommand ( szCmd );
         g_free ( szCmd );
       }
@@ -624,21 +601,13 @@ SetMovefilterCommands ( const char *sz,
 
 }
 
-extern void
-MoveFilterOK ( GtkWidget *pw, GtkWidget *pwMoveFilter ) {
-
-
-
-}
-
-
 extern void
 MoveFilterSetPredefined ( GtkWidget *pwMoveFilter,
                           const int i ) {
 
 
-  movefilterwidget *pmfw = 
-    gtk_object_get_user_data ( GTK_OBJECT ( pwMoveFilter ) );
+  movefilterwidget *pmfw = (movefilterwidget *)
+    g_object_get_data ( G_OBJECT ( pwMoveFilter ), "user_data" );
 
   if ( i < 0 )
     return;
@@ -646,8 +615,7 @@ MoveFilterSetPredefined ( GtkWidget *pwMoveFilter,
   memcpy ( pmfw->pmf, aaamfMoveFilterSettings[ i ],
            MAX_FILTER_PLIES * MAX_FILTER_PLIES * sizeof ( movefilter ) );
 
-  gtk_option_menu_set_history ( GTK_OPTION_MENU ( pmfw->pwOptionMenu ),
-                                i );
+  gtk_option_menu_set_history ( GTK_OPTION_MENU ( pmfw->pwOptionMenu ), (unsigned)i );
 
   MoveFilterChanged ( pmfw );
 
