@@ -5,7 +5,7 @@
  * Analysis move list
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,10 +17,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkmovelist.c,v 1.10 2006/06/26 19:00:44 c_anthon Exp $
+ * $Id: gtkmovelist.c,v 1.21 2008/01/15 18:08:51 c_anthon Exp $
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <gtk/gtk.h>
 #include <string.h>
@@ -34,15 +34,6 @@
 #include "gtkgame.h"
 #include "drawboard.h"
 
-extern void HintDoubleClick(GtkTreeView        *treeview,
-                       GtkTreePath        *path,
-                       GtkTreeViewColumn  *col,
-                       hintdata *phd);
-extern void HintSelect(GtkTreeSelection *selection, hintdata *phd);
-
-extern GdkColor wlCol;
-void ShowMove ( hintdata *phd, const int f );
-
 #define DETAIL_COLUMN_COUNT 11
 #define MIN_COLUMN_COUNT 5
 
@@ -65,7 +56,7 @@ enum
 
 void MoveListUpdate ( const hintdata *phd );
 
-void MoveListCreate(hintdata *phd)
+extern void MoveListCreate(hintdata *phd)
 {
     static char *aszTitleDetails[] = {
 	N_("Rank"), 
@@ -80,7 +71,7 @@ void MoveListCreate(hintdata *phd)
         N_("Diff."), 
         N_("Move")
     };
-	int i;
+	unsigned int i;
 	int showWLTree = showMoveListDetail && !phd->fDetails;
 
 /* Create list widget */
@@ -153,14 +144,12 @@ float rBest;
 
 GtkStyle *psHighlight = NULL;
 
-extern GtkWidget *pwMoveAnalysis;
-
-void MoveListRefreshSize()
+extern void MoveListRefreshSize(void)
 {
 	custom_cell_renderer_invalidate_size();
 	if (pwMoveAnalysis)
 	{
-		hintdata *phd = gtk_object_get_user_data(GTK_OBJECT(pwMoveAnalysis));
+		hintdata *phd = (hintdata *)gtk_object_get_user_data(GTK_OBJECT(pwMoveAnalysis));
 		MoveListUpdate(phd);
 	}
 }
@@ -170,9 +159,9 @@ void MoveListRefreshSize()
  * For example, after new evaluations, rollouts or toggle of MWC/Equity.
  *
  */
-void MoveListUpdate ( const hintdata *phd )
+extern void MoveListUpdate ( const hintdata *phd )
 {
-  int i, j, colNum;
+  unsigned int i, j, colNum;
   char sz[ 32 ];
   cubeinfo ci;
   movelist *pml = phd->pml;
@@ -200,7 +189,7 @@ if (!psHighlight)
 
   /* This function should only be called when the game state matches
      the move list. */
-  assert( ms.fMove == 0 || ms.fMove == 1 );
+  g_assert( ms.fMove == 0 || ms.fMove == 1 );
     
   GetMatchStateCubeInfo( &ci, &ms );
   rBest = pml->amMoves[ 0 ].rScore;
@@ -226,16 +215,18 @@ if (!psHighlight)
 	{
 		/* Lets count how many moves are possible to see if this is the last move */
 		movelist ml;
-		if (!ms.anDice[0])
+		int dice[2];
+		memcpy(dice, ms.anDice, sizeof(dice));
+		if (!dice[0])
 		{	/* If the dice have got lost, try to find them */
 			moverecord* pmr = (moverecord*)plLastMove->p;
 			if (pmr)
 			{
-				ms.anDice[0] = pmr->anDice[0];
-				ms.anDice[1] = pmr->anDice[1];
+				dice[0] = pmr->anDice[0];
+				dice[1] = pmr->anDice[1];
 			}
 		}
-		GenerateMoves(&ml, ms.anBoard, ms.anDice[0], ms.anDice[1], FALSE);
+		GenerateMoves(&ml, msBoard(), dice[0], dice[1], FALSE);
 		if (i < ml.cMoves - 1)
 			rankKnown = 0;
 	}
@@ -247,7 +238,7 @@ if (!psHighlight)
 
 	if (showWLTree)
 	{
-		gtk_list_store_set(store, &iter, 1, rankKnown ? i + 1 : -1, -1);
+		gtk_list_store_set(store, &iter, 1, rankKnown ? (int)i + 1 : -1, -1);
 		goto skipoldcode;
 	}
 	else
@@ -282,7 +273,7 @@ if (!psHighlight)
     }
 	
 	gtk_list_store_set(store, &iter, ML_COL_MOVE + offset,
-                        FormatMove( sz, ms.anBoard,
+                        FormatMove( sz, msBoard(),
                                     pml->amMoves[ i ].anMove ), -1);
 
 	/* highlight row */
@@ -292,6 +283,8 @@ if (!psHighlight)
 		sprintf(buf, "#%02x%02x%02x", psHighlight->fg[GTK_STATE_SELECTED].red / 256, psHighlight->fg[GTK_STATE_SELECTED].green / 256, psHighlight->fg[GTK_STATE_SELECTED].blue / 256);
 		gtk_list_store_set(store, &iter, ML_COL_FGCOL + offset, buf, -1);
 	}
+	else
+		gtk_list_store_set(store, &iter, ML_COL_FGCOL + offset, NULL, -1);
 skipoldcode:	/* Messy as 3 copies of code at moment... */
 	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
   }
@@ -301,25 +294,20 @@ skipoldcode:	/* Messy as 3 copies of code at moment... */
   UpdateStoredMoves ( pml, &ms );
 }
 
-int MoveListGetSelectionCount(const hintdata *phd)
-{
-	return gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(phd->pwMoves)));
-}
-
-GList *MoveListGetSelectionList(const hintdata *phd)
+extern GList *MoveListGetSelectionList(const hintdata *phd)
 {
 	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(phd->pwMoves));
 	GtkTreeSelection* sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(phd->pwMoves));
 	return gtk_tree_selection_get_selected_rows(sel, &model);
 }
 
-void MoveListFreeSelectionList(GList *pl)
+extern void MoveListFreeSelectionList(GList *pl)
 {
 	g_list_foreach (pl, (GFunc)gtk_tree_path_free, NULL);
 	g_list_free(pl);
 }
 
-move *MoveListGetMove(const hintdata *phd, GList *pl)
+extern move *MoveListGetMove(const hintdata *phd, GList *pl)
 {
 	move *m;
 	int showWLTree = showMoveListDetail && !phd->fDetails;
@@ -328,7 +316,8 @@ move *MoveListGetMove(const hintdata *phd, GList *pl)
 	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(phd->pwMoves));
 
 	gboolean check = gtk_tree_model_get_iter(model, &iter, (GtkTreePath*)(pl->data));
-	assert(check);
+	if (check == 0)
+		printf("Error in move list!\n");
 
 	if (showWLTree)
 		col = 0;
@@ -338,7 +327,7 @@ move *MoveListGetMove(const hintdata *phd, GList *pl)
 	return m;
 }
 
-void MoveListShowToggledClicked(GtkWidget *pw, hintdata *phd)
+extern void MoveListShowToggledClicked(GtkWidget *pw, hintdata *phd)
 {
 	int f = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON ( phd->pwShow ) );
 	if (f)
@@ -349,7 +338,7 @@ void MoveListShowToggledClicked(GtkWidget *pw, hintdata *phd)
 	ShowMove(phd, f);
 }
 
-gint MoveListClearSelection( GtkWidget *pw, GdkEventSelection *pes, hintdata *phd )
+extern gint MoveListClearSelection( GtkWidget *pw, GdkEventSelection *pes, hintdata *phd )
 {
 	gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(phd->pwMoves)));
     return TRUE;
