@@ -1,3 +1,4 @@
+/*lint -e818 */
 /*
  * gtkmovelistctrl.c
  * by Jon Kinsey, 2005
@@ -5,7 +6,7 @@
  * Analysis move list control
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,21 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkmovelistctrl.c,v 1.8 2006/09/30 09:24:33 Superfly_Jon Exp $
+ * $Id: gtkmovelistctrl.c,v 1.21 2009/03/13 16:25:48 c_anthon Exp $
  */
 
 #include "config.h"
+
+#include "gtkgame.h"
 #include "gtkmovelistctrl.h"
-#include "assert.h"
-#include "backgammon.h"
 #include "format.h"
 #include "drawboard.h"
-#include "gtkchequer.h"
-#include "string.h"
 
-
-extern float rBest;
-extern GtkStyle *psHighlight;
 GdkColor wlCol;
 
 static void     custom_cell_renderer_movelist_init       (CustomCellRendererMovelist      *cellprogress);
@@ -69,7 +65,7 @@ static   gpointer parent_class;
  *
  ***************************************************************************/
 
-GType
+static GType
 custom_cell_renderer_movelist_get_type (void)
 {
   static GType cell_progress_type = 0;
@@ -89,13 +85,14 @@ custom_cell_renderer_movelist_get_type (void)
       sizeof (CustomCellRendererMovelist),
       0,                                                        /* n_preallocs */
       (GInstanceInitFunc) custom_cell_renderer_movelist_init,
+      NULL
     };
 
     /* Derive from GtkCellRenderer */
     cell_progress_type = g_type_register_static (GTK_TYPE_CELL_RENDERER,
                                                  "CustomCellRendererMovelist",
                                                   &cell_progress_info,
-                                                  0);
+                                                  (GTypeFlags)0);
   }
 
   return cell_progress_type;
@@ -112,7 +109,7 @@ custom_cell_renderer_movelist_get_type (void)
 static void
 custom_cell_renderer_movelist_init (CustomCellRendererMovelist *cellrendererprogress)
 {
-  GTK_CELL_RENDERER(cellrendererprogress)->mode = GTK_CELL_RENDERER_MODE_INERT;
+  GTK_CELL_RENDERER(cellrendererprogress)->mode = (int)GTK_CELL_RENDERER_MODE_INERT;
   GTK_CELL_RENDERER(cellrendererprogress)->xpad = 2;
   GTK_CELL_RENDERER(cellrendererprogress)->ypad = 2;
 }
@@ -191,7 +188,7 @@ custom_cell_renderer_movelist_get_property (GObject    *object,
                                             GParamSpec *pspec)
 {
 	CustomCellRendererMovelist  *cellprogress = CUSTOM_CELL_RENDERER_MOVELIST(object);
-	assert(param_id == 1);
+	g_assert(param_id == 1);
 
 	g_value_set_pointer(value, cellprogress->pml);
 }
@@ -200,13 +197,13 @@ static void
 custom_cell_renderer_movelist_set_property (GObject      *object,
                                             guint         param_id,
                                             const GValue *value,
-                                            GParamSpec   *pspec)
+                                            GParamSpec   *UNUSED(notused))
 {
 	CustomCellRendererMovelist *cellprogress = CUSTOM_CELL_RENDERER_MOVELIST (object);
 	if (param_id == 1)
 		cellprogress->pml = g_value_get_pointer(value);
 	else
-		cellprogress->rank = g_value_get_int(value);
+		cellprogress->rank = (unsigned)g_value_get_int(value);
 }
 
 /***************************************************************************
@@ -232,7 +229,7 @@ custom_cell_renderer_movelist_new (void)
 
 static int fontheight = -1, minWidth;
 
-void custom_cell_renderer_invalidate_size()
+extern void custom_cell_renderer_invalidate_size(void)
 {
 	fontheight = -1;
 }
@@ -261,7 +258,7 @@ custom_cell_renderer_movelist_get_size (GtkCellRenderer *cell,
 {
 	gint calc_width;
 	gint calc_height;
-	char buf[20];
+	char buf[100];
 
 	if (fontheight == -1)
 	{	/* Calculate sizes (if not known) */
@@ -274,7 +271,8 @@ custom_cell_renderer_movelist_get_size (GtkCellRenderer *cell,
 		pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 		g_object_unref (layout);
 		fontheight = logical_rect.height;
-		_s_A = _s_E = _s_B = _s_C = _s_D = fontheight / 5;
+		_s_A = _s_E = _s_C = _s_D = fontheight / 5;
+		_s_B = 0;	/* Pack lines quite closely */
 
 		_s_Z = logical_rect.width;
 
@@ -303,7 +301,8 @@ custom_cell_renderer_movelist_get_size (GtkCellRenderer *cell,
 		g_object_unref (layout);
 		_s_a = logical_rect.width;
 
-		layout = gtk_widget_create_pango_layout (widget, "Cubeless 3-ply");
+		sprintf(buf, "%s 3-ply", _("Cubeless"));
+		layout = gtk_widget_create_pango_layout (widget, buf);
 		pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 		g_object_unref (layout);
 		_s_b = logical_rect.width;
@@ -379,9 +378,11 @@ custom_cell_renderer_movelist_render (GtkCellRenderer *cell,
 	float *ar;
 	GdkColor *pFontCol, *fg;
 	PangoRectangle logical_rect;
+	char *cmark_sz;
+	char *highlight_sz;
 	cubeinfo ci;
 	GetMatchStateCubeInfo( &ci, &ms );
-
+	/*lint --e(641)*/
 	selected = (flags & GTK_CELL_RENDERER_SELECTED) && GTK_WIDGET_HAS_FOCUS (widget);
 
 	gc = gdk_gc_new(window);
@@ -407,11 +408,12 @@ custom_cell_renderer_movelist_render (GtkCellRenderer *cell,
 	}
 
 	/* First line of control */
-
+        cmark_sz = cellprogress->pml->cmark ? "+" : "";
+        highlight_sz = (phd->piHighlight && cellprogress->rank - 1 == *phd->piHighlight ) ? "*" : "";
 	if (cellprogress->rank > 0)
-		sprintf(buf, "%d", cellprogress->rank);
+		sprintf(buf, "%d%s%s", cellprogress->rank, cmark_sz, highlight_sz );
 	else
-		strcpy(buf, "??");
+		sprintf(buf, "??%s%s", cmark_sz, highlight_sz);
 
 	pango_layout_set_text(layout, buf, -1);
 	pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
@@ -422,7 +424,7 @@ custom_cell_renderer_movelist_render (GtkCellRenderer *cell,
 	gdk_draw_layout_with_colors(window, gc, cell_area->x + x, cell_area->y + y, layout, pFontCol, 0);
 
 	x = _s_A + _s_a + _s_Y * 3;
-	FormatEval( buf, &cellprogress->pml->esMove );
+	(void)FormatEval( buf, &cellprogress->pml->esMove );
 	pango_layout_set_text(layout, buf, -1);
 	gdk_draw_layout_with_colors(window, gc, cell_area->x + x, cell_area->y + y, layout, pFontCol, 0);
 
@@ -450,7 +452,7 @@ custom_cell_renderer_movelist_render (GtkCellRenderer *cell,
 	pango_font_description_set_weight(pfd, PANGO_WEIGHT_BOLD);
 	pango_layout_set_font_description (layout, pfd);
 
-	pango_layout_set_text(layout, FormatMove(buf, ms.anBoard, cellprogress->pml->anMove ), -1);
+	pango_layout_set_text(layout, FormatMove(buf, msBoard(), cellprogress->pml->anMove ), -1);
 	gdk_draw_layout_with_colors(window, gc, cell_area->x + x, cell_area->y + y, layout, pFontCol, 0);
 
 	pango_font_description_set_weight(pfd, PANGO_WEIGHT_NORMAL);
