@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,13 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkoptions.h,v 1.2 2003/08/13 11:52:28 Superfly_Jon Exp $
+ * $Id: gtkoptions.h,v 1.5 2007/09/02 20:27:02 c_anthon Exp $
  */
 
 #ifndef _GTKOPTIONS_H_
 #define _GTKOPTIONS_H_
 
-extern void
-GTKSetOptions( void );
+extern void GTKSetOptions( void );
+extern void GTKSound(void);
 
 #endif /* _GTKOPTIONS_H_ */
