@@ -6,7 +6,7 @@
 *
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of version 2 of the GNU General Public License as
+* it under the terms of version 3 or later of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
@@ -18,51 +18,60 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: gtkpanels.c,v 1.28.2.1 2006/11/06 23:29:34 c_anthon Exp $
+* $Id: gtkpanels.c,v 1.60 2009/11/21 20:57:55 c_anthon Exp $
 */
 
 #include "config.h"
 
 #include <stdlib.h>
-#if HAVE_LIMITS_H
-#include <limits.h>
-#endif
 #include <ctype.h>
 #include "backgammon.h"
-#include <glib/gi18n.h>
 #include <string.h>
 #include "gtkboard.h"
 #include "gtkgame.h"
+#include "gtktoolbar.h"
 #include "positionid.h"
+#if USE_BOARD3D
+#include "fun3d.h"
+#endif
 
-extern GtkItemFactory *pif;
-extern GtkWidget *pom;
-extern GtkWidget *hpaned;
+static int fDisplayPanels = TRUE;
+static int fDockPanels = TRUE;
 
-extern GtkWidget* GL_Create();
-extern GtkWidget *StatsPixmapButton(GdkColormap *pcmap, char **xpm, void (*fn)());
+#define NUM_CMD_HISTORY 10
+#define KEY_TAB -247
 
-static void CreatePanel(gnubgwindow window, GtkWidget* pWidget, char* winTitle, char* windowRole);
+typedef struct _CommandEntryData_T {
+	GtkWidget *pwEntry, *pwHelpText, *cmdEntryCombo;
+	int showHelp;
+	int numHistory;
+	int completing;
+	char *cmdString;
+	char *cmdHistory[NUM_CMD_HISTORY];
+} CommandEntryData_T;
+
+static CommandEntryData_T cedPanel = {
+	NULL, NULL, NULL, 0, 0, 0, NULL,
+	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}
+};
 
 
-int fDisplayPanels = TRUE;
-int fDockPanels = TRUE;
+static GtkWidget *game_select_combo = NULL;
 
-/* Callbacks return TRUE as delete_event needs to be stopped */
-extern gboolean ShowAnalysis();
-extern gboolean DeleteAnalysis();
-extern gboolean ShowAnnotation();
-extern gboolean DeleteAnnotation();
-extern gboolean ShowGameWindow();
-extern gboolean DeleteGame();
-extern gboolean ShowMessage();
-extern gboolean DeleteMessage();
-extern gboolean ShowTheoryWindow();
-extern gboolean DeleteTheoryWindow();
-extern gboolean ShowCommandWindow();
-extern gboolean DeleteCommandWindow();
+typedef gboolean (*panelFun)(void);
 
-typedef gboolean (*panelFun)();
+static gboolean DeleteMessage ( void );
+static gboolean DeleteAnalysis( void );
+static gboolean DeleteAnnotation( void );
+static gboolean DeleteGame( void );
+static gboolean DeleteTheoryWindow ( void );
+static gboolean DeleteCommandWindow ( void );
+
+static gboolean ShowAnnotation( void );
+static gboolean ShowMessage( void );
+static gboolean ShowAnalysis( void );
+static gboolean ShowTheoryWindow( void );
+static gboolean ShowCommandWindow( void );
 
 typedef struct _windowobject {
 	char* winName;
@@ -85,7 +94,7 @@ windowobject woPanel[NUM_WINDOWS] =
 		TRUE, FALSE, FALSE, FALSE,
 		NULL, NULL,
 		0,
-		{0, 0, 20, 20}
+		{0, 0, 20, 20, FALSE}
 	},
 	/* game list */
 	{
@@ -93,15 +102,15 @@ windowobject woPanel[NUM_WINDOWS] =
 		TRUE, TRUE, TRUE, TRUE,
 		ShowGameWindow, DeleteGame,
 		0,
-		{ 250, 200, 20, 20 }
+		{ 250, 200, 20, 20, FALSE }
 	},
 	/* analysis */
 	{
 		"analysis",
-		FALSE, TRUE, TRUE, TRUE,
+		TRUE, TRUE, TRUE, TRUE,
 		ShowAnalysis, DeleteAnalysis,
 		0,
-		{ 0, 400, 20, 20 }
+		{ 0, 400, 20, 20, FALSE }
 	},
 	/* annotation */
 	{
@@ -109,7 +118,7 @@ windowobject woPanel[NUM_WINDOWS] =
 		FALSE, TRUE, TRUE, FALSE,
 		ShowAnnotation, DeleteAnnotation,
 		0,
-		{ 0, 400, 20, 20 }
+		{ 0, 400, 20, 20, FALSE }
 	},
 	/* hint */
 	{
@@ -117,7 +126,7 @@ windowobject woPanel[NUM_WINDOWS] =
 		FALSE, FALSE, FALSE, FALSE,
 		NULL, NULL,
 		0,
-		{ 0, 450, 20, 20 }
+		{ 0, 450, 20, 20, FALSE }
 	},
 	/* message */
 	{
@@ -125,7 +134,7 @@ windowobject woPanel[NUM_WINDOWS] =
 		FALSE, TRUE, TRUE, TRUE,
 		ShowMessage, DeleteMessage,
 		0,
-		{ 0, 500, 20, 20 }
+		{ 0, 500, 20, 20, FALSE }
 	},
 	/* command */
 	{
@@ -133,7 +142,7 @@ windowobject woPanel[NUM_WINDOWS] =
 		FALSE, TRUE, TRUE, TRUE,
 		ShowCommandWindow, DeleteCommandWindow,
 		0,
-		{ 0, 0, 20, 20 }
+		{ 0, 0, 20, 20, FALSE }
 	}, 
 	/* theory */
 	{
@@ -141,12 +150,12 @@ windowobject woPanel[NUM_WINDOWS] =
 		FALSE, TRUE, TRUE, TRUE,
 		ShowTheoryWindow, DeleteTheoryWindow,
 		0,
-		{ 0, 0, 20, 20 }
+		{ 0, 0, 20, 20, FALSE }
 	}
 };
 
 
-extern gboolean ShowPanel(gnubgwindow window)
+static gboolean ShowPanel(gnubgwindow window)
 {
 	setWindowGeometry(window);
 	if (!woPanel[window].docked && woPanel[window].pwWin->window)
@@ -164,14 +173,14 @@ extern gboolean ShowGameWindow( void )
 {
 	ShowPanel(WINDOW_GAME);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Game record")), TRUE);
+			  "/View/Panels/Game record")), TRUE);
 	return TRUE;
 }
 
-extern gboolean ShowAnnotation( void )
+static gboolean ShowAnnotation( void )
 {
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Commentary")), TRUE);
+			  "/View/Panels/Commentary")), TRUE);
 
 	woPanel[WINDOW_ANNOTATION].showing = TRUE;
 	/* Avoid showing before main window */
@@ -180,38 +189,57 @@ extern gboolean ShowAnnotation( void )
 	return TRUE;
 }
 
-extern gboolean ShowMessage( void )
+static gboolean ShowMessage( void )
 {
 	ShowPanel(WINDOW_MESSAGE);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Message")), TRUE);
+			  "/View/Panels/Message")), TRUE);
 	return TRUE;
 }
 
-extern gboolean ShowAnalysis( void )
+static gboolean ShowAnalysis( void )
 {
 	ShowPanel(WINDOW_ANALYSIS);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-		"/View/Analysis")), TRUE);
+		"/View/Panels/Analysis")), TRUE);
 	return TRUE;
 }
 
-extern gboolean ShowTheoryWindow( void )
+static gboolean ShowTheoryWindow( void )
 {
 	ShowPanel(WINDOW_THEORY);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Theory")), TRUE);
+			  "/View/Panels/Theory")), TRUE);
 	return TRUE;
 }
 
-extern gboolean ShowCommandWindow( void )
+static gboolean ShowCommandWindow( void )
 {
 	ShowPanel(WINDOW_COMMAND);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Command")), TRUE);
+			  "/View/Panels/Command")), TRUE);
 	return TRUE;
 }
 
+static void CreatePanel(gnubgwindow window, GtkWidget* pWidget, char* winTitle, char* windowRole)
+{
+	if (!woPanel[window].docked)
+	{
+		woPanel[window].pwWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_window_set_title(GTK_WINDOW(woPanel[window].pwWin), winTitle);
+		gtk_window_set_role( GTK_WINDOW( woPanel[window].pwWin ), windowRole );
+		gtk_window_set_type_hint(GTK_WINDOW(woPanel[window].pwWin), GDK_WINDOW_TYPE_HINT_UTILITY);
+
+		setWindowGeometry(window);
+		gtk_container_add(GTK_CONTAINER(woPanel[window].pwWin), pWidget);
+		gtk_window_add_accel_group(GTK_WINDOW(woPanel[window].pwWin), pagMain);
+
+		g_signal_connect(G_OBJECT(woPanel[window].pwWin), "delete_event", G_CALLBACK(woPanel[window].hideFun), NULL);
+	}
+	else
+		woPanel[window].pwWin = pWidget;
+}
+
 static void CreateMessageWindow( void )
 {
         GtkWidget *psw;
@@ -229,9 +257,8 @@ static void CreateMessageWindow( void )
 
 GtkWidget *pwTheoryList = NULL;
 
-extern char *ReturnHits( int anBoard[ 2 ][ 25 ] );
 
-void UpdateTheoryData(BoardData* bd, int UpdateType, int points[2][25])
+void UpdateTheoryData(BoardData* bd, int UpdateType, const TanBoard points)
 {
 	char* pc;
 
@@ -243,7 +270,7 @@ void UpdateTheoryData(BoardData* bd, int UpdateType, int points[2][25])
 		if (ms.gs != GAME_NONE)
 		{
 			int diff;
-			int anPip[ 2 ];
+			unsigned int anPip[ 2 ];
 			PipCount(points, anPip);
 
 			diff = anPip[0] - anPip[1];
@@ -280,7 +307,7 @@ void UpdateTheoryData(BoardData* bd, int UpdateType, int points[2][25])
 
 	if (UpdateType & TT_RETURNHITS)
 	{
-		int anBoard[ 2 ][ 25 ];
+		TanBoard anBoard;
 		pc = NULL;
 		if ( bd->valid_move )
 		{
@@ -301,7 +328,7 @@ void UpdateTheoryData(BoardData* bd, int UpdateType, int points[2][25])
 		if (ms.gs != GAME_NONE)
 		{
 			float fKC;
-			int anPip[ 2 ];
+			unsigned int anPip[ 2 ];
 			PipCount(points, anPip);
 
 			fKC = KleinmanCount(anPip[1], anPip[0]);
@@ -340,8 +367,236 @@ static GtkWidget *CreateTheoryWindow( void )
 	return woPanel[WINDOW_THEORY].pwWin;
 }
 
-extern struct CommandEntryData_T cedPanel;
+/* Display help for command (pStr) in widget (pwText) */
+static void ShowHelp(GtkWidget * pwText, char *pStr)
+{
+	command *pc, *pcFull;
+	char szCommand[128], szUsage[128], szBuf[255], *cc, *pTemp;
+	command cTop = { NULL, NULL, NULL, NULL, acTop };
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
 
+	/* Copy string as token striping corrupts string */
+	pTemp = malloc(strlen(pStr) + 1);
+	strcpy(pTemp, pStr);
+	cc = CheckCommand(pTemp, acTop);
+
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pwText));
+
+	if (cc) {
+		sprintf(szBuf, _("Unknown keyword: %s\n"), cc);
+		gtk_text_buffer_set_text(buffer, szBuf, -1);
+	} else if ((pc = FindHelpCommand(&cTop, pStr, szCommand, szUsage)) != NULL) {
+		gtk_text_buffer_set_text(buffer, "", -1);
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		if (!*pStr) {
+			gtk_text_buffer_insert(buffer, &iter,
+					       _("Available commands:\n"),
+					       -1);
+		} else {
+			if (!pc->szHelp) {	/* The command is an abbreviation, search for the full version */
+				for (pcFull = acTop; pcFull->sz; pcFull++) {
+					if (pcFull->pf == pc->pf
+					    && pcFull->szHelp) {
+						pc = pcFull;
+						strcpy(szCommand, pc->sz);
+						break;
+					}
+				}
+			}
+
+			sprintf(szBuf, "Command: %s\n", szCommand);
+			gtk_text_buffer_insert(buffer, &iter, szBuf, -1);
+			gtk_text_buffer_insert(buffer, &iter,
+					       gettext(pc->szHelp), -1);
+			sprintf(szBuf, "\n\nUsage: %s", szUsage);
+			gtk_text_buffer_insert(buffer, &iter, szBuf, -1);
+
+			if (!(pc->pc && pc->pc->sz))
+				gtk_text_buffer_insert(buffer, &iter, "\n",
+						       -1);
+			else {
+				gtk_text_buffer_insert(buffer, &iter,
+						       _("<subcommand>\n"),
+						       -1);
+				gtk_text_buffer_insert(buffer, &iter,
+						       _
+						       ("Available subcommands:\n"),
+						       -1);
+			}
+		}
+
+		pc = pc->pc;
+
+		while (pc && pc->sz) {
+			if (pc->szHelp) {
+				sprintf(szBuf, "%-15s\t%s\n", pc->sz,
+					gettext(pc->szHelp));
+				gtk_text_buffer_insert(buffer, &iter,
+						       szBuf, -1);
+			}
+			pc++;
+		}
+	}
+	free(pTemp);
+}
+
+static void PopulateCommandHistory(CommandEntryData_T *pData)
+{
+	GList *glist;
+	int i;
+
+	glist = NULL;
+	for (i = 0; i < pData->numHistory; i++)
+		glist = g_list_append(glist, pData->cmdHistory[i]);
+	if (glist) {
+		gtk_combo_set_popdown_strings(GTK_COMBO
+					      (pData->cmdEntryCombo),
+					      glist);
+		g_list_free(glist);
+	}
+}
+
+static void CommandOK(GtkWidget *UNUSED(pw), CommandEntryData_T *pData)
+{
+	int i, found = -1;
+	pData->cmdString =
+	    gtk_editable_get_chars(GTK_EDITABLE(pData->pwEntry), 0, -1);
+
+	/* Update command history */
+
+	/* See if already in history */
+	for (i = 0; i < pData->numHistory; i++) {
+		if (!StrCaseCmp(pData->cmdString, pData->cmdHistory[i])) {
+			found = i;
+			break;
+		}
+	}
+	if (found != -1) {	/* Remove old entry */
+		free(pData->cmdHistory[found]);
+		pData->numHistory--;
+		for (i = found; i < pData->numHistory; i++)
+			pData->cmdHistory[i] = pData->cmdHistory[i + 1];
+	}
+
+	if (pData->numHistory == NUM_CMD_HISTORY) {
+		free(pData->cmdHistory[NUM_CMD_HISTORY - 1]);
+		pData->numHistory--;
+	}
+	for (i = pData->numHistory; i > 0; i--)
+		pData->cmdHistory[i] = pData->cmdHistory[i - 1];
+
+	pData->cmdHistory[0] = malloc(strlen(pData->cmdString) + 1);
+	strcpy(pData->cmdHistory[0], pData->cmdString);
+	pData->numHistory++;
+	PopulateCommandHistory(pData);
+	gtk_entry_set_text(GTK_ENTRY(pData->pwEntry), "");
+
+	if (pData->cmdString) {
+		UserCommand(pData->cmdString);
+		g_free(pData->cmdString);
+	}
+}
+
+static void CommandTextChange(GtkEntry * entry,
+			      CommandEntryData_T *pData)
+{
+	if (pData->showHelp) {
+		/* Make sure the message window is showing */
+		if (!PanelShowing(WINDOW_MESSAGE))
+			PanelShow(WINDOW_MESSAGE);
+
+		ShowHelp(pData->pwHelpText,
+			 gtk_editable_get_chars(GTK_EDITABLE(entry), 0,
+						-1));
+	}
+}
+
+static void CreateHelpText(CommandEntryData_T *pData)
+{
+	GtkWidget *psw;
+	pData->pwHelpText = gtk_text_view_new();
+	gtk_widget_set_size_request(pData->pwHelpText, 400, 300);
+	psw = gtk_scrolled_window_new(NULL, NULL);
+	gtk_container_add(GTK_CONTAINER(psw), pData->pwHelpText);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(psw),
+				       GTK_POLICY_NEVER,
+				       GTK_POLICY_AUTOMATIC);
+	CommandTextChange(GTK_ENTRY(pData->pwEntry), pData);
+}
+
+static void ShowHelpToggled(GtkWidget * widget,
+			    CommandEntryData_T *pData)
+{
+	if (!pData->pwHelpText)
+		CreateHelpText(pData);
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
+		pData->showHelp = 1;
+		CommandTextChange(GTK_ENTRY(pData->pwEntry), pData);
+	} else
+		pData->showHelp = 0;
+	gtk_widget_grab_focus(pData->pwEntry);
+}
+
+/* Capitalize first letter of each word */
+static void Capitalize(char *str)
+{
+	int cap = 1;
+	while (*str) {
+		if (cap) {
+			if (*str >= 'a' && *str <= 'z')
+				*str += 'A' - 'a';
+			cap = 0;
+		} else {
+			if (*str == ' ')
+				cap = 1;
+			if (*str >= 'A' && *str <= 'Z')
+				*str -= 'A' - 'a';
+		}
+		str++;
+	}
+}
+
+static gboolean CommandKeyPress(GtkWidget *UNUSED(widget), GdkEventKey * event,
+				CommandEntryData_T *pData)
+{
+	short k = (short)event->keyval;
+
+	if (k == KEY_TAB) {	/* Tab press - auto complete */
+		command *pc;
+		char szCommand[128], szUsage[128];
+		command cTop = { NULL, NULL, NULL, NULL, acTop };
+		if ((pc =
+		     FindHelpCommand(&cTop,
+				     gtk_editable_get_chars(GTK_EDITABLE
+							    (pData->
+							     pwEntry), 0,
+							    -1), szCommand,
+				     szUsage)) != NULL) {
+			Capitalize(szCommand);
+			gtk_entry_set_text(GTK_ENTRY(pData->pwEntry),
+					   szCommand);
+		}
+		/* Gtk 1 not good at stoping focus moving - so just move back later */
+		pData->completing = 1;
+	}
+	return FALSE;
+}
+
+static gboolean CommandFocusIn(GtkWidget * widget, GdkEventFocus *UNUSED(eventDetails),
+			       CommandEntryData_T *pData)
+{
+	if (pData->completing) {
+		/* Gtk 1 not good at stoping focus moving - so just move back now */
+		pData->completing = 0;
+		gtk_widget_grab_focus(pData->pwEntry);
+		gtk_editable_set_position(GTK_EDITABLE(pData->pwEntry),
+					  (int)strlen(gtk_editable_get_chars(GTK_EDITABLE(pData->pwEntry), 0, -1)));
+		return TRUE;
+	} else
+		return FALSE;
+}
 static GtkWidget *CreateCommandWindow( void )
 {
     GtkWidget *pwhbox;
@@ -360,9 +615,9 @@ static GtkWidget *CreateCommandWindow( void )
 
 	PopulateCommandHistory(&cedPanel);
 
-	gtk_signal_connect(GTK_OBJECT(cedPanel.pwEntry), "changed", GTK_SIGNAL_FUNC(CommandTextChange), &cedPanel);
-	gtk_signal_connect(GTK_OBJECT(cedPanel.pwEntry), "key-press-event", GTK_SIGNAL_FUNC(CommandKeyPress), &cedPanel);
-	gtk_signal_connect(GTK_OBJECT(cedPanel.pwEntry), "activate", GTK_SIGNAL_FUNC(CommandOK), &cedPanel);
+	g_signal_connect(G_OBJECT(cedPanel.pwEntry), "changed", G_CALLBACK(CommandTextChange), &cedPanel);
+	g_signal_connect(G_OBJECT(cedPanel.pwEntry), "key-press-event", G_CALLBACK(CommandKeyPress), &cedPanel);
+	g_signal_connect(G_OBJECT(cedPanel.pwEntry), "activate", G_CALLBACK(CommandOK), &cedPanel);
 
 	pwhbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start ( GTK_BOX ( pwvbox ), pwhbox, FALSE, FALSE, 0);
@@ -371,16 +626,16 @@ static GtkWidget *CreateCommandWindow( void )
 
 	pwShowHelp = gtk_toggle_button_new_with_label( _("Help") );
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pwShowHelp), cedPanel.showHelp);
-	gtk_signal_connect(GTK_OBJECT(pwShowHelp), "toggled", GTK_SIGNAL_FUNC(ShowHelpToggled), &cedPanel);
+	g_signal_connect(G_OBJECT(pwShowHelp), "toggled", G_CALLBACK(ShowHelpToggled), &cedPanel);
 
 	gtk_box_pack_start ( GTK_BOX ( pwhbox ), pwShowHelp, FALSE, FALSE, 5);
-	gtk_signal_connect(GTK_OBJECT(pwShowHelp), "focus-in-event", GTK_SIGNAL_FUNC(CommandFocusIn), &cedPanel);
+	g_signal_connect(G_OBJECT(pwShowHelp), "focus-in-event", G_CALLBACK(CommandFocusIn), &cedPanel);
 
 	return woPanel[WINDOW_COMMAND].pwWin;
 }
 
-static GtkWidget *CreateAnalysisWindow( void ) {
-
+static GtkWidget *CreateAnalysisWindow( void )
+{
     GtkWidget *pHbox, *sw;
     GtkTextBuffer *buffer;
 	if (!woPanel[WINDOW_ANALYSIS].docked)
@@ -400,11 +655,8 @@ static GtkWidget *CreateAnalysisWindow( void ) {
 		gtk_container_add( GTK_CONTAINER( woPanel[WINDOW_ANALYSIS].pwWin ), pwPaned); 
 		gtk_window_add_accel_group( GTK_WINDOW( woPanel[WINDOW_ANALYSIS].pwWin ), pagMain );
     
-		gtk_paned_pack1( GTK_PANED( pwPaned ),
-				 pwAnalysis = gtk_label_new( NULL ), TRUE, FALSE );
-    
-		gtk_paned_pack2( GTK_PANED( pwPaned ),
-				 pHbox = gtk_hbox_new( FALSE, 0 ), FALSE, TRUE );
+		gtk_paned_pack1( GTK_PANED( pwPaned ), pwAnalysis = gtk_label_new( NULL ), TRUE, FALSE );
+    	gtk_paned_pack2( GTK_PANED( pwPaned ), pHbox = gtk_hbox_new( FALSE, 0 ), FALSE, TRUE );
 	}
 	else
 	{
@@ -423,15 +675,15 @@ static GtkWidget *CreateAnalysisWindow( void ) {
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(sw), pwCommentary);
     gtk_box_pack_start(GTK_BOX(pHbox), sw, TRUE, TRUE, 0);
-    gtk_widget_set_size_request (sw, -1, 150);
+    gtk_widget_set_size_request (sw, 100, 150);
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pwCommentary));
     g_signal_connect( G_OBJECT( buffer ), "changed",
 		    G_CALLBACK( CommentaryChanged ), buffer );
 
 	if (!woPanel[WINDOW_ANALYSIS].docked)
 	{
-	    gtk_signal_connect( GTK_OBJECT( woPanel[WINDOW_ANALYSIS].pwWin ), "delete_event",
-			GTK_SIGNAL_FUNC(woPanel[WINDOW_ANALYSIS].hideFun), NULL );
+	    g_signal_connect( G_OBJECT( woPanel[WINDOW_ANALYSIS].pwWin ), "delete_event",
+			G_CALLBACK(woPanel[WINDOW_ANALYSIS].hideFun), NULL );
 		return woPanel[WINDOW_ANALYSIS].pwWin;
 	}
 	else
@@ -441,46 +693,81 @@ static GtkWidget *CreateAnalysisWindow( void ) {
 	}
 }
 
-static void ButtonCommand( GtkWidget *pw, char *szCommand ) {
-
-    UserCommand( szCommand );
+extern void GTKGameSelectDestroy(void)
+{
+	game_select_combo = NULL;
 }
 
-static GtkWidget *PixmapButton( GdkColormap *pcmap, char **xpm,
-				char *szCommand ) {
-    
-    GdkPixmap *ppm;
-    GdkBitmap *pbm;
-    GtkWidget *pw, *pwButton;
+extern void GTKPopGame(int i)
+{
+    GtkTreeIter iter;
+    GtkTreeModel *model;
 
-    ppm = gdk_pixmap_colormap_create_from_xpm_d( NULL, pcmap, &pbm, NULL,
-						 xpm );
-    pw = gtk_pixmap_new( ppm, pbm );
-    pwButton = gtk_button_new();
-    gtk_container_add( GTK_CONTAINER( pwButton ), pw );
-    
-    gtk_signal_connect( GTK_OBJECT( pwButton ), "clicked",
-			GTK_SIGNAL_FUNC( ButtonCommand ), szCommand );
-    
-    return pwButton;
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(game_select_combo));
+    while (gtk_tree_model_iter_nth_child(model, &iter, NULL, i))
+	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+}
+
+extern void GTKAddGame(moverecord * pmr)
+{
+    char sz[128];
+    GtkTreeModel *model;
+    gint last_game;
+
+    sprintf(sz, _("Game %d: %d, %d"), pmr->g.i + 1, pmr->g.anScore[0],
+	    pmr->g.anScore[1]);
+    gtk_combo_box_append_text(GTK_COMBO_BOX(game_select_combo), sz);
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(game_select_combo));
+    last_game = gtk_tree_model_iter_n_children(model, NULL);
+    GTKSetGame(last_game - 1);
+}
+
+extern void GTKRegenerateGames(void)
+{
+    listOLD *pl, *plGame;
+    int i = gtk_combo_box_get_active(GTK_COMBO_BOX(game_select_combo));
+
+    GL_SetNames();
+    GTKPopGame(0);
+    for (pl = lMatch.plNext; pl->p; pl = pl->plNext) {
+	plGame = pl->p;
+	GTKAddGame(plGame->plNext->p);
+    }
+
+    GTKSetGame(i);
+}
+
+extern void GTKSetGame(int i)
+{
+    gtk_combo_box_set_active(GTK_COMBO_BOX(game_select_combo), i);
+}
+
+static void SelectGame(GtkWidget * pw, void *UNUSED(data))
+{
+    listOLD *pl;
+    int i = 0;
+
+    if (!plGame)
+	return;
+
+    i = gtk_combo_box_get_active(GTK_COMBO_BOX(pw));
+	if (i == -1)
+		return;
+    for (pl = lMatch.plNext; i && pl->plNext->p; i--, pl = pl->plNext);
+
+    if (pl->p == plGame)
+	return;
+
+    ChangeGame(pl->p);
 }
 
 static void CreateGameWindow( void ) {
 
     GtkWidget *psw = gtk_scrolled_window_new( NULL, NULL ),
 	*pvbox = gtk_vbox_new( FALSE, 0 ),
-	*phbox = gtk_hbox_new( FALSE, 0 ),
-	*pm = gtk_menu_new(),
-	*pw;
+	*phbox = gtk_hbox_new( FALSE, 0 );
     GdkColormap *pcmap;
 
-#include "xpm/prevgame.xpm"
-#include "xpm/prevmove.xpm"
-#include "xpm/nextmove.xpm"
-#include "xpm/nextgame.xpm"
-#include "xpm/prevmarked.xpm"
-#include "xpm/nextmarked.xpm"
-
     pcmap = gtk_widget_get_colormap( pwMain );
 	if (!woPanel[WINDOW_GAME].docked)
 	{
@@ -499,47 +786,10 @@ static void CreateGameWindow( void ) {
 	}
     gtk_box_pack_start( GTK_BOX( pvbox ), phbox, FALSE, FALSE, 4 );
 
-    gtk_box_pack_start( GTK_BOX( phbox ),
-			pw = PixmapButton( pcmap, prevgame_xpm,
-					   "previous game" ),
-			FALSE, FALSE, 4 );
-    gtk_tooltips_set_tip( ptt, pw, _("Move back to the previous game"), "" );
-    gtk_box_pack_start( GTK_BOX( phbox ),
-			pw = PixmapButton( pcmap, prevmove_xpm,
-					   "previous roll" ),
-			FALSE, FALSE, 0 );
-    gtk_tooltips_set_tip( ptt, pw, _("Move back to the previous roll"), "" );
-    gtk_box_pack_start( GTK_BOX( phbox ),
-			pw = PixmapButton( pcmap, nextmove_xpm,
-					   "next roll" ),
-			FALSE, FALSE, 4 );
-    gtk_tooltips_set_tip( ptt, pw, _("Move ahead to the next roll"), "" );
-    gtk_box_pack_start( GTK_BOX( phbox ),
-			pw = PixmapButton( pcmap, nextgame_xpm,
-				      "next game" ),
-			FALSE, FALSE, 0 );
-    gtk_tooltips_set_tip( ptt, pw, _("Move ahead to the next game"), "" );
-
-    gtk_box_pack_start( GTK_BOX( phbox ),
-			pw = PixmapButton( pcmap, prevmarked_xpm,
-					   "previous marked" ),
-			FALSE, FALSE, 4 );
-    gtk_tooltips_set_tip( ptt, pw, _("Move back to the previous marked "
-				     "decision" ), "" );
-    gtk_box_pack_start( GTK_BOX( phbox ),
-			pw = PixmapButton( pcmap, nextmarked_xpm,
-					   "next marked" ),
-			FALSE, FALSE, 0 );
-    gtk_tooltips_set_tip( ptt, pw, _("Move ahead to the next marked "
-				     "decision" ), "" );
-        
-    gtk_menu_append( GTK_MENU( pm ), gtk_menu_item_new_with_label(
-	_("(no game)") ) );
-    gtk_widget_show_all( pm );
-    gtk_option_menu_set_menu( GTK_OPTION_MENU( pom = gtk_option_menu_new() ),
-			      pm );
-    gtk_option_menu_set_history( GTK_OPTION_MENU( pom ), 0 );
-    gtk_box_pack_start( GTK_BOX( phbox ), pom, TRUE, TRUE, 4 );
+    game_select_combo = gtk_combo_box_new_text();
+    g_signal_connect (G_OBJECT (game_select_combo), "changed",
+		    G_CALLBACK (SelectGame), NULL);
+    gtk_box_pack_start( GTK_BOX( phbox ), game_select_combo, TRUE, TRUE, 4 );
     
     gtk_container_add( GTK_CONTAINER( pvbox ), psw );
     gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( psw ),
@@ -549,8 +799,8 @@ static void CreateGameWindow( void ) {
 
 	if (!woPanel[WINDOW_GAME].docked)
 	{
-	    gtk_signal_connect( GTK_OBJECT( woPanel[WINDOW_GAME].pwWin ), "delete_event",
-			GTK_SIGNAL_FUNC(woPanel[WINDOW_GAME].hideFun), NULL );
+	    g_signal_connect( G_OBJECT( woPanel[WINDOW_GAME].pwWin ), "delete_event",
+			G_CALLBACK(woPanel[WINDOW_GAME].hideFun), NULL );
 	}
 	else
 		woPanel[WINDOW_GAME].pwWin = pvbox;
@@ -563,7 +813,7 @@ static void CreateHeadWindow(gnubgwindow panel, const char* sz, GtkWidget* pwWid
 	GtkWidget* pwVbox = gtk_vbox_new(FALSE, 0);
 	GtkWidget* pwHbox = gtk_hbox_new(FALSE, 0);
 	GdkColormap *pcmap = gtk_widget_get_colormap( pwMain );
-	GtkWidget* pwX = StatsPixmapButton(pcmap, x_xpm, (void (*)())woPanel[panel].hideFun);
+	GtkWidget* pwX = StatsPixmapButton(pcmap, x_xpm, (void(*)(GtkWidget*, char*))woPanel[panel].hideFun);
 
 	gtk_box_pack_start( GTK_BOX( pwVbox ), pwHbox, FALSE, FALSE, 0 );
 	gtk_box_pack_start( GTK_BOX( pwHbox ), pwLab, FALSE, FALSE, 10 );
@@ -573,7 +823,7 @@ static void CreateHeadWindow(gnubgwindow panel, const char* sz, GtkWidget* pwWid
 	woPanel[panel].pwWin = pwVbox;
 }
 
-static void CreatePanels()
+static void CreatePanels(void)
 {
    CreateGameWindow();
    gtk_box_pack_start( GTK_BOX( pwPanelVbox ), woPanel[WINDOW_GAME].pwWin, TRUE, TRUE, 0 );
@@ -593,51 +843,51 @@ static void CreatePanels()
    gtk_box_pack_start(GTK_BOX(pwPanelVbox), woPanel[WINDOW_THEORY].pwWin, FALSE, FALSE, 0 );
 }
 
-extern gboolean DeleteMessage ( void )
+static gboolean DeleteMessage ( void )
 {
 	HidePanel(WINDOW_MESSAGE);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Message")), FALSE);
+			  "/View/Panels/Message")), FALSE);
 	return TRUE;
 }
 
-extern gboolean DeleteAnalysis( void )
+static gboolean DeleteAnalysis( void )
 {
 	HidePanel(WINDOW_ANALYSIS);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Analysis")), FALSE);
+			  "/View/Panels/Analysis")), FALSE);
 	return TRUE;
 }
 
-extern gboolean DeleteAnnotation( void )
+static gboolean DeleteAnnotation( void )
 {
 	HidePanel(WINDOW_ANNOTATION);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Commentary")), FALSE);
+			  "/View/Panels/Commentary")), FALSE);
 	return TRUE;
 }
 
-extern gboolean DeleteGame( void )
+static gboolean DeleteGame( void )
 {
 	HidePanel(WINDOW_GAME);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Game record")), FALSE);
+			  "/View/Panels/Game record")), FALSE);
 	return TRUE;
 }
 
-extern gboolean DeleteTheoryWindow ( void )
+static gboolean DeleteTheoryWindow ( void )
 {
 	HidePanel(WINDOW_THEORY);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Theory")), FALSE);
+			  "/View/Panels/Theory")), FALSE);
 	return TRUE;
 }
 
-extern gboolean DeleteCommandWindow ( void )
+static gboolean DeleteCommandWindow ( void )
 {
 	HidePanel(WINDOW_COMMAND);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(pif,
-			  "/View/Command")), FALSE);
+			  "/View/Panels/Command")), FALSE);
 	return TRUE;
 }
 
@@ -727,42 +977,31 @@ extern void getWindowGeometry(gnubgwindow window)
 
 }
 
-static void CreatePanel(gnubgwindow window, GtkWidget* pWidget, char* winTitle, char* windowRole)
-{
-	if (!woPanel[window].docked)
-	{
-		woPanel[window].pwWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title(GTK_WINDOW(woPanel[window].pwWin), winTitle);
-		gtk_window_set_role( GTK_WINDOW( woPanel[window].pwWin ), windowRole );
-		gtk_window_set_type_hint(GTK_WINDOW(woPanel[window].pwWin), GDK_WINDOW_TYPE_HINT_UTILITY);
-
-		setWindowGeometry(window);
-		gtk_container_add(GTK_CONTAINER(woPanel[window].pwWin), pWidget);
-		gtk_window_add_accel_group(GTK_WINDOW(woPanel[window].pwWin), pagMain);
-
-		gtk_signal_connect(GTK_OBJECT(woPanel[window].pwWin), "delete_event", GTK_SIGNAL_FUNC(woPanel[window].hideFun), NULL);
-	}
-	else
-		woPanel[window].pwWin = pWidget;
-}
-
-void DockPanels()
+void DockPanels(void)
 {
 	int i;
 	int currentSelectedGame = -1;
 	if(!fX)
 	  return;
 
-	if (pom)
-		currentSelectedGame = gtk_option_menu_get_history( GTK_OPTION_MENU( pom ) );
+	if (game_select_combo)
+		currentSelectedGame = gtk_combo_box_get_active(GTK_COMBO_BOX(game_select_combo));
 
 	if (fDockPanels)
 	{
 		RefreshGeometries();	/* Get the current window positions */
 
-		gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Commentary"));
-		gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Hide panels"));
-		gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Restore panels"));
+		gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Panels/Commentary"));
+		if (fDisplayPanels)
+		{
+			gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Hide panels"));
+			gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Restore panels"));
+		}
+		else
+		{
+			gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Hide panels"));
+			gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Restore panels"));
+		}
 
 		for (i = 0; i < NUM_WINDOWS; i++)
 		{
@@ -774,14 +1013,14 @@ void DockPanels()
 		}
 		CreatePanels();
 		if (fDisplayPanels)
-			SwapBoardToPanel(TRUE);
+			SwapBoardToPanel(TRUE, TRUE);
 	}
 	else
 	{
 		if (fDisplayPanels)
-			SwapBoardToPanel(FALSE);
+			SwapBoardToPanel(FALSE, TRUE);
 
-		gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Commentary"));
+		gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Panels/Commentary"));
 		gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Hide panels"));
 		gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Restore panels"));
 
@@ -800,24 +1039,24 @@ void DockPanels()
 		CreateTheoryWindow();
 		CreateCommandWindow();
 	}
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Message"), !fDockPanels || fDisplayPanels);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Game record"), !fDockPanels || fDisplayPanels);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Commentary"), !fDockPanels || fDisplayPanels);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Analysis"), !fDockPanels || fDisplayPanels);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Command"), !fDockPanels || fDisplayPanels);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Theory"), !fDockPanels || fDisplayPanels);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Message"), !fDockPanels || fDisplayPanels);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Game record"), !fDockPanels || fDisplayPanels);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Commentary"), !fDockPanels || fDisplayPanels);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Analysis"), !fDockPanels || fDisplayPanels);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Command"), !fDockPanels || fDisplayPanels);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Theory"), !fDockPanels || fDisplayPanels);
 
 	if (!fDockPanels || fDisplayPanels)
 	{
 		for (i = 0; i < NUM_WINDOWS; i++)
 		{
-			if (woPanel[i].undockable && woPanel[i].showing)
+			if (woPanel[i].dockable && woPanel[i].showing)
 				woPanel[i].showFun();
 		}
 	}
 	/* Refresh panel contents */
 	GTKRegenerateGames();
-	GTKUpdateAnnotations();
+	ChangeGame(NULL);
 	if (currentSelectedGame != -1)
 	    GTKSetGame(currentSelectedGame);
 
@@ -828,9 +1067,7 @@ void DockPanels()
 	SetMainWindowSize();
 }
 
-
-extern void
-ShowAllPanels ( gpointer *p, guint n, GtkWidget *pw )
+extern void ShowAllPanels ( gpointer UNUSED(p), guint UNUSED(n), GtkWidget *UNUSED(pw) )
 {
 	BoardData *bd = BOARD( pwBoard )->board_data;
 	int i;
@@ -840,8 +1077,8 @@ ShowAllPanels ( gpointer *p, guint n, GtkWidget *pw )
 
 	/* Hide for smoother appearance */
 #if USE_BOARD3D
-	if (bd->rd->fDisplayType == DT_3D)
-		gtk_widget_hide(bd->bd3d.drawing_area3d);
+	if (display_is_3d(bd->rd))
+		gtk_widget_hide(GetDrawingArea3d(bd->bd3d));
 	else
 #endif
 		gtk_widget_hide(bd->drawing_area);
@@ -854,28 +1091,27 @@ ShowAllPanels ( gpointer *p, guint n, GtkWidget *pw )
 			woPanel[i].showFun();
 	}
 
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Hide panels"), TRUE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Restore panels"), FALSE);
+	gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Hide panels"));
+	gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Restore panels"));
 
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Message"), TRUE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Game record"), TRUE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Commentary"), TRUE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Analysis"), TRUE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Command"), TRUE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Theory"), TRUE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Message"), TRUE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Game record"), TRUE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Commentary"), TRUE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Analysis"), TRUE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Command"), TRUE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Theory"), TRUE);
 
-	SwapBoardToPanel(TRUE);
+	SwapBoardToPanel(TRUE, TRUE);
 
 #if USE_BOARD3D
-	if (bd->rd->fDisplayType == DT_3D)
-		gtk_widget_show(bd->bd3d.drawing_area3d);
+	if (display_is_3d(bd->rd))
+		gtk_widget_show(GetDrawingArea3d(bd->bd3d));
 	else
 #endif
 		gtk_widget_show(bd->drawing_area);
 }
 
-extern void
-HideAllPanels ( gpointer *p, guint n, GtkWidget *pw )
+void DoHideAllPanels(int updateEvents)
 {
 	BoardData *bd = BOARD( pwBoard )->board_data;
 	int i;
@@ -885,8 +1121,8 @@ HideAllPanels ( gpointer *p, guint n, GtkWidget *pw )
 
 	/* Hide for smoother appearance */
 #if USE_BOARD3D
-	if (bd->rd->fDisplayType == DT_3D)
-		gtk_widget_hide(bd->bd3d.drawing_area3d);
+	if (display_is_3d(bd->rd))
+		gtk_widget_hide(GetDrawingArea3d(bd->bd3d));
 	else
 #endif
 		gtk_widget_hide(bd->drawing_area);
@@ -902,30 +1138,36 @@ HideAllPanels ( gpointer *p, guint n, GtkWidget *pw )
 		}
 	}
 
+	gtk_widget_show(gtk_item_factory_get_widget(pif, "/View/Restore panels"));
 	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Restore panels"), TRUE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Hide panels"), FALSE);
+	gtk_widget_hide(gtk_item_factory_get_widget(pif, "/View/Hide panels"));
 
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Message"), FALSE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Game record"), FALSE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Commentary"), FALSE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Analysis"), FALSE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Theory"), FALSE);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Command"), FALSE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Message"), FALSE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Game record"), FALSE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Commentary"), FALSE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Analysis"), FALSE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Theory"), FALSE);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(pif, "/View/Panels/Command"), FALSE);
 
-	SwapBoardToPanel(FALSE);
+	SwapBoardToPanel(FALSE, updateEvents);
 
 	/* Resize screen */
 	SetMainWindowSize();
 
 #if USE_BOARD3D
-	if (bd->rd->fDisplayType == DT_3D)
-		gtk_widget_show(bd->bd3d.drawing_area3d);
+	if (display_is_3d(bd->rd))
+		gtk_widget_show(GetDrawingArea3d(bd->bd3d));
 	else
 #endif
 		gtk_widget_show(bd->drawing_area);
 }
 
-extern void ToggleDockPanels( gpointer *p, guint n, GtkWidget *pw )
+extern void HideAllPanels ( gpointer UNUSED(p), guint UNUSED(n), GtkWidget *UNUSED(pw) )
+{
+	DoHideAllPanels(TRUE);
+}
+
+extern void ToggleDockPanels( gpointer UNUSED(p), guint UNUSED(n), GtkWidget *pw )
 {
 	int newValue = GTK_CHECK_MENU_ITEM( pw )->active;
 	if (fDockPanels != newValue)
@@ -935,7 +1177,7 @@ extern void ToggleDockPanels( gpointer *p, guint n, GtkWidget *pw )
 	}
 }
 
-void DisplayWindows()
+extern void DisplayWindows(void)
 {
 	int i;
 	/* Display any other windows now */
@@ -995,10 +1237,9 @@ setWindowGeometry(gnubgwindow window)
                     ( pwo->wg.nPosY >= 0 ) ? pwo->wg.nPosY : 0 );
 
   if (pwo->wg.max)
-    gtk_window_maximize(GTK_WINDOW( pwo->pwWin ));
-
-
-
+    gtk_window_maximize(GTK_WINDOW(pwo->pwWin));
+  else
+    gtk_window_unmaximize(GTK_WINDOW(pwo->pwWin));
 }
 
 void ShowHidePanel(gnubgwindow panel)
@@ -1009,7 +1250,7 @@ void ShowHidePanel(gnubgwindow panel)
 		woPanel[panel].hideFun();
 }
 
-int SetMainWindowSize()
+int SetMainWindowSize(void)
 {
 	if (woPanel[WINDOW_MAIN].wg.nWidth && woPanel[WINDOW_MAIN].wg.nHeight)
 	{
@@ -1037,7 +1278,7 @@ RefreshGeometries ( void )
 {
 	int i;
 	for (i = 0; i < NUM_WINDOWS; i++)
-		getWindowGeometry(i);
+		getWindowGeometry((gnubgwindow)i);
 }
 
 extern void CommandSetAnnotation( char *sz ) {
@@ -1051,7 +1292,7 @@ extern void CommandSetMessage( char *sz ) {
 
 	SetToggle("message", &woPanel[WINDOW_MESSAGE].showing, sz,
 		_("Show window with messages"),
-		_("Do not show window with message."));
+		_("Do not show window with messages."));
 }
 
 extern void CommandSetTheoryWindow( char *sz ) {
@@ -1214,7 +1455,7 @@ CommandSetGeometryPosY ( char *sz ) {
 extern void
 CommandSetGeometryMax ( char *sz )
 {
-  int maxed = (strcasecmp(sz, "yes") == 0);
+  int maxed = (StrCaseCmp(sz, "yes") == 0);
   woPanel[pwoSetPanel].wg.max = maxed;
   outputf ( maxed ? _("%s window maximised.\n") : _("%s window unmaximised.\n"),
 	  woPanel[pwoSetPanel].winName);
@@ -1223,7 +1464,7 @@ CommandSetGeometryMax ( char *sz )
 		setWindowGeometry(pwoSetPanel);
 }
 
-extern void CommandSetDisplayPanels( char *sz ) {
+extern void CommandSetPanels( char *sz ) {
 
   SetToggle ("panels", &fDisplayPanels, sz, 
   _("Game list, Annotation and Message panels/windows will be displayed."),
@@ -1239,7 +1480,8 @@ extern void CommandSetDisplayPanels( char *sz ) {
     
 }
 
-extern void CommandShowDisplayPanels( char *sz ) {
+extern void CommandShowPanels( char *UNUSED(sz) )
+{
 	if (fDisplayPanels)
 	  outputf( _("Game list, Annotation and Message panels/windows "
 		        "will be displayed."));
@@ -1264,8 +1506,7 @@ static void GetGeometryDisplayString(char* buf, windowobject* pwo)
 		dispName, pwo->wg.nWidth, pwo->wg.nHeight, pwo->wg.nPosX, pwo->wg.nPosY);
 }
 
-extern void
-CommandShowGeometry ( char *sz )
+extern void CommandShowGeometry ( char *UNUSED(sz) )
 {
 	int i;
 	char szBuf[1024];
@@ -1278,17 +1519,17 @@ CommandShowGeometry ( char *sz )
 	}
 }
 
-int DockedPanelsShowing()
+int DockedPanelsShowing(void)
 {
 	return fDockPanels && fDisplayPanels;
 }
 
-int ArePanelsShowing()
+int ArePanelsShowing(void)
 {
 	return fDisplayPanels;
 }
 
-int ArePanelsDocked()
+int ArePanelsDocked(void)
 {
 	return fDockPanels;
 }
@@ -1313,7 +1554,7 @@ int PanelShowing(gnubgwindow window)
 	return woPanel[window].showing && (fDisplayPanels || !woPanel[window].docked);
 }
 
-void ClosePanels()
+extern void ClosePanels(void)
 {
 	/* loop through and close each panel (not main window) */
 	int window;
