@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 2000.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkprefs.h,v 1.10 2004/03/31 09:51:54 Superfly_Jon Exp $
+ * $Id: gtkprefs.h,v 1.12 2007/07/02 12:43:39 ace Exp $
  */
 
 #ifndef _GTKPREFS_H_
@@ -28,5 +28,8 @@ extern void BoardPreferences( GtkWidget *pwBoard );
 extern void BoardPreferencesStart( GtkWidget *pwBoard );
 extern void BoardPreferencesDone( GtkWidget *pwBoard );
 extern void Default3dSettings(BoardData* bd);
+extern void UpdatePreview(GtkWidget *notused);
+
+extern GtkWidget *pwPrevBoard;
 
 #endif
