@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,31 +16,23 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkrolls.c,v 1.16 2006/09/21 22:24:05 Superfly_Jon Exp $
+ * $Id: gtkrolls.c,v 1.28 2008/03/12 22:56:34 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <gtk/gtk.h>
+#include "config.h"
 
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <assert.h>
 
-#include "backgammon.h"
-#include "eval.h"
-#include "gtkrolls.h"
 #include "gtkgame.h"
+
 #include "drawboard.h"
 #include <glib/gi18n.h>
+
 #include "format.h"
 #include "gtkwindows.h"
+#include "gtkrolls.h"
 
 typedef struct _rollswidget {
 
@@ -57,15 +49,15 @@ typedef struct _rollswidget {
 
 static void
 add_level ( GtkTreeStore *model, GtkTreeIter *iter,
-            const int n, int anBoard[ 2 ][ 25 ],
+            const int n, const TanBoard anBoard,
             evalcontext *pec, cubeinfo *pci,
-            const int fInvert,
+            const gboolean fInvert,
             float arOutput[ NUM_ROLLOUT_OUTPUTS ] ) {
 
   int n0, n1;
   GtkTreeIter child_iter;
   cubeinfo ci;
-  int an[ 2 ][ 25 ];
+  TanBoard an;
   float ar[ NUM_ROLLOUT_OUTPUTS ];
   int anMove[ 8 ];
   int i;
@@ -95,7 +87,7 @@ add_level ( GtkTreeStore *model, GtkTreeIter *iter,
 
       if ( n ) {
 
-        add_level ( model, &child_iter, n - 1, an, pec, &ci, ! fInvert, ar );
+        add_level ( model, &child_iter, n - 1, (ConstTanBoard)an, pec, &ci, !fInvert, ar );
 		if (fInterrupt)
 			return;
 
@@ -106,7 +98,7 @@ add_level ( GtkTreeStore *model, GtkTreeIter *iter,
 
         ProgressValueAdd ( 1 );
 
-        if ( GeneralEvaluationE ( ar, an, &ci, pec ) < 0 )
+        if ( GeneralEvaluationE ( ar, (ConstTanBoard)an, &ci, pec ) < 0 )
           return;
 
       }
@@ -159,7 +151,7 @@ static gint
 sort_func ( GtkTreeModel *model,
             GtkTreeIter *a,
             GtkTreeIter *b,
-            gpointer user_data) {
+            gpointer notused) {
 
   char *sz0, *sz1;
   float r0, r1;
@@ -180,11 +172,10 @@ sort_func ( GtkTreeModel *model,
 }
 
 
-static GtkTreeModel *
-create_model ( const int n, evalcontext *pec, matchstate *pms ) {
-
+static GtkTreeModel *create_model(const int n, evalcontext *pec, const matchstate *pms)
+{
   GtkTreeStore *model;
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   cubeinfo ci;
   float arOutput[ NUM_ROLLOUT_OUTPUTS ];
   int i, j;
@@ -204,7 +195,7 @@ create_model ( const int n, evalcontext *pec, matchstate *pms ) {
 
   ProgressStartValue ( _("Calculating equities" ), j );
 
-  add_level ( model, NULL, n - 1, anBoard, pec, &ci, TRUE, arOutput );
+  add_level ( model, NULL, n - 1, (ConstTanBoard)anBoard, pec, &ci, TRUE, arOutput );
 
   ProgressEnd ();
 
@@ -222,13 +213,13 @@ create_model ( const int n, evalcontext *pec, matchstate *pms ) {
 
 
 static GtkWidget *
-RollsTree ( const int n, evalcontext *pec, matchstate *pms ) {
-
+RollsTree ( const int n, evalcontext *pec, const matchstate *pms )
+{
   GtkTreeModel *pm;
   GtkWidget *ptv;
   int i;
   GtkCellRenderer *renderer;
-  static char *aszColumn[] = {
+  static const char *aszColumn[] = {
     N_("Roll"),
     N_("Move"),
     N_("Equity")
@@ -302,7 +293,7 @@ static void DepthChanged ( GtkRange *pr, rollswidget *prw )
 	else
 	{
 		if (!prw->closing)
-			gtk_range_set_value(GTK_RANGE(prw->pScale), prw->nDepth);
+			gtk_range_set_value(GTK_RANGE(prw->pScale), (double)prw->nDepth);
 	}
 
 	pwGrab = pwOldGrab;
@@ -320,13 +311,13 @@ static void DepthChanged ( GtkRange *pr, rollswidget *prw )
 	}
 }
 
-void CancelRolls(GtkWidget* pButton)
+static void CancelRolls(GtkWidget* pButton)
 {
 	fInterrupt = TRUE;
 	gtk_widget_set_sensitive(pButton, FALSE);
 }
 
-static gint RollsClose(GtkWidget *widget, GdkEvent *event, rollswidget *prw)
+static gint RollsClose(GtkWidget *notused, GdkEvent *notused2, rollswidget *prw)
 {
 	if (pwOldGrab != pwGrab)
 	{	/* Mid-depth change - wait for it to cancel */
@@ -347,7 +338,7 @@ GTKShowRolls ( const gint nDepth, evalcontext *pec, matchstate *pms ) {
   GtkAdjustment *padj;
   int n;
 
-  rollswidget *prw = g_malloc ( sizeof ( rollswidget ) );
+  rollswidget *prw = (rollswidget *)g_malloc ( sizeof ( rollswidget ) );
 
   prw->closing = FALSE;
   prw->pDialog = GTKCreateDialog( _("Distribution of rolls"), DT_INFO, NULL, DIALOG_FLAG_MODAL, NULL, NULL );
@@ -363,7 +354,9 @@ GTKShowRolls ( const gint nDepth, evalcontext *pec, matchstate *pms ) {
   vbox = gtk_vbox_new ( FALSE, 8 );
   gtk_container_set_border_width ( GTK_CONTAINER ( vbox ), 8);
   gtk_container_add ( GTK_CONTAINER (DialogArea( prw->pDialog, DA_MAIN ) ), vbox );
-  gtk_object_set_data_full ( GTK_OBJECT ( vbox ), "user_data",
+
+  /* hook to free rollswidget on widget destruction */
+  g_object_set_data_full ( G_OBJECT ( vbox ), "rollswidget",
                              prw, g_free );
 
   /* scrolled window to hold tree widget */
@@ -388,7 +381,7 @@ GTKShowRolls ( const gint nDepth, evalcontext *pec, matchstate *pms ) {
                        FALSE, FALSE, 4 );
 
   /* Set page size to 1 */
-  padj = GTK_ADJUSTMENT(gtk_adjustment_new(1, 1, 5, 1, 1, 0));
+  padj = GTK_ADJUSTMENT(gtk_adjustment_new(1., 1., 5., 1., 1., 0.));
   prw->pScale = gtk_hscale_new( padj );
   gtk_widget_set_size_request( prw->pScale, 100, -1 );
   gtk_box_pack_start ( GTK_BOX ( hbox ), prw->pScale, FALSE, FALSE, 4 );
@@ -397,12 +390,12 @@ GTKShowRolls ( const gint nDepth, evalcontext *pec, matchstate *pms ) {
   gtk_range_set_update_policy( GTK_RANGE( prw->pScale ), GTK_UPDATE_DISCONTINUOUS );
 
   prw->pCancel = gtk_button_new_with_label( _("Cancel") );
-  gtk_signal_connect( GTK_OBJECT( prw->pCancel ), "clicked", 
-			GTK_SIGNAL_FUNC( CancelRolls ), NULL );
+  g_signal_connect( G_OBJECT( prw->pCancel ), "clicked", 
+			G_CALLBACK( CancelRolls ), NULL );
   gtk_box_pack_start ( GTK_BOX ( hbox ), prw->pCancel, FALSE, FALSE, 4 );
 
-  gtk_signal_connect ( GTK_OBJECT ( prw->pScale ), "value-changed",
-                       GTK_SIGNAL_FUNC ( DepthChanged ), prw );
+  g_signal_connect( G_OBJECT ( prw->pScale ), "value-changed",
+                       G_CALLBACK ( DepthChanged ), prw );
 
   /* tree  */
 
@@ -412,11 +405,8 @@ GTKShowRolls ( const gint nDepth, evalcontext *pec, matchstate *pms ) {
   }
   
   gtk_window_set_default_size( GTK_WINDOW( prw->pDialog ), 560, 400 ); 
-  gtk_widget_show_all( prw->pDialog );
-  gtk_signal_connect( GTK_OBJECT( prw->pDialog ), "delete_event",
-                      GTK_SIGNAL_FUNC(RollsClose), prw );	/* In case closed mid calculation */
+  g_signal_connect( G_OBJECT( prw->pDialog ), "delete_event",
+                      G_CALLBACK(RollsClose), prw );	/* In case closed mid calculation */
 
-  GTKDisallowStdin();
-  gtk_main();
-  GTKAllowStdin();
+  GTKRunDialog(prw->pDialog);
 }
