@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,35 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtksplash.c,v 1.10 2005/10/11 07:56:38 Superfly_Jon Exp $
+ * $Id: gtksplash.c,v 1.33 2011/08/31 00:50:44 mdpetch Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
+#include "gtklocdefs.h"
 
-#include <gtk/gtk.h>
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include "backgammon.h"
-#include "eval.h"
+#include "gtkgame.h"
+#include "util.h"
 #include "gtksplash.h"
-#include "gtktoolbar.h"
-#include "i18n.h"
-
-#if GTK_CHECK_VERSION(2,0,0)
-#define USLEEP(x) g_usleep(x)
-#elif HAVE_USLEEP
-#define USLEEP(x) usleep(x)
-#else
-#define USLEEP(x)
-#endif
 
 typedef struct _gtksplash {
   GtkWidget *pwWindow;
@@ -53,25 +33,21 @@ typedef struct _gtksplash {
 
 
 extern GtkWidget *
-CreateSplash () {
+CreateSplash (void) {
 
   gtksplash *pgs;
   GtkWidget *pwvbox, *pwFrame, *pwb;
   GtkWidget *pwImage;
+  gchar *fn;
   int i;
-#include "xpm/gnubg-big.xpm"
 
   pgs = (gtksplash *) g_malloc ( sizeof ( gtksplash ) );
 
   pgs->pwWindow = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
-#if GTK_CHECK_VERSION(2,0,0)
   gtk_window_set_role( GTK_WINDOW( pgs->pwWindow ),
 		       "splash screen" );
-#if GTK_CHECK_VERSION(2,2,0)
   gtk_window_set_type_hint( GTK_WINDOW( pgs->pwWindow ),
 			    GDK_WINDOW_TYPE_HINT_SPLASHSCREEN );
-#endif
-#endif
   gtk_window_set_title ( GTK_WINDOW ( pgs->pwWindow ), 
                          _("Starting " VERSION_STRING ) );
   gtk_window_set_position ( GTK_WINDOW ( pgs->pwWindow ), GTK_WIN_POS_CENTER );
@@ -87,7 +63,9 @@ CreateSplash () {
 
   /* image */
 
-  pwImage = image_from_xpm_d ( gnubg_big_xpm, GTK_WIDGET ( pgs->pwWindow ) );
+  fn = g_build_filename(getPkgDataDir(), "pixmaps", "gnubg-big.png", NULL);
+  pwImage = gtk_image_new_from_file(fn);
+  g_free(fn);
   gtk_box_pack_start ( GTK_BOX ( pwvbox ), pwImage, FALSE, FALSE, 0 );
 
   gtk_box_pack_start( GTK_BOX( pwvbox ),
@@ -108,13 +86,12 @@ CreateSplash () {
 
   /* signals */
 
-  gtk_object_set_data_full ( GTK_OBJECT ( pgs->pwWindow ), "user_data", 
+  g_object_set_data_full ( G_OBJECT ( pgs->pwWindow ), "gtksplash", 
                              pgs, g_free );
 
   gtk_widget_show_all ( GTK_WIDGET ( pgs->pwWindow ) );
 
-  while( gtk_events_pending() )
-    gtk_main_iteration();
+  ProcessEvents();
 
   return pgs->pwWindow;
 
@@ -126,33 +103,29 @@ DestroySplash ( GtkWidget *pwSplash ) {
 
   if ( ! pwSplash )
     return;
-  
-  USLEEP( 1000 );
+
+#ifndef WIN32
+  /* Don't bother with these pauses on windows? */
+  g_usleep ( 250 * 1000 );
+#endif
 
   gtk_widget_destroy ( pwSplash );
 
 }
 
 
-extern void
-PushSplash ( GtkWidget *pwSplash, 
-             const gchar *szText0, const gchar *szText1,
-             const unsigned long nMuSec ) {
-  
+extern void PushSplash ( GtkWidget *pwSplash, const gchar *szText0, const gchar *szText1 )
+{
   gtksplash *pgs;
 
   if ( ! pwSplash )
     return;
   
-  pgs = gtk_object_get_data ( GTK_OBJECT ( pwSplash ),
-                                         "user_data" );
+  pgs = (gtksplash *)g_object_get_data ( G_OBJECT ( pwSplash ),
+                                         "gtksplash");
 
   gtk_label_set_text ( GTK_LABEL ( pgs->apwStatus[ 0 ] ), szText0 );
   gtk_label_set_text ( GTK_LABEL ( pgs->apwStatus[ 1 ] ), szText1 );
 
-  while( gtk_events_pending() )
-    gtk_main_iteration();
-
-  USLEEP( nMuSec );
-
+  ProcessEvents();
 }
