@@ -7,7 +7,7 @@
  * http://www46.pair.com/sengoku/TempMap/English/TempMap.html
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -19,34 +19,30 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtktempmap.c,v 1.21 2005/02/25 11:42:35 Superfly_Jon Exp $
+ * $Id: gtktempmap.c,v 1.32 2008/03/12 22:56:34 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <assert.h>
+#include <glib.h>
 
 #include "backgammon.h"
 #include "eval.h"
 #include "gtktempmap.h"
 #include "gtkgame.h"
 #include "drawboard.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "format.h"
 #include "render.h"
 #include "renderprefs.h"
 #include "gtkboard.h"
 #include "gtkgame.h"
+#include "gtkwindows.h"
 
 #define SIZE_DIE 4
 #define SIZE_QUADRANT 40
@@ -97,7 +93,7 @@ TempMapEquities( evalcontext *pec, matchstate *pms,
 
   int i, j;
   float arOutput[ NUM_ROLLOUT_OUTPUTS ];
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   int aaan[ 6 ][ 6 ][ 8 ];
   float aar[ 6 ][ 6 ];
   cubeinfo ci;
@@ -135,7 +131,7 @@ TempMapEquities( evalcontext *pec, matchstate *pms,
       SwapSides ( anBoard );
       ci.fMove = ! ci.fMove;
 
-      if ( GeneralEvaluationE ( arOutput, anBoard, &ci, pec ) < 0 ) {
+      if ( GeneralEvaluationE ( arOutput, (ConstTanBoard)anBoard, &ci, pec ) < 0 ) {
         ProgressEnd();
         return -1;
       }
@@ -177,7 +173,7 @@ CalcTempMapEquities( evalcontext *pec, tempmapwidget *ptmw ) {
                           ptmw->atm[ i ].aarEquity, 
                           ptmw->atm[ i ].aaanMove,
                           ptmw->atm[ i ].szTitle,
-                          ptmw->atm[ i ].pms->nCube / ptmw->atm[ 0 ].pms->nCube ) < 0 ) 
+                          (float)(ptmw->atm[ i ].pms->nCube / ptmw->atm[ 0 ].pms->nCube) ) < 0 ) 
       return -1;
 
   return 0;
@@ -194,7 +190,7 @@ UpdateStyle( GtkWidget *pw, const float r ) {
 
   ps->bg[ GTK_STATE_NORMAL ].red = 0xFFFF;
   ps->bg[ GTK_STATE_NORMAL ].blue = 
-    ps->bg[ GTK_STATE_NORMAL ].green = ( 1.0 - r ) * 0xFFFF;
+  ps->bg[ GTK_STATE_NORMAL ].green = (guint16)(( 1.0f - r ) * 0xFFFF);
 
   gtk_widget_set_style( pw, ps ); 
 
@@ -206,7 +202,7 @@ SetStyle( GtkWidget *pw, const float rEquity,
           const float rMin, const float rMax, const int fInvert ) {
 
   float r = ( rEquity - rMin ) / ( rMax - rMin );
-  UpdateStyle( pw, fInvert ? ( 1.0 - r ) : r );
+  UpdateStyle( pw, fInvert ? ( 1.0f - r ) : r );
 
 }
 
@@ -219,7 +215,7 @@ GetEquityString( const float rEquity, cubeinfo *pci, const int fInvert ) {
   if ( fInvert ) {
     /* invert equity */
     if ( pci->nMatchTo )
-      r = 1.0 - rEquity;
+      r = 1.0f - rEquity;
     else
       r = -rEquity;
   }
@@ -282,7 +278,7 @@ UpdateTempMapEquities( tempmapwidget *ptmw ) {
           g_strdup_printf( "%s [%s]", 
                            GetEquityString( ptmw->atm[ m ].aarEquity[ i ][ j ],
                                             &ci, ptmw->fInvert ), 
-                           FormatMove( szMove, ptmw->atm[ m ].pms->anBoard, 
+                           FormatMove( szMove, (ConstTanBoard)ptmw->atm[ m ].pms->anBoard, 
                                        ptmw->atm[ m ].aaanMove[ i ][ j ] ) );
                            
 
@@ -367,7 +363,7 @@ ExposeQuadrant( GtkWidget *pw, GdkEventExpose *pev, tempmapwidget *ptmw ) {
 
     char szMove[ 100 ];
 
-    FormatMove( szMove, ptmw->atm[ m ].pms->anBoard, 
+    FormatMove( szMove, (ConstTanBoard)ptmw->atm[ m ].pms->anBoard, 
                 ptmw->atm[ m ].aaanMove[ i ][ j ] );
     gtk_draw_string( pw->style, pw->window, GTK_STATE_NORMAL,
                      10, 30, szMove );
@@ -408,8 +404,8 @@ ExposeDie( GtkWidget *pw, GdkEventExpose *pev,
       g_free( ptmw->achDice[ i ] );
       g_free( ptmw->achPips[ i ] );
 
-      ptmw->achDice[ i ] = g_malloc ( nSizeDie * nSizeDie * 7 * 7 * 4 );
-      ptmw->achPips[ i ] = g_malloc ( nSizeDie * nSizeDie * 3 );
+      ptmw->achDice[ i ] = (unsigned char*)g_malloc ( nSizeDie * nSizeDie * 7 * 7 * 4 );
+      ptmw->achPips[ i ] = (unsigned char*)g_malloc ( nSizeDie * nSizeDie * 3 );
     }
 
     RenderDice( &rd, ptmw->achDice[ 0 ], ptmw->achDice[ 1 ], nSizeDie * 7 * 4 );
@@ -530,11 +526,8 @@ GTKShowTempMap( const matchstate ams[], const int n,
 
   /* dialog */
 
-  pwDialog = GTKCreateDialog( _("Sho Sengoku Temperature Map - "
-                             "Distribution of rolls"), DT_INFO, NULL, NULL );
-  gtk_window_set_modal( GTK_WINDOW( pwDialog ), TRUE );
-  gtk_window_set_transient_for( GTK_WINDOW( pwDialog ),
-                                  GTK_WINDOW( pwMain ) );
+  pwDialog = GTKCreateDialog( _("Sho Sengoku Temperature Map - Distribution of rolls"),
+	  DT_INFO, NULL, DIALOG_FLAG_MODAL, NULL, NULL );
 
   ptmw = (tempmapwidget *) g_malloc( sizeof ( tempmapwidget ) );
   ptmw->fShowBestMove = FALSE;
@@ -557,7 +550,6 @@ GTKShowTempMap( const matchstate ams[], const int n,
   gtk_container_set_border_width ( GTK_CONTAINER ( pwv ), 8);
   gtk_container_add ( GTK_CONTAINER (DialogArea( pwDialog, DA_MAIN ) ), pwv );
 
-
   /* calculate number of rows and columns */
 
   for ( lm = 1; /**/; ++lm )
@@ -604,9 +596,7 @@ GTKShowTempMap( const matchstate ams[], const int n,
           
           ptm->aapwDA[ i ][ j ] = gtk_drawing_area_new();
           ptm->aapwe[ i ][ j ] = gtk_event_box_new();
-#if GTK_CHECK_VERSION(2,4,0)
 	gtk_event_box_set_visible_window(GTK_EVENT_BOX(ptm->aapwe[i][j]), FALSE);
-#endif
           
           gtk_container_add( GTK_CONTAINER( ptm->aapwe[ i ][ j ] ),
                              ptm->aapwDA[ i ][ j ] );
@@ -625,9 +615,9 @@ GTKShowTempMap( const matchstate ams[], const int n,
           gtk_object_set_data_full( GTK_OBJECT( ptm->aapwDA[ i ][ j ] ),
                                     "user_data", pi, g_free );
           
-          gtk_signal_connect( GTK_OBJECT( ptm->aapwDA[ i ][ j ] ),
+          g_signal_connect( G_OBJECT( ptm->aapwDA[ i ][ j ] ),
                               "expose_event",
-                              GTK_SIGNAL_FUNC( ExposeQuadrant ), ptmw );
+                              G_CALLBACK( ExposeQuadrant ), ptmw );
 
         }
 
@@ -646,9 +636,9 @@ GTKShowTempMap( const matchstate ams[], const int n,
         gtk_object_set_data_full( GTK_OBJECT( pw ),
                                   "user_data", pi, g_free );
         
-        gtk_signal_connect( GTK_OBJECT( pw ),
+        g_signal_connect( G_OBJECT( pw ),
                             "expose_event",
-                            GTK_SIGNAL_FUNC( ExposeDie ),
+                            G_CALLBACK( ExposeDie ),
                             ptmw );
         /* die */
     
@@ -665,9 +655,9 @@ GTKShowTempMap( const matchstate ams[], const int n,
         gtk_object_set_data_full( GTK_OBJECT( pw ),
                                   "user_data", pi, g_free );
         
-        gtk_signal_connect( GTK_OBJECT( pw ),
+        g_signal_connect( G_OBJECT( pw ),
                             "expose_event",
-                            GTK_SIGNAL_FUNC( ExposeDie ),
+                            G_CALLBACK( ExposeDie ),
                             ptmw );
     
       }
@@ -676,9 +666,7 @@ GTKShowTempMap( const matchstate ams[], const int n,
       
       ptm->pwAverage = gtk_drawing_area_new();
       ptm->pweAverage = gtk_event_box_new();
-#if GTK_CHECK_VERSION(2,4,0)
 	gtk_event_box_set_visible_window(GTK_EVENT_BOX(ptm->pweAverage), FALSE);
-#endif
 
       gtk_container_add( GTK_CONTAINER( ptm->pweAverage ), ptm->pwAverage );
 
@@ -695,9 +683,9 @@ GTKShowTempMap( const matchstate ams[], const int n,
       gtk_object_set_data_full( GTK_OBJECT( ptm->pwAverage ),
                                 "user_data", pi, g_free );
       
-      gtk_signal_connect( GTK_OBJECT( ptm->pwAverage ),
+      g_signal_connect( G_OBJECT( ptm->pwAverage ),
                           "expose_event",
-                          GTK_SIGNAL_FUNC( ExposeQuadrant ),
+                          G_CALLBACK( ExposeQuadrant ),
                           ptmw );
 
      }
@@ -724,11 +712,11 @@ GTKShowTempMap( const matchstate ams[], const int n,
     
     gtk_object_set_data( GTK_OBJECT( pw ), "user_data", NULL );
     
-    gtk_signal_connect( GTK_OBJECT( pw ),
+    g_signal_connect( G_OBJECT( pw ),
                         "expose_event",
-                        GTK_SIGNAL_FUNC( ExposeQuadrant ), NULL );
+                        G_CALLBACK( ExposeQuadrant ), NULL );
     
-    UpdateStyle( pw, 1.0 * i / 15.0 );
+    UpdateStyle( pw, 1.0f * i / 15.0f );
     
     
   }
@@ -767,8 +755,8 @@ GTKShowTempMap( const matchstate ams[], const int n,
     gtk_object_set_data_full( GTK_OBJECT( pw ),
                               "user_data", pi, g_free );
     
-    gtk_signal_connect( GTK_OBJECT( pw ), "toggled", 
-                        GTK_SIGNAL_FUNC( TempMapPlyToggled ), ptmw );
+    g_signal_connect( G_OBJECT( pw ), "toggled", 
+                        G_CALLBACK( TempMapPlyToggled ), ptmw );
                         
   }
 
@@ -783,15 +771,15 @@ GTKShowTempMap( const matchstate ams[], const int n,
   pw = gtk_check_button_new_with_label( _("Show equities") );
   gtk_box_pack_end( GTK_BOX( pwh ), pw, FALSE, FALSE, 0 );
   
-  gtk_signal_connect( GTK_OBJECT( pw ), "toggled",
-                      GTK_SIGNAL_FUNC( ShowEquityToggled ), ptmw );
+  g_signal_connect( G_OBJECT( pw ), "toggled",
+                      G_CALLBACK( ShowEquityToggled ), ptmw );
 
 
   pw = gtk_check_button_new_with_label( _("Show best move") );
   gtk_box_pack_end( GTK_BOX( pwh ), pw, FALSE, FALSE, 0 );
   
-  gtk_signal_connect( GTK_OBJECT( pw ), "toggled",
-                      GTK_SIGNAL_FUNC( ShowBestMoveToggled ), ptmw );
+  g_signal_connect( G_OBJECT( pw ), "toggled",
+                      G_CALLBACK( ShowBestMoveToggled ), ptmw );
 
 
   /* update */
@@ -804,6 +792,5 @@ GTKShowTempMap( const matchstate ams[], const int n,
   gtk_window_set_default_size( GTK_WINDOW( pwDialog ), 400, 500 ); 
   gtk_object_weakref( GTK_OBJECT( pwDialog ), DestroyDialog, ptmw );
 
-  gtk_widget_show_all( pwDialog );
-
+  GTKRunDialog(pwDialog);
 }
