@@ -4,7 +4,7 @@
  * by Joern Thyssen  <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,22 +16,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: html.c,v 1.172 2006/10/04 12:28:39 c_anthon Exp $
+ * $Id: html.c,v 1.215 2009/10/26 16:08:30 Superfly_Jon Exp $
  */
 
 #include "config.h"
 
-#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <time.h>
-#include <assert.h>
-#include <stdarg.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <stdlib.h>
-#include <glib.h>
 
 #include "analysis.h"
 #include "backgammon.h"
@@ -41,11 +35,15 @@
 #include "eval.h"
 #include "positionid.h"
 #include "matchid.h"
-#include "record.h"
-#include "path.h"
 #include "formatgs.h"
+#include "relational.h"
 
-#include <glib/gi18n.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+#ifdef WIN32
+#include <io.h>
+#endif
+#include "util.h"
 
 typedef enum _stylesheetclass {
   CLASS_MOVETABLE,
@@ -79,7 +77,7 @@ typedef enum _stylesheetclass {
   NUM_CLASSES 
 } stylesheetclass;
 
-static char *aaszStyleSheetClasses[ NUM_CLASSES ][ 2 ] = {
+static const char *aaszStyleSheetClasses[ NUM_CLASSES ][ 2 ] = {
   { "movetable", "background-color: #ddddee" },
   { "moveheader", "background-color: #89d0e2; padding: 0.5em" },
   { "movenumber", "width: 2em; text-align: right" },
@@ -121,36 +119,35 @@ static char *aaszStyleSheetClasses[ NUM_CLASSES ][ 2 ] = {
   { "cubecubelesstext", "font-style: italic" }
 };
 
-
 #define FORMATHTMLPROB(f) \
 ( ( f ) < 1.0f ) ? "&nbsp;" : "", \
 ( ( f ) < 0.1f ) ? "&nbsp;" : "", \
 ( f ) * 100.0f
    
 
-char *aszHTMLExportType[ NUM_HTML_EXPORT_TYPES ] = {
+const char *aszHTMLExportType[ NUM_HTML_EXPORT_TYPES ] = {
   "gnu",
   "bbs",
   "fibs2html" };
 
-char *aszHTMLExportCSS[ NUM_HTML_EXPORT_CSS ] = {
+const char *aszHTMLExportCSS[ NUM_HTML_EXPORT_CSS ] = {
   N_("in <head>"),
   N_("inline (inside tags)"),
   N_("external file (\"gnubg.css\")")
 };
 
-char *aszHTMLExportCSSCommand[ NUM_HTML_EXPORT_CSS ] = {
+const char *aszHTMLExportCSSCommand[ NUM_HTML_EXPORT_CSS ] = {
   "head", "inline", "external" };
 
 
 
 /* text for links on html page */
 
-static char *aszLinkText[] = {
-  N_ ("[First Game]"), 
-  N_ ("[Previous Game]"), 
-  N_ ("[Next Game]"), 
-  N_ ("[Last Game]") };
+static const char *aszLinkText[] = {
+  N_("[First Game]"), 
+  N_("[Previous Game]"), 
+  N_("[Next Game]"), 
+  N_("[Last Game]") };
 
 static const char*
 bullet = "&bull; ";
@@ -167,16 +164,16 @@ WriteStyleSheet ( FILE *pf, const htmlexportcss hecss ) {
 
     fputs( "\n"
            "/* CSS Stylesheet for " VERSION_STRING " */\n"
-           "/* $Id: html.c,v 1.172 2006/10/04 12:28:39 c_anthon Exp $ */\n",
+           "/* $Id: html.c,v 1.215 2009/10/26 16:08:30 Superfly_Jon Exp $ */\n",
            pf );
 
-    fputs( _("/* This file is distributed as a part of the "
+    fputs( "/* This file is distributed as a part of the "
              "GNU Backgammon program. */\n"
              "/* Copying and distribution of verbatim and modified "
              "versions of this file */\n"
              "/* is permitted in any medium provided the copyright notice "
              "and this */\n"
-             "/* permission notice are preserved. */\n\n"),
+             "/* permission notice are preserved. */\n\n",
            pf );
 
   for ( i = 0; i < NUM_CLASSES; ++i )
@@ -189,8 +186,8 @@ WriteStyleSheet ( FILE *pf, const htmlexportcss hecss ) {
   if ( hecss == HTML_EXPORT_CSS_HEAD )
     fputs ( "</style>\n", pf );
   else if ( hecss == HTML_EXPORT_CSS_EXTERNAL )
-    fputs( _("\n"
-             "/* end of file */\n"), pf );
+    fputs( "\n"
+             "/* end of file */\n", pf );
 
 }
 
@@ -277,27 +274,6 @@ GetStyleGeneral( const htmlexportcss hecss, ... ) {
 }
 
 
-static void
-HTMLPrintLegend( FILE *pf, const htmlexportcss hecss ) {
-
-  /* WRITE ME */
-
-
-}
-
-
-  
-#if 0
-static void
-WriteStyle ( FILE *pf, const stylesheetclass ssc, 
-             const htmlexportcss hecss ) {
-
-  fputs ( GetStyle ( ssc, hecss ), pf );
-
-}
-#endif
-
-
 static void
 printRolloutTable ( FILE *pf, 
                     char asz[][ 1024 ],
@@ -438,22 +414,15 @@ printRolloutTable ( FILE *pf,
 
 
 static void
-printStatTableHeader ( FILE *pf, const htmlexportcss hecss,
-                       const char *format, ... ) {
-
-  va_list val;
-
-  va_start( val, format );
+printStatTableHeader ( FILE *pf, const htmlexportcss hecss, const char *header) {
 
   fprintf ( pf, 
             "<tr %s>\n"
             "<th colspan=\"3\" style=\"text-align: center\">", 
             GetStyle ( CLASS_STATTABLEHEADER, hecss ) );
-  vfprintf ( pf, format, val );
+  fputs ( header, pf );
   fputs ( "</th>\n</tr>\n", pf );
 
-  va_end( val );
-
 }
 
 
@@ -463,7 +432,7 @@ printStatTableRow ( FILE *pf, const char *format1, const char *format2,
 
   va_list val;
   char *sz;
-  int l = 100 + strlen ( format1 ) + 2 * strlen ( format2 );
+  size_t l = 100 + strlen ( format1 ) + 2 * strlen ( format2 );
 
   va_start( val, format2 );
 
@@ -531,7 +500,7 @@ printPointBBS ( FILE *pf, const char *szImageDir, const char *szExtension,
 
   char sz[ 100 ];
   char szAlt[ 100 ];
-  char *aasz[ 2 ][ 2 ] = { { "dd", "dn" },
+  const char *aasz[ 2 ][ 2 ] = { { "dd", "dn" },
                            { "ud", "up" } };
 
   if ( iPoint0 ) {
@@ -567,8 +536,8 @@ printHTMLBoardBBS ( FILE *pf, matchstate *pms, int fTurn,
                     const char *szImageDir, const char *szExtension,
                     const htmlexportcss hecss ) {
 
-  int anBoard[ 2 ][ 25 ];
-  int anPips[ 2 ];
+  TanBoard anBoard;
+  unsigned int anPips[ 2 ];
   int acOff[ 2 ];
   int i, j;
   char sz[ 1024 ];
@@ -576,7 +545,7 @@ printHTMLBoardBBS ( FILE *pf, matchstate *pms, int fTurn,
   memcpy ( anBoard, pms->anBoard, sizeof ( anBoard ) );
 
   if ( pms->fMove ) SwapSides ( anBoard );
-  PipCount ( anBoard, anPips );
+  PipCount ( (ConstTanBoard)anBoard, anPips );
 
   for( i = 0; i < 2; i++ ) {
     acOff[ i ] = 15;
@@ -584,7 +553,7 @@ printHTMLBoardBBS ( FILE *pf, matchstate *pms, int fTurn,
       acOff[ i ] -= anBoard[ i ][ j ];
   }
 
-  PipCount ( anBoard, anPips );
+  PipCount ( (ConstTanBoard)anBoard, anPips );
 
   /* Begin table  and print for player 0*/
   fprintf ( pf, "<table><tr><th align=\"left\">%s<th align=\"right\">%d<tr><th align=\"center\" colspan=\"2\">", ap[ 0 ].szName,  anPips[ 1 ]);
@@ -743,8 +712,10 @@ printHTMLBoardBBS ( FILE *pf, matchstate *pms, int fTurn,
   fprintf( pf, "<span %s>", 
            GetStyle ( CLASS_POSITIONID, hecss ) );
 
-  fprintf ( pf, _("Position ID: <tt>%s</tt> Match ID: <tt>%s</tt><br /></span></table>\n"),
-            PositionID ( pms->anBoard ),
+  fprintf ( pf, "%s <tt>%s</tt> %s <tt>%s</tt><br /></span></table>\n",
+	  _("Position ID:"),
+            PositionID ( (ConstTanBoard)pms->anBoard ),
+	  _("Match ID:"),
             MatchIDFromMatchState ( pms ) );
 
 
@@ -816,13 +787,13 @@ printHTMLBoardF2H ( FILE *pf, matchstate *pms, int fTurn,
   int fColor;
   int i;
 
-  char *aszDieO[] = { "b-odie1", "b-odie2", "b-odie3",
+  const char *aszDieO[] = { "b-odie1", "b-odie2", "b-odie3",
                      "b-odie4", "b-odie5", "b-odie6" };
-  char *aszDieX[] = { "b-xdie1", "b-xdie2", "b-xdie3",
+  const char *aszDieX[] = { "b-xdie1", "b-xdie2", "b-xdie3",
                      "b-xdie4", "b-xdie5", "b-xdie6" };
 
-  int anBoard[ 2 ][ 25 ];
-  int anPips[ 2 ];
+  TanBoard anBoard;
+  unsigned int anPips[ 2 ];
 
   memcpy ( anBoard, pms->anBoard, sizeof ( anBoard ) );
 
@@ -1105,7 +1076,7 @@ printHTMLBoardF2H ( FILE *pf, matchstate *pms, int fTurn,
   printImage ( pf, szImageDir, "b-indent", szExtension, "", 
                hecss, HTML_EXPORT_TYPE_FIBS2HTML );
 
-  PipCount ( anBoard, anPips );
+  PipCount ( (ConstTanBoard)anBoard, anPips );
   fprintf ( pf, _("Pip counts: %s %d, %s %d<br />\n"),
             ap[ 0 ].szName, anPips[ 1 ], 
             ap[ 1 ].szName, anPips[ 0 ] );
@@ -1118,7 +1089,7 @@ printHTMLBoardF2H ( FILE *pf, matchstate *pms, int fTurn,
            GetStyle ( CLASS_POSITIONID, hecss ) );
 
   fprintf ( pf, _("Position ID: <tt>%s</tt> Match ID: <tt>%s</tt><br />\n"),
-            PositionID ( pms->anBoard ),
+            PositionID ( (ConstTanBoard)pms->anBoard ),
             MatchIDFromMatchState ( pms ) );
 
   fprintf( pf, "</span>" );
@@ -1192,8 +1163,8 @@ printHTMLBoardGNU ( FILE *pf, matchstate *pms, int fTurn,
   char szAlt[ 1000 ];
   int i, j;
 
-  int anBoard[ 2 ][ 25 ];
-  int anPips[ 2 ];
+  TanBoard anBoard;
+  unsigned int anPips[ 2 ];
   int acOff[ 2 ];
 
   memcpy ( anBoard, pms->anBoard, sizeof ( anBoard ) );
@@ -1592,7 +1563,7 @@ printHTMLBoardGNU ( FILE *pf, matchstate *pms, int fTurn,
 
   fputs ( "<p>", pf );
 
-  PipCount ( anBoard, anPips );
+  PipCount ( (ConstTanBoard)anBoard, anPips );
   fprintf ( pf, _("Pip counts: %s %d, %s %d<br />\n"),
             ap[ 0 ].szName, anPips[ 1 ], 
             ap[ 1 ].szName, anPips[ 0 ] );
@@ -1603,7 +1574,7 @@ printHTMLBoardGNU ( FILE *pf, matchstate *pms, int fTurn,
            GetStyle ( CLASS_POSITIONID, hecss ) );
 
   fprintf ( pf, _("Position ID: <tt>%s</tt> Match ID: <tt>%s</tt><br />\n"),
-            PositionID ( pms->anBoard ),
+            PositionID ( (ConstTanBoard)pms->anBoard ),
             MatchIDFromMatchState ( pms ) );
 
   fprintf( pf, "</span>" );
@@ -1701,7 +1672,7 @@ HTMLBoardHeader ( FILE *pf, const matchstate *pms,
 
     fprintf ( pf,
               _(" %s doubles to %d"),
-              ap[ pms->fMove ].szName,
+              ap[ !pms->fTurn ].szName,
               pms->nCube * 2
             );
 
@@ -1847,7 +1818,7 @@ HTMLEpilogue ( FILE *pf, const matchstate *pms, char *aszLinks[ 4 ],
   int fFirst;
   int i;
 
-  const char szVersion[] = "$Revision: 1.172 $";
+  const char szVersion[] = "$Revision: 1.215 $";
   int iMajor, iMinor;
 
   iMajor = atoi ( strchr ( szVersion, ' ' ) );
@@ -1927,7 +1898,7 @@ HTMLEpilogueComment ( FILE *pf ) {
 
   time_t t;
 
-  const char szVersion[] = "$Revision: 1.172 $";
+  const char szVersion[] = "$Revision: 1.215 $";
   int iMajor, iMinor;
   char *pc;
 
@@ -2002,12 +1973,12 @@ HTMLPrintCubeAnalysisTable ( FILE *pf,
 
   if ( pes->et == EVAL_NONE ) return; /* no evaluation */
 
-  cd = FindCubeDecision ( arDouble, GCCCONSTAHACK aarOutput, pci );
+  cd = FindCubeDecision ( arDouble,  aarOutput, pci );
 
   fActual = fDouble > 0;
   fClose = isCloseCubedecision ( arDouble ); 
   fMissed =  fDouble > -1 &&
-    isMissedDouble ( arDouble, GCCCONSTAHACK aarOutput, fDouble, pci );
+    isMissedDouble ( arDouble,  aarOutput, fDouble, pci );
 
   fDisplay = 
     ( fActual && exsExport.afCubeDisplay[ EXPORT_CUBE_ACTUAL ] ) ||
@@ -2234,7 +2205,7 @@ HTMLPrintCubeAnalysisTable ( FILE *pf,
 
   }
 
-  getCubeDecisionOrdering ( ai, arDouble, GCCCONSTAHACK aarOutput, pci );
+  getCubeDecisionOrdering ( ai, arDouble,  aarOutput, pci );
 
   for ( i = 0; i < 3; i++ ) {
 
@@ -2316,7 +2287,7 @@ HTMLPrintCubeAnalysisTable ( FILE *pf,
 
   if ( pes->et == EVAL_ROLLOUT && exsExport.afCubeParameters[ 1 ] ) {
 
-    char *sz = strdup ( OutputRolloutContext ( NULL, pes ) );
+    char *sz = g_strdup ( OutputRolloutContext ( NULL, &pes->rc ) );
     char *pcS = sz, *pcE;
 
     while ( ( pcE = strstr ( pcS, "\n" ) ) ) {
@@ -2364,6 +2335,9 @@ HTMLPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
                         const htmlexporttype het, const htmlexportcss hecss ) {
 
   cubeinfo ci;
+  /* we need to remember the double type to be able to do the right
+   * thing for beavers and racoons */
+  static doubletype dt = DT_NORMAL;
 
   GetMatchStateCubeInfo ( &ci, pms );
 
@@ -2378,11 +2352,18 @@ HTMLPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
                                  pmr->fPlayer,
                                  &pmr->CubeDecPtr->esDouble, &ci, FALSE, -1,
                                  pmr->stCube, SKILL_NONE, hecss );
+    dt = DT_NORMAL;
 
     break;
 
   case MOVE_DOUBLE:
 
+    dt = DoubleType ( pms->fDoubled, pms->fMove, pms->fTurn );
+    if (dt != DT_NORMAL)
+    {
+	    fprintf ( pf, "<p><span %s> Cannot analyse doubles nor raccoons!</span></p>\n", GetStyle ( CLASS_BLUNDER, hecss ));
+	    break;
+    }
     HTMLPrintCubeAnalysisTable ( pf, 
                                  pmr->CubeDecPtr->aarOutput, 
 				 pmr->CubeDecPtr->aarStdDev,
@@ -2398,6 +2379,12 @@ HTMLPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
 
     /* cube analysis from double, {take, drop, beaver} */
 
+    if (dt != DT_NORMAL)
+    {
+	    dt = DT_NORMAL;
+	    fprintf ( pf, "<p><span %s> Cannot analyse doubles nor raccoons!</span></p>\n", GetStyle ( CLASS_BLUNDER, hecss ));
+	    break;
+    }
     HTMLPrintCubeAnalysisTable ( pf, 
                                  pmr->CubeDecPtr->aarOutput, 
 				 pmr->CubeDecPtr->aarStdDev,
@@ -2411,7 +2398,7 @@ HTMLPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
 
   default:
 
-    assert ( FALSE );
+    g_assert ( FALSE );
 
 
   }
@@ -2439,7 +2426,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
                         const htmlexporttype het, const htmlexportcss hecss ) {
 
   char sz[ 64 ];
-  int i;
+  unsigned int i;
   float rEq, rEqTop;
 
   cubeinfo ci;
@@ -2455,7 +2442,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
 
   /* print alerts */
 
-  if ( badSkill(pmr->n.stMove) ) {
+  if ( badSkill(pmr->n.stMove) && pmr->n.iMove < pmr->ml.cMoves) {
 
     /* blunder or error */
 
@@ -2546,7 +2533,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
       /* move no */
 
       if ( i != pmr->n.iMove || i != pmr->ml.cMoves - 1 || 
-           pmr->ml.cMoves == 1 ) 
+           pmr->ml.cMoves == 1 || i < exsExport.nMoves - 1 ) 
         fprintf ( pf, 
                   "<td %s>%d</td>\n", 
                   GetStyle ( CLASS_MOVENUMBER, hecss ), i + 1 );
@@ -2580,7 +2567,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
       fprintf ( pf,
                 "<td %s>%s</td>\n",
                 GetStyle ( CLASS_MOVEMOVE, hecss ),
-                FormatMove ( sz, pms->anBoard,
+                FormatMove ( sz, (ConstTanBoard)pms->anBoard,
                              pmr->ml.amMoves[ i ].anMove ) );
 
       /* equity */
@@ -2688,9 +2675,9 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
 
           break;
 
-        case EVAL_ROLLOUT: {
-
-          char *sz = strdup ( OutputRolloutContext ( NULL, pes ) );
+        case EVAL_ROLLOUT:
+		{
+          char *sz = g_strdup ( OutputRolloutContext ( NULL, &pes->rc ) );
           char *pcS = sz, *pcE;
           
           while ( ( pcE = strstr ( pcS, "\n" ) ) ) {
@@ -2720,7 +2707,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
             
           }
 
-          free ( sz );
+          g_free ( sz );
 
           break;
 
@@ -2745,7 +2732,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
                 "<tr %s><td>&nbsp;</td><td>&nbsp;</td>"
                 "<td>&nbsp;</td><td>%s</td><td>&nbsp;</td></tr>\n",
                 GetStyle ( CLASS_MOVETHEMOVE, hecss ),
-                FormatMove ( sz, pms->anBoard, pmr->n.anMove ) );
+                FormatMove ( sz, (ConstTanBoard)pms->anBoard, pmr->n.anMove ) );
     else 
       /* no legal moves */
       /* FIXME: output equity?? */
@@ -2804,7 +2791,7 @@ HTMLAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
       fprintf ( pf,
                 _("%s%s moves %s"), bullet,
                 ap[ pmr->fPlayer ].szName,
-                FormatMove ( sz, pms->anBoard, pmr->n.anMove ) );
+                FormatMove ( sz, (ConstTanBoard)pms->anBoard, pmr->n.anMove ) );
     else if ( ! pmr->ml.cMoves )
       fprintf ( pf,
                 _("%s%s cannot move"), bullet,
@@ -2867,10 +2854,8 @@ HTMLAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
  */
 
 static void HTMLDumpStatcontext ( FILE *pf, const statcontext *psc,
-                                  matchstate *pms, const int iGame,
-                                  const htmlexportcss hecss ) {
-
-  const int fIsMatch = iGame < 0;
+                                  int nMatchTo, const int iGame,
+                                  const htmlexportcss hecss, const gchar *header ) {
 
   fprintf ( pf, "\n<!-- %s Statistics -->\n\n", 
             ( iGame >= 0 ) ? "Game" : "Match" );
@@ -2879,24 +2864,7 @@ static void HTMLDumpStatcontext ( FILE *pf, const statcontext *psc,
             "<table %s>\n",
             GetStyle ( CLASS_STATTABLE, hecss ) );
 
-  if ( iGame >= 0 ) {
-    printStatTableHeader ( pf,
-                           hecss, 
-                           _("Game statistics for game %d"),
-                           iGame + 1);
-  }
-  else {
-    
-    if ( pms->nMatchTo )
-      printStatTableHeader ( pf,
-                             hecss, 
-                             _( "Match statistics" ) );
-    else
-      printStatTableHeader ( pf,
-                             hecss,
-                             _( "Session statistics" ) );
-
-  }
+  printStatTableHeader ( pf, hecss, header); 
 
   fprintf ( pf,
             "<tr %s>\n"
@@ -2910,7 +2878,7 @@ static void HTMLDumpStatcontext ( FILE *pf, const statcontext *psc,
 
   if( psc->fMoves ) {
 
-    GList *list = formatGS( psc, pms->nMatchTo, fIsMatch, FORMATGS_CHEQUER );
+    GList *list = formatGS( psc, nMatchTo, FORMATGS_CHEQUER );
     GList *pl;
 
     printStatTableHeader ( pf, hecss, _("Checker play statistics") );
@@ -2932,7 +2900,7 @@ static void HTMLDumpStatcontext ( FILE *pf, const statcontext *psc,
 
   if( psc->fDice ) {
 
-    GList *list = formatGS( psc, pms->nMatchTo, fIsMatch, FORMATGS_LUCK );
+    GList *list = formatGS( psc, nMatchTo, FORMATGS_LUCK );
     GList *pl;
 
     printStatTableHeader ( pf, hecss, 
@@ -2955,7 +2923,7 @@ static void HTMLDumpStatcontext ( FILE *pf, const statcontext *psc,
 
   if( psc->fCube ) {
 
-    GList *list = formatGS( psc, pms->nMatchTo, fIsMatch, FORMATGS_CUBE );
+    GList *list = formatGS( psc, nMatchTo, FORMATGS_CUBE );
     GList *pl;
 
     printStatTableHeader ( pf, hecss, 
@@ -2978,7 +2946,7 @@ static void HTMLDumpStatcontext ( FILE *pf, const statcontext *psc,
 
   {
     
-    GList *list = formatGS( psc, pms->nMatchTo, fIsMatch, FORMATGS_OVERALL );
+    GList *list = formatGS( psc, nMatchTo, FORMATGS_OVERALL );
     GList *pl;
     
     printStatTableHeader ( pf, hecss, 
@@ -3010,40 +2978,7 @@ static void
 HTMLPrintComment ( FILE *pf, const moverecord *pmr,
                    const htmlexportcss hecss ) {
 
-  char *sz = NULL;
-
-  switch ( pmr->mt ) {
-
-  case MOVE_GAMEINFO:
-    sz = pmr->sz;
-    break;
-  case MOVE_DOUBLE:
-  case MOVE_TAKE:
-  case MOVE_DROP:
-    sz = pmr->sz;
-    break;
-  case MOVE_NORMAL:
-    sz = pmr->sz;
-    break;
-  case MOVE_RESIGN:
-    sz = pmr->sz;
-    break;
-  case MOVE_SETBOARD:
-    sz = pmr->sz;
-    break;
-  case MOVE_SETDICE:
-    sz = pmr->sz;
-    break;
-  case MOVE_SETCUBEVAL:
-    sz = pmr->sz;
-    break;
-  case MOVE_SETCUBEPOS:
-    sz = pmr->sz;
-    break;
-  case MOVE_TIME:
-	  /* ignore */
-	  break;
-  }
+  char *sz = pmr->sz;
 
   if ( sz ) {
 
@@ -3148,7 +3083,7 @@ HTMLMatchInfo ( FILE *pf, const matchinfo *pmi,
     tmx.tm_year = pmi->nYear - 1900;
     tmx.tm_mon = pmi->nMonth - 1;
     tmx.tm_mday = pmi->nDay;
-    strftime ( sz, sizeof ( sz ), _("%B %d, %Y"), &tmx );
+    strftime ( sz, sizeof ( sz ), "%B %d, %Y", &tmx );
     HTMLPrintMI( pf, _("Date"), sz );
   }
 
@@ -3166,92 +3101,6 @@ HTMLMatchInfo ( FILE *pf, const matchinfo *pmi,
 
 }
 
-static void
-HTMLDumpPlayerRecords ( FILE *pf, const htmlexportcss hecss ) {
-
-  /* dump the player records from file */
- 
-  playerrecord apr[ 2 ];
-  playerrecord *pr;
-  int i;
-  int f = FALSE;
-  int af[ 2 ] = { FALSE, FALSE };
-  
-  for ( i = 0; i < 2; ++i ) 
-    if ( ( pr = GetPlayerRecord ( ap[ i ].szName ) ) ) {
-      f = TRUE;
-      af[ i ] = TRUE;
-      memcpy ( &apr[ i ], pr, sizeof ( playerrecord ) );
-    }
-    else
-      memset ( &apr[ i ], 0, sizeof ( playerrecord ) );
-
-  if ( ! f )
-    return;
-
-  fputs ( "\n<!-- Player records -->\n\n", pf );
-
-  fprintf ( pf,
-            "<table %s>\n",
-            GetStyle ( CLASS_STATTABLE, hecss ) );
-
-  printStatTableHeader ( pf, hecss, _("Recorded player statistics") );
-  
-  fprintf ( pf,
-            "<tr %s>\n"
-            "<th>%s</th><th>%s</th><th>%s</th>\n"
-            "</tr>\n",
-            GetStyle ( CLASS_STATTABLEHEADER, hecss ),
-            _("Player"), 
-            ap[ 0 ].szName, ap[ 1 ].szName );
-
-  printStatTableRow ( pf,
-                      _("Chequer (Long-term error rate)"), "%5.3f",
-                      -apr[ 0 ].arErrorChequerplay[ EXPAVG_100 ],
-                      -apr[ 1 ].arErrorChequerplay[ EXPAVG_100 ] );
-
-  printStatTableRow ( pf,
-                      _("Cube (Long-term error rate)"), "%5.3f",
-                      -apr[ 0 ].arErrorCube[ EXPAVG_100 ],
-                      -apr[ 1 ].arErrorCube[ EXPAVG_100 ] );
-
-  printStatTableRow ( pf,
-                      _("Chequer (Short-term error rate)"), "%5.3f",
-                      -apr[ 0 ].arErrorChequerplay[ EXPAVG_20 ],
-                      -apr[ 1 ].arErrorChequerplay[ EXPAVG_20 ] );
-
-  printStatTableRow ( pf,
-                      _("Cube (Short-term error rate)"), "%5.3f",
-                      -apr[ 0 ].arErrorCube[ EXPAVG_20 ],
-                      -apr[ 1 ].arErrorCube[ EXPAVG_20 ] );
-
-  printStatTableRow ( pf,
-                      _("Chequer (total)"), "%5.3f",
-                      -apr[ 0 ].arErrorChequerplay[ TOTAL ],
-                      -apr[ 1 ].arErrorChequerplay[ TOTAL ] );
-
-  printStatTableRow ( pf,
-                      _("Cube (total)"), "%5.3f",
-                      -apr[ 0 ].arErrorCube[ TOTAL ],
-                      -apr[ 1 ].arErrorCube[ TOTAL ] );
-
-  printStatTableRow ( pf,
-                      _("Luck (total)"), "%5.3f",
-                      apr[ 0 ].arLuck[ TOTAL ],
-                      apr[ 1 ].arLuck[ TOTAL ] );
-
-  printStatTableRow ( pf,
-                      _("Games (total)"), "%d",
-                      apr[ 0 ].cGames,
-                      apr[ 1 ].cGames );
-
-  fprintf ( pf, "</table>\n" );
-
-  fprintf ( pf, "\n<!-- End Player Records -->\n\n" );
-
-}
-
-
 /*
  * Export a game in HTML
  *
@@ -3261,14 +3110,14 @@ HTMLDumpPlayerRecords ( FILE *pf, const htmlexportcss hecss ) {
  *
  */
 
-static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
+static void ExportGameHTML ( FILE *pf, listOLD *plGame, const char *szImageDir,
 			     const char *szExtension,
 			     const htmlexporttype het,
 			     const htmlexportcss hecss,
 			     const int iGame, const int fLastGame,
 			     char *aszLinks[ 4 ] )
 {
-    list *pl;
+    listOLD *pl;
     moverecord *pmr;
     matchstate msExport;
     matchstate msOrig;
@@ -3276,12 +3125,19 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
     statcontext *psc = NULL;
     static statcontext scTotal;
     xmovegameinfo *pmgi = NULL;
+    listOLD *pl_hint = NULL;
+    statcontext *psc_rel = NULL;
+
+    msOrig.nMatchTo = 0;
 
     if ( ! iGame )
       IniStatcontext ( &scTotal );
 
     updateStatisticsGame ( plGame );
 
+    if (game_is_last(plGame))
+	    pl_hint = game_add_pmr_hint(plGame);
+
     for( pl = plGame->plNext; pl != plGame; pl = pl->plNext ) {
 
       pmr = pl->p;
@@ -3331,6 +3187,7 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
 
         break;
 
+      case MOVE_DOUBLE:
       case MOVE_TAKE:
       case MOVE_DROP:
 
@@ -3360,6 +3217,9 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
 
     }
 
+    if (pl_hint)
+	    game_remove_pmr_hint(pl_hint);
+
     if( pmgi && pmgi->fWinner != -1 ) {
 
       /* print game result */
@@ -3374,22 +3234,26 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
     }
 
     if ( psc ) {
-      HTMLDumpStatcontext ( pf, psc, &msOrig, iGame, hecss );
-      HTMLDumpPlayerRecords ( pf, hecss );
+	    char *header = g_strdup_printf( _("Game statistics for game %d"), iGame + 1);
+      HTMLDumpStatcontext ( pf, psc, msOrig.nMatchTo, iGame, hecss, header );
+      g_free(header);
     }
 
 
     if ( fLastGame ) {
+	    const gchar *header;
 
       /* match statistics */
+    header = ms.nMatchTo ? _( "Match statistics" ) : _( "Session statistics" );
 
       fprintf ( pf, "<hr />\n" );
-      HTMLDumpStatcontext ( pf, &scTotal, &msOrig, -1, hecss );
-
-      /* legend */
-
-      if ( exsExport.fIncludeLegend )
-        HTMLPrintLegend( pf, hecss );
+      HTMLDumpStatcontext ( pf, &scTotal, msOrig.nMatchTo, -1, hecss, header );
+      psc_rel = relational_player_stats_get(ap[0].szName, ap[1].szName);
+      if (psc_rel)
+      {
+	      HTMLDumpStatcontext ( pf, psc_rel, 0, -1, hecss, _("Statistics from database") );
+	      g_free(psc_rel);
+      }
 
     }
 
@@ -3398,59 +3262,6 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
     
 }
 
-/*
- * get name number
- *
- * Input
- *    plGame: the game for which the game number is requested
- *
- * Returns:
- *    the game number
- *
- */
-
-extern int
-getGameNumber ( const list *plGame ) {
-
-  list *pl;
-  int iGame;
-
-  for( pl = lMatch.plNext, iGame = 0; pl != &lMatch; 
-       pl = pl->plNext, iGame++ )
-    if ( pl->p == plGame ) return iGame;
-
-  return -1;
-
-}
-
-
-/*
- * get move number
- *
- * Input
- *    plGame: the game 
- *    p: the move
- *
- * Returns:
- *    the move number
- *
- */
-
-extern int
-getMoveNumber ( const list *plGame, const void *p ) {
-
-  list *pl;
-  int iMove;
-
-  for( pl = plGame->plNext, iMove = 0; pl != plGame; 
-       pl = pl->plNext, iMove++ )
-    if ( p == pl->p ) return iMove;
-
-  return -1;
-
-}
-
-
 /*
  * Open file gnubg.css with same path as requested html file
  *
@@ -3469,13 +3280,12 @@ OpenCSS( const char *sz ) {
   
   pchCSS = g_build_filename(pchBase, "gnubg.css", NULL );
 
-  if ( !access( pchCSS, R_OK ) ) {
-    /* file exists */
+  if ( g_file_test(pchCSS, G_FILE_TEST_EXISTS) ) {
     outputf( _("gnubg.css is not written since it already exist in \"%s\"\n"),
              pchBase );
     pf = NULL;
   }
-  else if ( ! ( pf = fopen( pchCSS, "w" ) ) ) {
+  else if ( ! ( pf = g_fopen( pchCSS, "w" ) ) ) {
     outputerr( pchCSS );
   }
 
@@ -3488,6 +3298,20 @@ OpenCSS( const char *sz ) {
 }
 
 
+static void check_for_html_images(gchar *path)
+{
+	gchar *folder;
+	gchar *img;
+
+	folder  = g_path_get_dirname(path);
+	img = g_build_filename(folder, "html-images", NULL);
+	if (! g_file_test(img, G_FILE_TEST_EXISTS))
+		CommandExportHTMLImages(img);
+	g_free(img);
+	g_free(folder);
+}
+
+
 extern void CommandExportGameHtml( char *sz ) {
 
     FILE *pf;
@@ -3510,11 +3334,14 @@ extern void CommandExportGameHtml( char *sz ) {
 
     if( !strcmp( sz, "-" ) )
 	pf = stdout;
-    else if( !( pf = fopen( sz, "w" ) ) ) {
+    else if( !( pf = g_fopen( sz, "w" ) ) ) {
 	outputerr( sz );
 	return;
     }
 
+    if ( exsExport.het == HTML_EXPORT_TYPE_GNU )
+	    check_for_html_images(sz);
+
     ExportGameHTML( pf, plGame,
                     exsExport.szHTMLPictureURL, exsExport.szHTMLExtension, 
                     exsExport.het, exsExport.hecss,
@@ -3525,7 +3352,7 @@ extern void CommandExportGameHtml( char *sz ) {
     if( pf != stdout )
 	fclose( pf );
 
-    setDefaultFileName ( sz, PATH_HTML );
+    setDefaultFileName ( sz);
     
     /* external stylesheet */
 
@@ -3535,60 +3362,11 @@ extern void CommandExportGameHtml( char *sz ) {
 
 }
 
-/*
- * Get filename for html file:
- *
- * Number 0 gets the value of szBase.
- * Assume szBase is "blah.html", then number 1 will be
- * "blah_001.html", and so forth.
- *
- * Input:
- *   szBase: base filename
- *   iGame: game number
- *
- * Garbage collect:
- *   Caller must free returned pointer if not NULL
- * 
- */
-
-extern char *
-HTMLFilename ( const char *szBase, const int iGame ) {
-
-  if ( ! iGame )
-    return strdup ( szBase );
-  else {
-
-    char *sz = malloc ( strlen ( szBase ) + 5 );
-    char *szExtension = strrchr ( szBase, '.' );
-    char *pc;
-
-    if ( ! szExtension ) {
-
-      sprintf ( sz, "%s_%03d", szBase, iGame + 1 );
-      return sz;
-
-    }
-    else {
-
-      strcpy ( sz, szBase );
-      pc = strrchr ( sz, '.' );
-      
-      sprintf ( pc, "_%03d%s", iGame + 1, szExtension );
-
-      return sz;
-
-    }
-
-  }
-
-}
-
-
 
 extern void CommandExportMatchHtml( char *sz ) {
     
     FILE *pf;
-    list *pl;
+    listOLD *pl;
     int nGames;
     char *aszLinks[ 4 ], *filenames[4];
     char *szCurrent;
@@ -3602,6 +3380,9 @@ extern void CommandExportMatchHtml( char *sz ) {
 	return;
     }
 
+    if ( exsExport.het == HTML_EXPORT_TYPE_GNU )
+	    check_for_html_images(sz);
+
     /* Find number of games in match */
 
     for( pl = lMatch.plNext, nGames = 0; pl != &lMatch; 
@@ -3610,48 +3391,48 @@ extern void CommandExportMatchHtml( char *sz ) {
 
     for( pl = lMatch.plNext, i = 0; pl != &lMatch; pl = pl->plNext, i++ ) {
 
-      szCurrent = HTMLFilename ( sz, i );
-	  filenames[0] = HTMLFilename ( sz, 0 );
+      szCurrent = filename_from_iGame ( sz, i );
+	  filenames[0] = filename_from_iGame ( sz, 0 );
       aszLinks[ 0 ] = g_path_get_basename ( filenames[ 0 ] );
 	  filenames[ 1 ] = aszLinks[ 1 ] = NULL;
 	  if (i > 0) {
-		filenames[ 1 ] = HTMLFilename ( sz, i - 1 );
+		filenames[ 1 ] = filename_from_iGame ( sz, i - 1 );
 		aszLinks[ 1 ]  = g_path_get_basename ( filenames[ 1 ] );
 	  }
 		
 	  filenames[ 2 ] = aszLinks[ 2 ] = NULL;
 	  if (i < nGames - 1) {
-		filenames[ 2 ] = HTMLFilename ( sz, i + 1 );
+		filenames[ 2 ] = filename_from_iGame ( sz, i + 1 );
 		aszLinks[ 2 ]  = g_path_get_basename ( filenames[ 2 ] );
 	  }
 
 
 	  
-	  filenames[ 3 ] = HTMLFilename ( sz, nGames - 1 );
+	  filenames[ 3 ] = filename_from_iGame ( sz, nGames - 1 );
 	  aszLinks[ 3 ] = g_path_get_basename ( filenames[ 3 ] );
       if ( !i ) {
 
         if ( ! confirmOverwrite ( sz, fConfirmSave ) ) {
 		  for ( j = 0; j < 4; j++ )
-			free (filenames [ j ] );
+			g_free (filenames [ j ] );
 
-		  free ( szCurrent );
+		  g_free ( szCurrent );
           return;
 		}
 
-        setDefaultFileName ( sz, PATH_HTML );
+        setDefaultFileName ( sz);
 
       }
 
 
       if( !strcmp( szCurrent, "-" ) )
 	pf = stdout;
-      else if( !( pf = fopen( szCurrent, "w" ) ) ) {
+      else if( !( pf = g_fopen( szCurrent, "w" ) ) ) {
 	outputerr( szCurrent );
 		for ( j = 0; j < 4; j++ )
-		  free (filenames [ j ] );
+		  g_free (filenames [ j ] );
 
-		free ( szCurrent );
+		g_free ( szCurrent );
 	return;
       }
 
@@ -3662,9 +3443,9 @@ extern void CommandExportMatchHtml( char *sz ) {
                        aszLinks );
 
       for ( j = 0; j < 4; j++ )
-        free (filenames [ j ] );
+        g_free (filenames [ j ] );
 
-      free ( szCurrent );
+      g_free ( szCurrent );
 
       if( pf != stdout )
         fclose( pf );
@@ -3684,7 +3465,7 @@ extern void CommandExportPositionHtml( char *sz ) {
 
     FILE *pf;
     int fHistory;
-    moverecord *pmr = getCurrentMoveRecord ( &fHistory );
+    moverecord *pmr;
     int iMove;
 	
     sz = NextToken( &sz );
@@ -3699,17 +3480,20 @@ extern void CommandExportPositionHtml( char *sz ) {
 		 "position html').") );
 	return;
     }
-
+    pmr = get_current_moverecord ( &fHistory );
     if ( ! confirmOverwrite ( sz, fConfirmSave ) )
       return;
 
     if( !strcmp( sz, "-" ) )
 	pf = stdout;
-    else if( !( pf = fopen( sz, "w" ) ) ) {
+    else if( !( pf = g_fopen( sz, "w" ) ) ) {
 	outputerr( sz );
 	return;
     }
 
+    if ( exsExport.het == HTML_EXPORT_TYPE_GNU )
+	    check_for_html_images(sz);
+
     HTMLPrologue ( pf, &ms, getGameNumber ( plGame ), NULL, 
                    exsExport.het, exsExport.hecss );
 
@@ -3746,7 +3530,7 @@ extern void CommandExportPositionHtml( char *sz ) {
     if( pf != stdout )
 	fclose( pf );
 
-    setDefaultFileName ( sz, PATH_HTML );
+    setDefaultFileName ( sz);
 
     /* external stylesheet */
 
@@ -3761,8 +3545,14 @@ static void
 ExportPositionGammOnLine( FILE *pf )
 {
     int fHistory;
-    moverecord *pmr = getCurrentMoveRecord ( &fHistory );
     int iMove;
+    moverecord *pmr = get_current_moverecord ( &fHistory );
+
+    if (!pmr)
+    {
+	    outputerrf(_("Unable to export this position"));
+	    return;
+    }
 
     fputs ( "\n<!-- Score -->\n\n", pf );
 
@@ -3824,8 +3614,7 @@ ExportPositionGammOnLine( FILE *pf )
 }
 
 
-extern void
-CommandExportPositionGammOnLine ( char *sz ) {
+extern void CommandExportPositionGammOnLine ( char *sz ) {
 
     FILE *pf;
     
@@ -3847,7 +3636,7 @@ CommandExportPositionGammOnLine ( char *sz ) {
 
     if( !strcmp( sz, "-" ) )
 	pf = stdout;
-    else if( !( pf = fopen( sz, "w" ) ) ) {
+    else if( !( pf = g_fopen( sz, "w" ) ) ) {
 	outputerr( sz );
 	return;
     }
@@ -3860,24 +3649,21 @@ CommandExportPositionGammOnLine ( char *sz ) {
 }
 
 
-extern void
-CommandExportPositionGOL2Clipboard( char *sz ) {
-
+extern void CommandExportPositionGOL2Clipboard( char *sz )
+{
     char *szClipboard;
     long l;
-    FILE *pf;
+	FILE *pf;
+	char *tmpFile;
 
     if( ms.gs == GAME_NONE ) {
       outputl( _("No game in progress (type `new game' to start one).") );
       return;
     }
     
-    /* get tmp file name */
+    /* get tmp file */
 
-    if ( ! ( pf = tmpfile() ) ) {
-      outputerr("temporary file");
-      return;
-    }
+	pf = GetTemporaryFile(NULL, &tmpFile);
 
     /* generate file */
 
@@ -3901,7 +3687,13 @@ CommandExportPositionGOL2Clipboard( char *sz ) {
 
     szClipboard = (char *) malloc ( l + 1 );
 
-    fread( szClipboard, 1, l + 1, pf );
+    if (fread( szClipboard, 1, l, pf ) != (unsigned long) l)
+    {
+	    outputerr("temporary file");
+	    free(szClipboard);
+	    fclose(pf);
+    }
+
     szClipboard[ l ] = 0;
 
     TextToClipboard( szClipboard );
@@ -3909,6 +3701,7 @@ CommandExportPositionGOL2Clipboard( char *sz ) {
     free( szClipboard );
 
     fclose( pf );
-
+    g_unlink(tmpFile);
+    g_free(tmpFile);
 }
 
