@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 1997-2002.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,36 +16,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: htmlimages.c,v 1.34 2006/06/27 22:23:31 c_anthon Exp $
+ * $Id: htmlimages.c,v 1.51 2009/10/26 16:12:26 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include "config.h"
 #include <errno.h>
 
-#if HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
-#include <stdio.h>
 #include <string.h>
-
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
 #include <stdlib.h>
 #include <glib.h>
+#ifdef WIN32
+#include <io.h>
+#endif
 
 #include "backgammon.h"
 #include "export.h"
-#include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include "render.h"
 #include "renderprefs.h"
 #include "boardpos.h"
@@ -113,7 +103,7 @@ static void DrawArrow(int side, int player)
 #if USE_GTK
 {
 	int x, y;
-	ArrowPosition(side /* rd.fClockwise */, s, &x, &y);
+	ArrowPosition(side /* rd.fClockwise */, player, s, &x, &y);
 
 	AlphaBlendClip2( auchMidlb, boardStride,
 				x, y,
@@ -159,7 +149,7 @@ static void DrawPips(unsigned char *auchDest, int nStride,
 	}
 }
 
-static void WriteBorder(char* file, unsigned char *auchSrc, unsigned char *auchBoardSrc)
+static void WriteBorder(const char* file, unsigned char *auchSrc, unsigned char *auchBoardSrc)
 {
 	CopyArea(auchLabel, boardStride, auchBoardSrc, boardStride,
 				BOARD_WIDTH * s, BORDER_HEIGHT * s);
@@ -173,7 +163,7 @@ static void WriteBorder(char* file, unsigned char *auchSrc, unsigned char *auchB
 	Write(file, auchLabel, BOARD_WIDTH * s, BORDER_HEIGHT * s);
 }
 
-static void WriteImages()
+static void WriteImages(void)
 {
 	int i, j, k;
 	imagesWritten = 0;
@@ -611,7 +601,7 @@ static void WriteImages()
 			imagesWritten, NUM_IMAGES);
 }
 
-static void RenderObjects()
+static void RenderObjects(void)
 {
 	int clockwise;
 	renderdata rd;
@@ -619,14 +609,18 @@ static void RenderObjects()
 
 	rd.fLabels = TRUE; 
 	rd.nSize = s;
+#if USE_BOARD3D
+	/* Use 2d colours for dice */
+	rd.fDisplayType = DT_2D;
+#endif
 
 	RenderBoard( &rd, auchBoard, boardStride );
 	RenderChequers( &rd, auchChequer[ 0 ], auchChequer[ 1 ], asRefract[ 0 ],
 			asRefract[ 1 ], CHEQUER_WIDTH * s * 4 );
 	RenderChequerLabels( &rd, auchChequerLabels, CHEQUER_LABEL_WIDTH * s * 3 );
 
-#if USE_GTK
-	RenderArrows( &rd, auchArrow[0], auchArrow[1], s * ARROW_WIDTH * 4 );
+#if USE_GTK && HAVE_CAIRO
+	RenderArrows( &rd, auchArrow[0], auchArrow[1], s * ARROW_WIDTH * 4, rd.fClockwise );
 #endif
 
 	/* Render numbers in both directions */
@@ -637,12 +631,13 @@ static void RenderObjects()
 	RenderBoardLabels(&rd, auchLoRev, auchHiRev, BOARD_WIDTH * s * 4);
 	rd.fClockwise = clockwise;
 
-	/* cubes and dices are rendered a bit smaller */
+	/* cubes and dice are rendered a bit smaller */
 	rd.nSize = ss;
 
 	RenderCube( &rd, auchCube, CUBE_WIDTH * ss * 4 );
 	RenderCubeFaces( &rd, auchCubeFaces, CUBE_LABEL_WIDTH * ss * 3, auchCube, CUBE_WIDTH * ss * 4 );
-	RenderDice( &rd, auchDice[ 0 ], auchDice[ 1 ], DIE_WIDTH * ss * 4 );
+
+	RenderDice( &rd, auchDice[ 0 ], auchDice[ 1 ], DIE_WIDTH * ss * 4, TRUE);
 	RenderPips( &rd, auchPips[ 0 ], auchPips[ 1 ], ss * 3 );
 }
 
@@ -652,16 +647,12 @@ static char* GetFilenameBase(char* sz)
 
 	if( !sz || !*sz )
 	{
-		outputf( _("You must specify a file to export to (see `%s')\n" ),
+		outputf( _("You must specify a directory to export to (see `%s')\n" ),
 			"help export htmlimages" );
 		return 0;
 	}
 
-	if( mkdir( sz
-#ifndef WIN32
-		, 0777
-#endif
-		) < 0 && errno != EEXIST )
+	if (!g_file_test(sz, G_FILE_TEST_IS_DIR) && g_mkdir( sz, 0777) < 0 )
 	{
 		outputerr ( sz );
 		return 0;
@@ -678,7 +669,7 @@ static char* GetFilenameBase(char* sz)
 	return szFile;
 }
 
-static void AllocObjects()
+static void AllocObjects(void)
 {
 	int i, j, k;
 
@@ -719,7 +710,7 @@ static void AllocObjects()
 		asRefract[i] = malloc((CHEQUER_WIDTH * s * CHEQUER_HEIGHT * s) * sizeof(short));
 }
 
-static void TidyObjects()
+static void TidyObjects(void)
 {
 	int i, j, k;
 	free(auchMidlb);
