@@ -4,7 +4,7 @@
  * by ystein Johansen, 2000
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,18 +15,10 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * $Id: import.h,v 1.9 2005/02/21 23:23:08 jsegrave Exp $
+ * $Id: import.h,v 1.15 2007/11/16 21:13:18 c_anthon Exp $
  */
 
 #ifndef _IMPORT_H_
 #define _IMPORT_H_
 
-extern int ImportJF( FILE * fp, char *szFileName);
-extern int ImportMat( FILE *fp, char *szFilename );
-extern int ImportOldmoves( FILE *pf, char *szFilename );
-extern int ImportSGG( FILE *pf, char *szFilename );
-extern int ImportTMG( FILE *pf, const char *szFilename );
-extern int ImportBKG( FILE *pf, const char *szFilename );
-extern int ImportSnowieTxt( FILE *pf );
-
 #endif
