@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 2001, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: latex.c,v 1.34 2006/07/04 16:46:15 c_anthon Exp $
+ * $Id: latex.c,v 1.43 2008/02/25 22:18:50 c_anthon Exp $
  */
 
 #include "config.h"
@@ -351,8 +351,8 @@ static void PrintLaTeXComment( FILE *pf, char *pch ) {
 
 static void 
 PrintLaTeXCubeAnalysis( FILE *pf, const matchstate* pms, int fPlayer,
-                        const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
-                        const float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
+                        float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
+                        float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ], 
                         const evalsetup *pes ) { 
   
     cubeinfo ci;
@@ -375,13 +375,15 @@ static char *PlayerSymbol( int fPlayer ) {
     return fPlayer ? "\\textbullet{}" : "\\textopenbullet{}";
 }
 
-static void ExportGameLaTeX( FILE *pf, list *plGame ) {
+static void ExportGameLaTeX( FILE *pf, listOLD *plGame ) {
     
-    list *pl;
+    listOLD *pl;
     moverecord *pmr;
     matchstate msExport;
-    int fTook = FALSE, i;
+    int fTook = FALSE;
+	unsigned int i;
     char sz[ 1024 ];
+    doubletype dt = DT_NORMAL;
 
     updateStatisticsGame ( plGame );
 
@@ -390,6 +392,7 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
         FixMatchState ( &msExport, pmr );
 	switch( pmr->mt ) {
 	case MOVE_GAMEINFO:
+		dt = DT_NORMAL;
             fputs ( "\\noindent{\\Large ", pf );
 	    if( pmr->g.nMatch )
 		fprintf( pf, _("%d point match (game %d)"), 
@@ -415,6 +418,7 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
 	    break;
 	    
 	case MOVE_NORMAL:
+	    dt = DT_NORMAL;
 	    msExport.fTurn = msExport.fMove = pmr->fPlayer;
 	    if( fTook )
 		/* no need to print board following a double/take */
@@ -423,15 +427,15 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
 		PrintLaTeXBoard( pf, &msExport, pmr->fPlayer );
 	    
 	    PrintLaTeXCubeAnalysis( pf, &msExport, pmr->fPlayer,
-				    GCCCONSTAHACK pmr->CubeDecPtr->aarOutput,
-				    GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+				     pmr->CubeDecPtr->aarOutput,
+				     pmr->CubeDecPtr->aarStdDev,
                                     &pmr->CubeDecPtr->esDouble );
             /* FIXME: output cube skill */
 
 	    sprintf( sz, "%s %d%d%s: ", PlayerSymbol( pmr->fPlayer ),
 		     pmr->anDice[ 0 ], pmr->anDice[ 1 ],
 		     aszLuckTypeLaTeXAbbr[ pmr->lt ] );
-	    FormatMove( strchr( sz, 0 ), msExport.anBoard, pmr->n.anMove );
+	    FormatMove( strchr( sz, 0 ), (ConstTanBoard)msExport.anBoard, pmr->n.anMove );
 	    fprintf( pf, "\\begin{center}%s%s\\end{center}\n\n", sz,
 		     aszSkillTypeAbbr[ pmr->n.stMove ] );
 
@@ -455,11 +459,17 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
 	    break;
 	    
 	case MOVE_DOUBLE:
+    dt = DoubleType ( msExport.fDoubled, msExport.fMove, msExport.fTurn );
+    if (dt != DT_NORMAL)
+    {
+	    fprintf ( pf, "\\begin{center}\\emph{Cannot analyse doubles nor raccoons!}\\end{center}");
+	    break;
+    }
 	    PrintLaTeXBoard( pf, &msExport, pmr->fPlayer );
 
 	    PrintLaTeXCubeAnalysis(pf, &msExport, pmr->fPlayer,
-				   GCCCONSTAHACK pmr->CubeDecPtr->aarOutput, 
-				   GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev, 
+				    pmr->CubeDecPtr->aarOutput, 
+				    pmr->CubeDecPtr->aarStdDev, 
 				   &pmr->CubeDecPtr->esDouble );
 
 	    /* FIXME what about beavers? */
@@ -473,6 +483,12 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
 	    break;
 	    
 	case MOVE_TAKE:
+    if (dt != DT_NORMAL)
+    {
+	    dt = DT_NORMAL;
+	    fprintf ( pf, "\\begin{center}\\emph{Cannot analyse doubles nor raccoons!}\\end{center}");
+	    break;
+    }
 	    fTook = TRUE;
 	    fprintf( pf, "\\begin{center}%s %s%s\\end{center}\n\n",
 		     PlayerSymbol( pmr->fPlayer ),
@@ -484,6 +500,12 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
 	    break;
 	    
 	case MOVE_DROP:
+    if (dt != DT_NORMAL)
+    {
+	    dt = DT_NORMAL;
+	    fprintf ( pf, "\\begin{center}\\emph{Cannot analyse doubles nor raccoons!}\\end{center}");
+	    break;
+    }
 	    fprintf( pf, "\\begin{center}%s %s%s\\end{center}\n\n",
 		     PlayerSymbol( pmr->fPlayer ),
                      _("Drop"),
@@ -506,10 +528,6 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
 	    /* ignore */
 	    break;
 
-	case MOVE_TIME:
-		/* ignore */
-		break;
-	    
 	case MOVE_SETBOARD:
 	case MOVE_SETCUBEVAL:
 	case MOVE_SETCUBEPOS:
@@ -520,9 +538,7 @@ static void ExportGameLaTeX( FILE *pf, list *plGame ) {
 	ApplyMoveRecord( &msExport, plGame, pmr );
     }
     
-    if( ( GameStatus( msExport.anBoard, msExport.bgv ) ) )
 	/* FIXME print game result */
-	;
 }
 
 extern void CommandExportGameLaTeX( char *sz ) {
@@ -561,14 +577,14 @@ extern void CommandExportGameLaTeX( char *sz ) {
     if( pf != stdout )
 	fclose( pf );
 
-    setDefaultFileName ( sz, PATH_LATEX );
+    setDefaultFileName ( sz);
 
 }
 
 extern void CommandExportMatchLaTeX( char *sz ) {
     
     FILE *pf;
-    list *pl;
+    listOLD *pl;
 
     sz = NextToken( &sz );
     
@@ -600,6 +616,6 @@ extern void CommandExportMatchLaTeX( char *sz ) {
     if( pf != stdout )
 	fclose( pf );
 
-    setDefaultFileName ( sz, PATH_LATEX );
+    setDefaultFileName ( sz);
 
 }
