@@ -10,7 +10,7 @@
  * Copyright (C) 1995, 1996, 1997, 1999, 2000 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -22,18 +22,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: md5.c,v 1.4 2006/09/21 22:24:45 Superfly_Jon Exp $
+ * $Id: md5.c,v 1.9 2007/07/02 12:46:38 ace Exp $
  */
 
 /* License changed from the GNU LGPL to the GNU GPL (as permitted
    under Term 3 of the GNU LGPL) by Gary Wong for distribution
    with GNU Backgammon. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
-
-#include <sys/types.h>
+#include "config.h"
 
 #if STDC_HEADERS
 # include <stdlib.h>
@@ -63,6 +59,7 @@
 /* This array contains the bytes used to pad the buffer to the next
    64-byte boundary.  (RFC 1321, 3.1: Step 1)  */
 /* But this isn't used ???? */
+/*lint -e{785}  Ignore not enough initilizer warning */
 static const unsigned char fillbuf[64] = { 0x80, 0 /* , 0, 0, ...  */ };
 
 
@@ -88,7 +85,7 @@ md5_init_ctx (struct md5_ctx *ctx)
 void *
 md5_read_ctx (const struct md5_ctx *ctx, void *resbuf)
 {
-  char *r = resbuf;
+  char *r = (char*)resbuf;
   WRITE (r, ctx->A);
   WRITE (r + 4, ctx->B);
   WRITE (r + 8, ctx->C);
@@ -103,9 +100,7 @@ md5_read_ctx (const struct md5_ctx *ctx, void *resbuf)
    IMPORTANT: On some systems it is required that RESBUF is correctly
    aligned for a 32 bits value.  */
 void *
-md5_finish_ctx (ctx, resbuf)
-     struct md5_ctx *ctx;
-     void *resbuf;
+md5_finish_ctx (struct md5_ctx *ctx, void *resbuf)
 {
   /* Take yet unprocessed bytes into account.  */
   md5_uint32 bytes = ctx->buflen;
@@ -134,9 +129,7 @@ md5_finish_ctx (ctx, resbuf)
    resulting message digest number will be written into the 16 bytes
    beginning at RESBLOCK.  */
 int
-md5_stream (stream, resblock)
-     FILE *stream;
-     void *resblock;
+md5_stream (FILE *stream, void *resblock)
 {
   /* Important: BLOCKSIZE must be a multiple of 64.  */
 #define BLOCKSIZE 4096
@@ -148,7 +141,7 @@ md5_stream (stream, resblock)
   md5_init_ctx (&ctx);
 
   /* Iterate over full file contents.  */
-  while (1)
+  for(;;)
     {
       /* We read the file in blocks of BLOCKSIZE bytes.  One call of the
 	 computation function processes the whole buffer so that with the
@@ -157,7 +150,7 @@ md5_stream (stream, resblock)
       sum = 0;
 
       /* Read block.  Take care for partial reads.  */
-      do
+	do
 	{
 	  n = fread (buffer + sum, 1, BLOCKSIZE - sum, stream);
 
@@ -169,7 +162,7 @@ md5_stream (stream, resblock)
 
       /* If end of file is reached, end the loop.  */
       if (n == 0)
-	break;
+		break;
 
       /* Process buffer with BLOCKSIZE bytes.  Note that
 			BLOCKSIZE % 64 == 0
@@ -191,10 +184,7 @@ md5_stream (stream, resblock)
    output yields to the wanted ASCII representation of the message
    digest.  */
 void *
-md5_buffer (buffer, len, resblock)
-     const char *buffer;
-     size_t len;
-     void *resblock;
+md5_buffer (const char *buffer, size_t len, void *resblock)
 {
   struct md5_ctx ctx;
 
@@ -210,10 +200,7 @@ md5_buffer (buffer, len, resblock)
 
 
 void
-md5_process_bytes (buffer, len, ctx)
-     const void *buffer;
-     size_t len;
-     struct md5_ctx *ctx;
+md5_process_bytes (const void *buffer, size_t len, struct md5_ctx *ctx)
 {
   /* const void aligned_buffer = buffer; */
 
@@ -231,14 +218,14 @@ md5_process_bytes (buffer, len, ctx)
       ctx->buflen += (md5_uint32)add;
 
       if (ctx->buflen > 64)
-	{
-	  md5_process_block (ctx->buffer, ctx->buflen & ~63, ctx);
+      {
+		md5_process_block (ctx->buffer, ctx->buflen & ~63, ctx);
 
-	  ctx->buflen &= 63;
-	  /* The regions in the following copy operation cannot overlap.  */
-	  memcpy (ctx->buffer, &ctx->buffer[(left_over + add) & ~63],
-		  ctx->buflen);
-	}
+		ctx->buflen &= 63;
+		/* The regions in the following copy operation cannot overlap.  */
+		memcpy (ctx->buffer, &ctx->buffer[(left_over + add) & ~63],
+			ctx->buflen);
+      }
 
       buffer = (const char *) buffer + add;
       len -= add;
@@ -260,11 +247,11 @@ md5_process_bytes (buffer, len, ctx)
       memcpy (&ctx->buffer[left_over], buffer, len);
       left_over += len;
       if (left_over >= 64)
-	{
-	  md5_process_block (ctx->buffer, 64, ctx);
-	  left_over -= 64;
-	  memcpy (ctx->buffer, &ctx->buffer[64], left_over);
-	}
+      {
+		md5_process_block (ctx->buffer, 64, ctx);
+		left_over -= 64;
+		memcpy (ctx->buffer, &ctx->buffer[64], left_over);
+      }
       ctx->buflen = (md5_uint32)left_over;
     }
 }
@@ -283,13 +270,10 @@ md5_process_bytes (buffer, len, ctx)
    It is assumed that LEN % 64 == 0.  */
 
 void
-md5_process_block (buffer, len, ctx)
-     const void *buffer;
-     size_t len;
-     struct md5_ctx *ctx;
+md5_process_block (const void *buffer, size_t len, struct md5_ctx *ctx)
 {
   md5_uint32 correct_words[16];
-  const md5_uint32 *words = buffer;
+  const md5_uint32 *words = (const md5_uint32 *)buffer;
   size_t nwords = len / sizeof (md5_uint32);
   const md5_uint32 *endp = words + nwords;
   md5_uint32 A = ctx->A;
@@ -342,6 +326,7 @@ md5_process_block (buffer, len, ctx)
        */
 
       /* Round 1.  */
+	  /*lint --e{123, 717}  Ignore macro warning */
       OP (A, B, C, D,  7, 0xd76aa478);
       OP (D, A, B, C, 12, 0xe8c7b756);
       OP (C, D, A, B, 17, 0x242070db);
