@@ -44,11 +44,11 @@
 
    (1) #UNDEF main(), and other unused functions.
    (2) make mt and mti parameters
-   $Id: mt19937ar.c,v 1.3 2005/02/21 23:23:08 jsegrave Exp $
+   $Id: mt19937ar.c,v 1.6 2007/12/12 23:08:22 Superfly_Jon Exp $
 */
 
-#include <stdio.h>
-
+/* No configuration used in this file
+#include "config.h" */
 #include "mt19937ar.h"
 /* Period parameters */  
 /* #define N 624 */
@@ -67,7 +67,7 @@ void init_genrand(unsigned long s, int *mti, unsigned long mt[ N ] )
     mt[0]= s & 0xffffffffUL;
     for (*mti=1; *mti<N; (*mti)++) {
         mt[*mti] = 
-	    (1812433253UL * (mt[*mti-1] ^ (mt[*mti-1] >> 30)) + *mti); 
+	    /*lint --e(737) */ (1812433253UL * (mt[*mti-1] ^ (mt[*mti-1] >> 30)) + *mti); 
         /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
         /* In the previous versions, MSBs of the seed affect   */
         /* only MSBs of the array mt[].                        */
