@@ -4,7 +4,7 @@
  * by Gary Wong <gary@cs.arizona.edu>, 1997-1999.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,28 +16,28 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: makebearoff.c,v 1.36 2006/09/24 06:22:20 kaoru Exp $
+ * $Id: makebearoff.c,v 1.66 2008/03/18 22:51:34 c_anthon Exp $
  */
 
 #include "config.h"
 
-#include <assert.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <time.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <sys/types.h>
 #include <math.h>
 #include <errno.h>
-#include <stdarg.h>
 #include "eval.h"
 #include "positionid.h"
-#include "getopt.h"
-#include <glib/gi18n.h>
 #include "bearoff.h"
+#include "util.h"
+#include "backgammon.h"
+#include "multithread.h"
+#include <glib/gi18n.h>
 
 #if WIN32
 #include <windows.h>
@@ -50,6 +50,25 @@ HWND hdlg;
 int CancelPressed = FALSE;
 #endif
 
+#if USE_MULTITHREAD
+extern int MT_GetThreadID(void)
+{
+  return (0);
+}
+extern void MT_Release(void)
+{
+  return;
+}
+extern void MT_Exclusive(void)
+{
+  return;
+}
+#else
+extern void CallbackProgress(void)
+{
+}
+#endif
+
 typedef struct _xhashent {
   void *p;
   unsigned int iKey;
@@ -61,12 +80,6 @@ typedef struct _xhash {
   xhashent *phe;
 } xhash;
 
-/* ugly fixes */
-char *aszRNG[]; 
-char *aszSkillType[ 1 ]; 
-int exsExport;
-int ap;
-/* end ugly fixes */
 
 #if WIN32
 static void
@@ -185,8 +198,7 @@ XhashAdd ( xhash *ph, const unsigned int iKey,
 
 
 static void *
-XhashLookup ( xhash *ph, const unsigned int iKey,
-             void **data ) {
+XhashLookup ( xhash *ph, const unsigned int iKey ) {
 
 
   int l = XhashPosition ( ph, iKey );
@@ -196,7 +208,7 @@ XhashLookup ( xhash *ph, const unsigned int iKey,
   if ( ph->phe[ l ].p && ph->phe[ l ].iKey == iKey ) {
     /* hit */
     ++ph->nHits;
-    return (*data = ph->phe[ l].p);
+    return (ph->phe[ l].p);
   }
   else
     /* miss */
@@ -215,14 +227,14 @@ OSLookup ( const unsigned int iPos,
   int i, j;
   unsigned char ac[ 128 ];
 
-  assert ( pfOutput != stdin );
+  g_assert ( pfOutput != stdin );
 
   if ( fCompress ) {
 
     unsigned char ac[ 128 ];
-    int i, j;
-    off_t iOffset;
-    int nBytes;
+    unsigned int i, j;
+    long iOffset;
+    size_t nBytes;
     unsigned int ioff, nz, ioffg, nzg;
     unsigned short int us;
 
@@ -390,15 +402,16 @@ RollsOS ( const unsigned short int aus[ 32 ] ) {
 
 }
 
-static void BearOff( int nId, int nPoints, 
+static void BearOff( int nId, unsigned int nPoints, 
                      unsigned short int aOutProb[ 64 ],
                      const int fGammon,
                      xhash *ph, bearoffcontext *pbc,
                      const int fCompress, 
                      FILE *pfOutput, FILE *pfTmp ) {
 
-    int i, iBest, iMode, j, anRoll[ 2 ], anBoard[ 2 ][ 25 ],
-      anBoardTemp[ 2 ][ 25 ], aProb[ 64 ];
+    int iBest, iMode, j, anRoll[ 2 ], aProb[ 64 ];
+    unsigned int i;
+	TanBoard anBoard, anBoardTemp;
     movelist ml;
     int k;
     unsigned int us;
@@ -410,7 +423,7 @@ static void BearOff( int nId, int nPoints,
     unsigned int usGammonBest;
     unsigned short int ausGammonBest[ 32 ];
     int iGammonBest;
-    int nBack;
+    unsigned int nBack;
 
     /* get board for given position */
 
@@ -458,7 +471,7 @@ static void BearOff( int nId, int nPoints,
 	    
     for( anRoll[ 0 ] = 1; anRoll[ 0 ] <= 6; anRoll[ 0 ]++ )
 	for( anRoll[ 1 ] = 1; anRoll[ 1 ] <= anRoll[ 0 ]; anRoll[ 1 ]++ ) {
-	    GenerateMoves( &ml, anBoard, anRoll[ 0 ], anRoll[ 1 ], FALSE );
+	    GenerateMoves( &ml, (ConstTanBoard)anBoard, anRoll[ 0 ], anRoll[ 1 ], FALSE );
 
 	    usBest = 0xFFFFFFFF; iBest = -1;
             usGammonBest = 0xFFFFFFFF; iGammonBest = -1;
@@ -468,8 +481,8 @@ static void BearOff( int nId, int nPoints,
 
 		j = PositionBearoff( anBoardTemp[ 1 ], nPoints, 15 );
 
-		assert( j >= 0 );
-		assert( j < nId );
+		g_assert( j >= 0 );
+		g_assert( j < nId );
 
 
                 if ( ! j ) {
@@ -479,9 +492,10 @@ static void BearOff( int nId, int nPoints,
                   pusj[ 32 ] = 0xFFFF;
 
                 }
-                else if ( ! XhashLookup ( ph, j, (void **) &pusj ) ) {
+                else if ( ! (pusj = XhashLookup ( ph, j))) {
                   /* look up in file generated so far */
-                  OSLookup ( j, nPoints, pusj = ausj, fGammon, fCompress,
+		  pusj = ausj;
+                  OSLookup ( j, nPoints, pusj, fGammon, fCompress,
                              pfOutput, pfTmp );
 
                   XhashAdd ( ph, j, pusj, fGammon ? 128 : 64 );
@@ -506,8 +520,8 @@ static void BearOff( int nId, int nPoints,
 
 	    }
 
-	    assert( iBest >= 0 );
-            assert( iGammonBest >= 0 );
+	    g_assert( iBest >= 0 );
+            g_assert( iGammonBest >= 0 );
 
 	    if( anRoll[ 0 ] == anRoll[ 1 ] ) {
               for( i = 0; i < 31; i++ ) {
@@ -553,7 +567,7 @@ WriteOS ( const unsigned short int aus[ 32 ],
           const int fCompress, FILE *output ) {
 
   unsigned int iIdx, nNonZero;
-  int j;
+  unsigned int j;
 
   if ( fCompress )
     CalcIndex ( aus, &iIdx, &nNonZero );
@@ -614,7 +628,7 @@ WriteFloat ( const float r, FILE *output ) {
   int j;
   unsigned char *pc;
 
-  pc = (char *) &r;
+  pc = (unsigned char *) &r;
 
   for ( j = 0; j < 4; ++j )
     putc ( *(pc++), output );
@@ -649,9 +663,12 @@ generate_os ( const int nOS, const int fHeader,
   unsigned short int aus[ 64 ];
   xhash h;
   FILE *pfTmp = NULL;
-  time_t t;
   unsigned int npos;
-  char szTmp[ 11 ];
+  char *tmpfile;
+#ifndef WIN32
+  int fTTY = isatty(STDERR_FILENO);
+#endif
+
 #if WIN32
   HINSTANCE hInstance = (HINSTANCE) GetModuleHandle(NULL);
   HWND hwndPB;
@@ -696,13 +713,9 @@ generate_os ( const int nOS, const int fHeader,
 #if WIN32
     dlgprintf(127, "Opening temporary file." );
 #endif
-    time ( &t );
-    sprintf ( szTmp, "t%06lu.bd", (unsigned long) t % 100000 );
-    if ( ! ( pfTmp = fopen ( szTmp, "w+b" ) ) ) {
-      perror ( szTmp );
-      exit ( 2 );
-    }
-
+	pfTmp = GetTemporaryFile(NULL, &tmpfile);
+	if (pfTmp == NULL)
+      exit(2);
   }
     
 
@@ -732,7 +745,7 @@ generate_os ( const int nOS, const int fHeader,
     if (CancelPressed)
       break;
 #else
-    if (!(i % 100))
+    if (!(i % 100) && fTTY)
       fprintf (stderr, "1:%d/%d        \r", i, n);
 #endif
 
@@ -764,7 +777,7 @@ generate_os ( const int nOS, const int fHeader,
 
     fclose ( pfTmp );
 
-    unlink ( szTmp );
+    g_unlink ( tmpfile );
 
   }
 #if !WIN32
@@ -783,14 +796,14 @@ generate_os ( const int nOS, const int fHeader,
 
 
 static void
-NDBearoff ( const int iPos, const int nPoints, float ar[ 4 ], xhash *ph,
+NDBearoff ( const int iPos, const unsigned int nPoints, float ar[ 4 ], xhash *ph,
             bearoffcontext *pbc) {
 
   int d0, d1;
   movelist ml;
-  int anBoard[ 2 ][ 25 ];
-  int anBoardTemp[ 2 ][ 25 ];
-  int i, j, k;
+  TanBoard anBoard, anBoardTemp;
+  int ii, j, k;
+  unsigned int i;
   int iBest;
   float rBest;
   float rMean;
@@ -819,10 +832,10 @@ NDBearoff ( const int iPos, const int nPoints, float ar[ 4 ], xhash *ph,
    */
 
   if ( pbc ) {
-    for ( i = 24; i >= 0 && ! anBoard[ 1 ][ i ]; --i )
+    for ( ii = 24; ii >= 0 && ! anBoard[ 1 ][ ii ]; --ii )
       ;
 
-    if ( i < pbc->nPoints ) {
+    if ( ii < (int)pbc->nPoints ) {
       unsigned int nPosID = PositionBearoff ( anBoard[ 1 ], 
                                               pbc->nPoints, pbc->nChequers );
       BearoffDist ( pbc, nPosID, NULL, NULL, ar, NULL, NULL );
@@ -843,7 +856,7 @@ NDBearoff ( const int iPos, const int nPoints, float ar[ 4 ], xhash *ph,
   for ( d0 = 1; d0 <= 6; ++d0 ) 
     for ( d1 = 1; d1 <= d0; ++d1 ) {
 
-      GenerateMoves ( &ml, anBoard, d0, d1, FALSE );
+      GenerateMoves ( &ml, (ConstTanBoard)anBoard, d0, d1, FALSE );
 
       rBest = 1e10;
       rGammonBest = 1e10;
@@ -856,8 +869,9 @@ NDBearoff ( const int iPos, const int nPoints, float ar[ 4 ], xhash *ph,
 
         j = PositionBearoff ( anBoardTemp[ 1 ], nPoints, 15 );
 
-        if ( ! XhashLookup ( ph, j, (void **) &prj ) ) {
-          NDBearoff ( j, nPoints, prj = arj, ph, pbc );
+        if ( ! (prj = XhashLookup ( ph, j ))) {
+	  prj = arj;
+          NDBearoff ( j, nPoints, prj, ph, pbc );
           XhashAdd ( ph, j, prj, 16 );
         }
 
@@ -879,8 +893,8 @@ NDBearoff ( const int iPos, const int nPoints, float ar[ 4 ], xhash *ph,
 
       }
 
-      assert ( iBest >= 0 );
-      assert ( iGammonBest >= 0 );
+      g_assert ( iBest >= 0 );
+      g_assert ( iGammonBest >= 0 );
       
       rMean = 1.0f + arBest[ 0 ];
 
@@ -926,6 +940,9 @@ generate_nd ( const int nPoints,const int nHashSize, const int fHeader,
   int i, j;
   char sz[ 41 ];
   float ar[ 4 ];
+#ifndef WIN32
+  int fTTY = isatty(STDERR_FILENO);
+#endif
 
   xhash h;
 #if WIN32
@@ -985,7 +1002,7 @@ generate_nd ( const int nPoints,const int nHashSize, const int fHeader,
     if (!((i+1) % 100))
       SendMessage(hwndPB, PBM_STEPIT, 0, 0);
 #else
-    if (!(i % 100))
+    if (!(i % 100) && fTTY)
       fprintf (stderr, "1:%d/%d        \r", i, n);
 #endif
 
@@ -1111,7 +1128,9 @@ static void BearOff2( int nUs, int nThem,
                       const int n, const int fCubeful,
                       xhash *ph, bearoffcontext *pbc, FILE *pfTmp ) {
 
-    int i, j, anRoll[ 2 ], anBoard[ 2 ][ 25 ], anBoardTemp[ 2 ][ 25 ];
+    int j, anRoll[ 2 ];
+    unsigned int i;
+	TanBoard anBoard, anBoardTemp;
     movelist ml;
     int aiBest[ 4 ];
     int asiBest[ 4 ];
@@ -1148,7 +1167,7 @@ static void BearOff2( int nUs, int nThem,
 
     /* look for position in bearoff file */
 
-    if ( pbc && isBearoff ( pbc, anBoard ) ) {
+    if ( pbc && isBearoff ( pbc, (ConstTanBoard)anBoard ) ) {
       unsigned short int nUsL = 
         PositionBearoff ( anBoard[ 1 ], pbc->nPoints, pbc->nChequers );
       unsigned short int nThemL = 
@@ -1165,11 +1184,11 @@ static void BearOff2( int nUs, int nThem,
       return;
     }
 
-    aiTotal[ 0 ] = aiTotal[ 1 ] = aiTotal[ 2 ] = aiTotal[ 3 ] = 0.0;
+    aiTotal[ 0 ] = aiTotal[ 1 ] = aiTotal[ 2 ] = aiTotal[ 3 ] = 0;
     
     for( anRoll[ 0 ] = 1; anRoll[ 0 ] <= 6; anRoll[ 0 ]++ )
 	for( anRoll[ 1 ] = 1; anRoll[ 1 ] <= anRoll[ 0 ]; anRoll[ 1 ]++ ) {
-	    GenerateMoves( &ml, anBoard, anRoll[ 0 ], anRoll[ 1 ], FALSE );
+	    GenerateMoves( &ml, (ConstTanBoard)anBoard, anRoll[ 0 ], anRoll[ 1 ], FALSE );
 
             asiBest [ 0 ] = asiBest[ 1 ] = 
               asiBest[ 2 ] = asiBest [ 3 ] = -0xFFFF;
@@ -1180,20 +1199,19 @@ static void BearOff2( int nUs, int nThem,
 
 		j = PositionBearoff( anBoardTemp[ 1 ], nTSP, nTSC );
 
-		assert( j >= 0 );
-		assert( j < nUs ); 
+		g_assert( j >= 0 );
+		g_assert( j < nUs ); 
 
                 if ( ! nThem ) {
-                  psij = asij;
                   asij[ 0 ] = asij[ 1 ] = asij[ 2 ] = asij[ 3 ] = EQUITY_P1;
                 }
                 else if ( ! j ) {
-                  psij = asij;
                   asij[ 0 ] = asij[ 1 ] = asij[ 2 ] = asij[ 3 ] = EQUITY_M1;
                 }
-                if ( ! XhashLookup ( ph, n * nThem + j, (void **) &psij ) ) {
+                if ( ! (psij = XhashLookup ( ph, n * nThem + j)) ) {
                   /* lookup in file */
-                  TSLookup ( nThem, j, nTSP, nTSC, psij = asij, n,
+		  psij = asij;
+                  TSLookup ( nThem, j, nTSP, nTSC, psij, n,
                              fCubeful, pfTmp );
                   XhashAdd ( ph, n * nThem + j, psij, fCubeful ? 8 : 2 );
                 }
@@ -1236,10 +1254,10 @@ static void BearOff2( int nUs, int nThem,
 
 	    }
 
-	    assert( aiBest[ 0 ] >= 0 );
-	    assert( ! fCubeful || aiBest[ 1 ] >= 0 );
-	    assert( ! fCubeful || aiBest[ 2 ] >= 0 );
-	    assert( ! fCubeful || aiBest[ 3 ] >= 0 );
+	    g_assert( aiBest[ 0 ] >= 0 );
+	    g_assert( ! fCubeful || aiBest[ 1 ] >= 0 );
+	    g_assert( ! fCubeful || aiBest[ 2 ] >= 0 );
+	    g_assert( ! fCubeful || aiBest[ 3 ] >= 0 );
 	    
             if ( anRoll[ 0 ] == anRoll[ 1 ] )
               for ( k = 0; k < ( fCubeful ? 4 : 1 ); ++k )
@@ -1278,10 +1296,13 @@ generate_ts ( const int nTSP, const int nTSC,
     int n;
     short int asiEquity[ 4 ];
     xhash h;
-    char szTmp[ 11 ];
     FILE *pfTmp;
-    time_t t;
     unsigned char ac[ 8 ];
+    char *tmpfile;
+#ifndef WIN32
+  int fTTY = isatty(STDERR_FILENO);
+#endif
+
 #if WIN32
   HINSTANCE hInstance = (HINSTANCE) GetModuleHandle(NULL);
   HWND hwndPB;
@@ -1297,12 +1318,9 @@ generate_ts ( const int nTSP, const int nTSC,
 		  12, 300, 470, 20, hdlg, NULL, hInstance, NULL);
 #endif
 
-    time ( &t );
-    sprintf ( szTmp, "t%06lu.bd", (unsigned long) t % 100000 );
-    if ( ! ( pfTmp = fopen ( szTmp, "w+b" ) ) ) {
-      perror ( szTmp );
-      exit ( 2 );
-    }
+	pfTmp = GetTemporaryFile(NULL, &tmpfile);
+	if (pfTmp == NULL)
+      exit(2);
 
     /* initialise xhash */
     
@@ -1361,7 +1379,8 @@ generate_ts ( const int nTSP, const int nTSC,
 #if WIN32
       SendMessage(hwndPB, PBM_STEPIT, 0, 0);
 #else
-      fprintf( stderr, "%d/%d     \r", iPos, n * n );
+      if (fTTY)
+	      fprintf( stderr, "%d/%d     \r", iPos, n * n );
 #endif
     }
 
@@ -1387,7 +1406,8 @@ generate_ts ( const int nTSP, const int nTSC,
 #if WIN32
       SendMessage(hwndPB, PBM_STEPIT, 0, 0);
 #else
-      fprintf( stderr, "%d/%d     \r", iPos, n * n );
+      if (fTTY)
+	      fprintf( stderr, "%d/%d     \r", iPos, n * n );
 #endif
     }
 
@@ -1433,97 +1453,43 @@ generate_ts ( const int nTSP, const int nTSC,
 #endif
     fclose ( pfTmp );
 
-    unlink ( szTmp ); 
+    g_unlink ( tmpfile ); 
 
 }
 
 
 
 
-static void
-usage ( char *arg0 ) {
-#ifndef WIN32
-  printf ( "Usage: %s [options] -f filename\n"
-           "Options:\n"
-           "  -f, --outfile filename\n"
-           "                      Output to file\n"
-           "  -t, --two-sided PxC Number of points and number of chequers\n"
-           "                      for two-sided database\n"
-           "  -o, --one-sided P   Number of points for one-sided database\n"
-           "  -s, --xhash-size N   Use cache of size N bytes\n"
-           "  -O, --old-bearoff filename\n"
-           "                      Reuse already generated bearoff database\n"
-           "  -H, --no-header     Do not write header\n"
-           "  -C, --no-cubeful    Do not calculate cubeful equities for\n"
-           "                      two-sided databases\n"
-           "  -c, --no-compress   Do not use compression scheme "
-                                  "for one-sided databases\n"
-           "  -g, --no-gammons    Include gammon distribution for one-sided"
-                                  " databases\n"
-           "  -n, --normal-dist   Approximate one-sided bearoff database\n"
-           "                      with normal distributions\n"
-           "  -v, --version       Show version information and exit\n"
-           "  -h, --help          Display usage and exit\n"
-           "\n"
-           "To generate gnubg_os0.bd:\n"
-           "%s -o 6 -f gnubg_os0.bd\n"
-           "\n",
-           arg0, arg0 );
-#else
-    MessageBox (NULL, 
-           "Usage: makebearoff [options] -f filename\n\n"
-           "Options:\n"
-           "  -f, --outfile filename\tOutput to file\n"
-           "  -t, --two-sided PxC   \tNumber of points and number of chequers"
-           " for two-sided database\n"
-           "  -o, --one-sided P     \tNumber of points for one-sided database\n"
-           "  -s, --xhash-size N    \tUse cache of size N bytes\n"
-           "  -O, --old-bearoff filename\tReuse already generated bearoff database\n"
-           "  -H, --no-header       \tDo not write header\n"
-           "  -C, --no-cubeful      \tDo not calculate cubeful equities for"
-           " two-sided databases\n"
-           "  -c, --no-compress     \tDo not use compression scheme "
-                                  "for one-sided databases\n"
-           "  -g, --no-gammons      \tInclude gammon distribution for one-sided"
-                                  " databases\n"
-           "  -n, --normal-dist     \tApproximate one-sided bearoff database"
-           " with normal distributions\n"
-           "  -v, --version         \t\tShow version information and exit\n"
-           "  -h, --help            \t\tDisplay usage and exit\n"
-           "\n"
-           "To generate gnubg_os0.bd:\n"
-           "makebearoff -o 6 -f gnubg_os0.bd\n"
-           "\n",
-		     "Makebearoff", MB_OK);
-#endif
-}
-
 static void
 version ( void ) {
 #ifndef WIN32
-  printf ( "makebearoff $Revision: 1.36 $\n" );
+  printf ( "makebearoff $Revision: 1.66 $\n" );
 #else
-  MessageBox( NULL, "makebearoff $Revision: 1.36 $\n", "Makebearoff", MB_OK );
+  MessageBox( NULL, "makebearoff $Revision: 1.66 $\n", "Makebearoff", MB_OK );
 #endif
 }
 
 
-extern int main( int argc, char **argv ) {
+extern int main( int argc, char **argv )
+{
+  /* static storage for options to keep picky comipler happy */
+  static int nOS = 0;
+  static int fHeader = TRUE;
+  static int fCompress = TRUE;
+  static int fGammon = TRUE;
+  static int nHashSize = 100000000;
+  static int fCubeful = TRUE;
+  static char *szOldBearoff = NULL;
+  static int fND = FALSE;
+  static char *szOutput = NULL;
+  static char *szTwoSided = NULL;
+  static int show_version=0;
 
-  int nTSP = 0, nTSC = 0;
-  int nOS = 0;
-  int fHeader = TRUE;
-  char ch;
-  int fCompress = TRUE;
-  int fGammon = TRUE;
-  int nHashSize = 100000000;
-  int fCubeful = TRUE;
-  char *szOldBearoff = NULL;
-  int fND = FALSE;
   bearoffcontext *pbc = NULL;
   FILE *output = stdout;
-  char *szOutput = NULL;
   double r;
+  int nTSP = 0, nTSC = 0;
+
 #if WIN32
   int i;
   char *aszOS[] = {"Number of points:",
@@ -1547,74 +1513,58 @@ extern int main( int argc, char **argv ) {
                 "Reuse old bearoff database:",
                 " ", " "};
 #endif
-
-  static struct option ao[] = {
-    { "two-sided", required_argument, NULL, 't' },
-    { "one-sided", required_argument, NULL, 'o' },
-    { "xhash-size", required_argument, NULL, 's' },
-    { "old-bearoff", required_argument, NULL, 'O' },
-    { "no-header", no_argument, NULL, 'H' },
-    { "no-cubeful", no_argument, NULL, 'C' },
-    { "no-compress", no_argument, NULL, 'c' },
-    { "no-gammon", no_argument, NULL, 'g' },
-    { "normal-dist", no_argument, NULL, 'n' },
-    { "help", no_argument, NULL, 'h' },
-    { "version", no_argument, NULL, 'v' },
-    { NULL, 0, NULL, 0 }
+           
+  GOptionEntry ao[] = {
+	  { "two-sided", 't',  0, G_OPTION_ARG_STRING, &szTwoSided,
+		  "Number of points (P) and number of chequers (C) for two-sided database", "PxC"},
+	  { "one-sided", 'o', 0, G_OPTION_ARG_INT, &nOS, 
+		  "Number of points (P) for one-sided database", "P"},
+	  { "xhash-size", 's', 0, G_OPTION_ARG_INT, &nHashSize, 
+		  "Use cache of size N bytes", "N"},
+	  { "old-bearoff", 'O', 0, G_OPTION_ARG_STRING, &szOldBearoff, 
+		  "Reuse already generated bearoff database \"filename\"", "filename"},
+	  { "no-header", 'H', G_OPTION_FLAG_REVERSE,  G_OPTION_ARG_NONE, &fHeader, 
+		  "Do not write header", NULL},
+	  { "no-cubeful", 'C', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &fCubeful, 
+		  "Do not calculate cubeful equities for two-sided databases", NULL},
+	  { "no-compress", 'c', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &fCompress, 
+		  "Do not use compression scheme for one-sided databases", NULL},
+	  { "no-gammon", 'g', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &fGammon,
+		  "Include gammon distribution for one-sided databases", NULL},
+	  { "normal-dist", 'n', 0, G_OPTION_ARG_NONE, &fND, 
+		  "Approximate one-sided bearoff database with normal distributions", NULL},
+	  { "version", 'v', 0, G_OPTION_ARG_NONE, &show_version, 
+		  "Prints version and exits", NULL},
+	  { "outfile", 'f', 0, G_OPTION_ARG_STRING, &szOutput, 
+		  "Required output filename", "filename"}, 
+	  {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
-  while ( ( ch = getopt_long ( argc, argv, "t:o:s:O:f:HCcgnhv", ao, NULL ) ) !=
-          (char) -1 ) {
-    switch ( ch ) {
-    case 't': /* size of two-sided */
-      sscanf ( optarg, "%dx%d", &nTSP, &nTSC );
-      break;
-    case 'o': /* size of one-sided */
-      nOS = atoi ( optarg );
-      break;
-    case 's': /* xhash size */
-      nHashSize = atoi ( optarg );
-      break;
-    case 'O': /* old database */
-      szOldBearoff = strdup ( optarg );
-      break;
-    case 'H': /* no header */
-      fHeader = FALSE;
-      break;
-    case 'C': /* cubeful */
-      fCubeful = FALSE;
-      break;
-    case 'c': /* compress */
-      fCompress = FALSE;
-      break;
-    case 'g': /* no gammons */
-      fGammon = FALSE;
-      break;
-    case 'n': /* normal dist */
-      fND = TRUE;
-      break;
-    case 'h': /* help */
-      usage ( argv[ 0 ] );
-      exit ( 0 );
-      break;
-    case 'v': /* version */
-      version ();
-      exit ( 0 );
-      break;
-    case 'f':
-      szOutput = strdup ( optarg );
-      break;
-    default:
-      usage ( argv[ 0 ] );
-      exit ( 1 );
-    }
+  GError *error = NULL;
+  GOptionContext *context;
+
+  context = g_option_context_new(NULL);
+  g_option_context_add_main_entries(context, ao, PACKAGE);
+  g_option_context_parse(context, &argc, &argv, &error);
+  g_option_context_free(context);
+  if (error) {
+  	g_printerr("%s\n", error->message);
+  	exit(EXIT_FAILURE);
   }
 
-  /* open output file */
+
+  if (szTwoSided)
+      sscanf ( szTwoSided, "%dx%d", &nTSP, &nTSC );
+
+  if (show_version)
+  {
+	  version();
+	  exit(0);
+  }
 
   if ( ! szOutput ) {
-    usage ( argv[ 0 ] );
-    return EXIT_FAILURE;
+	  g_printerr("Required argument -f missing\n");
+    exit(EXIT_FAILURE);
   }
 
   if ( ! ( output = fopen ( szOutput, "w+b" ) ) ) {
@@ -1632,7 +1582,7 @@ extern int main( int argc, char **argv ) {
       exit ( 2 );
     }
 #if WIN32 
-    hdlg = CreateDialog(NULL, MAKEINTRESOURCE (DLG_MAKEBEAROFF), NULL, DlgProc);
+    hdlg = CreateDialog(NULL, MAKEINTRESOURCE (DLG_MAKEBEAROFF), NULL, (DLGPROC)DlgProc);
     /* error if NULL */
     for (i = 0; i < 9; i++)
        SendDlgItemMessage(hdlg, 101 + i, WM_SETTEXT, 0, (LPARAM) aszOS[i]);
@@ -1647,7 +1597,7 @@ extern int main( int argc, char **argv ) {
     dlgprintf( 123, "%d", nHashSize);
     dlgprintf( 124, "%s", szOldBearoff ? "yes" : "no");
     dlgprintf(130, "Generating one-sided bearoff database. Please wait." );
-    dlgprintf(131, "makebearoff $Revision: 1.36 $" );
+    dlgprintf(131, "makebearoff $Revision: 1.66 $" );
 #else
     fprintf ( stderr, 
               _("One-sided database:\n"
@@ -1709,7 +1659,7 @@ extern int main( int argc, char **argv ) {
     }
 
     if ( szOldBearoff &&
-         ! ( pbc = BearoffInit ( szOldBearoff, NULL, BO_NONE, NULL ) ) ) {
+         ! ( pbc = BearoffInit ( szOldBearoff, BO_NONE, NULL ) ) ) {
       dsplerr ( _("Error initialising old bearoff database!\n" ) );
       exit( 2 );
     }
@@ -1737,7 +1687,7 @@ extern int main( int argc, char **argv ) {
       fprintf ( stderr, "Number of reads in old database: %lu\n",
                 pbc->nReads );
 #endif
-      BearoffClose ( &pbc );
+      BearoffClose ( pbc );
     }
 
 #if !WIN32
@@ -1757,7 +1707,7 @@ extern int main( int argc, char **argv ) {
     r = n;
     r = r * r * ( fCubeful ? 8.0 : 2.0 );
 #if WIN32 
-    hdlg = CreateDialog(NULL, MAKEINTRESOURCE (DLG_MAKEBEAROFF), NULL, DlgProc);
+    hdlg = CreateDialog(NULL, MAKEINTRESOURCE (DLG_MAKEBEAROFF), NULL, (DLGPROC)DlgProc);
     /* error if NULL */
     for (i = 0; i < 11; i++)
        SendDlgItemMessage(hdlg, 101 + i, WM_SETTEXT, 0, (LPARAM) aszTS[i]);
@@ -1775,7 +1725,7 @@ extern int main( int argc, char **argv ) {
     dlgprintf(125, "" );
     dlgprintf(126, "" );
     dlgprintf(130, "Generating two-sided bearoff database. Please wait." );
-    dlgprintf(131, "makebearoff $Revision: 1.36 $" );
+    dlgprintf(131, "makebearoff $Revision: 1.66 $" );
 #else 
     fprintf ( stderr,
               _("Two-sided database:\n"
@@ -1803,7 +1753,7 @@ extern int main( int argc, char **argv ) {
     dlgprintf(127, "Initialising old bearoff database." );
 #endif
     if ( szOldBearoff &&
-         ! ( pbc = BearoffInit ( szOldBearoff, NULL, BO_NONE, NULL ) ) ) {
+         ! ( pbc = BearoffInit ( szOldBearoff, BO_NONE, NULL ) ) ) {
       dsplerr ( _("Error initialising old bearoff database!\n" ) );
       exit( 2 );
     }
@@ -1826,7 +1776,7 @@ extern int main( int argc, char **argv ) {
       fprintf ( stderr, "Number of reads in old database: %lu\n",
                 pbc->nReads );
 #endif
-      BearoffClose ( &pbc );
+      BearoffClose ( pbc );
     }
 #if !WIN32
     fprintf ( stderr, "Number of re-reads while generating: %ld\n", 
@@ -1874,5 +1824,3 @@ DlgProc (HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam){
    return FALSE;
 }
 #endif
-
-	   
