@@ -4,7 +4,7 @@
  * by Jrn Thyssen <jth@gnubg.org>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,23 +16,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: makehyper.c,v 1.9 2006/06/26 21:07:16 c_anthon Exp $
+ * $Id: makehyper.c,v 1.32 2008/10/22 19:46:08 c_anthon Exp $
  */
 
 #include "config.h"
 
-#include <assert.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
 #include <math.h>
+#include <multithread.h>
+#include "backgammon.h"
 
 #include "eval.h"
 #include "positionid.h"
-#include "getopt.h"
-#include <glib/gi18n.h>
 #include <locale.h>
 #include "bearoff.h"
 #include "drawboard.h"
@@ -40,13 +41,6 @@
 static cubeinfo ci;
 static cubeinfo ciJacoby;
 
-/* ugly fixes */
-char *aszRNG[]; 
-char *aszSkillType[ 1 ]; 
-int exsExport;
-int ap;
-/* end ugly fixes */
-
 typedef enum _hyperclass {
   HYPER_OVER,
   HYPER_BEAROFF,
@@ -70,11 +64,31 @@ typedef struct _hyperequity {
 
 } hyperequity;
 
+#if USE_MULTITHREAD
+extern int MT_GetThreadID(void)
+{
+  return (0);
+}
+extern void MT_Release(void)
+{
+  return;
+}
+extern void MT_Exclusive(void)
+{
+  return;
+}
+#else
+extern void CallbackProgress(void)
+{
+}
+#endif
+
+
 int aiNorm[ 10 ];
 
 
 static hyperclass
-ClassifyHyper ( int anBoard[ 2 ][ 25 ] ) {
+ClassifyHyper ( TanBoard anBoard ) {
 
   int i;
   int nOppBack, nBack;
@@ -111,9 +125,9 @@ ClassifyHyper ( int anBoard[ 2 ][ 25 ] ) {
 }
 
 static void
-HyperOver ( int anBoard[ 2 ][ 25 ], float ar[ NUM_OUTPUTS ], const int nC ) {
+HyperOver ( const TanBoard anBoard, float ar[ NUM_OUTPUTS ], const int nC ) {
 
-  EvalOver( anBoard, ar, VARIATION_HYPERGAMMON_1 + nC - 1 );
+  EvalOver( anBoard, ar, VARIATION_HYPERGAMMON_1 + nC - 1, NULL );
 
 }
 
@@ -121,9 +135,10 @@ HyperOver ( int anBoard[ 2 ][ 25 ], float ar[ NUM_OUTPUTS ], const int nC ) {
 static void
 StartGuessHyper ( hyperequity ahe[], const int nC, bearoffcontext *pbc ) {
 
-  int i, j, k;
+  int i, j;
+  unsigned int k;
   int nPos = Combination( 25 + nC, nC );
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
 
   int ai[ 4 ] = { 0,0,0,0 };
 
@@ -135,7 +150,7 @@ StartGuessHyper ( hyperequity ahe[], const int nC, bearoffcontext *pbc ) {
 
       switch ( ClassifyHyper ( anBoard ) ) {
       case HYPER_OVER:
-        HyperOver ( anBoard, ahe[ i * nPos + j ].arOutput, nC );
+        HyperOver ( (ConstTanBoard)anBoard, ahe[ i * nPos + j ].arOutput, nC );
 
         for ( k = 0; k < 5; ++k )
           ahe[ i * nPos + j ].arEquity[ k ] = 
@@ -178,7 +193,7 @@ StartGuessHyper ( hyperequity ahe[], const int nC, bearoffcontext *pbc ) {
 
       default:
 
-        assert ( FALSE );
+        g_assert ( FALSE );
         break;
 
       }
@@ -186,16 +201,12 @@ StartGuessHyper ( hyperequity ahe[], const int nC, bearoffcontext *pbc ) {
     }
   }
 
-  printf( _("Number of game-over positions:   %10d\n"
-            "Number of non-contact positions: %10d\n"
-            "Number of contact positions:     %10d\n"
-            "Total number of legal positions: %10d\n"
-            "Number of illegal positions:     %10d\n"
-            "Total number of positions in file%10d\n"),
-          ai[ 0 ], ai[ 2 ], ai[ 1 ], 
-          ai[ 0 ] + ai[ 2 ] + ai[ 1 ], 
-          ai[ 3 ],
-          nPos * nPos );
+  printf("%-25s: %10d\n", _("Number of game-over positions"), ai[ 0 ]);
+  printf("%-25s: %10d\n", _("Number of non-contact positions"), ai[ 2 ]);
+  printf("%-25s: %10d\n", _("Number of contact positions"), ai[ 1 ]);
+  printf("%-25s: %10d\n", _("Total number of legal positions"), ai[ 0 ] + ai[ 2 ] + ai[ 1 ]);
+  printf("%-25s: %10d\n", _("Number of illegal positions"), ai[ 3 ]);
+  printf("%-25s: %10d\n", _("Total number of positions in file"), nPos * nPos );
             
 }
 
@@ -211,7 +222,7 @@ StartFromDatabase( hyperequity ahe[], const int nC,
   int i, j, k;
   float r;
 
-  if ( ! ( pf = fopen ( szFilename, "r+b" ) ) ) {
+  if ( ! ( pf = g_fopen ( szFilename, "r+b" ) ) ) {
     perror ( szFilename );
     exit(2);
   }
@@ -223,12 +234,16 @@ StartFromDatabase( hyperequity ahe[], const int nC,
   for ( i = 0; i < nPos; ++i )
     for ( j = 0; j < nPos; ++j ) {
 
-      fread( ac, 1, 28, pf );
+      if (fread( ac, 1, 28, pf ) != 28)
+      {
+	      perror(szFilename);
+	      exit(EXIT_FAILURE);
+      }
 
       for ( k = 0; k < NUM_OUTPUTS; ++k ) {
         us = ac[ 3 * k ] | ( ac[ 3 * k + 1 ] ) << 8 | ( ac[ 3 * k + 2 ] ) << 16;
         r = us / 16777215.0;
-        assert( r >= 0 && r <= 1 );
+        g_assert( r >= 0 && r <= 1 );
         ahe[ i * nPos + j ].arOutput[ k ] = r;
       }
 
@@ -236,7 +251,7 @@ StartFromDatabase( hyperequity ahe[], const int nC,
         us = ac[ 15 + 3 * k ] | ( ac[ 15 + 3 * k + 1 ] ) << 8 |  
            ( ac[ 15 + 3 * k + 2 ] ) << 16;
         r = ( us / 16777215.0f - 0.5f ) * 6.0f;
-        assert( r >= -3 && r <= 3 );
+        g_assert( r >= -3 && r <= 3 );
         ahe[ i * nPos + j ].arEquity[ k + 1 ] = r;
       }
 
@@ -301,10 +316,11 @@ HyperEquity ( const int nUs, const int nThem,
               hyperequity *phe, const int nC, const hyperequity aheOld[],
               float arNorm[] ) {
 
-  int anBoard[ 2 ][ 25 ];
-  int anBoardTemp[ 2 ][ 25 ];
+  TanBoard anBoard;
+  TanBoard anBoardTemp;
   movelist ml;
-  int i, j, k;
+  int i, j;
+  unsigned int k;
   int nUsNew, nThemNew;
   hyperequity heBest;
   hyperequity heOld;
@@ -326,7 +342,7 @@ HyperEquity ( const int nUs, const int nThem,
   switch ( ( hc = ClassifyHyper ( anBoard ) ) ) {
   case HYPER_OVER:
 
-    HyperOver ( anBoard, phe->arOutput, nC );
+    HyperOver ( (ConstTanBoard)anBoard, phe->arOutput, nC );
 
     for ( k = 0; k < 5; ++k )
       phe->arEquity[ k ] = Utility ( phe->arOutput, &ci );
@@ -355,7 +371,7 @@ HyperEquity ( const int nUs, const int nThem,
     for ( i = 1; i <= 6; ++i ) 
       for ( j = 1; j <= i; ++j ) {
       
-        GenerateMoves ( &ml, anBoard, i, j, FALSE );
+        GenerateMoves ( &ml, (ConstTanBoard)anBoard, i, j, FALSE );
 
         if ( ml.cMoves ) {
 
@@ -371,9 +387,9 @@ HyperEquity ( const int nUs, const int nThem,
             nUsNew = PositionBearoff( anBoardTemp[ 1 ], 25, nC );
             nThemNew = PositionBearoff( anBoardTemp[ 0 ], 25, nC );
           
-            assert ( nUsNew >= 0 );
-            assert ( nUsNew < nUs );
-            assert ( nThemNew >= 0 );
+            g_assert ( nUsNew >= 0 );
+            g_assert ( nUsNew < nUs );
+            g_assert ( nThemNew >= 0 );
 
             /* cubeless */
 
@@ -538,7 +554,7 @@ WriteProb ( FILE *pf, const float r ) {
 
 
 static void
-WriteFile ( const char *szFilename, const hyperequity ahe[], 
+WriteHyperFile ( const char *szFilename, const hyperequity ahe[], 
             const int nC ) {
 
   int nPos = Combination( 25 + nC, nC );
@@ -547,7 +563,7 @@ WriteFile ( const char *szFilename, const hyperequity ahe[],
   FILE *pf;
 
   
-  if ( ! ( pf = fopen ( szFilename, "w+b" ) ) ) {
+  if ( ! ( pf = g_fopen ( szFilename, "w+b" ) ) ) {
     perror ( szFilename );
     return;
   }
@@ -577,31 +593,6 @@ version ( void ) {
 
 }
 
-static void
-usage( const char *arg0 ) {
-
-  printf ( _("Usage: %s [options] -f filename\n"
-             "Options:\n"
-             "  -f, --outfile filename\n"
-             "                      Output to file.\n"
-             "  -r, --restart filename\n"
-             "                      Restart calculation of database from "
-             "given file.\n"
-             "                      Default start guess is zero-vector.\n"
-             "  -c, --chequers number\n"
-             "                      The number of chequers. Default is 3 "
-             "corresponding to\n"
-             "                      normal 3-chequer hypergammon.\n"
-             "  -n, --no-checkpoint\n"
-             "                      Do not write a checkpoint file after "
-             "each iteration.\n"
-             "  -t, --threshold     The convergens threshold. Default is "
-             "1e-5\n"
-             "  -v, --version       Show version information and exit\n"
-             "  -h, --help          Display usage and exit\n"),
-           arg0 );
-
-}
 
 extern int
 main ( int argc, char **argv ) {
@@ -611,6 +602,7 @@ main ( int argc, char **argv ) {
   int nPos;
   float rNorm;
   float rEpsilon = 1.0e-5;
+  gchar *szEpsilon = NULL;
   bearoffcontext *pbc = NULL;
   int it;
   char szFilename[ 20 ];
@@ -618,20 +610,27 @@ main ( int argc, char **argv ) {
   time_t t0, t1, t2, t3;
   char *szOutput = NULL;
   char *szRestart = NULL;
-  char ch;
   int fCheckPoint = TRUE;
+  int show_version=0;
 
-  static struct option ao[] = {
-    { "chequers", required_argument, NULL, 'c' },
-    { "restart", required_argument, NULL, 'r' },
-    { "outfile", required_argument, NULL, 'f' },
-    { "threshold", required_argument, NULL, 't' },
-    { "no-checkpoint", no_argument, NULL, 'n' },
-    { "help", no_argument, NULL, 'h' },
-    { "version", no_argument, NULL, 'v' },
-    { NULL, 0, NULL, 0 }
+  GOptionEntry ao[] = {
+    { "chequers", 'c', 0, G_OPTION_ARG_INT, &nC, 
+	    "The number of chequers(0<C<4). Default is 3." , "C"},
+    { "restart", 'r', 0, G_OPTION_ARG_FILENAME, &szRestart, 
+	    "Restart calculation of database from \"filename\"", "filename"},
+    { "threshold", 't', 0, G_OPTION_ARG_STRING, &szEpsilon, 
+	    "The convergens threshold (T). Default is 1e-5", "T"},
+    { "no-checkpoint", 'n', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &fCheckPoint, 
+	    "Do not write a checkpoint file after each iteration.", NULL},
+    { "version", 'v', 0, G_OPTION_ARG_NONE, &show_version, 
+	    "Print version info and exit", NULL},
+    { "outfile", 'f', 0, G_OPTION_ARG_STRING, &szOutput, 
+	    "Required output filename", "filename"}, 
+    { NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
+  GError *error = NULL;
+  GOptionContext *context;
 
   /* i18n */
 
@@ -639,43 +638,34 @@ main ( int argc, char **argv ) {
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  /* parse options */
+  context = g_option_context_new(NULL);
+  g_option_context_add_main_entries(context, ao, PACKAGE);
+  g_option_context_parse(context, &argc, &argv, &error);
+  g_option_context_free(context);
 
-  while ( ( ch = getopt_long ( argc, argv, "c:r:f:t:nhv", ao, NULL ) ) !=
-          (char) -1 ) {
-    switch ( ch ) {
-    case 'h': /* help */
-      usage ( argv[ 0 ] );
-      exit ( 0 );
-      break;
-    case 'v': /* version */
-      version ();
-      exit ( 0 );
-      break;
-    case 'f': /* output file */
-      szOutput = strdup ( optarg );
-      break;
-    case 'r': /* restart file */
-      szRestart = strdup ( optarg );
-      break;
-    case 'c': /* number of chequers */
-      nC = atoi( optarg );
-      break;
-    case 'n': /* no check point files */
-      fCheckPoint = FALSE;
-      break;
-    case 't': /* convergense threshold */
-      rEpsilon = atof ( optarg );
-      break;
-    default:
-      usage ( argv[ 0 ] );
-      exit ( 1 );
-    }
+  if (error) {
+  	g_printerr("%s\n", error->message);
+  	exit(EXIT_FAILURE);
   }
 
+  /* parse options */
+  if (show_version)
+  {
+      version ();
+      exit ( 0 );
+  }
+
+  if (szEpsilon)
+	  rEpsilon = g_strtod (szEpsilon, NULL);
+  if (rEpsilon >1.0 || rEpsilon < 0.0)
+  {
+	  g_printerr("Valid threadholds are 0.0 - 1.0\n");
+	  exit(1);
+  }
+	  
   if ( ! szOutput || nC < 1 || nC > 3 ) {
-    /* illegal options */
-    usage( argv[ 0 ] );
+	  g_printerr("Illegal options. Try `makehyper --help' for usage "
+			  "information\n");
     exit(1);
   }
 
@@ -687,12 +677,11 @@ main ( int argc, char **argv ) {
   
   nPos = Combination( 25 + nC, nC );
 
-  printf ( _("Total number of one sided positions: %d\n"), nPos );
-  printf ( _("Total number of two sided positions: %d\n"), nPos * nPos );
-  printf ( _("Estimated size of file             : %d bytes\n"),
-           nPos * nPos * 28 + 40 );
-  printf ( _("Output file                        : '%s'\n"), szOutput );
-  printf ( _("Convergens threshold               : %e\n"), rEpsilon );
+  printf ("%-40s: %d\n", _("Total number of one sided positions"), nPos );
+  printf ("%-40s: %d\n", _("Total number of two sided positions"), nPos * nPos );
+  printf ("%-40s: %s %d\n", _("Estimated size of file"), _("bytes"), nPos * nPos * 28 + 40 );
+  printf ("%-40s: %s\n", _("Output file"), szOutput );
+  printf ("%-40s: %e\n", _("Convergens threshold"), rEpsilon );
 
   /* Iteration 0 */
 
@@ -743,7 +732,7 @@ main ( int argc, char **argv ) {
 
       sprintf ( szFilename, "%s.tmp", szOutput );
       if ( rNorm > rEpsilon )
-        WriteFile ( szFilename, aheEquity, nC );
+        WriteHyperFile ( szFilename, aheEquity, nC );
       else
         unlink( szFilename );
 
@@ -758,7 +747,7 @@ main ( int argc, char **argv ) {
 
   time ( &t0 );
 
-  WriteFile ( szOutput, aheEquity, nC );
+  WriteHyperFile ( szOutput, aheEquity, nC );
 
   time ( &t1 );
 
