@@ -3,7 +3,7 @@
  * by Joern Thyssen, 1999-2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: matchequity.h,v 1.19 2006/07/04 16:46:15 c_anthon Exp $
+ * $Id: matchequity.h,v 1.23 2009/06/13 00:05:41 c_anthon Exp $
  */
 
 
@@ -31,9 +31,9 @@
 
 typedef struct _metinfo {
 
-  unsigned char *szName;          /* Name of match equity table */
-  unsigned char *szFileName;     /* File name of met */
-  unsigned char *szDescription;  /* Description of met */
+  gchar *szName;          /* Name of match equity table */
+  gchar *szFileName;     /* File name of met */
+  gchar *szDescription;  /* Description of met */
   int nLength;                /* native length of met */
  
 } metinfo;
@@ -79,12 +79,12 @@ getMEAtScore( const int nScore0, const int nScore1, const int nMatchTo,
 /* Initialise match equity table */
 
 void
-InitMatchEquity ( const char *szFileName, const char *szDir );
+InitMatchEquity ( const char *szFileName );
 
 /* Get double points */
 
 extern int
-GetPoints ( float arOutput [ 5 ], cubeinfo *pci, float arCP[ 2 ] );
+GetPoints ( float arOutput [ 5 ], const cubeinfo *pci, float arCP[ 2 ] );
 
 extern float
 GetDoublePointDeadCube ( float arOutput [ 5 ], cubeinfo *pci );
