@@ -4,7 +4,7 @@
  * by Jrn Thyssen <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,19 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: matchid.h,v 1.2 2004/10/05 16:13:00 Superfly_Jon Exp $
+ * $Id: matchid.h,v 1.9 2009/03/21 21:46:32 c_anthon Exp $
  */
 
 #ifndef _MATCHID_H_
 #define _MATCHID_H_
 
-#include "backgammon.h"
-
+#define L_MATCHID 12
 extern int
 LogCube ( const int n );
 
 extern char*
-MatchID ( const int anDice[ 2 ],
+MatchID ( const unsigned int anDice[ 2 ],
           const int fTurn,
           const int fResigned,
           const int fDoubled,
@@ -38,13 +37,13 @@ MatchID ( const int anDice[ 2 ],
           const int nMatchTo,
           const int anScore[ 2 ],
           const int nCube,
-          const int gs );
+          const gamestate gs );
 
 extern char*
 MatchIDFromKey( unsigned char auchKey[ 8 ] );
 
 extern int
-MatchFromID ( int anDice[ 2 ],
+MatchFromID ( unsigned int anDice[ 2 ],
               int *pfTurn,
               int *pfResigned,
               int *pfDoubled,
@@ -54,7 +53,7 @@ MatchFromID ( int anDice[ 2 ],
               int *pnMatchTo,
               int anScore[ 2 ],
               int *pnCube,
-              int *pgs,
+              gamestate *pgs,
               const char *szMatchID );
 
 extern int
@@ -68,7 +67,7 @@ MatchFromKey ( int anDice[ 2 ],
                int *pnMatchTo,
                int anScore[ 2 ],
                int *pnCube,
-               int *pgs,
+               gamestate *pgs,
                const unsigned char *auchKey );
 
 extern char *
