@@ -6,7 +6,7 @@
  * (after inspiration from osr.cc from fibs2html <fibs2html.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,25 +18,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: osr.c,v 1.16 2006/04/12 06:53:35 Superfly_Jon Exp $
+ * $Id: osr.c,v 1.30 2008/02/28 14:15:51 c_anthon Exp $
  */
 
+/*! \file osr.c
+    \brief one-sided race rollouts
+*/
+
+
 #include <stdio.h>
-#include <assert.h>
+#include <glib.h>
 #include <string.h>
 
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
 
 #include "config.h"
 
 #include "backgammon.h"
-#include "eval.h"
 #include "positionid.h"
 #include "osr.h"
 #include "mt19937ar.h"
-#include "bearoff.h"
 
 #define MAX_PROBS        32
 #define MAX_GAMMON_PROBS 15
@@ -66,17 +66,11 @@ OSRQuasiRandomDice( const int iTurn, const int iGame, const int cGames,
 /* Fill aaProb with one sided bearoff probabilities for position with */
 /* bearoff id n.                                                      */
 
-static 
-#if defined( __GNUC__ )
-inline
-#endif
-void
-getBearoffProbs(const unsigned int n, unsigned short int aaProb[32])
+static inline void getBearoffProbs(const unsigned int n, unsigned short int aaProb[32])
 {
-
-  assert( pbc1 );
-  BearoffDist ( pbc1, n, NULL, NULL, NULL, aaProb, NULL );
-
+  g_assert( pbc1 );
+  if (BearoffDist ( pbc1, n, NULL, NULL, NULL, aaProb, NULL ))
+	puts("BearoffDist failed?");
 }
 
 
@@ -100,35 +94,30 @@ isCrossOver ( const int from, const int to ) {
  *
  */
 
-static int
-chequersout ( int anBoard[ 25 ] ) {
+static unsigned int chequersout(const unsigned int anBoard[25])
+{
+	unsigned int i, n = 0;
 
-  int i;
-  int n=0;
-
-  for ( i = 6; i < 25; i++ )
-    n += anBoard[i];
-
-  return n;
+	for (i = 6; i < 25; i++)
+		n += anBoard[i];
 
+	return n;
 }
 
-static int
-checkboard ( int anBoard[ 25 ] ) {
-
-  int i;
-
-  for ( i = 0; i < 25; ++i )
-    if ( anBoard[ i ] < 0 )
-      return 0;
-
-  return 1;
+/*! \brief checks that we haven't moved too many checkers of any point on the board
+ * but board need not contain all 15 checkers */
+static int checkboard(const unsigned int anBoard[25])
+{
+	unsigned int i;
 
+	for (i = 0; i < 25; i++)
+		if (anBoard[i] > 15)
+			return 0;
+	return 1;
 }
 
-static void
-FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
-
+static void FindBestMoveOSR2 ( unsigned int anBoard[ 25 ], const int anDice[ 2 ], unsigned int *pnOut )
+{
   int ifar, inear, iboth;
   int iused = 0;
   int i, j, lc;
@@ -145,18 +134,16 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
 
     --anBoard[ ifar ];
     --anBoard[ inear ];
-    assert ( checkboard ( anBoard ) );
+    g_assert ( checkboard ( anBoard ) );
     anBoard[ 5 ] += 2;
 
+    g_assert(*pnOut >= 2);
     *pnOut -= 2;
-    assert ( *pnOut >= 0 );
-    assert ( *pnOut == chequersout ( anBoard ) );
+    g_assert ( *pnOut == chequersout ( anBoard ) );
 
     return;
-
   }
 
-
   iboth = 5 + ( anDice[ 0 ] + anDice[ 1 ] );
 
   if ( anBoard[ iboth ] ) {
@@ -167,11 +154,11 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
 
     --anBoard[ iboth ];
     ++anBoard[ 5 ];
-    assert ( checkboard ( anBoard ) );
+    g_assert ( checkboard ( anBoard ) );
 
+    g_assert ( *pnOut > 0 );
     --*pnOut;
-    assert ( *pnOut >= 0 );
-    assert ( *pnOut == chequersout ( anBoard ) );
+    g_assert ( *pnOut == chequersout ( anBoard ) );
 
     return;
 
@@ -190,11 +177,11 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
 
       --anBoard[ 5 + anDice[ i ] ];
       ++anBoard[ 5 ];
-    assert ( checkboard ( anBoard ) );
+      g_assert ( checkboard ( anBoard ) );
 
+      g_assert ( *pnOut > 0 );
       --*pnOut;
-      assert ( *pnOut >= 0 );
-    assert ( *pnOut == chequersout ( anBoard ) );
+      g_assert ( *pnOut == chequersout ( anBoard ) );
 
       ++iused;
 
@@ -221,7 +208,7 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
 
         --anBoard[ j ];
         ++anBoard[ j - anDice[ i ] ];
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         ++iused;
         found = TRUE;
         /* FIXME: increment lc if needed */
@@ -245,15 +232,16 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
 
           --anBoard[ j ];
           ++anBoard[ j - anDice[ i ] ];
-          assert ( checkboard ( anBoard ) );
+          g_assert ( checkboard ( anBoard ) );
           ++iused;
 
           if ( j - anDice[ i ] < 6 )
-            /* we've moved inside home quadrant */
+		  {	/* we've moved inside home quadrant */
+            g_assert ( *pnOut > 0 );
             --*pnOut;
+		  }
 
-          assert ( *pnOut >= 0 );
-          assert ( *pnOut == chequersout ( anBoard ) );
+          g_assert ( *pnOut == chequersout ( anBoard ) );
 
           break;
 
@@ -273,7 +261,7 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
     if ( anBoard[ anDice[ 1 ] - 1 ] ) {
       /* bear-off */
       --anBoard[ anDice[ 1 ] - 1 ];
-    assert ( checkboard ( anBoard ) );
+      g_assert ( checkboard ( anBoard ) );
       return;
     }
 
@@ -287,7 +275,7 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
         --anBoard[ j ];
         ++anBoard[ j - anDice[ 1 ] ];
 
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         return;
         
       }
@@ -309,7 +297,7 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
         /* else */
         /*   bearoff */
 
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         return;
 
       }
@@ -318,7 +306,7 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
 
   }
 
-  assert ( iused == 2 );
+  g_assert ( iused == 2 );
   return;
 
 }
@@ -337,9 +325,8 @@ FindBestMoveOSR2 ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
  *
  */
 
-static void
-FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
-
+static void FindBestMoveOSR4(unsigned int anBoard[ 25 ], const int nDice, unsigned int *pnOut)
+{
   int nd = 4;
   int i, n = 0;
   int first, any;
@@ -347,21 +334,21 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
 
   /* check for exact bear-ins */
 
-  while ( nd > 0 && *pnOut > 0 && anBoard[ 5 + nDice ] ) {
-
+  while ( nd > 0 && *pnOut > 0 && anBoard[ 5 + nDice ] )
+  {
     --anBoard[ 5 + nDice ];
     ++anBoard[ 5 ];
-    assert ( checkboard ( anBoard ) );
+    g_assert ( checkboard ( anBoard ) );
 
     --nd;
+    g_assert ( *pnOut > 0 );
     --*pnOut;
-    assert ( *pnOut >= 0 );
-    assert ( *pnOut == chequersout ( anBoard ) );
+    g_assert ( *pnOut == chequersout ( anBoard ) );
 
   }
 
-  if ( *pnOut > 0 ) {
-
+  if (*pnOut > 0)
+  {
     /* check for 4, 3, or 2 chequers move exactly into home quadrant */
     
     for ( n = nd; n > 1; --n ) {
@@ -372,23 +359,20 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
         
         --anBoard[ i ];
         ++anBoard[ 5 ];
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         
         nd -= n;
+        g_assert (*pnOut > 0);
         --*pnOut;
-        assert ( *pnOut >= 0 );
-        assert ( *pnOut == chequersout ( anBoard ) );
+        g_assert ( *pnOut == chequersout ( anBoard ) );
         
         n =nd; /* restart loop */
-        
       }
-      
     }
-
   }
 
-  if ( *pnOut > 0 && nd > 0 ) {
-
+  if ( *pnOut > 0 && nd > 0 )
+  {
     first = TRUE;
 
     /* find rearest chequer */
@@ -407,80 +391,75 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
 
           /* move chequers */
           
-          while ( anBoard[ i ] && nd && *pnOut ) {
-            
+          while ( anBoard[ i ] && nd && *pnOut )
+		  {
             --anBoard[ i ];
             ++anBoard[ i - nDice ];
-            assert ( checkboard ( anBoard ) );
+            g_assert ( checkboard ( anBoard ) );
             
             if ( i - nDice < 6 )
-              /* we move into homeland */
-              --*pnOut;
+			{	/* we move into homeland */
+				g_assert ( *pnOut > 0 );
+				--*pnOut;
+			}
+
             
-            assert ( *pnOut >= 0 );
-            assert ( *pnOut == chequersout ( anBoard ) );
+            g_assert ( *pnOut == chequersout ( anBoard ) );
             
             --nd;
-            
           }
 
           if ( ! *pnOut || ! nd )
             break;
-          
+
           /* did we move all chequers from that point */
 
           first = ! anBoard[ i ];
-
         }
-
       }
-
     }
 
     /* move chequers from the rear */
 
-    while ( *pnOut && nd ) {
-
-      for ( i = lc; i > 5; --i ) {
-
-        if( anBoard[ i ] ) {
-
-          while ( anBoard[ i ] && nd && *pnOut ) {
-
+    while ( *pnOut && nd )
+	{
+      for ( i = lc; i > 5; --i ) 
+	  {
+        if( anBoard[ i ] )
+		{
+          while ( anBoard[ i ] && nd && *pnOut )
+		  {
             --anBoard[ i ];
             ++anBoard[ i - nDice ];
-            assert ( checkboard ( anBoard ) );
+            g_assert ( checkboard ( anBoard ) );
 
             if ( i - nDice < 6 )
+			{
+              g_assert ( *pnOut > 0 );
               --*pnOut;
+			}
             
-            assert ( *pnOut >= 0 );
-            assert ( *pnOut == chequersout ( anBoard ) );
+            g_assert ( *pnOut == chequersout ( anBoard ) );
 
             --nd;
-
           }
 
           if ( ! n || ! *pnOut )
             break;
-
         }
       }
     }
   }
 
-  
-  if ( ! *pnOut ) {
-
-    /* all chequers inside home quadrant */
-
-    while ( nd ) {
-
+  if (!*pnOut )
+  {		/* all chequers inside home quadrant */
+    while ( nd )
+	{
       if ( anBoard[ nDice - 1 ] ) {
         /* perfect bear-off */
         --anBoard[ nDice - 1 ];
         --nd;
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         continue;
       }
 
@@ -488,7 +467,7 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
         /* bear  double 1s, 2s, and 3s off, e.g., 4/2/0 */
         --anBoard[ 2 * nDice - 1 ];
         nd -= 2;
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         continue;
       }
 
@@ -497,7 +476,7 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
            double 2s off from 6 point (6/4/2/0) */
         --anBoard[ 3 * nDice - 1 ];
         nd -= 3;
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         continue;
       }
 
@@ -505,7 +484,7 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
         /* hmmm, this should not be possible... */
         /* bear off double 1s: 4/3/2/1/0 */
         --anBoard[ 4 * nDice - 1 ];
-        assert ( checkboard ( anBoard ) );
+        g_assert ( checkboard ( anBoard ) );
         nd -= 4;
       }
 
@@ -526,7 +505,7 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
 
           if ( i - nDice > -1 )
             ++anBoard[ i - nDice ];
-          assert ( checkboard ( anBoard ) );
+          g_assert ( checkboard ( anBoard ) );
 
         }
 
@@ -535,11 +514,8 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
       if ( ! any )
         /* no more chequers left */
         nd = 0;
-
     }
-
   }
-
 }
 
 
@@ -557,17 +533,14 @@ FindBestMoveOSR4 ( int anBoard[ 25 ], const int nDice, int *pnOut ) {
  *
  */
 
-static void
-FindBestMoveOSR ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
-
-  if ( anDice[ 0 ] != anDice[ 1 ] )
-    FindBestMoveOSR2 ( anBoard, anDice, pnOut );
-  else
-    FindBestMoveOSR4 ( anBoard, anDice[ 0 ], pnOut );
-
+static void FindBestMoveOSR(unsigned int anBoard[ 25 ], const int anDice[ 2 ], unsigned int *pnOut)
+{
+	if (anDice[0] != anDice[1])
+		FindBestMoveOSR2(anBoard, anDice, pnOut);
+	else
+		FindBestMoveOSR4(anBoard, anDice[0], pnOut);
 }
 
-
 /*
  * osr: one sided rollout.
  *
@@ -581,36 +554,29 @@ FindBestMoveOSR ( int anBoard[ 25 ], const int anDice[ 2 ], int *pnOut ) {
  *          quadrant.
  */
 
-static int
-osr ( int anBoard[ 25 ], const int iGame, const int nGames, const int nOut ) {
+static int osr(unsigned int anBoard[25], const int iGame, const int nGames, unsigned int nOut)
+{
+	int iTurn = 0;
+	int anDice[ 2 ];
 
-  int iTurn = 0;
-  int anDice[ 2 ];
-  int n = nOut;
+	/* loop until all chequers are in home quadrant */
 
-  /* loop until all chequers are in home quadrant */
+	while (nOut)
+	{
+		/* roll dice */
+		if ( OSRQuasiRandomDice ( iTurn, iGame, nGames, anDice ) < 0 )
+			return -1;
 
-  while ( n ) {
+		if ( anDice[ 0 ] < anDice[ 1 ] )
+			swap ( anDice, anDice + 1 );
 
-    /* roll dice */
+		/* find and move best move */
+		FindBestMoveOSR(anBoard, anDice, &nOut);
 
-    if ( OSRQuasiRandomDice ( iTurn, iGame, nGames, anDice ) < 0 )
-      return -1;
-
-    if ( anDice[ 0 ] < anDice[ 1 ] )
-      swap ( anDice, anDice + 1 );
-
-    /* find and move best move */
-
-    FindBestMoveOSR ( anBoard, anDice, &n );
-
-    iTurn++;
-
-  }
-
-
-  return iTurn;
+		iTurn++;
+	}
 
+	return iTurn;
 }
 
 
@@ -629,20 +595,17 @@ osr ( int anBoard[ 25 ], const int iGame, const int nGames, const int nOut ) {
  */
 
 static void
-rollOSR ( const int nGames, const int anBoard[ 25 ], const int nOut,
-          float arProbs[], const int nMaxProbs,
-          float arGammonProbs[], const int nMaxGammonProbs ) {
+rollOSR ( const int nGames, const unsigned int anBoard[ 25 ], const int nOut,
+          float arProbs[], const unsigned int nMaxProbs,
+          float arGammonProbs[], const unsigned int nMaxGammonProbs ) {
 
-  int an[ 25 ];
+  unsigned int an[ 25 ];
   unsigned short int anProb[ 32 ];
-  int i, n, m;
+  unsigned int i;
+  int n, m;
   int iGame;
   
-#if __GNUC__ || !HAVE_ALLOCA
-  int anCounts[nMaxGammonProbs];
-#else
-  int *anCounts = (int*)alloca(sizeof(int) * nMaxGammonProbs);
-#endif
+  int *anCounts = (int*) g_alloca(nMaxGammonProbs * sizeof(int));
 
   memset(anCounts, 0, sizeof(int) * nMaxGammonProbs);
 
@@ -667,7 +630,7 @@ rollOSR ( const int nGames, const int anBoard[ 25 ], const int nOut,
 
     /* update counts */
 
-    ++anCounts[ MIN( m == 15 ? n + 1 : n, nMaxGammonProbs - 1 ) ];
+    ++anCounts[ MIN( m == 15 ? n + 1 : n, (int)nMaxGammonProbs - 1 ) ];
 
     /* get prob. from bearoff1 */
 
@@ -680,7 +643,7 @@ rollOSR ( const int nGames, const int anBoard[ 25 ], const int nOut,
 
   /* scale resulting probabilities */
 
-  for ( i = 0; i < nMaxProbs; ++i ) {
+  for ( i = 0; i < (unsigned int)nMaxProbs; ++i ) {
     arProbs[ i ] /= nGames;
     /* printf ( "arProbs[%d]=%f\n", i, arProbs[ i ] ); */
   }
@@ -713,8 +676,8 @@ rollOSR ( const int nGames, const int anBoard[ 25 ], const int nOut,
  */
 
 static int
-osp ( const int anBoard[ 25 ], const int nGames,
-      int an[ 25 ], float arProbs[ MAX_PROBS ], 
+osp ( const unsigned int anBoard[ 25 ], const int nGames,
+      unsigned int an[ 25 ], float arProbs[ MAX_PROBS ], 
       float arGammonProbs[ MAX_GAMMON_PROBS ] ) {
 
   int i, n;
@@ -774,7 +737,7 @@ osp ( const int anBoard[ 25 ], const int nGames,
 
 
 static float
-bgProb ( const int anBoard[ 25 ],
+bgProb ( const unsigned int anBoard[ 25 ],
          const int fOnRoll,
          const int nTotal,
          const float arProbs[],
@@ -806,7 +769,7 @@ bgProb ( const int anBoard[ 25 ],
       
       /* FIXME: this ignores chequers on the bar */
 
-      getBearoffProbs ( PositionBearoff ( anBoard+18, 6, 15 ), anProb );
+      getBearoffProbs ( PositionBearoff ( anBoard + 18, 6, 15 ), anProb );
 
       for ( i = 0; i < nMaxProbs; ++i ) {
 
@@ -846,11 +809,11 @@ bgProb ( const int anBoard[ 25 ],
  */
 
 extern void
-raceProbs ( int anBoard[ 2 ][ 25 ], const int nGames,
+raceProbs ( const TanBoard anBoard, const int nGames,
             float arOutput[ NUM_OUTPUTS ],
             float arMu[ 2 ] ) {
   
-  int an[ 2 ][ 25 ];
+  TanBoard an;
   float aarProbs[ 2 ][ MAX_PROBS ];
   float aarGammonProbs[ 2 ][ MAX_PROBS ];
   float arG[ 2 ], arBG[ 2 ];
@@ -943,7 +906,7 @@ raceProbs ( int anBoard[ 2 ][ 25 ], const int nGames,
     for ( i = 0; i < 2; ++i ) {
       arMu[ i ] = 0.0f;
       for ( j = 0; j < MAX_PROBS; ++j )
-        arMu[ i ] += 1.0 * j * aarProbs[ i ][ j ];
+        arMu[ i ] += 1.0f * j * aarProbs[ i ][ j ];
 
     }
 
