@@ -4,7 +4,7 @@
  * by Jrn Thyssen <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,14 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: osr.h,v 1.2 2003/05/12 18:41:01 thyssen Exp $
+ * $Id: osr.h,v 1.5 2007/12/29 14:32:31 Superfly_Jon Exp $
  */
 
 #ifndef _OSR_H_
 #define _OSR_H_
 
 extern void
-raceProbs ( int anBoard[ 2 ][ 25 ], const int nGames,
+raceProbs ( const TanBoard anBoard, const int nGames,
             float arOutput[ NUM_OUTPUTS ],
             float arMu[ 2 ] );
 
