@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: progress.c,v 1.25 2006/09/21 22:24:06 Superfly_Jon Exp $
+ * $Id: progress.c,v 1.41 2008/03/12 22:56:35 Superfly_Jon Exp $
  */
 
 #include "config.h"
@@ -74,8 +74,8 @@ typedef struct _rolloutprogress {
 } rolloutprogress;
 
 #if USE_GTK
-void AllocTextList(rolloutprogress *prp)
-{	// 2d array to cache displayed widget text
+static void AllocTextList(rolloutprogress *prp)
+{	/* 2d array to cache displayed widget text */
 	int i;
 	int lines = prp->n;
 	prp->pListText = malloc(sizeof(char*) * lines * 2);
@@ -89,8 +89,8 @@ void AllocTextList(rolloutprogress *prp)
 	}
 }
 
-void FreeTextList(rolloutprogress *prp)
-{	// destroy list
+static void FreeTextList(rolloutprogress *prp)
+{	/* destroy list */
 	int i;
 	int lines = prp->n;
 
@@ -102,8 +102,8 @@ void FreeTextList(rolloutprogress *prp)
 	free(prp->pListText);
 }
 
-void SetRolloutText(rolloutprogress *prp, int x, int y, char* sz)
-{	// Cache set text to reduce flicker (and speed things up a bit)
+static void SetRolloutText(rolloutprogress *prp, int x, int y, char* sz)
+{	/* Cache set text to reduce flicker (and speed things up a bit) */
 	if (!prp->pListText[x][y] || strcmp(prp->pListText[x][y - 1], sz))
 	{
 		gtk_clist_set_text(GTK_CLIST(prp->pwRolloutResult), x, y, sz);
@@ -189,6 +189,7 @@ GTKStatPageWin ( const rolloutstat *prs, const int cGames ) {
   static char *aszRow[ 7 ];
   int i;
   int anTotal[ 6 ];
+  int cGamesCount=0;
 
   pw = gtk_vbox_new ( FALSE, 0 );
 
@@ -259,14 +260,18 @@ GTKStatPageWin ( const rolloutstat *prs, const int cGames ) {
 
 
   sprintf ( aszRow[ 0 ], "%%" );
-  for ( i = 0; i < 6; i++ )
+  for ( i = 0; i < 6; i++ ) {
     sprintf ( aszRow[ i + 1 ], "%6.2f%%",
 	      100.0 * (float) anTotal[ i ] / (float) cGames );
+    cGamesCount+=anTotal[i];
+  }
   gtk_clist_append ( GTK_CLIST ( pwStat ), aszRow );
 
 
   for ( i = 0; i < 7; i++ ) free ( aszRow[ i ] );
 
+  /* allow for one missing (could just be a stopped rollout) */
+  if (cGamesCount < (cGames-1)) outputerrf (_("Win statistics invalid due to (race) truncation, or  (%d != %d)"), cGamesCount, cGames);
 
   return pw;
 
@@ -397,9 +402,9 @@ GTKStatPageBearoff ( const rolloutstat *prs, const int cGames ) {
 
   /* column titles */
 
-  aszColumnTitle[ 0 ] = strdup ( "" );
+  aszColumnTitle[ 0 ] = g_strdup ( "" );
   for ( i = 0; i < 2; i++ )
-    aszColumnTitle[ i + 1 ] = strdup ( ap[ i ].szName );
+    aszColumnTitle[ i + 1 ] = g_strdup ( ap[ i ].szName );
 
   /* table with results */
 
@@ -479,9 +484,9 @@ GTKStatPageClosedOut ( const rolloutstat *prs, const int cGames ) {
 
   /* column titles */
 
-  aszColumnTitle[ 0 ] = strdup ( "" );
+  aszColumnTitle[ 0 ] = g_strdup ( "" );
   for ( i = 0; i < 2; i++ )
-    aszColumnTitle[ i + 1 ] = strdup ( ap[ i ].szName );
+    aszColumnTitle[ i + 1 ] = g_strdup ( ap[ i ].szName );
 
   /* table with results */
 
@@ -556,9 +561,9 @@ GTKStatPageHit ( const rolloutstat *prs, const int cGames ) {
 
   /* column titles */
 
-  aszColumnTitle[ 0 ] = strdup ( "" );
+  aszColumnTitle[ 0 ] = g_strdup ( "" );
   for ( i = 0; i < 2; i++ )
-    aszColumnTitle[ i + 1 ] = strdup ( ap[ i ].szName );
+    aszColumnTitle[ i + 1 ] = g_strdup ( ap[ i ].szName );
 
   /* table with results */
 
@@ -700,7 +705,6 @@ GTKViewRolloutStatistics(GtkWidget *widget, gpointer data){
   
   rolloutprogress *prp = (rolloutprogress *) data;
   rolloutstat *prs = prp->prs;
-  // int cGames = gtk_progress_get_value ( GTK_PROGRESS( prp->pwRolloutProgress ) );
   int cGames = prp->nGamesDone;
   int nRollouts = GTK_CLIST( prp->pwRolloutResult )->rows / 2;
   int i;
@@ -736,28 +740,16 @@ GTKViewRolloutStatistics(GtkWidget *widget, gpointer data){
 		GTKRolloutStatPage ( &prs[ i * 2 ], cGames ), gtk_label_new ( sz ) );
   }
 
-  gtk_widget_show_all( pwDialog );
-  gtk_main();
+  GTKRunDialog(pwDialog);
 }
 
-static void RolloutCancel( GtkObject *po, rolloutprogress *prp ) {
-
+static void RolloutCancel( GtkObject *po, rolloutprogress *prp )
+{
     pwGrab = pwOldGrab;
     prp->pwRolloutDialog = NULL;
     prp->pwRolloutResult = NULL;
     prp->pwRolloutProgress = NULL;
     fInterrupt = TRUE;
-    /* FIXME: we have a small problem here:
-     *
-     * rollout.c: calls fnAction, which calls
-     *            GTKSuspendInput
-     *            then the rollout dialog is canceled, which leads
-     *            to pwGrab being reset to pwOldGrab
-     *            GTKResumeInput, which tries to remove signal from
-     *            pwGrab, but pwGrab has just be replaced...
-     */
-
-
 }
 
 static void RolloutStop( GtkObject *po, gpointer p ) {
@@ -798,7 +790,8 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
 
   AllocTextList(prp);
 
-  prp->pwRolloutDialog = GTKCreateDialog( _("GNU Backgammon - Rollout"), DT_INFO, NULL, DIALOG_FLAG_MODAL, NULL, NULL );
+  prp->pwRolloutDialog = GTKCreateDialog( _("GNU Backgammon - Rollout"), DT_INFO, NULL,
+	  DIALOG_FLAG_MODAL | DIALOG_FLAG_MINMAXBUTTONS | DIALOG_FLAG_NOTIDY, NULL, NULL );
   prp->pwRolloutViewStat = gtk_button_new_with_label ( _("View statistics") );
   prp->pwRolloutStop = gtk_button_new_with_label( _("Stop") );
     
@@ -806,8 +799,8 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
   pwGrab = prp->pwRolloutDialog;
     
   prp->nRolloutSignal = 
-    gtk_signal_connect( GTK_OBJECT( prp->pwRolloutDialog ),
-                        "destroy", GTK_SIGNAL_FUNC( RolloutCancel ), prp );
+    g_signal_connect( G_OBJECT( prp->pwRolloutDialog ),
+                        "destroy", G_CALLBACK( RolloutCancel ), prp );
 
   /* Buttons */
 
@@ -816,7 +809,7 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
 
   gtk_container_add( GTK_CONTAINER( pwButtons ), prp->pwRolloutStop );
     
-  if ( aars )
+  if ( aars && (prc->nGamesDone == 0) )
     gtk_container_add( GTK_CONTAINER( pwButtons ), prp->pwRolloutViewStat );
 
   gtk_widget_set_sensitive( prp->pwRolloutOK, FALSE );
@@ -824,11 +817,11 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
     
   /* Setup signal */
 
-  gtk_signal_connect( GTK_OBJECT( prp->pwRolloutStop ), "clicked",
-                      GTK_SIGNAL_FUNC( RolloutStop ), prp );
+  g_signal_connect( G_OBJECT( prp->pwRolloutStop ), "clicked",
+                      G_CALLBACK( RolloutStop ), prp );
     
-  gtk_signal_connect( GTK_OBJECT( prp->pwRolloutViewStat ), "clicked",
-                      GTK_SIGNAL_FUNC( GTKViewRolloutStatistics ), prp );
+  g_signal_connect( G_OBJECT( prp->pwRolloutViewStat ), "clicked",
+                      G_CALLBACK( GTKViewRolloutStatistics ), prp );
 
   pwVbox = gtk_vbox_new( FALSE, 4 );
 	
@@ -839,9 +832,6 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
   gtk_clist_column_titles_passive( GTK_CLIST( prp->pwRolloutResult ) );
     
   prp->pwRolloutProgress = gtk_progress_bar_new();
-#if !USE_GTK
-  gtk_progress_set_show_text ( GTK_PROGRESS ( prp->pwRolloutProgress ), TRUE );
-#endif
   
   gtk_box_pack_start( GTK_BOX( pwVbox ), prp->pwRolloutResult, TRUE, TRUE, 0 );
   gtk_box_pack_start( GTK_BOX( pwVbox ), prp->pwRolloutProgress, FALSE, FALSE,
@@ -864,12 +854,6 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
                         _("Standard error") );
   }
 
-#if !USE_GTK
-  gtk_progress_configure( GTK_PROGRESS( prp->pwRolloutProgress ), 0, 0, 
-                          prc->nTrials );
-  gtk_progress_set_format_string( GTK_PROGRESS( prp->pwRolloutProgress ),
-                                  "%v/%u (%p%%)" );
-#endif
   
   /* time elapsed and left */
 
@@ -910,13 +894,6 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
   /* record start time */
   time( &prp->tStart );
 
-#if 0
-  GTKDisallowStdin();
-  while( gtk_events_pending() )
-    gtk_main_iteration();
-  GTKAllowStdin();
-#endif
-
 }
 
 static void
@@ -934,10 +911,8 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 
     char sz[ 32 ];
     int i;
-#if USE_GTK
     gchar *gsz;
     double frac;
-#endif
 
     if( !prp ||  !prp->pwRolloutResult )
       return;
@@ -989,10 +964,6 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 	  SetRolloutText(prp, iAlternative * 2, i + 1, "n/a");
 	}
 	  
-#if !USE_GTK
-    gtk_progress_configure( GTK_PROGRESS( prp->pwRolloutProgress ),
-                            iGame + 1, 0, prc->nTrials );
-#else
     frac = (iGame + 1) * 1.0 / (prc->nTrials * 1.0); 
     /* why doesn't type casting work? */
     gsz = g_strdup_printf( "%d/%d (%.0f%%)" ,
@@ -1003,11 +974,10 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 		    frac );
     gtk_progress_bar_set_text(GTK_PROGRESS_BAR( prp->pwRolloutProgress), gsz );
     g_free( gsz );
-#endif
     
     /* calculate estimate time left */
 
-    if ( iAlternative == ( prp->n - 1 ) ) {
+    if ( (iAlternative == ( prp->n - 1 )) && iGame > 10 ) {
       /* The code in estimatedTimeLeft allows that it's called on every
          iAlternative, but it gives a lot of updates on the gui. If someone
          thinks otherwise, we can consider removing the if-condition above */
@@ -1043,21 +1013,12 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 
     }
       
-
-#if 0
-    GTKDisallowStdin();
-    while( gtk_events_pending() )
-      gtk_main_iteration();
-    GTKAllowStdin();
-#endif
     return;
 }
 
 static void GTKRolloutProgressEnd( void **pp ) {
     
-#if USE_GTK
     gchar *gsz;
-#endif
 
     rolloutprogress *prp = *pp;
 
@@ -1083,14 +1044,12 @@ static void GTKRolloutProgressEnd( void **pp ) {
                                     gsz );
     g_free( gsz );
 
-    gtk_signal_disconnect( GTK_OBJECT( prp->pwRolloutDialog ), 
+    g_signal_handler_disconnect( G_OBJECT( prp->pwRolloutDialog ), 
                            prp->nRolloutSignal );
-    gtk_signal_connect( GTK_OBJECT( prp->pwRolloutDialog ), "destroy",
-                        GTK_SIGNAL_FUNC( gtk_main_quit ), NULL );
+    g_signal_connect( G_OBJECT( prp->pwRolloutDialog ), "destroy",
+                        G_CALLBACK( gtk_main_quit ), NULL );
 
-    GTKDisallowStdin();
-    gtk_main();
-    GTKAllowStdin();
+	gtk_main();
 
     prp->pwRolloutProgress = NULL;
 
@@ -1156,15 +1115,17 @@ TextRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 
   /* write progress 1/10th trial */
 
-  if ( iGame == prp->iNextGame && iAlternative == prp->iNextAlternative ) {
+  if ( iGame >= prp->iNextGame && iAlternative == prp->iNextAlternative ) {
 
     if ( ! iAlternative )
       outputl( "" );
 
     pch = OutputRolloutResult( NULL, 
                                (char(*) [1024]) prp->ppch[ iAlternative ], 
-                               GCCCONSTAHACK aarOutput[ iAlternative ],
-                               GCCCONSTAHACK aarStdDev[ iAlternative ],
+                               ( float (*)[NUM_ROLLOUT_OUTPUTS] )
+                                aarOutput[ iAlternative ],
+                                ( float (*)[NUM_ROLLOUT_OUTPUTS] )
+                                aarStdDev[ iAlternative ],
                                &aci[ iAlternative ], 1, prc->fCubeful );
 
     if ( fShowRanks && iGame > 1 ) {
