@@ -4,7 +4,7 @@
  * by Joern Thyssen <jth@gnubg.org>, 2004.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,24 +16,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: relational.h,v 1.4 2004/11/05 14:46:22 Superfly_Jon Exp $
+ * $Id: relational.h,v 1.13 2009/03/24 23:48:46 c_anthon Exp $
  */
 
 #ifndef _RELATIONAL_H_
 #define _RELATIONAL_H_
 
-typedef struct _RowSet
-{
-	int cols, rows;
-	char ***data;
-	int *widths;
-} RowSet;
+#include <dbprovider.h>
+#include <stddef.h>
+#include <sys/types.h>
+#include "analysis.h"
 
-extern int RelationalMatchExists();
-extern void RelationalUpdatePlayerDetails(int player_id, const char* newName,
-										  const char* newNotes);
-extern int RunQuery(RowSet* pRow, char *sz);
-extern void FreeRowset(RowSet* pRow);
-extern void RelationalLinkNick(char* nick, char* env, char* player);
+#define DB_VERSION 1
+
+
+extern int RelationalUpdatePlayerDetails(const char* oldName, const char* newName, const char* newNotes);
+extern float Ratio(float a, int b);
+extern statcontext *relational_player_stats_get(const char *player0, const char *player1);
 
 #endif /* _RELATIONAL_H_ */
