@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 1997-2002.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,46 +16,48 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: render.c,v 1.48.2.1 2006/11/10 11:34:40 c_anthon Exp $
+ * $Id: render.c,v 1.88 2009/10/26 16:12:26 Superfly_Jon Exp $
  */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
+#include "common.h"
 
-#include <assert.h>
+#include <glib.h>
 #include <stdio.h>
 #include <string.h>
 #if HAVE_FREETYPE
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_GLYPH_H
-#include "path.h"
 #endif
 #include <isaac.h>
 #include <math.h>
 #include <stdlib.h>
 
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
 #include "render.h"
 #include "renderprefs.h"
 #include "boarddim.h"
 #include "backgammon.h"
+#include "util.h"
 
 #if USE_GTK
 #include <gtk/gtk.h>
 #include <cairo.h>
 #endif
+#if USE_BOARD3D
+#include "fun3d.h"
+#endif
 
 static randctx rc;
 #define RAND irand( &rc )
 
+#define FONT_VERA "fonts/Vera.ttf"
+#define FONT_VERA_SERIF_BOLD "fonts/VeraSeBd.ttf"
+#define FONT_VERA_BOLD "fonts/VeraBd.ttf"
+
 /* aaanPositions[Clockwise][x][point number][x, y. deltay] */
 int positions[ 2 ][ 30 ][ 3 ] = { {
-    { BAR_X, BAR_Y_0, CHEQUER_HEIGHT },      /* bar - player 0 */
+    { BAR_X, BAR_Y_1, -CHEQUER_HEIGHT },      /* bar - player 1 */
     { POINT_X(1),  TOP_POINT_Y,  CHEQUER_HEIGHT }, 
     { POINT_X(2),  TOP_POINT_Y,  CHEQUER_HEIGHT }, 
     { POINT_X(3),  TOP_POINT_Y,  CHEQUER_HEIGHT }, 
@@ -80,7 +82,7 @@ int positions[ 2 ][ 30 ][ 3 ] = { {
     { POINT_X(22), BOT_POINT_Y, -CHEQUER_HEIGHT }, 
     { POINT_X(23), BOT_POINT_Y, -CHEQUER_HEIGHT }, 
     { POINT_X(24), BOT_POINT_Y, -CHEQUER_HEIGHT }, 
-    { BAR_X, BAR_Y_1, -(CHEQUER_HEIGHT) },        /* bar player 1 */
+    { BAR_X, BAR_Y_0, (CHEQUER_HEIGHT) },        /* bar player 0 */
     { BEAROFF_RIGHT_X, TOP_POINT_Y,  CHEQUER_WIDTH }, /* player 0 tray right */
     { BEAROFF_RIGHT_X, BOT_POINT_Y, -CHEQUER_WIDTH }, /* player 0 tray right */
     { BEAROFF_LEFT_X, TOP_POINT_Y,  CHEQUER_WIDTH },  /* player 0 tray left */
@@ -120,11 +122,6 @@ int positions[ 2 ][ 30 ][ 3 ] = { {
 
 #if HAVE_FREETYPE
 FT_Library ftl;
-
-#if ( FREETYPE_MINOR < 1 ) || ( ( FREETYPE_MINOR == 1 ) && ( FREETYPE_PATCH < 3 ) )
-#define FT_GLYPH_FORMAT_BITMAP ft_glyph_format_bitmap
-#define FT_PIXEL_MODE_GRAY ft_pixel_mode_grays
-#endif
 #endif
 
 renderdata rdDefault = {
@@ -138,17 +135,16 @@ renderdata rdDefault = {
       { 0xFF, 0x5F, 0x5F, 0xFF }, { 0xBF, 0xBF, 0xBF, 0xFF } }, /* aanBoardC */
     { 25, 25, 25, 25 }, /* aSpeckle */
     { 1.5, 1.5 }, /* arRefraction */
-    { 0.2, 1.0 }, /* arCoefficient */
+    { 0.2f, 1.0f }, /* arCoefficient */
     { 3.0, 30.0 }, /* arExponent */
-    { 0.2, 1.0 }, /* arDiceCoefficient */
+    { 0.2f, 1.0f }, /* arDiceCoefficient */
     { 3.0, 30.0 }, /* arDiceExponent */
-    { -0.55667, 0.32139, 0.76604 }, /* arLight */
+    { -0.55667f, 0.32139f, 0.76604f }, /* arLight */
     0.5, /* rRound */
     -1, /* nSize */
     TRUE, /* fHinges */
     TRUE, /* fLabels */
     FALSE, /* fClockwise */
-    TRUE, /* position+match ids */
     FALSE, /* Dice area */
     TRUE, /* Show game info */
     TRUE,  /* dynamic labels */
@@ -180,21 +176,21 @@ renderdata rdDefault = {
 	0, 0,	/* Used at runtime */
 	/* Default 3d colours - black+white, should never be used -
 		if no 3d settings the first design will be used */
-    {{{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0},
-	 {{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, 0}},
-    {{{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0},
-	 {{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, 0}},
-	{{{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, 0},
-     {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0}},
-    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0},
-	{{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, 0},
-    {{.5, .5, .5, .5}, {.5, .5, .5, .5}, {.5, .5, .5, .5}, 100, 0, 0},
-    {{{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0},
-	 {{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, 0}},
-	{{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, 0},
-    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0},
-    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0},
-    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, 0}
+    {{{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL},
+	 {{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, NULL, NULL}},
+    {{{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL},
+	 {{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, NULL, NULL}},
+	{{{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, NULL, NULL},
+     {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL}},
+    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL},
+	{{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, NULL, NULL},
+    {{.5, .5, .5, .5}, {.5, .5, .5, .5}, {.5, .5, .5, .5}, 100, 0, NULL, NULL},
+    {{{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL},
+	 {{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, NULL, NULL}},
+	{{0, 0, 0, 1}, {0, 0, 0, 1}, {0, 0, 0, 1}, 100, 0, NULL, NULL},
+    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL},
+    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL},
+    {{1, 1, 1, 1}, {1, 1, 1, 1}, {1, 1, 1, 1}, 100, 0, NULL, NULL}
 #endif
 };
 
@@ -452,7 +448,7 @@ extern void AlphaBlendClip( unsigned char *puchDest, int nDestStride,
 		cx, cy );
 }
 
-extern void AlphaBlend2( unsigned char *puchDest, int nDestStride,
+static void AlphaBlend2( unsigned char *puchDest, int nDestStride,
 			 unsigned char *puchBack, int nBackStride,
 			 unsigned char *puchFore, int nForeStride,
 			 int cx, int cy ) {
@@ -655,14 +651,15 @@ static void RenderBorder( unsigned char *puch, int nStride, int x0, int y0,
     }
 }
 
-static void RenderFramePainted( renderdata *prd, unsigned char *puch,
-				int nStride ) {
-    int i, ix;
+static void RenderFramePainted(renderdata *prd, unsigned char *puch, int nStride)
+{
+    int i;
+    unsigned int ix;
     float x, z, cos_theta, diffuse, specular;
-	VARIABLE_ARRAY(unsigned char, colours, 4 * 3 * prd->nSize)
+	unsigned char *colours = (unsigned char*) g_alloca(4 * 3 * prd->nSize * sizeof(unsigned char));
 
-    diffuse = 0.8 * prd->arLight[ 2 ] + 0.2;
-    specular = pow( prd->arLight[ 2 ], 20 ) * 0.6;
+    diffuse = 0.8f * prd->arLight[ 2 ] + 0.2f;
+    specular = pow( prd->arLight[ 2 ], 20 ) * 0.6f;
 
     /* fill whole board area with flat colour */
     FillArea( puch, nStride, prd->nSize * BOARD_WIDTH, prd->nSize * ( BOARD_HEIGHT - 1 ),
@@ -1026,8 +1023,12 @@ static void WoodPixel( float x, float y, float z, unsigned char auch[ 3 ],
 	break;
 	
     default:
-	assert( FALSE );
+	g_assert( FALSE );
     }
+#if USE_GTK
+	if (showingGray)
+		GrayScaleColC(auch);
+#endif
 }
 
 static void RenderFrameWood( renderdata *prd, unsigned char *puch,
@@ -1039,17 +1040,17 @@ static void RenderFrameWood( renderdata *prd, unsigned char *puch,
 	s = prd->nSize;
     unsigned char a[ 3 ];
     float rx, rz, cos_theta, rDiffuseTop, rHeight, rDiffuse;
-#if __GNUC__ || !HAVE_ALLOCA
-	int anSpecular[4][s];
-	float arDiffuse[4][s];
-	float arHeight[s];
-#else
-	int *anSpecularData = alloca( 4 * s * sizeof( int ) );
-	int *anSpecular[4] = { anSpecularData, anSpecularData + s, anSpecularData + 2 * s, anSpecularData + 3 * s };
-	float *arDiffuseData = alloca( 4 * s * sizeof( float ) );
-	float *arDiffuse[4] = { arDiffuseData, arDiffuseData + s, arDiffuseData + 2 * s, arDiffuseData + 3 * s };
-	float *arHeight = alloca( s * sizeof( float ) );
-#endif
+	int *anSpecular[4], *anSpecularData = g_alloca( 4 * s * sizeof( int ) );
+	float *arDiffuse[4], *arDiffuseData = g_alloca( 4 * s * sizeof( float ) );
+	float *arHeight = g_alloca( s * sizeof( float ) );
+	anSpecular[0] = anSpecularData;
+	anSpecular[1] = anSpecularData + s;
+	anSpecular[2] = anSpecularData + 2 * s;
+	anSpecular[3] = anSpecularData + 3 * s;
+	arDiffuse[0] = arDiffuseData;
+	arDiffuse[1] = arDiffuseData + s;
+	arDiffuse[2] = arDiffuseData + 2 * s;
+	arDiffuse[3] = arDiffuseData + 3 * s;
 
 
     nSpecularTop = pow( prd->arLight[ 2 ], 20 ) * 0.6 * 0x100;
@@ -1264,11 +1265,14 @@ static void HingePixel( renderdata *prd, float xNorm, float yNorm,
     float arReflection[ 3 ], arAuxLight[ 2 ][ 3 ] = {
 	{ 0.6, 0.7, 0.5 },
 	{ 0.5, -0.6, 0.7 } };
-    float *arLight[ 3 ] = { prd->arLight, arAuxLight[ 0 ], arAuxLight[ 1 ] };
     float zNorm, zEye;
     float diffuse, specular = 0, cos_theta;
     float l;
     int i;
+    float *arLight[ 3 ];
+	arLight[ 0 ] = prd->arLight;
+	arLight[ 1 ] = arAuxLight[ 0 ];
+	arLight[ 2 ] = arAuxLight[ 1 ];
 
     zNorm = ssqrt( 1.0 - xNorm * xNorm - yNorm * yNorm );
     
@@ -1425,7 +1429,7 @@ static void RenderGlyph( unsigned char *puch, int nStride, FT_Glyph pftg,
     FT_Bitmap *pb;
     int x, y, x0 = 0, y0 = 0;
     
-    assert( pftg->format == FT_GLYPH_FORMAT_BITMAP );
+    g_assert( pftg->format == FT_GLYPH_FORMAT_BITMAP );
 
     pftbg = (FT_BitmapGlyph) pftg;
     pb = &pftbg->bitmap;
@@ -1443,7 +1447,7 @@ static void RenderGlyph( unsigned char *puch, int nStride, FT_Glyph pftg,
 	yOff = 0;
     }
     
-    assert( pb->pixel_mode == FT_PIXEL_MODE_GRAY );
+    g_assert( pb->pixel_mode == FT_PIXEL_MODE_GRAY );
 
     puch += yOff * nStride + xOff * 3;
     nStride -= 3 * pb->width;
@@ -1507,23 +1511,22 @@ RenderBasicLabels( renderdata *prd, unsigned char *puch, int nStride,
 
 static void
 RenderLabels( renderdata *prd, unsigned char *puch, int nStride,
-               const int iStart, const int iEnd, const int iDelta ) {
-
-
+               const int iStart, const int iEnd, const int iDelta )
+{
 #if HAVE_FREETYPE
     FT_Face ftf;
     int i;
     FT_Glyph aftg[ 10 ];
     char *file;
 
-    file = PathSearch(FONT_VERA, szDataDirectory);
+    file = BuildFilename(FONT_VERA);
     if( FT_New_Face( ftl, file, 0, &ftf ) )
 	{
 		RenderBasicLabels( prd, puch, nStride, iStart, iEnd, iDelta );
-                free(file);
+                g_free(file);
 		return;
 	}
-    free(file);
+    g_free(file);
 
     if( FT_Set_Pixel_Sizes( ftf, 0, prd->nSize * 5 / 2 ) )
 	{
@@ -1555,9 +1558,8 @@ RenderLabels( renderdata *prd, unsigned char *puch, int nStride,
 
 }
 
-static unsigned char BoardPixel( renderdata *prd, int i, int antialias,
-				  int j ) {
-
+static unsigned char BoardPixel(renderdata *prd, int i, int antialias, int j)
+{
     return clamp( ( ( (int) prd->aanBoardColour[ 0 ][ j ] -
 		      (int) prd->aSpeckle[ 0 ] / 2 +
 		      (int) RAND % ( prd->aSpeckle[ 0 ] + 1 ) ) *
@@ -1570,7 +1572,8 @@ static unsigned char BoardPixel( renderdata *prd, int i, int antialias,
 
 extern void RenderBoard( renderdata *prd, unsigned char *puch, int nStride ) {
 
-    int ix, iy, antialias;
+    unsigned int ix, iy;
+    int antialias;
 
 #define BUF( y, x, i ) ( puch[ (y) * nStride + (x) * 3 + (i) ] )
 
@@ -1710,7 +1713,7 @@ extern void RenderChequers( renderdata *prd, unsigned char *puch0,
     int ix, iy, in, fx, fy, i;
     float x, y, z, x_loop, y_loop, diffuse, specular_x, specular_o, cos_theta,
 	r, x1, y1, len;
-    
+
 #define BUFX( y, x, i ) puch0[ ( (y) * size + (x) ) * 4 + (i) ]
 #define BUFO( y, x, i ) puch1[ ( (y) * size + (x) ) * 4 + (i) ]
 
@@ -1846,14 +1849,15 @@ extern void RenderChequers( renderdata *prd, unsigned char *puch0,
 
 extern void RenderChequerLabels( renderdata *prd, unsigned char *puch,
 				 int nStride ) {
-    int i, ip;
+    int i;
+    unsigned int ip;
 #if HAVE_FREETYPE
     FT_Face ftf;
     FT_Glyph aftg[ 10 ];
     int fFreetype = FALSE;
     char *file;
     
-    file = PathSearch(FONT_VERA, szDataDirectory);
+    file = BuildFilename(FONT_VERA);
     if( !FT_New_Face( ftl, file, 0, &ftf ) &&
 	!FT_Set_Pixel_Sizes( ftf, 0, 2 * prd->nSize ) ) {
 	fFreetype = TRUE;
@@ -1864,7 +1868,7 @@ extern void RenderChequerLabels( renderdata *prd, unsigned char *puch,
 
 	FT_Done_Face( ftf );
     }
-    free(file);
+    g_free(file);
 #endif
 
     for( i = 0; i < 12; i++ ) {
@@ -2006,7 +2010,7 @@ RenderBasicCube ( const float arLight[ 3 ], const int nSize,
 
 
 
-extern void
+static void
 RenderResign( renderdata *prd, unsigned char *puch, int nStride ) {
 
   const double arColour[ 4 ] = { 1.0, 1.0, 1.0, 0.0 }; /* white */
@@ -2033,9 +2037,9 @@ extern void RenderCubeFaces( renderdata *prd, unsigned char *puch,
     int fFreetype = FALSE;
     char *file;
     
-    file = PathSearch(FONT_VERA_SERIF_BOLD, szDataDirectory);
+    file = BuildFilename(FONT_VERA_SERIF_BOLD);
     if( !FT_New_Face( ftl, file, 0, &ftf ) &&
-	!FT_Set_Pixel_Sizes( ftf, 0, 4.5 * prd->nSize ) ) {
+	!FT_Set_Pixel_Sizes( ftf, 0, 3 * prd->nSize ) ) {
 	fFreetype = TRUE;
 	
 	for( i = 0; i < 10; i++ ) {
@@ -2052,9 +2056,9 @@ extern void RenderCubeFaces( renderdata *prd, unsigned char *puch,
 	
 	FT_Done_Face( ftf );
     }
-    free(file);
+    g_free(file);
 #endif
-    
+
     for( i = 0; i < 6; i++ ) {
 	/* Clear destination buffer (no blending at present - so all overwriten anyway) */
 	memset(puch, 0, nStride * CUBE_LABEL_HEIGHT * prd->nSize);
@@ -2065,8 +2069,8 @@ extern void RenderCubeFaces( renderdata *prd, unsigned char *puch,
 
 #if HAVE_FREETYPE
 	if( fFreetype )
-	    RenderNumber( puch, nStride, aftg, 2 << i, 3 * prd->nSize,
-			  4.8 * prd->nSize, 0, 0, 0x80 );
+	    RenderNumber( puch, nStride, aftg, 2 << i, 
+		2 * prd->nSize, 3 * prd->nSize, 0, 0, 0x80 );
 	else
 #endif
 	    RenderBasicNumber( puch, nStride, 4 * prd->nSize, 2 << i,
@@ -2085,8 +2089,8 @@ extern void RenderCubeFaces( renderdata *prd, unsigned char *puch,
 
 #if HAVE_FREETYPE
 	if( fFreetype )
-	    RenderNumber( puch, nStride, aftgSmall, 2 << i, 3 * prd->nSize,
-			  63 * prd->nSize / 16, 0, 0, 0x80 );
+	    RenderNumber( puch, nStride, aftgSmall, 2 << i, 2 * prd->nSize,
+			  3 * prd->nSize, 0, 0, 0x80 );
 	else
 #endif
 	    RenderBasicNumber( puch, nStride, 3 * prd->nSize, 2 << i,
@@ -2105,7 +2109,7 @@ extern void RenderCubeFaces( renderdata *prd, unsigned char *puch,
 }
 
 
-extern void RenderResignFaces( renderdata *prd, unsigned char *puch,
+static void RenderResignFaces( renderdata *prd, unsigned char *puch,
 			     int nStride, unsigned char *puchCube,
 			     int nStrideCube ) {
     int i;
@@ -2116,9 +2120,9 @@ extern void RenderResignFaces( renderdata *prd, unsigned char *puch,
     int fFreetype = FALSE;
     char *file;
     
-    file = PathSearch(FONT_VERA_SERIF_BOLD, szDataDirectory);
+    file = BuildFilename(FONT_VERA_SERIF_BOLD);
     if( !FT_New_Face( ftl, file, 0, &ftf ) &&
-	!FT_Set_Pixel_Sizes( ftf, 0, 5 * prd->nSize ) ) {
+	!FT_Set_Pixel_Sizes( ftf, 0, 4 * prd->nSize ) ) {
 	fFreetype = TRUE;
 	
 	for( i = 0; i < 10; i++ ) {
@@ -2126,7 +2130,7 @@ extern void RenderResignFaces( renderdata *prd, unsigned char *puch,
 	    FT_Get_Glyph( ftf->glyph, aftg + i );
 	}
 	
-	FT_Set_Pixel_Sizes( ftf, 0, 21 * prd->nSize / 8 );
+	FT_Set_Pixel_Sizes( ftf, 0, 3 * prd->nSize );
 	
 	for( i = 0; i < 10; i++ ) {
 	    FT_Load_Char( ftf, '0' + i, FT_LOAD_RENDER );
@@ -2135,7 +2139,7 @@ extern void RenderResignFaces( renderdata *prd, unsigned char *puch,
 	
 	FT_Done_Face( ftf );
     }
-    free(file);
+    g_free(file);
 #endif
     
     for( i = 0; i < 3; i++ ) {
@@ -2149,8 +2153,8 @@ extern void RenderResignFaces( renderdata *prd, unsigned char *puch,
 #if HAVE_FREETYPE
 	if( fFreetype )
             RenderNumber( puch, nStride, aftg, i + 1,
-                          3 * prd->nSize,
-			  78 * prd->nSize / 16, 0, 0, 0x80 );
+                          2 * prd->nSize,
+			  3.5 * prd->nSize, 0, 0, 0x80 );
 	else
 #endif
             RenderBasicNumber( puch, nStride, 4 * prd->nSize, 
@@ -2172,9 +2176,10 @@ extern void RenderResignFaces( renderdata *prd, unsigned char *puch,
 #define circ(x,y) ssqrt( (x*x) + (y*y) )
 
 extern void RenderDice( renderdata *prd, unsigned char *puch0,
-			unsigned char *puch1, int nStride ) {
+			unsigned char *puch1, int nStride, int alpha ) {
     
-    int ix, iy, in, fx, fy, i;
+    unsigned int ix, iy;
+    int in, fx, fy, i;
     float x, y, x_loop, y_loop, diffuse, specular_x, specular_o, cos_theta,
 	x_norm, y_norm, z_norm;
     double *aarDiceColour[ 2 ];
@@ -2247,12 +2252,14 @@ extern void RenderDice( renderdata *prd, unsigned char *puch0,
 	    for( i = 0; i < 3; i++ )
 		*puch0++ = clamp( ( diffuse * aarDiceColour[ 0 ][ i ] +
 				    specular_x ) * 64.0 );
-	    *puch0++ = 255 * ( 4 - in ) / 4; /* alpha channel */
+		if (alpha)
+		    *puch0++ = 255 * ( 4 - in ) / 4; /* alpha channel */
 	    
 	    for( i = 0; i < 3; i++ )
 		*puch1++ = clamp( ( diffuse * aarDiceColour[ 1 ][ i ] +
 				    specular_o ) * 64.0 );
-	    *puch1++ = 255 * ( 4 - in ) / 4; /* alpha channel */
+		if (alpha)
+		    *puch1++ = 255 * ( 4 - in ) / 4; /* alpha channel */
 	    
 	    x_loop += 2.0 / ( DIE_WIDTH * prd->nSize );
 	}
@@ -2265,7 +2272,8 @@ extern void RenderDice( renderdata *prd, unsigned char *puch0,
 extern void RenderPips( renderdata *prd, unsigned char *puch0,
 			unsigned char *puch1, int nStride ) {
 
-    int ix, iy, in, fx, fy, i;
+    unsigned int ix, iy;
+    int in, fx, fy, i;
     float x, y, z, x_loop, y_loop, diffuse, specular_x, specular_o, cos_theta,
 	dice_top[ 2 ][ 3 ];
     double *aarDiceColour[ 2 ];
@@ -2382,31 +2390,42 @@ static void Copy_RGB_to_RGBA( unsigned char *puchDest, int nDestStride,
     }
 }
 
-#if USE_GTK
-static void RenderArrow(unsigned char* puch, double arColour[4], int nSize, int up)
+#if USE_GTK && HAVE_CAIRO
+static double Highlight(double c)
 {
-	cairo_surface_t* surface = cairo_image_surface_create_for_data (puch, CAIRO_FORMAT_RGB24,
+	if (c < .75)
+		return c + .25;
+	else
+		return c - .25;
+}
+
+static void RenderArrow(unsigned char* puch, double arColour[4], int nSize, int left)
+{
+	cairo_surface_t* surface = cairo_image_surface_create_for_data (puch, CAIRO_FORMAT_ARGB32,
 												nSize * ARROW_WIDTH,
 												nSize * ARROW_HEIGHT,
 												nSize * ARROW_WIDTH * 4);
 	cairo_t* cr = cairo_create(surface);
 
-#define AR_LINE_WIDTH 0.01
-#define AR_WIDTH 0.4
-#define AR_HEAD_SIZE 0.4
+#define AR_LINE_WIDTH 0.06
+#define AR_WIDTH 0.35
+#define AR_HEAD_SIZE 0.35
 
 	cairo_scale(cr, nSize * ARROW_WIDTH, nSize * ARROW_HEIGHT);
 
-	if (!up)
-	{	/* Rotate by 180 degrees */
-		cairo_translate(cr, .5, .5);
-		cairo_rotate(cr, PI);
-		cairo_translate(cr, -.5, -.5);
-	}
+	/* Point arrows correct way */
+	cairo_translate(cr, .5, .5);
+	if (left)
+		cairo_rotate(cr, -G_PI / 2);
+	else
+		cairo_rotate(cr, G_PI / 2);
+	cairo_translate(cr, -.5, -.5);
 
 	cairo_set_line_width (cr, AR_LINE_WIDTH);
 
-	cairo_set_source_rgba (cr, 0, 0, 0, 1);
+	/* clean the canvas */
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_set_source_rgba (cr, 0, 0, 0, 0);
 	cairo_rectangle(cr, 0, 0, 1, 1);
 	cairo_fill(cr);
 
@@ -2421,7 +2440,7 @@ static void RenderArrow(unsigned char* puch, double arColour[4], int nSize, int
 
 	cairo_set_source_rgba (cr, arColour[2], arColour[1], arColour[0], 1);
 	cairo_fill_preserve (cr);
-	cairo_set_source_rgba (cr, 0, 0, 0, 1);
+	cairo_set_source_rgba (cr, Highlight(arColour[2]), Highlight(arColour[1]), Highlight(arColour[0]), 1);
 	cairo_stroke(cr);
 
 	cairo_destroy (cr);
@@ -2429,10 +2448,10 @@ static void RenderArrow(unsigned char* puch, double arColour[4], int nSize, int
 }
 
 extern void RenderArrows( renderdata *prd, unsigned char* puch0,
-			  unsigned char* puch1, int nStride )
+			  unsigned char* puch1, int nStride, int fClockwise )
 {
-	RenderArrow(puch0, prd->aarColour[0], prd->nSize, TRUE);
-	RenderArrow(puch1, prd->aarColour[1], prd->nSize, FALSE);
+	RenderArrow(puch0, prd->aarColour[0], prd->nSize, fClockwise);
+	RenderArrow(puch1, prd->aarColour[1], prd->nSize, fClockwise);
 }
 #endif
 
@@ -2458,7 +2477,7 @@ static void DrawChequers( renderdata *prd, unsigned char *puch, int nStride,
 			  int x, int y, int cx, int cy ) {
     int i, c, yChequer;
 
-    c = ( !iPoint || iPoint == 25 ) ? 3 : 5;
+    c = ( iPoint == 0 || iPoint == 25 ) ? 3 : 5;
     yChequer = positions[ prd->fClockwise ][ iPoint ][ 1 ] * prd->nSize;
     
     for( i = 0; i < n; i++ ) {
@@ -2490,7 +2509,7 @@ static void DrawChequers( renderdata *prd, unsigned char *puch, int nStride,
 }
 
 extern void CalculateArea( renderdata *prd, unsigned char *puch, int nStride,
-			   renderimages *pri, int anBoard[ 2 ][ 25 ],
+			   renderimages *pri, TanBoard anBoard,
 			   int *anOff, int anDice[ 2 ],
 			   int anDicePosition[ 2 ][ 2 ],
 			   int fDiceColour, int anCubePosition[ 2 ],
@@ -2516,10 +2535,10 @@ extern void CalculateArea( renderdata *prd, unsigned char *puch, int nStride,
 	y = 0;
     }
 
-    if( x + cx > BOARD_WIDTH * prd->nSize )
+    if( x + cx > BOARD_WIDTH * (int)prd->nSize )
 	cx = BOARD_WIDTH * prd->nSize - x;
     
-    if( y + cy > BOARD_HEIGHT * prd->nSize )
+    if( y + cy > BOARD_HEIGHT * (int)prd->nSize )
 	cy = BOARD_HEIGHT * prd->nSize - y;    
     
     if( cx <= 0 || cy <= 0 )
@@ -2601,10 +2620,10 @@ extern void CalculateArea( renderdata *prd, unsigned char *puch, int nStride,
 			      n > 0, x, y, cx, cy );
 	}
     }
-    
+
     /* draw dice */
     for( i = 0; i < 2; i++ )
-	if( intersects( x, y, cx, cy, anDicePosition[ i ][ 0 ] * prd->nSize,
+	if( anDice[i] && intersects( x, y, cx, cy, anDicePosition[ i ][ 0 ] * prd->nSize,
 			anDicePosition[ i ][ 1 ] * prd->nSize, 
 			DIE_WIDTH * prd->nSize,
 			DIE_HEIGHT * prd->nSize ) ) {
@@ -2710,11 +2729,9 @@ extern void CalculateArea( renderdata *prd, unsigned char *puch, int nStride,
     if( prd->showMoveIndicator &&
 			intersects( x, y, cx, cy,
                     anArrowPosition[ 0 ], anArrowPosition[ 1 ],
-                    ARROW_WIDTH * prd->nSize, ARROW_HEIGHT * prd->nSize ) ) {
-
-	assert( anArrowPosition );
-
-	AlphaBlendClip2( puch, nStride,
+                    ARROW_WIDTH * prd->nSize, ARROW_HEIGHT * prd->nSize ) )
+	{
+		AlphaBlendClip2( puch, nStride,
 			 anArrowPosition[ 0 ] - x, anArrowPosition[ 1 ] - y,
 			 cx, cy,
 			 puch, nStride,
@@ -2723,7 +2740,7 @@ extern void CalculateArea( renderdata *prd, unsigned char *puch, int nStride,
 			 prd->nSize * ARROW_WIDTH * 4,
 			 0, 0,
 			 prd->nSize * ARROW_WIDTH, prd->nSize * ARROW_HEIGHT );
-		}
+	}
 #endif
 }
 
@@ -2731,7 +2748,7 @@ extern void
 RenderBoardLabels( renderdata *prd, 
                    unsigned char *achLo, unsigned char *achHi, int nStride ) {
 
-    unsigned char *achTemp = malloc( BOARD_WIDTH * prd->nSize * 3 * prd->nSize * 3 );
+    unsigned char *achTemp = malloc( BOARD_WIDTH * prd->nSize * 5 * prd->nSize * 5 );
 
     /* 12 11 10 9 8 7 - 6 5 4 3 2 1 */
     
@@ -2765,22 +2782,22 @@ extern void RenderImages( renderdata *prd, renderimages *pri ) {
 				   CHEQUER_HEIGHT * 4 );
     pri->achChequer[ 1 ] = malloc( nSize * nSize * CHEQUER_WIDTH * 
 				   CHEQUER_HEIGHT * 4 );
-    pri->achChequerLabels = malloc( nSize * nSize * CHEQUER_LABEL_WIDTH *
-				    CHEQUER_LABEL_HEIGHT * 3 * 12 );
+    pri->achChequerLabels = malloc( nSize * nSize * CHEQUER_WIDTH *
+				    CHEQUER_HEIGHT * 3 * 12 );
     pri->achDice[ 0 ] = malloc( nSize * nSize * DIE_WIDTH * DIE_HEIGHT * 4 );
     pri->achDice[ 1 ] = malloc( nSize * nSize * DIE_WIDTH * DIE_HEIGHT * 4 );
     pri->achPip[ 0 ] = malloc( nSize * nSize * 3 );
     pri->achPip[ 1 ] = malloc( nSize * nSize * 3 );
     pri->achCube = malloc( nSize * nSize * CUBE_WIDTH * CUBE_HEIGHT * 4 );
-    pri->achCubeFaces = malloc( nSize * nSize * CUBE_LABEL_WIDTH *
-				CUBE_LABEL_HEIGHT * 3 * 12 );
+    pri->achCubeFaces = malloc( nSize * nSize * CUBE_WIDTH *
+				CUBE_HEIGHT * 3 * 12 );
     pri->asRefract[ 0 ] = malloc( nSize * nSize * CHEQUER_WIDTH * CHEQUER_HEIGHT *
 				  sizeof (unsigned short) );
     pri->asRefract[ 1 ] = malloc( nSize * nSize * CHEQUER_WIDTH * CHEQUER_HEIGHT *
 				  sizeof (unsigned short) );
     pri->achResign = malloc ( nSize * nSize * RESIGN_WIDTH * RESIGN_HEIGHT * 4 );
-    pri->achResignFaces = malloc ( nSize * nSize * RESIGN_LABEL_WIDTH * 
-				   RESIGN_LABEL_HEIGHT * 3 * 3 );
+    pri->achResignFaces = malloc ( nSize * nSize * RESIGN_WIDTH * 
+				   RESIGN_HEIGHT * 3 * 3 );
 #if USE_GTK
     pri->auchArrow[0] = malloc(prd->nSize * prd->nSize * ARROW_WIDTH * ARROW_HEIGHT * 4 );
     pri->auchArrow[1] = malloc(prd->nSize * prd->nSize * ARROW_WIDTH * ARROW_HEIGHT * 4 );
@@ -2798,7 +2815,7 @@ extern void RenderImages( renderdata *prd, renderimages *pri ) {
 		    nSize * CHEQUER_WIDTH * 4 );
     RenderChequerLabels( prd, pri->achChequerLabels, nSize * 
 			 CHEQUER_LABEL_WIDTH * 3 );
-    RenderDice( prd, pri->achDice[ 0 ], pri->achDice[ 1 ], nSize * DIE_WIDTH * 4 );
+    RenderDice( prd, pri->achDice[ 0 ], pri->achDice[ 1 ], nSize * DIE_WIDTH * 4, TRUE );
     RenderPips( prd, pri->achPip[ 0 ], pri->achPip[ 1 ], nSize * 3 );
     RenderCube( prd, pri->achCube, nSize * CUBE_WIDTH * 4 );
     RenderCubeFaces( prd, pri->achCubeFaces, nSize * CUBE_LABEL_WIDTH * 3,
@@ -2806,9 +2823,9 @@ extern void RenderImages( renderdata *prd, renderimages *pri ) {
     RenderResign( prd, pri->achResign, nSize * RESIGN_WIDTH * 4 );
     RenderResignFaces( prd, pri->achResignFaces, nSize * RESIGN_LABEL_WIDTH * 3,
 		       pri->achResign, nSize * RESIGN_WIDTH * 4 );
-#if USE_GTK
+#if USE_GTK && HAVE_CAIRO
 	if (prd->showMoveIndicator)
-		RenderArrows( prd, pri->auchArrow[0], pri->auchArrow[1], nSize * ARROW_WIDTH * 4 );
+		RenderArrows( prd, pri->auchArrow[0], pri->auchArrow[1], nSize * ARROW_WIDTH * 4, prd->fClockwise );
 #endif
 
     RenderBoardLabels( prd, pri->achLabels[ 0 ], pri->achLabels[ 1 ],
@@ -2860,12 +2877,12 @@ extern void RenderFinalise( void )
 #endif
 }
 
-int TolComp(double f1, double f2)
+static int TolComp(double f1, double f2)
 {
 	return fabs(f1 - f2) < .005;
 }
 
-int ColourCompare(double c1[4], double c2[4])
+static int ColourCompare(double c1[4], double c2[4])
 {
 	return TolComp(c1[0], c2[0]) &&
 		TolComp(c1[1], c2[1]) &&
@@ -2893,19 +2910,19 @@ int MaterialCompare(Material* pMat1, Material* pMat2)
 		pMat1->alphaBlend == pMat2->alphaBlend;
 }
 
-int MaterialTextCompare(Material* pMat1, Material* pMat2)
+static int MaterialTextCompare(Material* pMat1, Material* pMat2)
 {
 	return MaterialCompare(pMat1, pMat2) &&
 		((!pMat1->textureInfo && !pMat2->textureInfo) ||
 			(pMat1->textureInfo && pMat2->textureInfo &&
-			!strcmp(pMat1->textureInfo->name, pMat2->textureInfo->name)));
+			!strcmp(MaterialGetTextureFilename(pMat1), MaterialGetTextureFilename(pMat2))));
 }
 #endif
 
 extern int PreferenceCompare(renderdata *prd1, renderdata *prd2)
 {
 #if USE_BOARD3D
-	if (prd1->fDisplayType == DT_3D)
+	if (display_is_3d(prd1))
 	{	/* 3d settings */
 		return (prd1->pieceType == prd2->pieceType &&
 			prd1->fHinges3d == prd2->fHinges3d &&
