@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 2000, 2003
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: renderprefs.h,v 1.4 2004/03/31 09:51:56 Superfly_Jon Exp $
+ * $Id: renderprefs.h,v 1.6 2007/07/02 12:43:40 ace Exp $
  */
 
 #ifndef _RENDERPREFS_H_
@@ -27,7 +27,7 @@
 #endif
 
 extern char *aszWoodName[];
-extern renderdata* GetMainAppearance();
+extern renderdata* GetMainAppearance(void);
 extern void CopyAppearance(renderdata* prd);
 
 extern void RenderPreferencesParam( renderdata *prd, char *szParam,
