@@ -4,7 +4,7 @@
  * by Gary Wong <gtw@gnu.org>, 2000, 2001.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,12 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: sgf.c,v 1.102.2.1 2006/11/06 23:29:34 c_anthon Exp $
+ * $Id: sgf.c,v 1.132 2009/10/01 21:05:54 c_anthon Exp $
  */
 
 #include "config.h"
 
-#include <assert.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <ctype.h>
 #include <math.h>
 #include <stdio.h>
@@ -37,9 +38,8 @@
 #include "analysis.h"
 #include "positionid.h"
 #include "sgf.h"
-#include <glib/gi18n.h>
 
-static char *szFile;
+static const char *szFile;
 static int fError;
 
 static int CheckSGFVersion(const char **sz);
@@ -52,7 +52,7 @@ static void ErrorHandler(char *sz, int fParseError)
     }
 }
 
-static void FreeList(list * pl, int nLevel)
+static void FreeList(listOLD * pl, int nLevel)
 {
     /* Levels are:
      *  0 - GameTreeSeq
@@ -84,16 +84,16 @@ static void FreeList(list * pl, int nLevel)
 	free(pl);
 }
 
-static void FreeGameTreeSeq(list * pl)
+static void FreeGameTreeSeq(listOLD * pl)
 {
 
     FreeList(pl, 0);
 }
 
-static list *LoadCollection(char *sz)
+static listOLD *LoadCollection(char *sz)
 {
 
-    list *plCollection, *pl, *plRoot, *plProp;
+    listOLD *plCollection, *pl, *plRoot, *plProp;
     FILE *pf;
     int fBackgammon;
     property *pp;
@@ -102,7 +102,7 @@ static list *LoadCollection(char *sz)
     SGFErrorHandler = ErrorHandler;
 
     if (strcmp(sz, "-")) {
-	if (!(pf = fopen(sz, "r"))) {
+	if (!(pf = g_fopen(sz, "r"))) {
 	    outputerr(sz);
 	    return NULL;
 	}
@@ -122,7 +122,7 @@ static list *LoadCollection(char *sz)
     if (plCollection) {
 	pl = plCollection->plNext;
 	while (pl != plCollection) {
-	    plRoot = ((list *) ((list *) pl->p)->plNext->p)->plNext->p;
+	    plRoot = ((listOLD *) ((listOLD *) pl->p)->plNext->p)->plNext->p;
 	    fBackgammon = FALSE;
 	    for (plProp = plRoot->plNext; plProp != plRoot;
 		 plProp = plProp->plNext) {
@@ -162,14 +162,14 @@ static void CopyName(int i, char *sz)
 
     /* FIXME sanity check the name as in CommandSetPlayerName */
 
-    pc = strdup(sz);
+    pc = g_strdup(sz);
 
     if (strlen(pc) > 31)
 	pc[31] = 0;
 
     strcpy(ap[i].szName, pc);
 
-    free(pc);
+    g_free(pc);
 
 }
 
@@ -180,7 +180,7 @@ static void SetScore(xmovegameinfo * pmgi, int fBlack, int n)
 	pmgi->anScore[fBlack] = n;
 }
 
-static void RestoreMI(list * pl, moverecord * pmr)
+static void RestoreMI(listOLD * pl, moverecord * pmr)
 {
 
     char *pch;
@@ -199,21 +199,10 @@ static void RestoreMI(list * pl, moverecord * pmr)
 		pmgi->i = 0;
 	} else if (!strncmp(pch, "ws:", 3) || !strncmp(pch, "bs:", 3))
 	    SetScore(pmgi, *pch == 'b', atoi(pch + 3));
-#if USE_TIMECONTROL
-	else if (!strncmp(pch, "wtime:", 6)) {
-	    pmr->tl[0].tv_sec = atoi(pch + 6);
-	} else if (!strncmp(pch, "btime:", 6)) {
-	    pmr->tl[1].tv_sec = atoi(pch + 6);
-	} else if (!strncmp(pch, "wtimeouts:", 10)) {
-	    pmgi->nTimeouts[0] = atoi(pch + 10);
-	} else if (!strncmp(pch, "btimeouts:", 10)) {
-	    pmgi->nTimeouts[1] = atoi(pch + 10);
-	}
-#endif
 
 }
 
-static void RestoreGS(list * pl, statcontext * psc)
+static void RestoreGS(listOLD * pl, statcontext * psc)
 {
 
     char *pch;
@@ -361,13 +350,12 @@ static void RestoreText(char *sz, char **ppch)
 
 static void RestoreRules(xmovegameinfo * pmgi, const char *sz)
 {
-
     char *pch;
     char *pchx;
 
     /* split string at colons */
 
-    pch = strdup(sz);
+    pch = g_strdup(sz);
 
     pchx = strtok(pch, ":");
     while (pchx) {
@@ -393,13 +381,12 @@ static void RestoreRules(xmovegameinfo * pmgi, const char *sz)
 
     }
 
-    free(pch);
-
+    g_free(pch);
 }
 
 
 
-static void RestoreRootNode(list * pl)
+static void RestoreRootNode(listOLD * pl)
 {
 
     property *pp;
@@ -424,11 +411,6 @@ static void RestoreRootNode(list * pl)
     pmr->g.nAutoDoubles = 0;
     pmr->g.bgv = VARIATION_STANDARD;
     pmr->g.fCubeUse = TRUE;
-#if USE_TIMECONTROL
-    pmr->tl[0].tv_sec = pmr->tl[1].tv_sec = 0;
-    pmr->tl[0].tv_usec = pmr->tl[1].tv_usec = 0;
-    pmr->g.nTimeouts[0] = pmr->g.nTimeouts[1] = 0;
-#endif
     IniStatcontext(&pmr->g.sc);
 
     for (pl = pl->plNext; (pp = pl->p); pl = pl->plNext)
@@ -538,7 +520,7 @@ static void RestoreRolloutScore(move * pm, const char *sz)
     pm->rScore2 = g_ascii_strtod(pc, &pc);
 }
 
-static void RestoreRolloutTrials(int *piTrials, const char *sz)
+static void RestoreRolloutTrials(unsigned int *piTrials, const char *sz)
 {
 
     char *pc = strstr(sz, "Trials");
@@ -548,7 +530,7 @@ static void RestoreRolloutTrials(int *piTrials, const char *sz)
     if (!pc)
 	return;
 
-    sscanf(pc, "Trials %d", piTrials);
+    sscanf(pc, "Trials %ud", piTrials);
 
 }
 
@@ -578,11 +560,7 @@ static void InitEvalContext(evalcontext * pec)
 
     pec->nPlies = 0;
     pec->fCubeful = FALSE;
-#if defined( REDUCTION_CODE )
-    pec->nReduced = 0;
-#else
     pec->fUsePrune = FALSE;
-#endif
     pec->fDeterministic = FALSE;
     pec->rNoise = 0.0;
 
@@ -604,18 +582,13 @@ static void RestoreEvalContext(evalcontext * pec, char *pc)
     }
     if (ver < 3) {
 	red = strtol(pc, &pc, 10);
-#if defined( REDUCTION_CODE )
-	pec->nReduced = red;
-#endif
 	pec->fDeterministic = strtol(pc, &pc, 10);
 	pec->rNoise = g_ascii_strtod(pc, &pc);
     } else {
 	pec->fDeterministic = strtol(pc, &pc, 10);
 	pec->rNoise = g_ascii_strtod(pc, &pc);
 	fUsePrune = strtol(pc, &pc, 10);
-#if !REDUCTION_CODE
 	pec->fUsePrune = fUsePrune;
-#endif
     }
 }
 
@@ -670,7 +643,7 @@ RestoreRolloutRolloutContext(rolloutcontext * prc, const char *sz)
     if (!pc)
 	return;
 
-    sscanf(pc, "RC %d %d %d %hu %u \"%[^\"]\" %d %d %d %d",
+    sscanf(pc, "RC %d %d %d %hu %u \"%[^\"]\" %lu %d %d %d",
 	   &fCubeful,
 	   &fVarRedn,
 	   &fInitial,
@@ -696,8 +669,7 @@ RestoreRolloutRolloutContext(rolloutcontext * prc, const char *sz)
 
 static void RestoreRollout(move * pm, const char *sz)
 {
-
-    int n;
+    unsigned int n;
 
     pm->esMove.et = EVAL_ROLLOUT;
     RestoreRolloutScore(pm, sz);
@@ -804,7 +776,7 @@ RestoreExtendedRolloutContext(rolloutcontext * prc, const char *sz)
     if (!pc)
 	return;
 
-    if (sscanf(pc, "RC %d %d %d %d %d %d %hu %d %d %hu \"%[^\"]\" %d",
+    if (sscanf(pc, "RC %d %d %d %d %d %d %hu %d %d %hu \"%[^\"]\" %lu",
 	       &fCubeful,
 	       &fVarRedn,
 	       &fInitial,
@@ -950,11 +922,8 @@ static void RestoreDoubleAnalysis(property * pp,
 	    pes->ec.rNoise = g_ascii_strtod(pch, &pch);
 	    fUsePrune = strtol(pch, &pch, 10);
 	}
-#if defined( REDUCTION_CODE )
-	pes->ec.nReduced = nReduced;
-#else
 	pes->ec.fUsePrune = fUsePrune;
-#endif
+
 	for (i = 0; i < 2; i++)
 	    for (j = 0; j < 7; j++)
 		aarOutput[i][j] = g_ascii_strtod(pch, &pch);
@@ -978,18 +947,29 @@ static void RestoreDoubleAnalysis(property * pp,
 }
 
 
+static void RestoreMarkedMoves(property * pp, movelist * pml)
+{
+	int val;
+	char *pch = pp->pl->plNext->p;
+	while ((val = strtol(pch, &pch, 10))) {
+		if (val > (int) pml->cMoves || val < 1)
+			continue;
+		pml->amMoves[val-1].cmark = CMARK_ROLLOUT;
+	}
+}
+
 static void RestoreMoveAnalysis(property * pp, int fPlayer,
-				movelist * pml, int *piMove,
+				movelist * pml, unsigned int *piMove,
 				evalsetup * pesChequer,
 				const matchstate * pms)
 {
-    list *pl = pp->pl->plNext;
+    listOLD *pl = pp->pl->plNext;
     const char *pc;
     char *pch;
     char ch;
     move *pm;
     int i, fDeterministic, nReduced, fUsePrune = 0;
-    int anBoardMove[2][25];
+    TanBoard anBoardMove;
     int ver;
     *piMove = atoi(pl->p);
 
@@ -1023,7 +1003,7 @@ static void RestoreMoveAnalysis(property * pp, int fPlayer,
 
 	memcpy(anBoardMove, pms->anBoard, sizeof(anBoardMove));
 	ApplyMove(anBoardMove, pm->anMove, FALSE);
-	PositionKey(anBoardMove, pm->auch);
+	PositionKey((ConstTanBoard)anBoardMove, pm->auch);
 
 	pch = (char *) pc;
 	while (isspace(ch = *pch++)) {
@@ -1071,11 +1051,7 @@ static void RestoreMoveAnalysis(property * pp, int fPlayer,
 		pm->esMove.ec.rNoise = g_ascii_strtod(pch, &pch);
 		fUsePrune = strtol(pch, &pch, 10);
 	    }
-#if defined( REDUCTION_CODE )
-	    pm->esMove.ec.nReduced = nReduced;
-#else
 	    pm->esMove.ec.fUsePrune = fUsePrune;
-#endif
 	    break;
 
 	case 'R':
@@ -1100,7 +1076,7 @@ static void RestoreMoveAnalysis(property * pp, int fPlayer,
     }
 }
 
-static void PointList(list * pl, int an[])
+static void PointList(listOLD * pl, int an[])
 {
 
     int i;
@@ -1123,10 +1099,10 @@ static void PointList(list * pl, int an[])
     }
 }
 
-static void RestoreNode(list * pl)
+static void RestoreNode(listOLD * pl)
 {
 
-    property *pp, *ppDA = NULL, *ppA = NULL, *ppC = NULL;
+    property *pp, *ppDA = NULL, *ppA = NULL, *ppC = NULL, *ppMR = NULL, *ppCR = NULL;
     moverecord *pmr = NULL;
     char *pch;
     int i, fPlayer = 0, fSetBoard = FALSE, an[25];
@@ -1134,9 +1110,6 @@ static void RestoreNode(list * pl)
     lucktype lt = LUCK_NONE;
     float rLuck = ERR_VAL;
 
-#if USE_TIMECONTROL
-    int fTimeset = 0;
-#endif
 
     for (pl = pl->plNext; (pp = pl->p); pl = pl->plNext) {
 	if (pp->ach[1] == 0 && (pp->ach[0] == 'B' || pp->ach[0] == 'W')) {
@@ -1282,6 +1255,12 @@ static void RestoreNode(list * pl)
 	else if (pp->ach[0] == 'A' && !pp->ach[1])
 	    /* move analysis */
 	    ppA = pp;
+	else if (pp->ach[0] == 'M' && pp->ach[1] == 'R')
+	    /* marked moves */
+	    ppMR = pp;
+	else if (pp->ach[0] == 'C' && pp->ach[1] == 'R')
+	    /* marked moves */
+	    ppCR = pp;
 	else if (pp->ach[0] == 'C' && !pp->ach[1])
 	    /* comment */
 	    ppC = pp;
@@ -1307,23 +1286,6 @@ static void RestoreNode(list * pl)
 	    /* good for white */
 	    lt = *((char *) pp->pl->plNext->p) == '2' ? LUCK_VERYBAD :
 		LUCK_BAD;
-#if USE_TIMECONTROL
-	else if (pp->ach[0] == 'W' && pp->ach[1] == 'L') {
-	    pmr->tl[0].tv_sec = atoi((char *) pp->pl->plNext->p);
-	    fTimeset |= 1;
-	} else if (pp->ach[0] == 'B' && pp->ach[1] == 'L') {
-	    pmr->tl[1].tv_sec = atoi((char *) pp->pl->plNext->p);
-	    fTimeset |= 2;
-	} else if ((pp->ach[0] == 'W' || pp->ach[0] == 'B')
-		   && pp->ach[1] == 'X') {
-	    pmr = NewMoveRecord();
-
-	    pmr->mt = MOVE_TIME;
-	    pmr->fPlayer = (pp->ach[0] == 'W');
-
-	    pmr->t.nPoints = atoi((char *) pp->pl->plNext->p);
-	}
-#endif
     }
 
     if (fSetBoard && !pmr) {
@@ -1331,16 +1293,8 @@ static void RestoreNode(list * pl)
 
 	pmr->mt = MOVE_SETBOARD;
 	ClosestLegalPosition(ms.anBoard);
-	PositionKey(ms.anBoard, pmr->sb.auchKey);
+	PositionKey(msBoard(), pmr->sb.auchKey);
     }
-#if USE_TIMECONTROL
-    /* Compensate for mising timestamps */
-    if (pmr && !(fTimeset & 1))
-	pmr->tl[0] = ((moverecord *) (plLastMove->p))->tl[0];
-
-    if (pmr && !(fTimeset & 2))
-	pmr->tl[1] = ((moverecord *) (plLastMove->p))->tl[1];
-#endif
 
     if (pmr && ppC)
 	pmr->sz = CopyEscapedString(ppC->pl->plNext->p);
@@ -1356,9 +1310,13 @@ static void RestoreNode(list * pl)
 				      pmr->CubeDecPtr->aarOutput,
 				      pmr->CubeDecPtr->aarStdDev,
 				      &pmr->CubeDecPtr->esDouble);
+	    if (ppCR)
+		    pmr->CubeDecPtr->cmark = CMARK_ROLLOUT;
 	    if (ppA)
 		RestoreMoveAnalysis(ppA, pmr->fPlayer, &pmr->ml,
 				    &pmr->n.iMove, &pmr->esChequer, &ms);
+	    if (ppMR)
+		RestoreMarkedMoves(ppMR, &pmr->ml);
 	    /* FIXME: separate st's */
 	    pmr->n.stMove = ast[0];
 	    pmr->stCube = ast[1];
@@ -1374,6 +1332,8 @@ static void RestoreNode(list * pl)
 				      pmr->CubeDecPtr->aarOutput,
 				      pmr->CubeDecPtr->aarStdDev,
 				      &pmr->CubeDecPtr->esDouble);
+	    if (ppCR)
+		    pmr->CubeDecPtr->cmark = CMARK_ROLLOUT;
 	    pmr->stCube = ast[0];
 	    break;
 
@@ -1391,7 +1351,7 @@ static void RestoreNode(list * pl)
     }
 }
 
-static void RestoreSequence(list * pl, int fRoot)
+static void RestoreSequence(listOLD * pl, int fRoot)
 {
 
     pl = pl->plNext;
@@ -1404,7 +1364,7 @@ static void RestoreSequence(list * pl, int fRoot)
 	RestoreNode(pl->p);
 }
 
-static void RestoreTree(list * pl, int fRoot)
+static void RestoreTree(listOLD * pl, int fRoot)
 {
 
     pl = pl->plNext;
@@ -1417,7 +1377,7 @@ static void RestoreTree(list * pl, int fRoot)
     /* FIXME restore other variations, once we can handle them */
 }
 
-static void RestoreGame(list * pl)
+static void RestoreGame(listOLD * pl)
 {
 
     moverecord *pmr, *pmrResign;
@@ -1435,22 +1395,14 @@ static void RestoreGame(list * pl)
     ms.nCube = 1;
     ms.fTurn = ms.fMove = ms.fCubeOwner = -1;
     ms.gs = GAME_NONE;
-#if USE_TIMECONTROL
-    ms.nTimeouts[0] = ms.nTimeouts[1] = 0;
-    ms.gc.pc[0].tc.timing = ms.gc.pc[1].tc.timing = TC_NONE;
-#endif
 
     RestoreTree(pl, TRUE);
 
     pmr = plGame->plNext->p;
-    assert(pmr->mt == MOVE_GAMEINFO);
+    g_assert(pmr->mt == MOVE_GAMEINFO);
 
     AddGame(pmr);
 
-#if USE_TIMECONTROL
-    if (pmr->tl[0].tv_sec && pmr->tl[1].tv_sec)
-	ms.gc.pc[0].tc.timing = ms.gc.pc[1].tc.timing = TC_UNKNOWN;
-#endif
     if (pmr->g.fResigned) {
 	/* setting fTurn = fMove = -1 results in the board being
 	   inverted when shown. /jth 2003-10-12 
@@ -1476,7 +1428,7 @@ static void RestoreGame(list * pl)
 extern void CommandLoadGame(char *sz)
 {
 
-    list *pl;
+    listOLD *pl;
 
     sz = NextToken(&sz);
 
@@ -1487,16 +1439,8 @@ extern void CommandLoadGame(char *sz)
     }
 
     if ((pl = LoadCollection(sz))) {
-	if (ms.gs == GAME_PLAYING && fConfirm) {
-	    if (fInterrupt)
+	if (!get_input_discard())
 		return;
-
-	    if (!GetInputYN
-		(_
-		 ("Are you sure you want to load a saved game, "
-		  "and discard the one in progress? ")))
-		return;
-	}
 #if USE_GTK
 	if (fX) {		/* Clear record to avoid ugly updates */
 	    GTKClearMoveRecord();
@@ -1521,7 +1465,7 @@ extern void CommandLoadGame(char *sz)
 	    GTKSet(ap);
 	}
 
-	setDefaultFileName(sz, PATH_SGF);
+	setDefaultFileName(sz);
 
 #endif
 
@@ -1535,7 +1479,7 @@ extern void CommandLoadGame(char *sz)
 extern void CommandLoadPosition(char *sz)
 {
 
-    list *pl;
+    listOLD *pl;
 
     sz = NextToken(&sz);
 
@@ -1546,16 +1490,8 @@ extern void CommandLoadPosition(char *sz)
     }
 
     if ((pl = LoadCollection(sz))) {
-	if (ms.gs == GAME_PLAYING && fConfirm) {
-	    if (fInterrupt)
+	if (!get_input_discard())
 		return;
-
-	    if (!GetInputYN
-		(_
-		 ("Are you sure you want to load a saved position, "
-		  "and discard the match in progress? ")))
-		return;
-	}
 #if USE_GTK
 	if (fX) {		/* Clear record to avoid ugly updates */
 	    GTKClearMoveRecord();
@@ -1580,7 +1516,7 @@ extern void CommandLoadPosition(char *sz)
 	    GTKSet(ap);
 	}
 
-	setDefaultFileName(sz, PATH_SGF);
+	setDefaultFileName(sz);
 
 #endif
 
@@ -1591,7 +1527,7 @@ extern void CommandLoadPosition(char *sz)
 extern void CommandLoadMatch(char *sz)
 {
 
-    list *pl;
+    listOLD *pl;
 
     sz = NextToken(&sz);
 
@@ -1604,16 +1540,8 @@ extern void CommandLoadMatch(char *sz)
     if ((pl = LoadCollection(sz))) {
 	/* FIXME make sure the root nodes have MI properties; if not,
 	   we're loading a session. */
-	if (ms.gs == GAME_PLAYING && fConfirm) {
-	    if (fInterrupt)
+	if (!get_input_discard())
 		return;
-
-	    if (!GetInputYN
-		(_
-		 ("Are you sure you want to load a saved match, "
-		  "and discard the game in progress? ")))
-		return;
-	}
 #if USE_GTK
 	if (fX) {		/* Clear record to avoid ugly updates */
 	    GTKClearMoveRecord();
@@ -1639,7 +1567,7 @@ extern void CommandLoadMatch(char *sz)
 	}
 #endif
 
-	setDefaultFileName(sz, PATH_SGF);
+	setDefaultFileName(sz);
 
 	if (fGotoFirstGame)
 	    CommandFirstGame(NULL);
@@ -1682,30 +1610,18 @@ static void WriteEscapedString(FILE * pf, char *pch, int fEscapeColons)
 
 static void WriteEvalContext(FILE * pf, const evalcontext * pec)
 {
-
     gchar buffer[G_ASCII_DTOSTR_BUF_SIZE];
     g_ascii_formatd(buffer, sizeof(buffer), "%.6f", pec->rNoise);
-#if defined( REDUCTION_CODE )
-    fprintf(pf, "ver %d %d%s %d %d %s %d",
-	    SGF_FORMAT_VER,
-	    pec->nPlies,
-	    pec->fCubeful ? "C" : "",
-	    pec->nReduced, pec->fDeterministic, buffer,
-/*         , pec->fUsePrune */
-	    0);
-#else
     fprintf(pf, "ver %d %d%s %d %s %d",
 	    SGF_FORMAT_VER,
 	    pec->nPlies,
 	    pec->fCubeful ? "C" : "",
 	    pec->fDeterministic, buffer, pec->fUsePrune);
-#endif
-
 }
 
 static void
 WriteMoveFilters(FILE * pf,
-		 const movefilter mf[MAX_FILTER_PLIES][MAX_FILTER_PLIES],
+		 movefilter mf[MAX_FILTER_PLIES][MAX_FILTER_PLIES],
 		 int nPlies)
 {
     int i;
@@ -1724,7 +1640,7 @@ static void WriteRolloutContext(FILE * pf, const rolloutcontext * prc)
 
     int i;
 
-    fprintf(pf, "RC %d %d %d %d %d %d %d %d %d %d \"%s\" %d ",
+    fprintf(pf, "RC %d %d %d %d %d %d %d %d %d %d \"%s\" %lu ",
 	    prc->fCubeful,
 	    prc->fVarRedn,
 	    prc->fInitial,
@@ -1745,7 +1661,7 @@ static void WriteRolloutContext(FILE * pf, const rolloutcontext * prc)
 	WriteEvalContext(pf, &prc->aecChequer[i]);
 	if (prc->aecChequer[i].nPlies) {
 	    fprintf(pf, " filt%d ", i);
-	    WriteMoveFilters(pf, prc->aaamfChequer[i],
+	    WriteMoveFilters(pf, (movefilter (*)[4])prc->aaamfChequer[i],
 			     prc->aecChequer[i].nPlies);
 	}
     }
@@ -1759,7 +1675,7 @@ static void WriteRolloutContext(FILE * pf, const rolloutcontext * prc)
 	WriteEvalContext(pf, &prc->aecChequerLate[i]);
 	if (prc->aecChequerLate[i].nPlies) {
 	    fprintf(pf, " latefilt%d ", i);
-	    WriteMoveFilters(pf, prc->aaamfLate[i],
+	    WriteMoveFilters(pf, (movefilter (*)[4])prc->aaamfLate[i],
 			     prc->aecChequerLate[i].nPlies);
 	}
     }
@@ -1834,7 +1750,7 @@ static void WriteRolloutAnalysis(FILE * pf, int fIsMove,
 static void WriteDoubleAnalysis(FILE * pf,
 				float aarOutput[][NUM_ROLLOUT_OUTPUTS],
 				float aarStdDev[][NUM_ROLLOUT_OUTPUTS],
-				evalsetup * pes)
+				evalsetup * pes, int cmark)
 {
 
     gchar buffer[G_ASCII_DTOSTR_BUF_SIZE];
@@ -1846,21 +1762,12 @@ static void WriteDoubleAnalysis(FILE * pf,
     switch (pes->et) {
     case EVAL_EVAL:
 	g_ascii_formatd(buffer, sizeof(buffer), "%.6f", pes->ec.rNoise);
-#if defined( REDUCTION_CODE )
-	fprintf(pf, "E ver %d %d%s %d %d %s %d",
-		SGF_FORMAT_VER,
-		pes->ec.nPlies,
-		pes->ec.fCubeful ? "C" : "",
-		pes->ec.nReduced, pes->ec.fDeterministic, buffer,
-/*	     pes->ec.fUsePrune, */
-		0);
-#else
 	fprintf(pf, "E ver %d %d%s %d %s %d",
 		SGF_FORMAT_VER,
 		pes->ec.nPlies,
 		pes->ec.fCubeful ? "C" : "",
 		pes->ec.fDeterministic, buffer, pes->ec.fUsePrune);
-#endif
+
 	for (i = 0; i < 2; i++) {
 	    for (j = 0; j < 7; j++) {
 		g_ascii_formatd(buffer, sizeof(buffer), "%.6f",
@@ -1879,10 +1786,12 @@ static void WriteDoubleAnalysis(FILE * pf,
 	break;
 
     default:
-	assert(FALSE);
+	g_assert(FALSE);
     }
 
     fputc(']', pf);
+    if (cmark)
+	    fputs("CR[]", pf);
 
 }
 
@@ -1908,10 +1817,10 @@ static void WriteMove(FILE * pf, int fPlayer, int anMove[])
 
 
 static void WriteMoveAnalysis(FILE * pf, int fPlayer, movelist * pml,
-			      int iMove)
+			      unsigned int iMove)
 {
 
-    int i;
+    unsigned int i;
     gchar buffer[G_ASCII_DTOSTR_BUF_SIZE];
 
     fprintf(pf, "A[%d]", iMove);
@@ -1949,18 +1858,9 @@ static void WriteMoveAnalysis(FILE * pf, int fPlayer, movelist * pml,
 	    fprintf(pf, "%d%s %d %d %s %d",
 		    pml->amMoves[i].esMove.ec.nPlies,
 		    pml->amMoves[i].esMove.ec.fCubeful ? "C" : "",
-#if defined( REDUCTION_CODE )
-		    pml->amMoves[i].esMove.ec.nReduced,
-#else
-		       /* FIXME: reduced shouldn't be saved/loaded */
 		    0,
-#endif
 		    pml->amMoves[i].esMove.ec.fDeterministic, buffer,
-#if !defined( REDUCTION_CODE )
 		    pml->amMoves[i].esMove.ec.fUsePrune
-#else
-		    0
-#endif
 		);
 	    break;
 
@@ -1975,11 +1875,17 @@ static void WriteMoveAnalysis(FILE * pf, int fPlayer, movelist * pml,
 
 
 	default:
-	    assert(FALSE);
+	    g_assert(FALSE);
 	}
 
 	fputc(']', pf);
     }
+    fprintf(pf, "MR[");
+    for (i = 0; i < pml->cMoves; i++) {
+	    if (pml->amMoves[i].cmark)
+		    fprintf(pf, "%d ", i+1);
+    }
+    fputc(']', pf);
 }
 
 static void WriteLuck(FILE * pf, int fPlayer, float rLuck, lucktype lt)
@@ -2188,7 +2094,7 @@ static void WriteStatContext(FILE * pf, statcontext * psc)
     }
 }
 
-static void WriteProperty(FILE * pf, char *szName, char *szValue)
+static void WriteProperty(FILE * pf, const char *szName, char *szValue)
 {
 
     if (!szValue || !*szValue)
@@ -2208,18 +2114,19 @@ static void AddRule(FILE * pf, const char *sz, int *pfFirst)
 
 }
 
-static void SaveGame(FILE * pf, list * plGame)
+extern void SaveGame(FILE * pf, listOLD * plGame)
 {
-
-    list *pl;
+    listOLD *pl;
     moverecord *pmr;
-    int i, j, anBoard[2][25];
+    unsigned int i, j;
+	TanBoard anBoard;
+    listOLD *pl_hint = NULL;
 
     updateStatisticsGame(plGame);
 
     pl = plGame->plNext;
     pmr = pl->p;
-    assert(pmr->mt == MOVE_GAMEINFO);
+    g_assert(pmr->mt == MOVE_GAMEINFO);
 
     /* Fixed header */
     fputs("(;FF[4]GM[6]CA[UTF-8]AP[GNU Backgammon:" VERSION "]", pf);
@@ -2228,17 +2135,8 @@ static void SaveGame(FILE * pf, list * plGame)
     /* FIXME: isn't it always appropriate to write this? */
     /* If not, money games will be loaded without score and game number */
     /* if( pmr->g.nMatch ) */
-#if USE_TIMECONTROL
-    fprintf(pf, "MI[length:%d][game:%d][ws:%d][bs:%d]"
-	    "[wtime:%d][btime:%d][wtimeouts:%d][btimeouts:%d]",
-	    pmr->g.nMatch, pmr->g.i,
-	    pmr->g.anScore[0], pmr->g.anScore[1],
-	    (int) pmr->tl[0].tv_sec, (int) pmr->tl[1].tv_sec,
-	    pmr->g.nTimeouts[0], pmr->g.nTimeouts[1]);
-#else
     fprintf(pf, "MI[length:%d][game:%d][ws:%d][bs:%d]",
 	    pmr->g.nMatch, pmr->g.i, pmr->g.anScore[0], pmr->g.anScore[1]);
-#endif
 
     /* Names */
     fputs("PW[", pf);
@@ -2267,7 +2165,7 @@ static void SaveGame(FILE * pf, list * plGame)
     if (pmr->g.fCrawford || pmr->g.fJacoby ||
 	pmr->g.bgv != VARIATION_STANDARD || !pmr->g.fCubeUse) {
 
-	static char *aszSGFVariation[NUM_VARIATIONS] =
+	static const char *aszSGFVariation[NUM_VARIATIONS] =
 	    { NULL, "Nackgammon", "Hypergammon1", "Hypergammon2",
 "Hypergammon3" };
 	int fFirst = TRUE;
@@ -2298,24 +2196,30 @@ static void SaveGame(FILE * pf, list * plGame)
     if (pmr->g.sc.fMoves || pmr->g.sc.fCube || pmr->g.sc.fDice)
 	WriteStatContext(pf, &pmr->g.sc);
 
+    if (game_is_last(plGame))
+	    pl_hint = game_add_pmr_hint(plGame);
+
     for (pl = pl->plNext; pl != plGame; pl = pl->plNext) {
 	pmr = pl->p;
 	switch (pmr->mt) {
 	case MOVE_NORMAL:
+		/* sanitise the move if from a hint record */
+		if ( pmr->ml.cMoves && pmr->n.iMove > pmr->ml.cMoves ) {
+			memcpy( pmr->n.anMove, pmr->ml.amMoves[ 0 ].anMove,
+					sizeof( pmr->n.anMove ) );
+			pmr->n.iMove = 0;
+		}
 	    fprintf(pf, "\n;%c[%d%d", pmr->fPlayer ? 'B' : 'W',
 		    pmr->anDice[0], pmr->anDice[1]);
 	    WriteMove(pf, pmr->fPlayer, pmr->n.anMove);
 	    putc(']', pf);
 
-#if USE_TIMECONTROL
-	    fprintf(pf, "WL[%d]BL[%d]",
-		    (int) pmr->tl[0].tv_sec, (int) pmr->tl[1].tv_sec);
-#endif
 	    if (pmr->CubeDecPtr->esDouble.et != EVAL_NONE)
 		WriteDoubleAnalysis(pf,
 				    pmr->CubeDecPtr->aarOutput,
 				    pmr->CubeDecPtr->aarStdDev,
-				    &pmr->CubeDecPtr->esDouble);
+				    &pmr->CubeDecPtr->esDouble,
+				    pmr->CubeDecPtr->cmark);
 
 	    if (pmr->ml.cMoves)
 		WriteMoveAnalysis(pf, pmr->fPlayer, &pmr->ml,
@@ -2331,15 +2235,12 @@ static void SaveGame(FILE * pf, list * plGame)
 	case MOVE_DOUBLE:
 	    fprintf(pf, "\n;%c[double]", pmr->fPlayer ? 'B' : 'W');
 
-#if USE_TIMECONTROL
-	    fprintf(pf, "WL[%d]BL[%d]",
-		    (int) pmr->tl[0].tv_sec, (int) pmr->tl[1].tv_sec);
-#endif
 	    if (pmr->CubeDecPtr->esDouble.et != EVAL_NONE)
 		WriteDoubleAnalysis(pf,
 				    pmr->CubeDecPtr->aarOutput,
 				    pmr->CubeDecPtr->aarStdDev,
-				    &pmr->CubeDecPtr->esDouble);
+				    &pmr->CubeDecPtr->esDouble,
+				    pmr->CubeDecPtr->cmark);
 
 	    WriteSkill(pf, pmr->stCube);
 
@@ -2348,15 +2249,12 @@ static void SaveGame(FILE * pf, list * plGame)
 	case MOVE_TAKE:
 	    fprintf(pf, "\n;%c[take]", pmr->fPlayer ? 'B' : 'W');
 
-#if USE_TIMECONTROL
-	    fprintf(pf, "WL[%d]BL[%d]",
-		    (int) pmr->tl[0].tv_sec, (int) pmr->tl[1].tv_sec);
-#endif
 	    if (pmr->CubeDecPtr->esDouble.et != EVAL_NONE)
 		WriteDoubleAnalysis(pf,
 				    pmr->CubeDecPtr->aarOutput,
 				    pmr->CubeDecPtr->aarStdDev,
-				    &pmr->CubeDecPtr->esDouble);
+				    &pmr->CubeDecPtr->esDouble,
+				    pmr->CubeDecPtr->cmark);
 
 	    WriteSkill(pf, pmr->stCube);
 
@@ -2365,15 +2263,12 @@ static void SaveGame(FILE * pf, list * plGame)
 	case MOVE_DROP:
 	    fprintf(pf, "\n;%c[drop]", pmr->fPlayer ? 'B' : 'W');
 
-#if USE_TIMECONTROL
-	    fprintf(pf, "WL[%d]BL[%d]", (int) pmr->tl[0].tv_sec,
-		    (int) pmr->tl[1].tv_sec);
-#endif
 	    if (pmr->CubeDecPtr->esDouble.et != EVAL_NONE)
 		WriteDoubleAnalysis(pf,
 				    pmr->CubeDecPtr->aarOutput,
 				    pmr->CubeDecPtr->aarStdDev,
-				    &pmr->CubeDecPtr->esDouble);
+				    &pmr->CubeDecPtr->esDouble,
+				    pmr->CubeDecPtr->cmark);
 
 
 	    WriteSkill(pf, pmr->stCube);
@@ -2426,17 +2321,8 @@ static void SaveGame(FILE * pf, list * plGame)
 	    fprintf(pf, "\n;CP[%c]", "cwb"[pmr->scp.fCubeOwner + 1]);
 	    break;
 
-#if USE_TIMECONTROL
-	case MOVE_TIME:
-	    fprintf(pf, "\n;%cX[%d]", !pmr->fPlayer ? 'B' : 'W',
-		    pmr->t.nPoints);
-	    fprintf(pf, "WL[%d]BL[%d]",
-		    (int) pmr->tl[0].tv_sec, (int) pmr->tl[1].tv_sec);
-	    break;
-#endif
-
 	default:
-	    assert(FALSE);
+	    g_assert(FALSE);
 	}
 
 	if (pmr->sz) {
@@ -2446,6 +2332,9 @@ static void SaveGame(FILE * pf, list * plGame)
 	}
     }
 
+    if (pl_hint)
+	    game_remove_pmr_hint(pl_hint);
+
     /* FIXME if the game is not over and the player on roll is the last
        player to move, add a PL property */
 
@@ -2476,7 +2365,7 @@ extern void CommandSaveGame(char *sz)
 
     if (!strcmp(sz, "-"))
 	pf = stdout;
-    else if (!(pf = fopen(sz, "w"))) {
+    else if (!(pf = g_fopen(sz, "w"))) {
 	outputerr(sz);
 	return;
     }
@@ -2486,7 +2375,7 @@ extern void CommandSaveGame(char *sz)
     if (pf != stdout)
 	fclose(pf);
 
-    setDefaultFileName(sz, PATH_SGF);
+    setDefaultFileName(sz);
 
 }
 
@@ -2494,7 +2383,7 @@ extern void CommandSaveMatch(char *sz)
 {
 
     FILE *pf;
-    list *pl;
+    listOLD *pl;
 
     sz = NextToken(&sz);
 
@@ -2516,29 +2405,31 @@ extern void CommandSaveMatch(char *sz)
 
     if (!strcmp(sz, "-"))
 	pf = stdout;
-    else if (!(pf = fopen(sz, "w"))) {
+    else if (!(pf = g_fopen(sz, "w"))) {
 	outputerr(sz);
 	return;
     }
 
     for (pl = lMatch.plNext; pl != &lMatch; pl = pl->plNext)
-	SaveGame(pf, pl->p);
+	    SaveGame(pf, pl->p);
 
     if (pf != stdout)
 	fclose(pf);
 
-    setDefaultFileName(sz, PATH_SGF);
+    setDefaultFileName(sz);
 
+    delete_autosave();
 }
 
 extern void CommandSavePosition(char *sz)
 {
 
     FILE *pf;
-    list l;
+    listOLD l;
     moverecord *pmgi;
     moverecord *pmsb;
-    moverecord *pmsd;
+    moverecord *pmsd = NULL;
+    moverecord *pmr_cur = NULL;
     moverecord *pmscv;
     moverecord *pmscp;
 
@@ -2560,7 +2451,7 @@ extern void CommandSavePosition(char *sz)
 
     if (!strcmp(sz, "-"))
 	pf = stdout;
-    else if (!(pf = fopen(sz, "w"))) {
+    else if (!(pf = g_fopen(sz, "w"))) {
 	outputerr(sz);
 	return;
     }
@@ -2596,7 +2487,7 @@ extern void CommandSavePosition(char *sz)
     pmsb->mt = MOVE_SETBOARD;
     if (ms.fMove)
 	SwapSides(ms.anBoard);
-    PositionKey(ms.anBoard, pmsb->sb.auchKey);
+    PositionKey(msBoard(), pmsb->sb.auchKey);
     if (ms.fMove)
 	SwapSides(ms.anBoard);
     ListInsert(&l, pmsb);
@@ -2622,15 +2513,20 @@ extern void CommandSavePosition(char *sz)
     /* FIXME if the dice are not rolled, this should be done with a PL
        property (which is SaveGame()'s job) */
 
-    pmsd = NewMoveRecord();
-
-    pmsd->mt = MOVE_SETDICE;
-    pmsd->fPlayer = ms.fMove;
-    pmsd->anDice[0] = ms.anDice[0];
-    pmsd->anDice[1] = ms.anDice[1];
-    pmsd->lt = LUCK_NONE;
-    pmsd->rLuck = ERR_VAL;
-    ListInsert(&l, pmsd);
+    pmr_cur = get_current_moverecord(NULL);
+    if (pmr_cur && (pmr_cur->ml.cMoves > 0 || pmr_cur->CubeDecPtr->esDouble.et != EVAL_NONE))
+	    ListInsert(&l, pmr_cur);
+    else if (ms.anDice[0] > 0)
+    {
+	    pmsd = NewMoveRecord();
+	    pmsd->mt = MOVE_SETDICE;
+	    pmsd->fPlayer = ms.fMove;
+	    pmsd->anDice[0] = ms.anDice[0];
+	    pmsd->anDice[1] = ms.anDice[1];
+	    pmsd->lt = LUCK_NONE;
+	    pmsd->rLuck = ERR_VAL;
+	    ListInsert(&l, pmsd);
+    }
 
     /* FIXME add MOVE_DOUBLE record(s) as appropriate */
 
@@ -2644,9 +2540,10 @@ extern void CommandSavePosition(char *sz)
 
     free(pmgi);
     free(pmsb);
-    free(pmsd);
+    if (pmsd)
+	    free(pmsd);
     free(pmscv);
     free(pmscp);
 
-    setDefaultFileName(sz, PATH_SGF);
+    setDefaultFileName(sz);
 }
