@@ -4,7 +4,7 @@
  * by Joern Thyssen  <jthyssen@dk.ibm.com>, 2002
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 2 of the GNU General Public License as
+ * it under the terms of version 3 or later of the GNU General Public License as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: text.c,v 1.73 2006/10/04 12:28:39 c_anthon Exp $
+ * $Id: text.c,v 1.88 2008/02/25 22:18:51 c_anthon Exp $
  */
 
 #include "config.h"
@@ -25,43 +25,31 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
-#include <assert.h>
+#include <glib.h>
 #include <stdarg.h>
 
 #include "backgammon.h"
 #include "drawboard.h"
 #include "export.h"
 #include "format.h"
-#include "eval.h"
 #include "positionid.h"
 #include "matchid.h"
 #include "record.h"
-#if USE_TIMECONTROL
-#include "timecontrol.h"
-#endif
 
 #include <glib/gi18n.h>
 
 
 /* "Color" of chequers */
 
-static char *aszColorName[] = { "O", "X" };
-
-
 static void
 printTextBoard ( FILE *pf, const matchstate *pms ) {
 
-  int anBoard[ 2 ][ 25 ];
+  TanBoard anBoard;
   char szBoard[ 2048 ];
   char sz[ 32 ], szCube[ 32 ], szPlayer0[ MAX_NAME_LEN + 3 ], szPlayer1[ MAX_NAME_LEN + 3 ],
     szScore0[ 35 ], szScore1[ 35 ], szMatch[ 35 ];
-#if USE_TIMECONTROL
-    char szTime0[20], szTime1[20];
-    char *apch[ 9 ] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-#else
     char *apch[ 7 ] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-#endif
-  int anPips[ 2 ];
+  unsigned int anPips[ 2 ];
 
   memcpy ( anBoard, pms->anBoard, sizeof ( anBoard ) );
 
@@ -93,7 +81,7 @@ printTextBoard ( FILE *pf, const matchstate *pms ) {
     if( pms->anDice[ 0 ] )
       sprintf( sz, 
                _("Rolled %d%d"), pms->anDice[ 0 ], pms->anDice[ 1 ] );
-    else if( !GameStatus( anBoard, pms->bgv ) )
+    else if( !GameStatus( (ConstTanBoard)anBoard, pms->bgv ) )
       strcpy( sz, _("On roll") );
     else
       sz[ 0 ] = 0;
@@ -107,8 +95,10 @@ printTextBoard ( FILE *pf, const matchstate *pms ) {
                  pms->nCube );
       else
         sprintf( szCube, _("(Cube: %d)"), pms->nCube );
-    } else {
-      int cch = strlen( ap[ pms->fCubeOwner ].szName );
+    }
+	else
+	{
+      size_t cch = strlen( ap[ pms->fCubeOwner ].szName );
 		
       if( cch > 20 )
         cch = 20;
@@ -124,13 +114,6 @@ printTextBoard ( FILE *pf, const matchstate *pms ) {
     }
   }
     
-#if USE_TIMECONTROL
-  apch[7] = apch[8] = 0;
-  if (ms.gc.pc[0].tc.timing != TC_NONE)
-    apch[7] = FormatClock(&ms.tvTimeleft[0], szTime0);
-  if (ms.gc.pc[1].tc.timing != TC_NONE)
-    apch[8] = FormatClock(&ms.tvTimeleft[1], szTime1);
-#endif
 
 
   if( pms->fResigned )
@@ -141,11 +124,11 @@ printTextBoard ( FILE *pf, const matchstate *pms ) {
     SwapSides( anBoard );
 	
 
-  fputs ( DrawBoard( szBoard, anBoard, pms->fMove, apch,
+  fputs ( DrawBoard( szBoard, (ConstTanBoard)anBoard, pms->fMove, apch,
                      MatchIDFromMatchState ( pms ), anChequers[ ms.bgv ] ),
           pf);
 
-  PipCount ( anBoard, anPips );
+  PipCount ( (ConstTanBoard)anBoard, anPips );
 
   fprintf ( pf, "Pip counts: O %d, X %d\n\n",
             anPips[ 0 ], anPips[ 1 ] );
@@ -163,20 +146,22 @@ printTextBoard ( FILE *pf, const matchstate *pms ) {
  *
  */
 
-static void 
-TextBoardHeader ( FILE *pf, const matchstate *pms, 
-                  const int iGame, const int iMove ) {
+extern void 
+TextBoardHeader ( GString *gsz, const matchstate *pms, 
+                  const int notused/*iGame*/, const int iMove ) {
 
   if ( iMove >= 0 )
-    fprintf ( pf, _("Move number %d: "), iMove + 1 );
+    g_string_append_printf(gsz, _("Move number %d: "), iMove + 1 );
 
   if ( pms->fResigned ) 
     
     /* resignation */
 
-    fprintf ( pf,
-              _("%s resigns %d points\n\n"), 
-              aszColorName[ pms->fTurn ],
+    g_string_append_printf(gsz,
+              ngettext( " %s resigns %d point",
+                        " %s resigns %d points",
+                        pms->fResigned * pms->nCube ), 
+              ap[ pms->fTurn ].szName,
               pms->fResigned * pms->nCube
             );
   
@@ -184,9 +169,9 @@ TextBoardHeader ( FILE *pf, const matchstate *pms,
 
     /* chequer play decision */
 
-    fprintf ( pf,
-              _("%s to play %d%d\n\n"),
-              aszColorName[ pms->fTurn ],
+    g_string_append_printf(gsz,
+              _(" %s to play %d%d\n\n"),
+              ap[ pms->fMove ].szName,
               pms->anDice[ 0 ], pms->anDice[ 1 ] 
             );
 
@@ -194,9 +179,9 @@ TextBoardHeader ( FILE *pf, const matchstate *pms,
 
     /* take decision */
 
-    fprintf ( pf,
-              _("%s doubles to %d\n\n"),
-              aszColorName[ pms->fMove ],
+    g_string_append_printf(gsz,
+              _(" %s doubles to %d"),
+              ap[ !(pms->fTurn) ].szName,
               pms->nCube * 2
             );
 
@@ -204,11 +189,9 @@ TextBoardHeader ( FILE *pf, const matchstate *pms,
 
     /* cube decision */
 
-    fprintf ( pf,
-              _("%s on roll, cube decision?\n\n"),
-              aszColorName[ pms->fTurn ]
-            );
-
+    g_string_append_printf(gsz,
+              _(" %s on roll, cube decision?"),
+              ap[ pms->fMove ].szName );
 
 }
 
@@ -222,10 +205,10 @@ TextBoardHeader ( FILE *pf, const matchstate *pms,
  *
  */
 
-static void 
-TextPrologue ( FILE *pf, const matchstate *pms, const int iGame ) {
+extern void 
+TextPrologue ( GString *gsz, const matchstate *pms, const int iGame ) {
 
-  fprintf( pf, pms->cGames == 1 ? 
+  g_string_append_printf( gsz, pms->cGames == 1 ? 
            _("The score (after %d game) is: %s %d, %s %d") :
            _("The score (after %d games) is: %s %d, %s %d"),
            pms->cGames, 
@@ -233,18 +216,18 @@ TextPrologue ( FILE *pf, const matchstate *pms, const int iGame ) {
            ap[ 1 ].szName, pms->anScore[ 1 ] );
 
   if ( pms->nMatchTo > 0 ) {
-    fprintf( pf,
+    g_string_append_printf( gsz,
              ngettext( " (match to %d point)", 
                        " (match to %d points)",
                        pms->nMatchTo ),
              pms->nMatchTo );
     if ( pms->fCrawford )
-       fputs( _(", Crawford game"), pf );
+       g_string_append(gsz,  _(", Crawford game"));
     if ( pms->fPostCrawford )
-       fputs( _(", post-Crawford play"), pf );
+       g_string_append(gsz, _(", post-Crawford play"));
   }
 
-  fputs( "\n", pf );
+  g_string_append(gsz, "\n");
 
 }
 
@@ -263,7 +246,7 @@ TextEpilogue ( FILE *pf, const matchstate *pms ) {
 
   time_t t;
 
-  const char szVersion[] = "$Revision: 1.73 $";
+  const char szVersion[] = "$Revision: 1.88 $";
   int iMajor, iMinor;
 
   iMajor = atoi ( strchr ( szVersion, ' ' ) );
@@ -297,9 +280,9 @@ TextEpilogue ( FILE *pf, const matchstate *pms ) {
  */
 
 static void
-TextPrintCubeAnalysisTable ( FILE *pf, 
-                             const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
-                             const float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+TextPrintCubeAnalysisTable ( GString *gsz, 
+                             float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
+                             float aarStdDev[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
                              int fPlayer,
                              const evalsetup* pes, const cubeinfo* pci,
                              int fDouble, int fTake,
@@ -314,11 +297,11 @@ TextPrintCubeAnalysisTable ( FILE *pf,
 
   if ( pes->et == EVAL_NONE ) return; /* no evaluation */
 
-  FindCubeDecision ( arDouble, GCCCONSTAHACK aarOutput, pci );
+  FindCubeDecision ( arDouble,  aarOutput, pci );
 
   fActual = fDouble;
   fClose = isCloseCubedecision ( arDouble ); 
-  fMissed = isMissedDouble ( arDouble, GCCCONSTAHACK aarOutput, fDouble, pci );
+  fMissed = isMissedDouble ( arDouble,  aarOutput, fDouble, pci );
 
   fDisplay = 
     ( fActual && exsExport.afCubeDisplay[ EXPORT_CUBE_ACTUAL ] ) ||
@@ -330,13 +313,12 @@ TextPrintCubeAnalysisTable ( FILE *pf,
   if ( ! fDisplay )
     return;
 
-  fputs ( OutputCubeAnalysisFull ( aarOutput,
+  g_string_append(gsz, OutputCubeAnalysisFull ( aarOutput,
                                    aarStdDev,
                                    pes,
                                    pci,
                                    fDouble, fTake, 
-                                   stDouble, stTake ),
-          pf );
+                                   stDouble, stTake ));
 
 }
 
@@ -356,32 +338,43 @@ TextPrintCubeAnalysisTable ( FILE *pf,
  */
 
 static void
-TextPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
+TextPrintCubeAnalysis ( GString *gsz, const matchstate *pms, moverecord *pmr ) {
 
   cubeinfo ci;
+  /* we need to remember the double type to be able to do the right
+   * thing for beavers and racoons */
+  static doubletype dt = DT_NORMAL;
 
   GetMatchStateCubeInfo ( &ci, pms );
 
+
   switch ( pmr->mt ) {
 
   case MOVE_NORMAL:
 
     /* cube analysis from move */
 
-    TextPrintCubeAnalysisTable ( pf, 
-                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput,
-				 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+    TextPrintCubeAnalysisTable ( gsz, 
+                                  pmr->CubeDecPtr->aarOutput,
+				  pmr->CubeDecPtr->aarStdDev,
                                  pmr->fPlayer,
                                  &pmr->CubeDecPtr->esDouble, &ci, FALSE, -1,
                                  pmr->stCube, SKILL_NONE );
+    dt = DT_NORMAL;
 
     break;
 
   case MOVE_DOUBLE:
 
-    TextPrintCubeAnalysisTable ( pf, 
-                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput, 
-				 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+    dt = DoubleType ( pms->fDoubled, pms->fMove, pms->fTurn );
+    if (dt != DT_NORMAL)
+    {
+	    g_string_append(gsz, _("Cannot analyse beaver nor raccoons!\n"));
+	    break;
+    }
+    TextPrintCubeAnalysisTable ( gsz, 
+                                  pmr->CubeDecPtr->aarOutput, 
+				  pmr->CubeDecPtr->aarStdDev,
                                  pmr->fPlayer,
                                  &pmr->CubeDecPtr->esDouble, 
 								 &ci, TRUE, -1,
@@ -394,9 +387,15 @@ TextPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
 
     /* cube analysis from double, {take, drop, beaver} */
 
-    TextPrintCubeAnalysisTable ( pf, 
-                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput, 
-				 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+    if (dt != DT_NORMAL)
+    {
+	    dt = DT_NORMAL;
+	    g_string_append(gsz, _("Cannot analyse beaver nor raccoons!\n"));
+	    break;
+    }
+    TextPrintCubeAnalysisTable ( gsz, 
+                                  pmr->CubeDecPtr->aarOutput, 
+				  pmr->CubeDecPtr->aarStdDev,
                                  pmr->fPlayer,
                                  &pmr->CubeDecPtr->esDouble, 
 								 &ci, TRUE, 
@@ -408,7 +407,7 @@ TextPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
 
   default:
 
-    assert ( FALSE );
+    g_assert ( FALSE );
 
 
   }
@@ -431,11 +430,11 @@ TextPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
  */
 
 static void
-TextPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
+TextPrintMoveAnalysis ( GString *gsz, const matchstate *pms, moverecord *pmr ) {
 
   char szBuf[ 1024 ];
   char sz[ 64 ];
-  int i;
+  unsigned int i;
 
   cubeinfo ci;
 
@@ -452,16 +451,16 @@ TextPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
 
     /* blunder or error */
 
-    fprintf ( pf, 
+    g_string_append_printf ( gsz, 
               _("Alert: %s move"),
               gettext ( aszSkillType[ pmr->n.stMove ] ) );
     
     if ( !pms->nMatchTo || ( pms->nMatchTo && ! fOutputMWC ) )
-      fprintf ( pf, " (%+7.3f)\n",
+      g_string_append_printf(gsz, " (%+7.3f)\n",
                 pmr->ml.amMoves[ pmr->n.iMove ].rScore -
                 pmr->ml.amMoves[ 0 ].rScore  );
     else
-      fprintf ( pf, " (%+6.3f%%)\n",
+      g_string_append_printf(gsz, " (%+6.3f%%)\n",
                 100.0f *
                 eq2mwc ( pmr->ml.amMoves[ pmr->n.iMove ].rScore, &ci ) - 
                 100.0f *
@@ -473,27 +472,27 @@ TextPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
 
     /* joker */
 
-    fprintf ( pf, 
+    g_string_append_printf(gsz, 
               _("Alert: %s roll!"),
               gettext ( aszLuckType[ pmr->lt ] ) );
     
     if ( !pms->nMatchTo || ( pms->nMatchTo && ! fOutputMWC ) )
-      fprintf ( pf, " (%+7.3f)\n", pmr->rLuck );
+      g_string_append_printf(gsz, " (%+7.3f)\n", pmr->rLuck );
     else
-      fprintf ( pf, " (%+6.3f%%)\n",
+      g_string_append_printf(gsz, " (%+6.3f%%)\n",
                 100.0f * eq2mwc ( pmr->rLuck, &ci ) - 
                 100.0f * eq2mwc ( 0.0f, &ci ) );
 
   }
 
-  fputs ( "\n", pf );
+  g_string_append(gsz, "\n" );
 
-  fprintf( pf, _("Rolled %d%d"), pmr->anDice[ 0 ], pmr->anDice[ 1 ] );
+  g_string_append_printf(gsz, _("Rolled %d%d"), pmr->anDice[ 0 ], pmr->anDice[ 1 ] );
 
   if( pmr->rLuck != ERR_VAL )
-    fprintf( pf, " (%s):\n", GetLuckAnalysis( pms, pmr->rLuck ) );
+    g_string_append_printf(gsz, " (%s):\n", GetLuckAnalysis( pms, pmr->rLuck ) );
   else
-    fprintf( pf, ":" );
+    g_string_append_printf(gsz, ":" );
 
   if ( pmr->ml.cMoves ) {
 	
@@ -501,15 +500,14 @@ TextPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
       if( i >= exsExport.nMoves && i != pmr->n.iMove )
         continue;
 
-      fputc( i == pmr->n.iMove ? '*' : ' ', pf );
-      fputs( FormatMoveHint( szBuf, pms, &pmr->ml, i,
+      g_string_append( gsz, i == pmr->n.iMove ? "*" : " " );
+      g_string_append(gsz,FormatMoveHint( szBuf, pms, &pmr->ml, i,
                              i != pmr->n.iMove ||
                              i != pmr->ml.cMoves - 1 ||
                              pmr->ml.cMoves == 1,
                              exsExport.fMovesDetailProb,
                              exsExport.afMovesParameters 
-                             [ pmr->ml.amMoves[ i ].esMove.et - 1 ] ), 
-             pf );
+                             [ pmr->ml.amMoves[ i ].esMove.et - 1 ] ) );
 
 
     }
@@ -519,19 +517,19 @@ TextPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
 
     if ( pmr->n.anMove[ 0 ] >= 0 )
       /* no movelist saved */
-      fprintf ( pf,
+      g_string_append_printf(gsz,
                 "*    %s\n",
                 FormatMove ( sz, pms->anBoard, pmr->n.anMove ) );
     else 
       /* no legal moves */
       /* FIXME: output equity?? */
-      fprintf ( pf,
+      g_string_append_printf(gsz,
                 "*    %s\n",
                 _("Cannot move" ) );
 
   }
 
-  fputs ( "\n\n", pf );
+  g_string_append(gsz, "\n\n");
 
   return;
 
@@ -551,8 +549,8 @@ TextPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
  *
  */
 
-static void
-TextAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
+extern void
+TextAnalysis ( GString *gsz, const matchstate *pms, moverecord *pmr ) {
 
 
   char sz[ 1024 ];
@@ -562,21 +560,21 @@ TextAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
   case MOVE_NORMAL:
 
     if ( pmr->n.anMove[ 0 ] >= 0 )
-      fprintf ( pf,
+      g_string_append_printf( gsz, 
                 _("* %s moves %s"),
                 ap[ pmr->fPlayer ].szName,
                 FormatMove ( sz, pms->anBoard, pmr->n.anMove ) );
     else if ( ! pmr->ml.cMoves )
-      fprintf ( pf,
+      g_string_append_printf ( gsz,
                 _("* %s cannot move"),
                 ap[ pmr->fPlayer ].szName );
 
-    fputs ( "\n", pf );
+    g_string_append (gsz, "\n" );
 
     if ( exsExport.fIncludeAnalysis ) {
-      TextPrintCubeAnalysis ( pf, pms, pmr );
+      TextPrintCubeAnalysis ( gsz, pms, pmr );
     
-      TextPrintMoveAnalysis ( pf, pms, pmr );
+      TextPrintMoveAnalysis ( gsz, pms, pmr );
     }
 
     break;
@@ -586,17 +584,17 @@ TextAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
   case MOVE_DROP:
 
     if ( pmr->mt == MOVE_DOUBLE ) 
-      fprintf ( pf,
+      g_string_append_printf(gsz,
                 "* %s doubles\n\n",
                 ap[ pmr->fPlayer ].szName );
     else
-      fprintf ( pf,
+      g_string_append_printf(gsz,
                 "* %s %s\n\n",
                 ap[ pmr->fPlayer ].szName,
                 ( pmr->mt == MOVE_TAKE ) ? _("accepts") : _("rejects") );
     
     if ( exsExport.fIncludeAnalysis )
-      TextPrintCubeAnalysis ( pf, pms, pmr );
+      TextPrintCubeAnalysis ( gsz, pms, pmr );
 
     break;
 
@@ -667,10 +665,6 @@ TextPrintComment ( FILE *pf, const moverecord *pmr ) {
   case MOVE_SETCUBEPOS:
     sz = pmr->sz;
     break;
-  case MOVE_TIME:
-	  /* ignore */
-	  break;
-
   }
 
   if ( sz ) {
@@ -796,10 +790,10 @@ TextDumpPlayerRecords ( FILE *pf ) {
  *
  */
 
-static void ExportGameText ( FILE *pf, list *plGame, 
+static void ExportGameText ( FILE *pf, listOLD *plGame, 
                              const int iGame, const int fLastGame ) {
 
-    list *pl;
+    listOLD *pl;
     moverecord *pmr;
     matchstate msExport;
     matchstate msOrig;
@@ -807,6 +801,7 @@ static void ExportGameText ( FILE *pf, list *plGame,
     statcontext *psc = NULL;
     static statcontext scTotal;
     xmovegameinfo *pmgi = NULL;
+    GString *gsz;
 
     if ( ! iGame )
       IniStatcontext ( &scTotal );
@@ -825,7 +820,10 @@ static void ExportGameText ( FILE *pf, list *plGame,
 
         ApplyMoveRecord ( &msExport, plGame, pmr );
 
-        TextPrologue( pf, &msExport, iGame );
+	gsz = g_string_new(NULL);
+        TextPrologue( gsz, &msExport, iGame );
+	fputs(gsz->str, pf);
+	g_string_free(gsz, TRUE);
 
         if ( exsExport.fIncludeMatchInfo )
           TextMatchInfo ( pf, &mi );
@@ -851,23 +849,37 @@ static void ExportGameText ( FILE *pf, list *plGame,
         msExport.anDice[ 0 ] = pmr->anDice[ 0 ];
         msExport.anDice[ 1 ] = pmr->anDice[ 1 ];
 
-        TextBoardHeader ( pf, &msExport, iGame, iMove );
+	gsz = g_string_new(NULL);
+        TextBoardHeader ( gsz, &msExport, iGame, iMove );
+	fputs(gsz->str, pf);
+        g_string_free(gsz, TRUE);
 
         printTextBoard( pf, &msExport );
-        TextAnalysis ( pf, &msExport, pmr );
+	gsz = g_string_new(NULL);
+        TextAnalysis ( gsz, &msExport, pmr );
+	fputs(gsz->str, pf);
+        g_string_free(gsz, TRUE);
         
         iMove++;
 
         break;
 
+      case MOVE_DOUBLE:
       case MOVE_TAKE:
       case MOVE_DROP:
 
-        TextBoardHeader ( pf,&msExport, iGame, iMove );
+	gsz = g_string_new(NULL);
+	TextBoardHeader ( gsz, &msExport, iGame, iMove );
+	fputs(gsz->str, pf);
+	g_string_free(gsz, TRUE);
+
 
         printTextBoard( pf, &msExport );
         
-        TextAnalysis ( pf, &msExport, pmr );
+	gsz = g_string_new(NULL);
+        TextAnalysis ( gsz, &msExport, pmr );
+	fputs(gsz->str, pf);
+        g_string_free(gsz, TRUE);
         
         iMove++;
 
@@ -950,7 +962,7 @@ extern void CommandExportGameText( char *sz ) {
     if( pf != stdout )
 	fclose( pf );
 
-    setDefaultFileName ( sz, PATH_TEXT );
+    setDefaultFileName ( sz);
 }
 
 
@@ -959,7 +971,7 @@ extern void CommandExportGameText( char *sz ) {
 extern void CommandExportMatchText( char *sz ) {
     
     FILE *pf;
-    list *pl;
+    listOLD *pl;
     int nGames;
     char *szCurrent;
     int i;
@@ -980,14 +992,14 @@ extern void CommandExportMatchText( char *sz ) {
 
     for( pl = lMatch.plNext, i = 0; pl != &lMatch; pl = pl->plNext, i++ ) {
 
-      szCurrent = HTMLFilename ( sz, i );
+      szCurrent = filename_from_iGame ( sz, i );
 
       if ( !i ) {
 
         if ( ! confirmOverwrite ( sz, fConfirmSave ) )
           return;
 
-        setDefaultFileName ( sz, PATH_TEXT );
+        setDefaultFileName ( sz);
 
       }
 
@@ -1017,6 +1029,7 @@ extern void CommandExportPositionText( char *sz ) {
     int fHistory;
     moverecord *pmr = getCurrentMoveRecord ( &fHistory );
     int iMove;
+    GString *gsz;
 	
     sz = NextToken( &sz );
     
@@ -1042,7 +1055,10 @@ extern void CommandExportPositionText( char *sz ) {
 	return;
     }
 
-    TextPrologue ( pf, &ms, getGameNumber ( plGame ) );
+    gsz = g_string_new(NULL);
+    TextPrologue ( gsz, &ms, getGameNumber ( plGame ) );
+    fputs(gsz->str, pf);
+    g_string_free(gsz, TRUE);
 
     if ( exsExport.fIncludeMatchInfo )
       TextMatchInfo ( pf, &mi );
@@ -1054,14 +1070,21 @@ extern void CommandExportPositionText( char *sz ) {
     else
       iMove = -1;
 
-    TextBoardHeader ( pf, &ms, 
-                      getGameNumber ( plGame ), iMove );
+    gsz = g_string_new(NULL);
+    TextBoardHeader ( gsz, &ms, 
+		    getGameNumber ( plGame ), iMove );
+    fputs(gsz->str, pf);
+    g_string_free(gsz, TRUE);
+
 
     printTextBoard( pf, &ms );
 
     if( pmr ) {
 
-      TextAnalysis ( pf, &ms, pmr );
+      gsz = g_string_new(NULL);
+      TextAnalysis ( gsz, &ms, pmr );
+      fputs(gsz->str, pf);
+      g_string_free(gsz, TRUE);
 
       if ( exsExport.fIncludeAnnotation )
         TextPrintComment ( pf, pmr );
@@ -1073,7 +1096,7 @@ extern void CommandExportPositionText( char *sz ) {
     if( pf != stdout )
 	fclose( pf );
 
-    setDefaultFileName ( sz, PATH_TEXT );
+    setDefaultFileName ( sz);
 
 }
 
