@@ -6,7 +6,7 @@
 *
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of version 2 of the GNU General Public License as
+* it under the terms of version 3 or later of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
@@ -18,11 +18,12 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: timer.c,v 1.7 2005/09/14 21:37:07 Superfly_Jon Exp $
+* $Id: timer.c,v 1.15 2009/10/31 20:49:28 c_anthon Exp $
 */
 
 #include "config.h"
 #include <time.h>
+#include <backgammon.h>
 #if HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -30,9 +31,9 @@
 #ifdef WIN32
 #include "windows.h"
 
-double perFreq = 0;
+static double perFreq = 0;
 
-int setup_timer()
+static int setup_timer()
 {
 	LARGE_INTEGER freq;
 	if (!QueryPerformanceFrequency(&freq))
@@ -48,7 +49,7 @@ int setup_timer()
 
 double get_time()
 {	/* Return elapsed time in milliseconds */
-	LARGE_INTEGER time;
+	LARGE_INTEGER timer;
 
 	if (!perFreq)
 	{
@@ -56,15 +57,13 @@ double get_time()
 			return clock() / 1000.0;
 	}
 
-	QueryPerformanceCounter(&time);
-	return time.QuadPart / perFreq;
+	QueryPerformanceCounter(&timer);
+	return timer.QuadPart / perFreq;
 }
 
 #else
 
-#if 1
-
-double get_time()
+extern double get_time(void)
 {	/* Return elapsed time in milliseconds */
 	struct timeval tv;
 	gettimeofday(&tv, 0);
@@ -72,31 +71,4 @@ double get_time()
 	return tv.tv_sec * 1000 + tv.tv_usec / 1000;
 }
 
-#else
-
-double perFreq = 0;
-
-int setup_timer()
-{
-    perFreq = __get_clockfreq() / 1000.0;
-     return 1;
-}
-
-double get_time()
-{    /* Return elapsed time in milliseconds */
-    if (!perFreq)
-    {
-        if (!setup_timer())
-            return clock() / 1000.0;
-    }
-{
-    unsigned long long int val;
-    __asm__ __volatile__("rdtsc" : "=A" (val) : );
-
-    return val / perFreq;
-}
-}
-
-#endif
-
 #endif
